function [ sim ] = BOWsimilarity( BOW, type, options )
%BOWSIMILARITY Summary of this function goes here
%   Detailed explanation goes here

if nargin == 1,
    type = 'histInter';
end

nrWords = size(BOW, 2);

switch type,
    case 'histInter'
        sim = NaN(nrWords);
        for i = 1:nrWords,
            for j = 1:nrWords,
                nrm = min(sum(BOW(:,i)), sum(BOW(:,j)));
                if nrm < 1e-6,
                    % warning('Maybe you supplied an invalid bow.');
                    nrm = 1;
                end
                %sim(i,j) = exp(-norm(BOW(:,i)- BOW(:,j),1));
                sim(i,j) = sum(min(BOW(:,i), BOW(:,j))) / nrm;
                sim(j,i) = sim(i,j);
            end
        end
    case 'l1exp'
        sim = NaN(nrWords);
        dist = NaN(nrWords);
        for i = 1:nrWords,
            for j = 1:nrWords,
                dist(i,j) = pdist2(BOW(:,i)', BOW(:,j)', 'L1');
            end
        end
        for i = 1:nrWords,
            for j = 1:nrWords,
                sim(i,j) = exp(-dist(i,j) / 4);
            end
        end
    case 'emd'
        if nargin < 3,
            error('You must supply options of histogram generation for this to make sense.');
        end
        
        sim = NaN(nrWords);
        warning('Assuming quantizer type was scalar.');
        bins = options.bins;
        dist = NaN(nrWords);
        for i = 1:nrWords,
            for j = 1:nrWords,
                dist(i,j) = 0;
                idx = 1;
                for k = 1:3,
                    idxE = idx + bins(k) -1;
                    tmp = pdist2(BOW(idx:idxE,i)', BOW(idx:idxE,j)', 'emd');
                    dist(i,j) = dist(i,j) + tmp;
                    idx = idx + bins(k);
                end
            end
        end
        for i = 1:nrWords,
            for j = 1:nrWords,
                sim(i,j) = exp(-dist(i,j) / 4);
            end
        end
    otherwise
        error('Invalid type.');
end

if any(any(isnan(sim))),
    error('Invalid similarity.');
end

end

