#include <cuda.h>
#include "device_functions.h"

extern "C"
__global__ void Fidelity(int nrS, int* globS, int nrCandidates, int *globCandidates, int nrElements, int nrWords, float *bow, float threshold, float *val)
{
   extern __shared__ float s[];
   int *S = (int*) s;
   float *sumbow = s + nrS + 1;
   float *tempSum = sumbow + nrWords;
   float *output = &val[blockIdx.x];
   
   int i = threadIdx.x;
   
   // Copy S to internal memory
   if (i < nrS) {
     S[i] = globS[i];
   }
   S[nrS] = globCandidates[blockIdx.x];
   __syncthreads();
   
   // compute sum bow 
   while (i < nrWords) {
     sumbow[i] = 0.0;
     
     int bowIdx = i*nrElements - 1;
     for (int j = 0; j <= nrS; j++) {
       sumbow[i] += bow[bowIdx + S[j]];
     }
     
     i += blockDim.x;
   } 
   __syncthreads();
   
   // sum up over words and threshold results
   i = threadIdx.x;
   tempSum[i] = 0.0;
   int idx = i;
   for (int j = 0; j < nrWords; j++) {
     tempSum[i] += fminf(sumbow[j], bow[idx]);
     idx += nrElements;
   }
   //int idx = i*nrWords;
   //for (int j = 0; j < nrWords; j++) {
   //  tempSum[i] += fminf(sumbow[j], bowT[idx]);
     idx++;
   //}
   tempSum[i] = fminf(tempSum[i], threshold);
   __syncthreads(); 
   
   // sum
   if (threadIdx.x == 0) {
      float tval = 0.0;
      for (int k = 0; k < 100; k++) {
        tval += tempSum[k];
      }
      *output = tval / (100.0 * threshold);
   } 
}

