public class ClusteredDiversity implements Function {
	private final int nrClusters;
	private final int[] clusterAssignment;
	private final double[] rewards;
	private final double a;
	private final double normalization;
	
	public ClusteredDiversity(int[] clusterAssignment, double[] rewards, double a) {
		this.clusterAssignment = clusterAssignment;
		this.rewards = rewards;
		this.a = a;
		
		// number of clusters
		int nrClusters = 0;
		for (int i = 0; i < clusterAssignment.length; i++) {
			if (clusterAssignment[i] > nrClusters) {
				nrClusters = clusterAssignment[i];
			}
		}
		this.nrClusters = nrClusters;
		
		// sanity check
		for (int i = 0; i < clusterAssignment.length; i++) {
			if (clusterAssignment[i] <= 0 || clusterAssignment[i] > nrClusters) {
				throw new RuntimeException("Invalid cluster assignments.");
			}
		}
		
		// compute normalization
		double[] accumRewards = new double[nrClusters];
		for (int i = 1; i <= clusterAssignment.length; i++) {
			int cluster = clusterAssignment[i - 1] - 1;
			accumRewards[cluster] += rewards[i - 1];
		}
		
		double normalization = 0.0;
		for (int i = 0; i < nrClusters; i++) {
			normalization += Math.pow(accumRewards[i], a);
		}
		
		this.normalization = normalization;
	}
	
	public double eval(int[] S) {
		if (S == null) {
			return 0.0;
		}
		
		double[] accumRewards = new double[nrClusters];
		for (int i : S) {
			int cluster = clusterAssignment[i - 1] - 1;
			accumRewards[cluster] += rewards[i - 1];
		}
		
		double val = 0.0;
		for (int i = 0; i < nrClusters; i++) {
			val += Math.pow(accumRewards[i], a);
		}
		val /= normalization;
		
		return val;
	}
	
	public double[] eval(int[] B, int[] candidates) {
		if (candidates == null)
			candidates = new int[] {};
		
		int nrCandidates = candidates.length;
		double[] values = new double[nrCandidates];
		
		double[] accumRewards = new double[nrClusters];
		for (int i : B) {
			int cluster = clusterAssignment[i - 1] - 1;
			accumRewards[cluster] += rewards[i - 1];
		}
		
		for (int c = 0; c < nrCandidates; c++) {
			int candidate = candidates[c];
			int cluster = clusterAssignment[candidate - 1] - 1;
			accumRewards[cluster] += rewards[candidate - 1];
			for (int i = 0; i < nrClusters; i++) {
				values[c] += Math.pow(accumRewards[i], a);
			}
			accumRewards[cluster] -= rewards[candidate - 1];
			values[c] /= normalization;
		}
		
		return values;
	}
	
	// >> For octave compatibility >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
	
	public double eval(int St) {
		int[] S = new int[] { St };
		return eval(S);
	}
	
	public double[] eval(int St, int[] candidates) {
		int[] S = new int[] { St };
		return eval(S, candidates);
	}
	
	public double[] eval(int[] S, int candidatest) {
		int[] candidates = new int[] { candidatest };
		return eval(S, candidates);
	}
}
