public class CompressedWordCount implements Function {
	private final double[][] bow;
	private final double normalization;
	private final int nrWords;
	private final int nrImages;
	
	public CompressedWordCount(Matrix bow) {
		this(bow.getMatrix());
	}
	
	public CompressedWordCount(double[][] bow) {
		this.bow = bow;
		this.nrWords = bow.length;
		this.nrImages = bow[0].length;
		
		// compute normalization
		double normalization = 0.0;
		for (int i = 0; i < nrWords; i++) {
			double[] imBow = bow[i];
			double tval = 0.0;
			for (int j = 0; j < nrImages; j++) {
				tval += imBow[j];
			}
			normalization += Math.sqrt(tval);
		}
		this.normalization = normalization;
	}
	
	public double eval(int[] S) {
		if (S == null) {
			return 0.0;
		}
		
		// Function:
		// sum(sqrt(sum(bow(:,S),2))) / sum(sqrt(sum(bow,2)))
		
		// sumBow = sum(bow(:, S), 2);
		double[] sumBow = new double[nrWords];
		for (int j = 0; j < nrWords; j++) {
			double[] imBow = bow[j];
			for (int i : S) {
				sumBow[j] += imBow[i - 1];
			}
			
		}
		
		double val = 0.0;
		for (int j = 0; j < nrWords; j++) {
			val += Math.sqrt(sumBow[j]);
		}
		return val / normalization;
	}
	
	public double[] eval(int[] B, int[] candidates) {
		if (candidates == null)
			candidates = new int[] {};
		
		int nrCandidates = candidates.length;
		double[] values = new double[nrCandidates];
		
		// Function:
		// sum(sqrt(sum(bow(:,S),2))) / sum(sqrt(sum(bow,2)))
		
		// sumBow = sum(bow(:, S), 2);
		double[] sumBow = new double[nrWords];
		for (int j = 0; j < nrWords; j++) {
			double[] imBow = bow[j];
			for (int i : B) {
				sumBow[j] += imBow[i - 1];
			}
			
		}
		
		for (int c = 0; c < nrCandidates; c++) {
			int candidate = candidates[c];
			for (int j = 0; j < nrWords; j++) {
				values[c] += Math.sqrt(sumBow[j] + bow[j][candidate - 1]);
			}
			values[c] /= normalization;
		}
		
		return values;
	}
	
	// >> For octave compatibility >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
	
	public double eval(int St) {
		int[] S = new int[] { St };
		return eval(S);
	}
	
	public double[] eval(int St, int[] candidates) {
		int[] S = new int[] { St };
		return eval(S, candidates);
	}
	
	public double[] eval(int[] S, int candidatest) {
		int[] candidates = new int[] { candidatest };
		return eval(S, candidates);
	}
	
}
