import java.util.Arrays;

public class CutCoverage implements Function {
	private final double[][] sim;
	private final double normalization;
	
	public CutCoverage(Matrix sim) {
		this(sim.getMatrix());
	}
	
	public CutCoverage(double[][] sim) {
		this.sim = sim;
		
		double normalization = 0;
		for (int i = 0; i < sim.length; i++) {
			for (int j = 0; j < sim[0].length; j++) {
				normalization += sim[i][j];
			}
		}
		this.normalization = normalization;
	}
	
	public double eval(int[] S) {
		if (S == null)
			S = new int[] {};
		
		// To be safe, sort S
		Arrays.sort(S);
		
		// find S complement
		int len = sim.length;
		
		int[] Scompl = new int[len - S.length];
		int idx1 = 0;
		int idx2 = 0;
		for (int i = 1; i <= len; i++) {
			if (idx1 < S.length && S[idx1] == i) {
				idx1++;
			} else {
				Scompl[idx2] = i;
				idx2++;
			}
		}
		
		// compute function
		double val = 0;
		for (int j : Scompl) {
			for (int i : S) {
				val += sim[i - 1][j - 1];
			}
		}
		return val / normalization;
	}
	
	@Override
	public double[] eval(int[] B, int[] cand) {
		if (cand == null)
			cand = new int[] {};
		
		int nrCandidates = cand.length;
		double[] values = new double[nrCandidates];
		
		int[] S = new int[B.length + 1];
		for (int c = 0; c < nrCandidates; c++) {
			int candidate = cand[c];
			for (int i = 0; i < B.length; i++) {
				S[i] = B[i];
			}
			S[B.length] = candidate;
			Arrays.sort(S);
			
			values[c] = eval(S);
		}
		
		return values;
	}
	
	// >> For octave compatibility >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
	
	public double eval(int St) {
		int[] S = new int[] { St };
		return eval(S);
	}
	
	public double[] eval(int St, int[] candidates) {
		int[] S = new int[] { St };
		return eval(S, candidates);
	}
	
	public double[] eval(int[] S, int candidatest) {
		int[] candidates = new int[] { candidatest };
		return eval(S, candidates);
	}
}
