public class EncodingAccuracy implements Function {
	private final double[][] encodingCosts;
	private final double normalization;
	
	public EncodingAccuracy(Matrix encodingCosts) {
		this(encodingCosts.getMatrix());
	}
	
	public EncodingAccuracy(double[][] encodingCosts) {
		this.encodingCosts = encodingCosts;
		
		this.normalization = 2.0 * encodingCosts.length;
	}
	
	public double eval(int[] S) {
		if (S == null)
			S = new int[] {};
		
		// compute function
		double val = 0;
		int len = encodingCosts.length;
		for (int j = 0; j < len; j++) {
			double minVal = Double.POSITIVE_INFINITY;
			for (int i : S) {
				if (encodingCosts[j][i - 1] < minVal) {
					minVal = encodingCosts[j][i - 1];
				}
			}
			val += minVal;
		}
		return 1.0 - (val / normalization);
	}
	
	public double[] eval(int[] B, int[] candidates) {
		if (candidates == null)
			candidates = new int[] {};
		
		int nrCandidates = candidates.length;
		double[] values = new double[nrCandidates];
		
		// compute function
		int len = encodingCosts.length;
		double[] baseCosts = new double[len];
		for (int j = 0; j < len; j++) {
			double minVal = Double.POSITIVE_INFINITY;
			for (int i : B) {
				if (encodingCosts[j][i - 1] < minVal) {
					minVal = encodingCosts[j][i - 1];
				}
			}
			baseCosts[j] = minVal;
		}
		
		for (int c = 0; c < nrCandidates; c++) {
			int candidate = candidates[c];
			double val = 0.0;
			for (int j = 0; j < len; j++) {
				if (baseCosts[j] < encodingCosts[j][candidate - 1]) {
					val += baseCosts[j];
				} else {
					val += encodingCosts[j][candidate - 1];
				}
			}
			values[c] = 1 - (val / normalization);
		}
		
		return values;
	}
	
	// >> For octave compatibility >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
	
	public double eval(int St) {
		int[] S = new int[] { St };
		return eval(S);
	}
	
	public double[] eval(int St, int[] candidates) {
		int[] S = new int[] { St };
		return eval(S, candidates);
	}
	
	public double[] eval(int[] S, int candidatest) {
		int[] candidates = new int[] { candidatest };
		return eval(S, candidates);
	}
}
