public class FacilityLocation implements Function {
	private final double[][] sim;
	private final int nrElements;
	
	public FacilityLocation(double[][] sim) {
		this.sim = sim;
		this.nrElements = sim.length;
	}
	
	public FacilityLocation(Matrix sim) {
		this(sim.getMatrix());
	}
	
	public double eval(int[] S) {
		if (S == null) {
			return 0.0;
		}
		
		double val = 0.0;
		for (int j = 0; j < nrElements; j++) {
			double maxVal = 0.0;
			for (int i : S) {
				if (sim[j][i - 1] > maxVal) {
					maxVal = sim[j][i - 1];
				}
			}
			val += maxVal;
		}
		return val / 100.0;
	}
	
	public double[] eval(int[] B, int[] candidates) {
		if (candidates == null)
			candidates = new int[] {};
		
		int nrCandidates = candidates.length;
		double[] values = new double[nrCandidates];
		
		// compute maximum for all elements in B
		double[] maxValues = new double[nrElements];
		for (int j = 0; j < nrElements; j++) {
			double maxVal = Double.NEGATIVE_INFINITY;
			for (int i : B) {
				if (sim[j][i - 1] > maxVal) {
					maxVal = sim[j][i - 1];
				}
			}
			maxValues[j] = maxVal;
		}
		
		// evaluate candidates
		for (int c = 0; c < nrCandidates; c++) {
			int candidate = candidates[c];
			double val = 0.0;
			
			for (int j = 0; j < nrElements; j++) {
				double maxVal = maxValues[j];
				if (sim[j][candidate - 1] > maxVal) {
					val += sim[j][candidate - 1];
				} else {
					val += maxVal;
				}
			}
			
			values[c] = val / 100.0;
		}
		
		return values;
	}
	
	// >> For octave compatibility >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
	
	public double eval(int St) {
		int[] S = new int[] { St };
		return eval(S);
	}
	
	public double[] eval(int St, int[] candidates) {
		int[] S = new int[] { St };
		return eval(S, candidates);
	}
	
	public double[] eval(int[] S, int candidatest) {
		int[] candidates = new int[] { candidatest };
		return eval(S, candidates);
	}
}
