public class Fidelity implements Function {
	private double[][] bow;
	private double threshold;
	private int nrWords;
	private int nrImages;
	
	public Fidelity(Matrix bow, double threshold) {
		this(bow.getMatrix(), threshold);
	}
	
	public Fidelity(double[][] bow, double threshold) {
		this.bow = bow;
		this.threshold = threshold;
		this.nrWords = bow.length;
		this.nrImages = bow[0].length;
	}
	
	public double eval(int[] S) {
		if (S == null)
			S = new int[] {};
		
		// sumBow = sum(bow(:, S), 2);
		double[] sumBow = new double[nrWords];
		for (int j = 0; j < nrWords; j++) {
			double[] imBow = bow[j];
			for (int i : S) {
				sumBow[j] += imBow[i - 1];
			}
			
		}
		
		// sim = min(repmat(sumBow, 1, 100), bow);
		// val = min(sum(sim, 1), alpha*ones(1,100));
		double[] sum = new double[nrImages];
		for (int j = 0; j < bow.length; j++) {
			double sumBowVal = sumBow[j];
			double[] imBow = bow[j];
			for (int i = 0; i < nrImages; i++) {
				if (sumBowVal < imBow[i])
					sum[i] += sumBowVal;
				else
					sum[i] += imBow[i];
			}
		}
		
		// val = sum(val) / (100*alpha);
		double val = 0;
		for (int j = 0; j < nrImages; j++) {
			if (sum[j] < threshold)
				val += sum[j];
			else
				val += threshold;
		}
		return val / (100.0 * threshold);
	}
	
	@Override
	public double[] eval(int[] B, int[] cand) {
		if (cand == null)
			cand = new int[] {};
		
		int nrCandidates = cand.length;
		double[] values = new double[nrCandidates];
		
		// sumBow = sum(bow(:, S), 2);
		double[] sumBow = new double[nrWords];
		for (int j = 0; j < nrWords; j++) {
			double[] imBow = bow[j];
			for (int i : B) {
				sumBow[j] += imBow[i - 1];
			}
		}
		
		// go on
		
		// sim = min(repmat(sumBow, 1, 100), bow);
		// val = min(sum(sim, 1), alpha*ones(1,100));
		double[][] sum = new double[nrImages][nrCandidates];
		for (int c = 0; c < nrCandidates; c++) {
			int candidate = cand[c];
			for (int j = 0; j < bow.length; j++) {
				double sumBowVal = sumBow[j] + bow[j][candidate - 1];
				double[] imBow = bow[j];
				for (int i = 0; i < nrImages; i++) {
					if (sumBowVal < imBow[i])
						sum[i][c] += sumBowVal;
					else
						sum[i][c] += imBow[i];
				}
			}
		}
		
		// val = sum(val) / (100*alpha);
		for (int c = 0; c < nrCandidates; c++) {
			double val = 0.0;
			for (int j = 0; j < nrImages; j++) {
				if (sum[j][c] < threshold)
					val += sum[j][c];
				else
					val += threshold;
			}
			values[c] = val / (100.0 * threshold);
		}
		return values;
	}
	
	// >> For octave compatibility >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
	
	public double eval(int St) {
		int[] S = new int[] { St };
		return eval(S);
	}
	
	public double[] eval(int St, int[] candidates) {
		int[] S = new int[] { St };
		return eval(S, candidates);
	}
	
	public double[] eval(int[] S, int candidatest) {
		int[] candidates = new int[] { candidatest };
		return eval(S, candidates);
	}
	
}
