import static org.junit.Assert.*;

import java.util.Random;

import org.junit.Test;


public class FidelityTest {

	@Test
	public void test() {
		int nrSummaries = 400;
		int nrWords = 1000;
		int nrImages = 100;
		int summaryLength = 10;
		
		// create summaries, bow and bowSummaries
		Random rg = new Random(12345);
		int[][] summaries = UtilsTest.generateSummaries(nrSummaries, summaryLength, nrImages, rg);
		double[][] bow = UtilsTest.generateBOW(nrImages, nrWords, rg);
		double[][] bowSummaries = UtilsTest.computeBOWofSummaries(summaries, bow);
		
		// create rouge loss
		Fidelity fidelity = new Fidelity(bow, 1000.0);
		
		// time
		int[] S = new int[1];
		int[] cand = new int[50];
		for (int j = 0; j < S.length; j++) {
			int idx = (rg.nextInt() % nrImages);
			if (idx < 0)
				idx = -idx;
			S[j] = idx + 1;
		}
		for (int j = 0; j < cand.length; j++) {
			cand[j] = j + 1;
		}
		long tstart = System.nanoTime();
		double val1 = fidelity.eval(S);
		long tend = System.nanoTime();
		System.out.println("Evaluation took " + ((double) (tend-tstart))/1000.0 + " us.");
		System.out.println("Val classic: " + val1);
		
		double val2 = fidelity.eval(S);
		System.out.println("Val fast: " + val2);
		
		// Output: 0.1055835265704429
		assertEquals(0.3362774045451081, val1, 1e-6);
		assertEquals(0.3362774045451081, val2, 1e-6);
	}

}
