import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;

public class Greedy {
	private Function function;
	
	public Greedy(Function function) {
		this.function = function;
	}
	
	public int[] maximize(int nrElements, int len) {
		Set<Integer> selected = new HashSet<Integer>();
		double curScore = 0.0;
		for (int i = 0; i < len; i++) {
			List<Integer> candidates = new ArrayList<Integer>(nrElements);
			for (int j = 1; j <= nrElements; j++) {
				candidates.add(j);
			}
			candidates.removeAll(selected);
			
			double[] gains = new double[candidates.size()];
			int j = 0;
			for (Integer e : candidates) {
				int[] Stest = new int[selected.size() + 1];
				int k = 0;
				for (Integer e2 : selected) {
					Stest[k] = e2;
					k++;
				}
				Stest[k] = e; // add candidate
				
				gains[j] = function.eval(Stest); // - curScore;
				
				j++;
			}
			
			// find maximum
			double maxGain = Double.NEGATIVE_INFINITY;
			Integer maxEl = -1;
			j = 0;
			for (Integer e : candidates) {
				if (gains[j] >= maxGain) {
					maxGain = gains[j];
					maxEl = e;
				}
				j++;
			}
			
			selected.add(maxEl);
			curScore = curScore + maxGain;
		}
		
		int[] S = new int[selected.size()];
		int i = 0;
		for (Integer e : selected) {
			S[i] = e;
			i++;
		}
		return S;
	}
	
	// /////
	public int[] maximizeadv(int nrElements, int len) {
		Set<Integer> selected = new HashSet<Integer>();
		double curScore = 0.0;
		for (int i = 0; i < len; i++) {
			List<Integer> candidates = new ArrayList<Integer>(nrElements);
			for (int j = 1; j <= nrElements; j++) {
				candidates.add(j);
			}
			candidates.removeAll(selected);
			
			int[] candidatesInt = new int[candidates.size()];
			int j = 0;
			for (Integer e : candidates) {
				candidatesInt[j] = e;
				j++;
			}
			
			int[] selectedInt = new int[selected.size()];
			j = 0;
			for (Integer e : selected) {
				selectedInt[j] = e;
				j++;
			}
			
			double[] gains = function.eval(selectedInt, candidatesInt);
			
			// find maximum
			double maxGain = Double.NEGATIVE_INFINITY;
			Integer maxEl = -1;
			j = 0;
			for (Integer e : candidates) {
				if (gains[j] > maxGain) {
					maxGain = gains[j];
					maxEl = e;
				}
				j++;
			}
			if (maxEl == -1) {
				throw new RuntimeException("No gain was larger than -inf.");
			}
			
			selected.add(maxEl);
			curScore = curScore + maxGain;
		}
		
		int[] S = new int[selected.size()];
		int i = 0;
		for (Integer e : selected) {
			S[i] = e;
			i++;
		}
		return S;
	}
	
	// ///////
	
	public int[] maximizep(int nrElements, int len) {
		try {
			Broker2 broker = new Broker2();
			
			int nrThreads = 8;
			ExecutorService threadPool = Executors.newFixedThreadPool(nrThreads);
			Collection<Callable<Object>> threads = new ArrayList<Callable<Object>>();
			double[] gains = new double[nrElements];
			for (int k = 0; k < nrThreads; k++) {
				threads.add(Executors.callable(new GreedyConsumer(k, broker, gains, function)));
			}
			
			Set<Integer> selected = new HashSet<Integer>();
			double curScore = 0.0;
			for (int i = 0; i < len; i++) {
				List<Integer> candidates = new ArrayList<Integer>();
				for (int j = 1; j <= nrElements; j++) {
					candidates.add(j);
				}
				candidates.removeAll(selected);
				
				int j = 0;
				for (Integer e : candidates) {
					int[] Stest = new int[selected.size() + 1];
					int k = 0;
					for (Integer e2 : selected) {
						Stest[k] = e2;
						k++;
					}
					Stest[k] = e; // add candidate
					
					JobPair a = new JobPair(j, Stest);
					broker.put(a);
					
					j++;
				}
				
				threadPool.invokeAll(threads);
				
				j = 0;
				for (Integer e : candidates) {
					gains[j] -= curScore;
					
					j++;
				}
				
				// find maximum
				double maxGain = Double.NEGATIVE_INFINITY;
				Integer maxEl = -1;
				j = 0;
				for (Integer e : candidates) {
					if (gains[j] >= maxGain) {
						maxGain = gains[j];
						maxEl = e;
					}
					j++;
				}
				
				selected.add(maxEl);
				curScore = curScore + maxGain;
			}
			
			threadPool.shutdown();
			threadPool.awaitTermination(Integer.MAX_VALUE, TimeUnit.SECONDS);
			
			int[] S = new int[selected.size()];
			int i = 0;
			for (Integer e : selected) {
				S[i] = e;
				i++;
			}
			return S;
			
		} catch (Exception e) {
			e.printStackTrace();
			
		}
		
		return null;
	}
}
