import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.Set;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;

public class GreedyNew {
	private Function function;
	private int nrThreads;
	private final Broker2New broker;
	private ExecutorService threadPool;
	private Collection<Callable<Object>> threads;
	
	public GreedyNew() {
	  this.nrThreads = 1;
	  this.function = null;
	  
	  broker = new Broker2New();
	  
	  startPool();
	}
	
	public GreedyNew(Function function) {
	  this();
	  this.function = function;
	}
	
	public void setNrThreads(int nrThreads) throws InterruptedException {
	  if (threadPool != null) {
	    shutdownPool();
	  }
	  this.nrThreads = nrThreads;
	  
	  startPool();
	}
	
	protected void finalize() throws InterruptedException {
	    shutdownPool();
	}
	
	private void startPool() {
	  threadPool = Executors.newFixedThreadPool(nrThreads);
	  threads = new ArrayList<Callable<Object>>();
	  for (int k = 0; k < nrThreads; k++) {
		  threads.add(Executors.callable(new GreedyConsumerNew(k, broker)));
	  }
	}
	
	private void shutdownPool() throws InterruptedException {
	    threadPool.shutdown();
	    threadPool.awaitTermination(Integer.MAX_VALUE, TimeUnit.SECONDS);	    
	}
	
	public int[] maximize(int nrElements, int len) {
		Set<Integer> selected = new HashSet<Integer>();
		double curScore = 0.0;
		for (int i = 0; i < len; i++) {
			Set<Integer> candidates = new HashSet<Integer>();
			for (int j = 1; j <= 100; j++) {
				candidates.add(j);
			}
			candidates.removeAll(selected);
			
			double[] gains = new double[candidates.size()];
			int j = 0;
			for (Integer e : candidates) {
				int[] Stest = new int[selected.size() + 1];
				int k = 0;
				for (Integer e2 : selected) {
					Stest[k] = e2;
					k++;
				}
				Stest[k] = e; // add candidate
				
				gains[j] = function.eval(Stest) - curScore;
				
				j++;
			}
			
			// find maximum
			double maxGain = Double.NEGATIVE_INFINITY;
			Integer maxEl = -1;
			j = 0;
			for (Integer e : candidates) {
				if (gains[j] >= maxGain) {
					maxGain = gains[j];
					maxEl = e;
				}
				j++;
			}
			
			selected.add(maxEl);
			curScore = curScore + maxGain;
		}
		
		int[] S = new int[selected.size()];
		int i = 0;
		for (Integer e : selected) {
			S[i] = e;
			i++;
		}
		return S;
	}
	
	// ///////
	
	public int[] maximizep(int nrElements, int len) {
		try {		
			Set<Integer> selected = new HashSet<Integer>();
			double curScore = 0.0;
			double[] gains = new double[nrElements];
			for (int i = 0; i < len; i++) {
				Set<Integer> candidates = new HashSet<Integer>();
				for (int j = 1; j <= 100; j++) {
					candidates.add(j);
				}
				candidates.removeAll(selected);
				
				int j = 0;
				for (Integer e : candidates) {
					int[] Stest = new int[selected.size() + 1];
					int k = 0;
					for (Integer e2 : selected) {
						Stest[k] = e2;
						k++;
					}
					Stest[k] = e; // add candidate
					
					JobPairNew a = new JobPairNew(j, Stest, function, gains);
					broker.put(a);
					
					j++;
				}
				
				threadPool.invokeAll(threads);
				
				j = 0;
				for (Integer e : candidates) {
					gains[j] -= curScore;
					
					j++;
				}
				
				// find maximum
				double maxGain = Double.NEGATIVE_INFINITY;
				Integer maxEl = -1;
				j = 0;
				for (Integer e : candidates) {
					if (gains[j] >= maxGain) {
						maxGain = gains[j];
						maxEl = e;
					}
					j++;
				}
				
				selected.add(maxEl);
				curScore = curScore + maxGain;
			}
			
			int[] S = new int[selected.size()];
			int i = 0;
			for (Integer e : selected) {
				S[i] = e;
				i++;
			}
			return S;
			
		} catch (Exception e) {
			e.printStackTrace();
			
		}
		
		return null;
	}
}
