public class Indicator implements Function {
	private final int idx;
	
	public Indicator(int idx) {
		this.idx = idx;
	}
	
	public double eval(int[] S) {
		if (S == null) {
			return 0.0;
		}
		
		for (int i : S) {
			if (i == idx) {
				return 1.0;
			}
		}
		
		return 0.0;
	}
	
	public double[] eval(int[] B, int[] cand) {
		if (cand == null)
			cand = new int[] {};
		
		int nrCandidates = cand.length;
		double[] values = new double[nrCandidates];
		
		for (int i : B) {
			if (i == idx) {
				for (int c = 0; c < nrCandidates; c++) {
					values[c] = 1.0;
				}
				return values;
			}
		}
		
		for (int c = 0; c < nrCandidates; c++) {
			int candidate = cand[c];
			if (candidate == idx) {
				values[c] = 1.0;
			}
		}
		
		return values;
	}
	
	// >> For octave compatibility >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
	
	public double eval(int St) {
		int[] S = new int[] { St };
		return eval(S);
	}
	
	public double[] eval(int St, int[] candidates) {
		int[] S = new int[] { St };
		return eval(S, candidates);
	}
	
	public double[] eval(int[] S, int candidatest) {
		int[] candidates = new int[] { candidatest };
		return eval(S, candidates);
	}
}
