import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.Random;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.TimeUnit;

public class LinCombFunction implements Function {
	
	private ArrayList<Function> functions;
	private double pruneThreshold;
	private double[] weights;
	private int[] order;
	
	public LinCombFunction() {
		functions = new ArrayList<Function>();
		this.pruneThreshold = 0;
		weights = new double[0];
		
		// ----
		/*
		 * // Enable exceptions and omit all subsequent error checks JCudaDriver.setExceptionsEnabled(true); // Initialize the driver and create a context for the first device. int errorFlag =
		 * cudaSetDeviceFlags(cudaDeviceMapHost); System.out.println("Error when setting flag: " + errorFlag); cuInit(0); CUdevice device = new CUdevice(); cuDeviceGet(device, 0); errorFlag =
		 * cudaSetDeviceFlags(cudaDeviceMapHost); System.out.println("Error when setting flag: " + errorFlag); CUcontext context = new CUcontext(); cuCtxCreate(context, cudaDeviceMapHost, device); //
		 * Check if the device supports mapped host memory cudaDeviceProp deviceProperties = new cudaDeviceProp(); cudaGetDeviceProperties(deviceProperties, 0); if (deviceProperties.canMapHostMemory
		 * == 0) { System.err.println("This device can not map host memory"); System.err.println(deviceProperties.toFormattedString()); System.exit(-1); } //System.out.println("PROPS: " +
		 * deviceProperties); System.out.println("We can use mapped memory."); // Set the flag indicating that mapped memory will be used //cudaSetDeviceFlags(cudaDeviceMapHost);
		 */
		
	}
	
	public void setPruneThreshold(double pruneThreshold) {
		this.pruneThreshold = pruneThreshold;
	}
	
	// For octave compatibility
	/*public void setWeights(double weight) {
		weights = new double[1];
		weights[0] = weight;
	}*/
	
	public void setWeights(double[] weights) {
	  	if (weights == null)
	  	  	weights = new double[0];
		this.weights = weights;
	}
	
	public void addFunction(Function fct) {
		functions.add(fct);
	}
	
	// For octave compatibility
	public double eval(int St) {
		int[] S = new int[1];
		S[0] = St;
		return eval(S);
	}
	
	public void timeFunctions() {
		timeFunctions(false);
	}
	
	public void timeFunctions(boolean verbose) {
		int nrTries = 10;
		int nrFunctions = functions.size();
		
		List<Integer> SList = new ArrayList<Integer>();
		int[] S = new int[10];
		Random rg = new Random();
		for (int j = 0; j < S.length; j++) {
			// int idx = (rg.nextInt() % 100);
			// if (idx < 0)
			// idx = -idx;
			int idx = j;
			S[j] = idx + 1;
			SList.add(idx + 1);
		}
		List<Integer> candList = new ArrayList<Integer>();
		for (int j = 1; j <= 100; j++) {
			candList.add(j);
		}
		candList.removeAll(SList);
		int[] cand = new int[candList.size()];
		for (int i = 0; i < candList.size(); i++) {
			cand[i] = candList.get(i);
		}
		
		Arrays.sort(S);
		final int[] runtimes = new int[nrFunctions];
		for (int f = 0; f < nrFunctions; f++) {
			long tstart = System.nanoTime();
			
			try {
				for (int t = 0; t < nrTries; t++) {
					functions.get(f).eval(S, cand);
				}
			} catch (java.lang.ArrayIndexOutOfBoundsException e) {
				System.out.println("Error in timing.");
			}
			
			long tend = System.nanoTime();
			runtimes[f] = (int) (tend - tstart);
			if (verbose) {
				System.out.println("Function " + f + " (" + functions.get(f) + ") took " + (tend - tstart) / 1000000000.0 + " seconds.");
			}
		}
		
		// order runtimes
		List<Integer> indices = new ArrayList<Integer>();
		for (int i = 0; i < runtimes.length; i++) {
			indices.add(i);
		}
		
		Comparator<Integer> comparator = new Comparator<Integer>() {
			public int compare(Integer i, Integer j) {
				if (runtimes[i] < runtimes[j])
					return -1;
				if (runtimes[i] > runtimes[j])
					return 1;
				return 0;
			}
		};
		
		Collections.sort(indices, comparator);
		
		order = new int[indices.size()];
		for (int i = 0; i < indices.size(); i++) {
			order[i] = indices.get(indices.size() - 1 - i);
		}
	}
	
	public double eval(int[] S) {
		// sanity checks
		if (weights == null) {
			throw new RuntimeException("Weights not set.");
		}
		
		if (weights.length != functions.size()) {
			throw new RuntimeException("Number of weights does not match number of functions.");
		}
		
		if (S == null) {
			S = new int[0];
		}
		
		// sort S
		Arrays.sort(S);
		
		double val = 0;
		int nrFunctions = functions.size();
		for (int i = 0; i < nrFunctions; i++) {
			// if (Math.abs(weights[i]) > pruneThreshold) {
			val += weights[i] * functions.get(i).eval(S);
			// System.out.println("Function " + functions.get(i) + ": val=" + functions.get(i).eval(S));
			// }
		}
		return val;
	}
	
	// public double evalp(int[] S) {
	// // sanity checks
	// if (weights == null) {
	// throw new RuntimeException("Weights not set.");
	// }
	//
	// if (weights.length != functions.size()) {
	// throw new
	// RuntimeException("Number of weights does not match number of functions.");
	// }
	//
	// // Sort S
	// Arrays.sort(S);
	//
	// double[] values = new double[weights.length];
	//
	// try {
	// Broker broker = new Broker();
	//
	// // Produce jobs
	// int nrFunctions = functions.size();
	// for (int i = 0; i < nrFunctions; i++) {
	// if (Math.abs(weights[i]) > pruneThreshold) {
	// broker.put(i);
	// }
	// }
	// // System.out.println("Queue contains " + broker.queue.size() +
	// " jobs.");
	//
	// ExecutorService threadPool = Executors.newFixedThreadPool(3);
	// threadPool.execute(new Consumer("1", broker, S, values, functions));
	// threadPool.execute(new Consumer("2", broker, S, values, functions));
	// threadPool.execute(new Consumer("3", broker, S, values, functions));
	// threadPool.shutdown();
	// threadPool.awaitTermination(Integer.MAX_VALUE, TimeUnit.SECONDS);
	// } catch (Exception e) {
	// e.printStackTrace();
	//
	// }
	//
	// double val = 0;
	// for (int i = 0; i < weights.length; i++) {
	// val += weights[i] * values[i];
	// }
	// return val;
	// }
	
	public double[] eval(int[] B, int[] cand) {
		// sanity checks
		if (weights == null) {
			throw new RuntimeException("Weights not set.");
		}
		
		if (weights.length != functions.size()) {
			throw new RuntimeException("Number of weights does not match number of functions.");
		}
		
		int nrCandidates = cand.length;
		double[] values = new double[nrCandidates];
		/*if (B == null) {
			return values;
		}*/
		
		// sort S
		Arrays.sort(B);
		
		int nrFunctions = functions.size();
		int nrThreads = 8;
		LinCombBroker broker = new LinCombBroker();
		ExecutorService threadPool = Executors.newFixedThreadPool(nrThreads);
		Collection<Callable<Object>> threads = new ArrayList<Callable<Object>>();
		double[][] gains = new double[nrFunctions][nrCandidates];
		for (int k = 0; k < nrThreads; k++) {
			threads.add(Executors.callable(new LinCombConsumer(k, broker)));
		}
		
		// Add works
		for (int f = 0; f < nrFunctions; f++) {
			int fn = f;
			if (order != null) {
				fn = order[f];
			}
			if (weights[fn] > pruneThreshold) {
				LinCombJobData jobData = new LinCombJobData(fn, B, cand, gains, functions);
				try {
					broker.put(jobData);
				} catch (InterruptedException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}
		
		try {
			threadPool.invokeAll(threads);
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		threadPool.shutdown();
		try {
			threadPool.awaitTermination(Integer.MAX_VALUE, TimeUnit.SECONDS);
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		for (int i = 0; i < nrFunctions; i++) {
			for (int j = 0; j < nrCandidates; j++) {
				values[j] += weights[i] * gains[i][j];
			}
		}
		
		// int nrFunctions = functions.size();
		// for (int i = 0; i < nrFunctions; i++) {
		// double[] tvalues = functions.get(i).eval(B, cand);
		// for (int j = 0; j < nrCandidates; j++) {
		// values[j] += weights[i] * tvalues[j];
		// }
		// }
		return values;
	}
	
	public class LinCombBroker {
		public LinkedBlockingQueue<LinCombJobData> queue;
		
		public LinCombBroker() {
			this.queue = new LinkedBlockingQueue<LinCombJobData>();
		}
		
		public void put(LinCombJobData data) throws InterruptedException {
			this.queue.put(data);
		}
		
		public LinCombJobData get() throws InterruptedException {
			return this.queue.poll();
		}
	}
	
	private class LinCombConsumer implements Runnable {
		private int id;
		private LinCombBroker broker;
		
		public LinCombConsumer(int id, LinCombBroker broker) {
			this.id = id;
			this.broker = broker;
		}
		
		@Override
		public void run() {
			try {
				LinCombJobData data = broker.get();
				
				while (data != null) {
					int functionIndex = data.getI();
					double[] lgains = data.getFunctions().get(functionIndex).eval(data.getB(), data.getCandidates());
					double[][] gains = data.getGains();
					
					for (int i = 0; i < lgains.length; i++) {
						gains[functionIndex][i] = lgains[i];
					}
					
					data = broker.get();
				}
				
				// System.out.println("Comsumer " + this.name +
				// " finished its job; terminating.");
			} catch (InterruptedException ex) {
				ex.printStackTrace();
			}
		}
	}
	
	private class LinCombJobData {
		private int i;
		private int[] B;
		private int[] candidates;
		private double[][] gains;
		private List<Function> functions;
		
		public LinCombJobData(int i, int[] B, int[] candidates, double[][] gains, List<Function> functions) {
			this.i = i;
			this.B = B;
			this.gains = gains;
			this.functions = functions;
			this.candidates = candidates;
		}
		
		public int getI() {
			return i;
		}
		
		public int[] getB() {
			return B;
		}
		
		public List<Function> getFunctions() {
			return functions;
		}
		
		public double[][] getGains() {
			return gains;
		}
		
		public int[] getCandidates() {
			return candidates;
		}
	}
	
}
