public class SumCoverage implements Function {
	private final double[][] sim;
	private final int nrElements;
	private final double normalization;
	
	public SumCoverage(Matrix sim) {
		this(sim.getMatrix());
	}
	
	public SumCoverage(double[][] sim) {
		this.sim = sim;
		this.nrElements = sim.length;
		
		double normalization = 0;
		for (int i = 0; i < sim.length; i++) {
			for (int j = 0; j < sim[0].length; j++) {
				normalization += sim[i][j];
			}
		}
		this.normalization = normalization;
	}
	
	public double eval(int[] S) {
		if (S == null) {
			return 0.0;
		}
		
		double val = 0.0;
		for (int j = 0; j < nrElements; j++) {
			for (int i : S) {
				val += sim[i - 1][j];
			}
		}
		return val / normalization;
	}
	
	public double[] eval(int[] B, int[] candidates) {
		if (candidates == null)
			candidates = new int[] {};
		
		int nrCandidates = candidates.length;
		double[] values = new double[nrCandidates];
		
		// compute maximum for all elements in B
		double sumValue = 0.0;
		for (int j = 0; j < nrElements; j++) {
			for (int i : B) {
				sumValue += sim[i - 1][j];
			}
		}
		
		// evaluate candidates
		for (int c = 0; c < nrCandidates; c++) {
			int candidate = candidates[c];
			double val = sumValue;
			
			for (int j = 0; j < nrElements; j++) {
				val += sim[candidate - 1][j];
			}
			
			values[c] = val / normalization;
		}
		
		return values;
	}
	
	// >> For octave compatibility >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
	
	public double eval(int St) {
		int[] S = new int[] { St };
		return eval(S);
	}
	
	public double[] eval(int St, int[] candidates) {
		int[] S = new int[] { St };
		return eval(S, candidates);
	}
	
	public double[] eval(int[] S, int candidatest) {
		int[] candidates = new int[] { candidatest };
		return eval(S, candidates);
	}
	
}
