public class SurrogateLoss implements Function {
	private final double[][] bow;
	private final double[][] summariesAsBow;
	private final double[] singletonValues;
	private final int nrWords;
	private final double normalization;
	private final double[] weights;
	private final int nrSummaries;
	private final int nrImages;
	
	// private final double[][] tbow;
	
	public SurrogateLoss(double[][] bow, double[][] summariesAsBow) {
		this.bow = bow;
		this.summariesAsBow = summariesAsBow;
		this.nrWords = bow.length;
		this.nrSummaries = summariesAsBow.length;
		this.nrImages = bow[0].length;
		
		// compute weights
		double[] weights = new double[nrSummaries];
		double[] t = new double[nrWords];
		for (int i = 0; i < nrWords; i++) {
			for (int j = 0; j < nrImages; j++) {
				weights[i] += bow[i][j] / nrImages;
			}
		}
		for (int i = 0; i < nrWords; i++) {
			for (int j = 0; j < nrSummaries; j++) {
				t[i] += summariesAsBow[j][i] / nrSummaries;
			}
		}
		for (int i = 0; i < nrWords; i++) {
			weights[i] /= t[i];
		}
		// remove NaNs
		for (int i = 0; i < nrWords; i++) {
			if (Double.isNaN(weights[i])) {
				weights[i] = 0;
			}
		}
		// find max
		double max = Double.NEGATIVE_INFINITY;
		for (int i = 0; i < nrWords; i++) {
			if (!Double.isInfinite(weights[i]) && weights[i] > max) {
				max = weights[i];
			}
		}
		// replace infs by 10*max
		for (int i = 0; i < nrWords; i++) {
			if (Double.isInfinite(weights[i])) {
				weights[i] = 10 * max;
			}
		}
		this.weights = weights;
		
		// compute tbow
		double[][] tbow = new double[nrWords][nrSummaries];
		
		// compute normalization
		double normalization = 0.0;
		
		this.normalization = normalization;
		
		// precompute singleton values
		int nrSingletons = nrImages;
		double[] singletonValues = new double[nrSingletons];
		for (int i = 0; i < nrSingletons; i++) {
			int[] S = new int[1];
			S[0] = i + 1;
			singletonValues[i] = eval(S);
		}
		this.singletonValues = singletonValues;
	}
	
	public SurrogateLoss(Matrix bow, Matrix summariesAsBow) {
		this(bow.getMatrix(), summariesAsBow.getMatrix());
	}
	
	// For octave compatibility
	public double eval(int St) {
		return singletonValues[St - 1];
	}
	
	public double eval(int[] S) {
		if (S == null) {
			return 0.0;
		}
		
		// sumBow = sum(bow(:, S), 2);
		double[] sumBow = new double[nrWords];
		for (int i = 0; i < nrWords; i++) {
			for (int j : S) {
				sumBow[i] += bow[i][j];
			}
		}
		
		double val = 0.0;
		
		return val / normalization;
	}
	
	@Override
	public double[] eval(int[] B, int[] cand) {
		// TODO Auto-generated method stub
		return null;
	}
	
}
