import java.io.Serializable;

public class SurrogateLossNew implements Function, Serializable {
	private final double[][] bow;
	private final double[][] summariesAsBow;
	private final int nrWords;
	private double normalization;
	private final int nrSummaries;
	private final int nrImages;
	
	private double offset;
	private double[] weights;
	private double[] thresholds;
	
	public SurrogateLossNew(double[][] bow, double[][] summariesAsBow) {
		this.bow = bow;
		this.summariesAsBow = summariesAsBow;
		this.nrWords = bow.length;
		this.nrSummaries = summariesAsBow.length;
		this.nrImages = bow[0].length;
		
		this.offset = 0;
		this.weights = new double[nrWords];
		this.thresholds = new double[nrWords];
		for (int w = 0; w < nrWords; w++) {
			this.weights[w] = 1.0;
			this.thresholds[w] = 1.0;
		}
		
		computeNormalization();
	}
	
	public void setOffset(double offset) {
		this.offset = offset;
	}
	
	public void setWeights(double[] weights) {
		if (weights.length != this.weights.length) {
			throw new IllegalArgumentException("Provided weight vector has invalid length.");
		}
		
		this.weights = weights;
		computeNormalization();
	}
	
	public void setThresholds(double[] thresholds) {
		if (thresholds.length != this.thresholds.length) {
			throw new IllegalArgumentException("Provided thresholds vector has invalid length.");
		}
		
		this.thresholds = thresholds;
		computeNormalization();
	}
	
	private void computeNormalization() {
		double normalization = 0.0;
		
		for (int w = 0; w < nrWords; w++) {
			normalization += weights[w] * thresholds[w];
		}
		normalization *= nrSummaries;
		
		this.normalization = normalization;
	}
	
	public SurrogateLossNew(Matrix bow, Matrix summariesAsBow) {
		this(bow.getMatrix(), summariesAsBow.getMatrix());
	}
	
	// For octave compatibility
	public double eval(int St) {
		return eval(new int[] { St });
	}
	
	public double eval(int[] S) {
		if (S == null) {
			S = new int[] {};
		}
		
		// sumBow = sum(bow(:, S), 2);
		double[] sumBow = new double[nrWords];
		for (int w = 0; w < nrWords; w++) {
			for (int j : S) {
				sumBow[w] += bow[w][j - 1];
			}
		}
		
		double val = 0.0;
		for (int s = 0; s < nrSummaries; s++) {
			for (int w = 0; w < nrWords; w++) {
				double nthresh = thresholds[w] - summariesAsBow[s][w];
				if (sumBow[w] < nthresh) {
					val += weights[w] * sumBow[w];
				} else {
					val += weights[w] * nthresh;
				}
			}
		}
		
		return (offset + val / normalization);
	}
	
	public double[] evalSumBow(double[][] sumBows) {
		int nrSamples = sumBows.length;
		double[] vals = new double[nrSamples];
		
		for (int i = 0; i < nrSamples; i++) {
			double[] sumBow = sumBows[i];
			
			double val = 0.0;
			for (int s = 0; s < nrSummaries; s++) {
				double[] tSummaryAsBow = summariesAsBow[s];
				for (int w = 0; w < nrWords; w++) {
					double nthresh = thresholds[w] - tSummaryAsBow[w];
					if (sumBow[w] < nthresh) {
						val += weights[w] * sumBow[w];
					} else {
						val += weights[w] * nthresh;
					}
				}
			}
			
			vals[i] = (offset + val / normalization);
		}
		
		return vals;
	}
	
	public double[] computeGradient(int[] S) {
		double[] grad = new double[1 + weights.length + thresholds.length];
		
		grad[0] = 1;
		
		throw new RuntimeException("Implementation not finished");
		// return grad;
	}
	
	@Override
	public double[] eval(int[] B, int[] cand) {
		// TODO Auto-generated method stub
		return null;
	}
	
}
