public class ThresholdedFeatureCounts implements Function {
	private final double[][] bow;
	private final double threshold;
	private final double[] normalization;
	private final int nrImages;
	private final int nrWords;
	
	public ThresholdedFeatureCounts(Matrix bow, double threshold) {
		this(bow.getMatrix(), threshold);
	}
	
	public ThresholdedFeatureCounts(double[][] bow, double threshold) {
		this.bow = bow;
		this.threshold = threshold;
		this.nrWords = bow.length;
		this.nrImages = bow[0].length;
		
		double[] normalization = new double[nrWords];
		for (int i = 0; i < nrWords; i++) {
			for (int j = 0; j < nrImages; j++) {
				normalization[i] += bow[i][j];
			}
		}
		this.normalization = normalization;
	}
	
	public double eval(int[] S) {
		if (S == null) {
			return 0.0;
		}
		
		double val = 0.0;
		
		double[] sumBow = new double[nrWords];
		for (int j = 0; j < nrWords; j++) {
			double[] imBow = bow[j];
			for (int i : S) {
				sumBow[j] += imBow[i - 1];
			}
			
		}
		
		// sum and threshold
		for (int i = 0; i < nrWords; i++) {
			double tval = sumBow[i] / normalization[i];
			if (tval < threshold) {
				val += tval;
			} else {
				val += threshold;
			}
		}
		
		return val / (threshold * nrWords);
	}
	
	public double[] eval(int[] B, int[] candidates) {
		if (candidates == null)
			candidates = new int[] {};
		
		int nrCandidates = candidates.length;
		double[] values = new double[nrCandidates];
		
		double[] sumBow = new double[nrWords];
		for (int j = 0; j < nrWords; j++) {
			double[] imBow = bow[j];
			for (int i : B) {
				sumBow[j] += imBow[i - 1];
			}
			
		}
		
		// sum and threshold
		for (int c = 0; c < nrCandidates; c++) {
			int candidate = candidates[c];
			double val = 0.0;
			for (int i = 0; i < nrWords; i++) {
				double tval = (sumBow[i] + bow[i][candidate - 1]) / normalization[i];
				if (tval < threshold) {
					val += tval;
				} else {
					val += threshold;
				}
			}
			values[c] = val / (threshold * nrWords);
		}
		
		return values;
	}
	
	// >> For octave compatibility >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
	
	public double eval(int St) {
		int[] S = new int[] { St };
		return eval(S);
	}
	
	public double[] eval(int St, int[] candidates) {
		int[] S = new int[] { St };
		return eval(S, candidates);
	}
	
	public double[] eval(int[] S, int candidatest) {
		int[] candidates = new int[] { candidatest };
		return eval(S, candidates);
	}
	
}
