import java.util.ArrayList;
import java.util.List;
import java.util.Random;

public class UtilsTest {
	public static int[] randomSet(int nrElements, int maxElement, Random rg) {
		int[] S = new int[nrElements];
		
		for (int i = 0; i < nrElements; i++) {
			S[i] = 1 + (rg.nextInt(maxElement));
		}
		
		return S;
	}
	
	public static int[] generateCandidatesSet(int[] S, int maxElement, Random rg) {
		List<Integer> universe = new ArrayList<Integer>(maxElement);
		for (int i = 1; i <= maxElement; i++) {
			universe.add(i);
		}
		
		List<Integer> SasCollection = new ArrayList<Integer>(S.length);
		for (int e : S) {
			SasCollection.add(e);
		}
		
		universe.removeAll(SasCollection);
		
		int[] candidates = new int[universe.size()];
		int idx = 0;
		for (int e : universe) {
			candidates[idx] = e;
			idx++;
		}
		
		return candidates;
	}
	
	public static double[][] generateSimiliarityMatrix(int nrDim, Random rg) {
		double[][] sim = new double[nrDim][nrDim];
		
		Random randomGenerator = new Random(124);
		for (int i = 0; i < nrDim; i++) {
			for (int j = 0; j <= i; j++) {
				sim[i][j] = randomGenerator.nextDouble();
				sim[j][i] = sim[i][j];
				if (i == j) {
					sim[i][j] = 1.0;
				}
			}
		}
		
		return sim;
	}
	
	public static int[][] generateSummaries(int nrSummaries, int length, int nrImages, Random rg) {
		int[][] summaries = new int[nrSummaries][10];
		
		for (int s = 0; s < nrSummaries; s++) {
			for (int j = 0; j < length; j++) {
				int idx = rg.nextInt() % nrImages;
				if (idx < 0) {
					idx = -idx;
				}
				summaries[s][j] = idx + 1;
			}
		}
		
		return summaries;
	}
	
	public static double[][] generateBOW(int nrImages, int nrWords, Random rg) {
		double[][] bow = new double[nrWords][nrImages];
		
		for (int w = 0; w < nrWords; w++) {
			for (int j = 0; j < nrImages; j++) {
				bow[w][j] = rg.nextDouble();
			}
		}
		
		return bow;
	}
	
	public static double[][] computeBOWofSummaries(int[][] summaries, double[][] bow) {
		int nrWords = bow.length;
		int nrSummaries = summaries[0].length;
		double[][] bowSummaries = new double[nrWords][nrSummaries];
		
		for (int w = 0; w < nrWords; w++) {
			for (int s = 0; s < nrSummaries; s++) {
				for (int j = 0; j < 10; j++) {
					bowSummaries[w][s] += bow[w][summaries[s][j] - 1];
				}
			}
		}
		
		return bowSummaries;
	}
}
