%% analyzes the convergence of the max-margin training.

clear all;
clc;

%% >> settings >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
ds = 1;
overlap = 0.1;
experiment = 'SurrogateRougeAsComponent';
loss = 'OneMinusRouge';
inference = 'greedy';
%% << settings <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

%% >> setup >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
addpath('external/sfo');
%% << setup <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

%% >> load data >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
basePath = fullfile(getPreDir(), '../BatchSystem/experiments', experiment);
fprintf(1, 'Base path: %s\n', basePath);

resultsFile = fullfile(basePath, 'results', sprintf('results_dataset_%d_overlap_%.2f_loss_%s_algorithm_%s.mat', ds, overlap, loss, inference));
fprintf(1, 'Loading results from ''%s''.\n', resultsFile);
load(resultsFile);

%% setup evaluation
summaries = pruneSummaries(ds, get_pruning_info(ds, overlap));
bowLoss = get_BOW_evaluation(ds);
optionsEvaluationLoss = struct();
optionsEvaluationLoss.bow = bowLoss;
optionsEvaluationLoss.summariesAsBow = computeBOWSummary(summaries, bowLoss);
evalFct = @(x) summaryEvaluation(x, summaries, 'rouge', optionsEvaluationLoss);

%%
nrIter = size(allWeights,2);
scores = [];
scoresRand = [];
scoresFinal = [];
iters = ceil(linspace(1, nrIter, 10));
for iter = iters,
    fprintf(1, 'Iteration %d (max %d)\n', iter, nrIter);
    
    weights = mean(allWeights(:,max(1,iter-5):iter),2);
    %weights = mean(allWeights(:,1:iter),2);
    
    tfcts = fcts(ds,:);
    eval(sprintf('scoreFct = %s;', constructScoreFunction(weights, 'tfcts')));
    eval(sprintf('scoreFctRand = %s;', constructScoreFunction(rand(size(weights)), 'tfcts')));
    eval(sprintf('scoreFctFinal = %s;', constructScoreFunction(allWeights(:,iter), 'tfcts')));
    
    greedyMax = greedySummary(scoreFct, 10);
    %greedyMaxRand = greedySummary(scoreFctRand, 10);
    greedyMaxFinal = greedySummary(scoreFctFinal, 10);
    
    scores(end+1) = summaryEvaluation(greedyMax, summaries, 'rouge', optionsEvaluationLoss);
    %scoresRand(end+1) = summaryEvaluation(greedyMaxRand, summaries, 'rouge', optionsEvaluationLoss);
    scoresFinal(end+1) = summaryEvaluation(greedyMaxFinal, summaries, 'rouge', optionsEvaluationLoss);

    fprintf(1, 'Score of greedy max       = %f\n', scores(end));
    %fprintf(1, 'Score of greedy max rand  = %f\n', scoresRand(end));
    fprintf(1, 'Score of greedy max final = %f\n', scoresFinal(end));
end

%% plot
plot(iters, scores, 'r')
hold on
plot(iters, scoresFinal, 'g');
legend('Score average weights', 'Score final weights');
