clear all;
clc;

resultsPath = '../BatchSystem/experiments/TwentyPasses/results';

fprintf(1, 'Analyzing results...\n');

for ds = 1:14,
    fprintf(1, '>> DS %d <<<<<<<<\n', ds);
    
    resultsFile = fullfile(resultsPath, sprintf('results_dataset_%d_overlap_0.10_loss_OneMinusRouge_algorithm_greedy.mat', ds));
    if ~exist(resultsFile, 'file'),
        continue;
    end
    load(resultsFile);
    
    %% find overfeat components
    overfeatComponents = []; % indices of overfeat components
    for c = 1:length(components),
        component = components(c);
        if isfield(component, 'resources'),
            resources = component.resources;
            for r = 1:length(resources),
                resource = resources(r);
                if length(fieldnames(resource)) > 0 && ~isempty(strfind(resource.url, 'overfeat')),
                    overfeatComponents(end+1) = c;
                    break;
                end
            end
        end
    end
    
    %% group components by types
    types = {};
    typesIndices = {};
    for c = 1:length(components),
        component = components(c);
        t = cellfun(@(x) strcmpi(x, component.type), types);
        if ~any(t),
            types{end+1} = component.type;
            typesIndices{end+1} = [];
        end
    end
    for c = 1:length(components),
        component = components(c);
        t = cellfun(@(x) strcmpi(x, component.type), types);
        typesIndices{find(t)} = [typesIndices{find(t)},c];
    end
    
    %% remove bias
    t = cellfun(@(x) strcmpi('Bias', x), types);
    idx = find(t);

    types(idx) = [];
    typesIndices(idx) = [];
    
    %% spit our results    
    totalWeight = sum(mean(allWeights, 2));
    overfeatWeight = sum(mean(allWeights(overfeatComponents,:),2));
    fprintf(1, '** Fraction overfeat weights: %2.2f %%\n', overfeatWeight/totalWeight*100);
    
    % compute fraction of weight assigned to certain types of functions
    a = zeros(size(types));
    b = zeros(size(types));
    for t = 1:length(typesIndices),
        totalWeight = sum(mean(allWeights, 2));
        componentWeight = sum(mean(allWeights(typesIndices{t},:),2));
        
        frComponentWeight = componentWeight/totalWeight*100;
        
        weighta = sum(mean(allWeights(intersect(typesIndices{t},overfeatComponents),:),2));
        weightb = sum(mean(allWeights(typesIndices{t},:),2));
        frOverfeatWeight = weighta / weightb*100;
        fprintf(1, 'Fraction ''%s'' weights: %2.2f %% (overfeat: %2.2f %%)\n', types{t}, frComponentWeight, frOverfeatWeight);
        
        a(t) = frComponentWeight;
        b(t) = frOverfeatWeight;
    end
    fprintf(1, '\n');
    
    %% create bar plot
    clf;
    bar(a, 'b');
    hold on
    bar(a.*b/100, 'k');
     set(gca, 'XTickLabel', types);
     set(gca, 'Position', [0.13 0.30 0.775 0.615]);
     rotateticklabel(gca, 45);
    title(sprintf('Dataset %d', ds));
    ylabel('Percentage');
    legend('Total Weight', 'Overfeat', 'Location', 'NorthWest');
    
    % save
    fname = sprintf('../results/weights_ds_%d.pdf',ds);
    print('-dpdf','-r300',fname);
end