clear all;
clc;

% determine how many summaries must be pruned to achieve 
% a certain threshold
thresholds = 0.50:-0.05:0.05;
nrPruneTh = -1*ones(14, length(thresholds));
perPruneTh = zeros(14, length(thresholds));

for dataset = 1:14,
    % analyze precomputed pruning
    filename = sprintf('%s/summary_pruning_set%d', getPreDir(), dataset);
    load(filename);
    nrSummaries(dataset) = length(pruningInformation);

    % determine number of summaries to prune
    nrPrune = -1;
    for i = 1:size(pruningInformation),
        humanScores = pruningInformation{i}.humanScores;
        randScores = pruningInformation{i}.randScores;
        
        th = min(humanScores);
        overlap = sum(randScores > th) / length(randScores);
        for thIdx = 1:length(thresholds),
	  if overlap < thresholds(thIdx) & nrPruneTh(dataset,thIdx) == -1,
	      nrPruneTh(dataset, thIdx) = i;
	      perPruneTh(dataset, thIdx) = i / nrSummaries(dataset);
	  end
        end
    end
end

analyze_vrouge_table(thresholds, nrPruneTh, perPruneTh);
return

% number of summaries to prune for every dataset
nrPruneDatasets = zeros(14,1); 
nrSummaries = zeros(14,1);
for dataset = 1:14,
    % analyze precomputed pruning
    filename = sprintf('%s/summary_pruning_set%d', getPreDir(), dataset);
    load(filename);
    nrSummaries(dataset) = length(pruningInformation);

    % determine number of summaries to prune
    nrPrune = -1;
    for i = 1:size(pruningInformation),
        humanScores = pruningInformation{i}.humanScores;
        randScores = pruningInformation{i}.randScores;
        
        th = min(humanScores);
        overlap = sum(randScores > th) / length(randScores) * 100;
        if overlap < 5.0 & nrPrune == -1,
            nrPrune = i;
            break
        end
    end
    nrPruneDatasets(dataset) = nrPrune;
end

% print pruning information
for dataset = 1:14,
    fprintf(1, '[Dataset %2d] #(to prune) = %3d; #(summaries) = %3d; %%(to prune) = %2.2f\n', dataset, nrPruneDatasets(dataset), nrSummaries(dataset), nrPruneDatasets(dataset) / nrSummaries(dataset) * 100);
end

%% settings
confidence = 0.95;
bins = 0.0:0.02:1.0;
fbins = 0.0:0.005:1.0;

for dataset = 1:14,
    
    close all;
    
    images = get_images(dataset);

    % analyze precomputed pruning
    filename = sprintf('%s/summary_pruning_set%d', getPreDir(), dataset);
    load(filename);

    allHumanScores = pruningInformation{1}.humanScores;
    allRandScores = pruningInformation{1}.randScores;

    % determine number of summaries to prune
    nrPrune = nrPruneDatasets(dataset);

    nrPrune

    % plot
    % F(length(pruningInformation)) = struct('cdata',[],'colormap',[]);
    figure('Renderer','opengl');
    writerObj = VideoWriter(sprintf('%s/pruning_%d.avi', getPreDir(), dataset));
    writerObj.FrameRate = 5;
    open(writerObj);
    for i = 1:length(pruningInformation),
        fprintf(1, '.');
        if mod(i,80) == 0,
            fprintf(1, '\n');
        end
        humanScores = pruningInformation{i}.humanScores;
        randScores = pruningInformation{i}.randScores;
        meanHuman = mean(humanScores);
        meanRand = mean(randScores);
        stdHuman = std(humanScores);
        stdRand = std(randScores);
        
        % compute overlap as random summaries exceeding the worst human
        % summary
        th = min(humanScores);
        overlap = sum(randScores > th) / length(randScores) * 100;

        if i == 1,
            % plot original histogram
            subplot(4,1,1);
            analyze_vrouge_plot(randScores, humanScores);
            title(sprintf('Histogram without pruning (overlap = %2.2f %%)', overlap));
        end

        % plot histogram with pruning
        if i <= nrPrune,
            subplot(4,1,2);
            analyze_vrouge_plot(randScores, humanScores);
            title(sprintf('Histogram after %d pruning steps (stop at iter %d; overlap = %2.2f %%)', i, nrPrune, overlap));
        end

        % plot histogram at iteration i
        subplot(4,1,3);
        analyze_vrouge_plot(randScores, humanScores);
        title(sprintf('Histogram after %d pruning steps (overlap = %2.2f %%)', i, overlap));
        
        % worst summary
        subplot(4,1,4);
        cla;
        s = pruningInformation{i}.removedSummariersIter(end,:);
        montage(images, 'Indices', s, 'Size', [1, 10]);
        title(sprintf('Worst summary at iteration %d', i));
        
        
        %return

        % F(i) = getframe(gcf);
        writeVideo(writerObj, getframe(gcf));
    end
    close(writerObj);
    fprintf(1, '\n');
end

% %% use 1st frame to get dimensions
% [h, w, p] = size(F(1).cdata);
% hf = figure; 
% % resize figure based on frame's w x h, and place at (150, 150)
% set(hf,'Position', [150 150 w h]);
% axis off
% % Place frames at bottom left
% movie(hf,F,1,30,[0 0 0 0]);

%%%%%%%%%%%%%%%%%%%%%%

%rng(123456);
%randomSummaries = randomSummary(100, 10, 1000);

%skewVals = [];
%
% %% settings
% dataset = 2;
% nrSummaries = size(pruneSummaries(dataset, 0), 1);
% idx = 1;
% for nrSummariesToPrune = 1:5:(nrSummaries-5),
%     fid = 1;
% 
%     %% load summaries and compute histograms of scores
%     [summaries, prunedSummaries] = pruneSummaries(dataset, nrSummariesToPrune);
% 
%     bowEvaluate = get_BOW_evaluation(dataset);
% 
%     summariesAsBow = computeBOWSummary(summaries, bowEvaluate);
%     optionsEvaluationLoss = struct();
%     optionsEvaluationLoss.bow = bowEvaluate;
%     optionsEvaluationLoss.summariesAsBow = summariesAsBow;
% 
%     [a1,b1] = summaryEvaluation(summaries, summaries, 'rouge', optionsEvaluationLoss);
%     [a2,b2] = summaryEvaluation(randomSummaries, summaries, 'rouge', optionsEvaluationLoss);
%     fprintf(fid, 'Average human score=%f [min=%f,max=%f]\n', mean(b1), min(b1), max(b1));
%     fprintf(fid, 'Average random score=%f [min=%f,max=%f]\n', mean(b2), min(b2), max(b2));
% 
%     % clf
%     % subplot 211
%     % hist(b1, 20)
%     % subplot 212
%     % hist(b2, 20)
%     p1 = skewness(b1);
%     p2 = skewness(b2);
%     
%     skewVals(idx,1) = p1;
%     skewVals(idx,2) = p2;
%     
%     idx = idx + 1;
% end
% 
% %%
% a = find(skewVals > 0);
% fprintf(1, 'Numbers of summaries to prune until skew is non-negative: %d\n', a(1)*5);
% b = find(skewVals(:,1) > skewVals(:,2));
% fprintf(1, 'Numbers of summaries to prune until skew is approx. equal: %d\n', b(1)*5);

