%% script to browse the user generated summaries

%% cleanup
clear all
clc

%% settings
dataset = 2;
k = 3;
evaluation = 'combinatorial';
summariesPerView = 10; % number of summaries to display per view

%% (1) load images
[imagescolor, images] = get_images(dataset);

%% (2) load summaries
summaryfile = sprintf('../Data/set%d/summary.txt', dataset);
summary = load(summaryfile);
nrSummaries = size(summary, 1);

%% (2b) compute scores
fprintf(1, 'Computing scores for summaries...\n');
truescore = NaN(size(summary,1),1);
for summaryno = 1 : size(summary, 1)
    currsummary = summary(summaryno, :);
    currsummarylist = [summary(1:summaryno - 1, :); summary(summaryno+1:size(summary,1), :)];
    addscore = summaryEvaluation(currsummary, currsummarylist, evaluation, k)*100;
    truescore(summaryno) = addscore;
end
truescorecum = sum(truescore)/size(summary,1);

%% (3) display images and summaries
subplot(1,2,1);
montage(imagescolor, 'size', [10,10]);
set(gca,'position',[0 0 0.45 0.9],'units','normalized')
title('original image collection');

subplot(1,2,2);
startidx = 1;
for k = 1:ceil(nrSummaries / summariesPerView),
    endidx = startidx + summariesPerView - 1;
    if endidx > nrSummaries,
        endidx = nrSummaries;
    end
    
    summaryImages = zeros((endidx - startidx + 1), 10);
    for s = 1:(endidx - startidx + 1),
        for l = 1:10,
            summaryImages(s,l) = summary(startidx + s - 1,l);
        end
    end
    summaryImages = summaryImages';
    
%     summaryImages = zeros(100, 100, 3, (endidx - startidx + 1)*10);
%     for s = 1:(endidx - startidx + 1),
%         for l = 1:10,
%             summaryImages(:,:,:,((s-1)*10)+l) = imagescolor(:,:,:,summary(startidx + s - 1,l));
%         end
%     end
    
    montage(imagescolor, 'size', [10,10], 'Indices', summaryImages(:));
    set(gca,'position',[0.55 0 0.45 0.9],'units','normalized')
    % print scores
    for l = 1:summariesPerView,
        a = text(-20, 50+(l-1)*100, {sprintf('%f', truescore(startidx + l - 1)); sprintf('(%f)', truescore(startidx + l - 1) - truescorecum)}, 'HorizontalAlignment', 'right');
        set(a, 'Color' ,'r');
    end
    title(sprintf('summary (one per row), %d/%d (k=%d, average score=%f)', k, ceil(nrSummaries / summariesPerView), k, truescorecum));
    pause
    
    
    startidx = startidx + summariesPerView;
end