function [encodingCosts] = computeEncodingCosts(ds)

%% settings
nrEncodingPatches = 100;
patchSize = 15;
overlap = 5;
epsNorm = 1e-2;

fprintf(1, 'Computing encoding costs: ');

%% load images and extract patches
imagesrgb = get_images(ds, false);
imgWidth = size(imagesrgb(:,:,1,1), 1);

%% extract patches to encode
nr = floor(imgWidth / patchSize);
patches = zeros(100*nr*nr, 3*patchSize*patchSize, 'single');
idx = 1;
maxNorm = 0;
for img = 1:100,
    for i = 1:nr,
        for j = 1:nr,
            xstart = 1 + (i-1)*patchSize;
            xend   = xstart + patchSize - 1;
            ystart = 1 + (j-1)*patchSize;
            yend   = ystart + patchSize - 1;

            tmp = imagesrgb(xstart:xend,ystart:yend,:,img);
            tmp = tmp(:);
            tmp = (tmp - mean(tmp)) / sqrt(var(tmp) + epsNorm);
            patches(idx,:) = tmp;
            
            if norm(tmp) > maxNorm,
                maxNorm = norm(tmp);
            end

            idx = idx + 1;
        end
    end
end

%% encode
encodingCosts = zeros(100*nr*nr, 100, 'single');
nmax = (imgWidth - patchSize) / overlap;
for img = 1:100,
    encodingPatches = zeros(nrEncodingPatches, 3*patchSize*patchSize, 'single');
    for i = 1:nrEncodingPatches,
        xp = ceil(rand(1)*nmax);
        yp = ceil(rand(1)*nmax);

        xstart = 1 + (xp-1)*overlap;
        xend   = xstart + patchSize - 1;
        ystart = 1 + (yp-1)*overlap;
        yend   = ystart + patchSize - 1;

        tmp = imagesrgb(xstart:xend,ystart:yend,:,img);
        tmp = tmp(:);
        tmp = (tmp - mean(tmp)) / sqrt(var(tmp) + epsNorm);
        encodingPatches(i,:) = tmp;
    end

    % compute encoding costs
    adist = vl_alldist(patches', encodingPatches');
    encCost = min(adist, [], 2) / (2*maxNorm^2);
    encCost = max(0, encCost);
    encCost = min(1, encCost);
    encodingCosts(:,img) = encCost;
    fprintf(1, '.');
end
fprintf(1, '\n');


end