clear all;
clc;

%% setup
fid = 1;

%% similarity based components
simResources = {'superPixels/histIntersection', 'superPixels/tfidf/histIntersection', 'colorHistogram/histIntersection', 'colorHistogram/tfidf/histIntersection', 'overfeat/classAssignments/histIntersection', 'overfeat/slidingOutputWindowAcrossLayers/histIntersection', 'overfeat/slidingOutputWindow/histIntersection', 'overfeat/slidingInputWindow/histIntersection', 'evaluation/histIntersection', 'gist/lambda1', 'gist/lambda2', 'gist/lambda3'};
for idx = 1:length(simResources),
    res_desc = simResources{idx};
    fprintf(fid, 'Component: FacilityLocation\n');
    fprintf(fid, '	Resource: Similarity:%s\n', res_desc);
    fprintf(fid, 'Component: SumCoverage\n');
    fprintf(fid, '	Resource: Similarity:%s\n', res_desc);
    fprintf(fid, 'Component: CutCoverage\n');
    fprintf(fid, '	Resource: Similarity:%s\n', res_desc);
end

% Cosine Similarity
simResources = {'superPixels/ip', 'superPixels/tfidf/ip', 'colorHistogram/ip', 'colorHistogram/tfidf/ip', 'overfeat/classAssignments/ip', 'overfeat/slidingOutputWindowAcrossLayers/ip', 'overfeat/slidingOutputWindow/ip', 'overfeat/slidingInputWindow/ip', 'evaluation/ip'};
thresholdRange = linspace(0.1, 1.0, 4); 
for idx = 1:length(simResources),
    res_desc = simResources{idx};
    for threshold = thresholdRange,
        fprintf(fid, 'Component: CosineSimilarity\n');
        fprintf(fid, '	Resource: Similarity:%s\n', res_desc);
        fprintf(fid, '	Resource: value:%f\n', threshold);
    end
end

%% BOW-based
bowResources = {'superPixels', 'superPixels/tfidf', 'colorHistogram', 'colorHistogram/tfidf', 'overfeat/classAssignments', 'overfeat/slidingOutputWindowAcrossLayers', 'overfeat/slidingOutputWindow', 'overfeat/slidingInputWindow', 'evaluation'};

% Compressed Word Count
for idx = 1:length(bowResources),
    res_desc = bowResources{idx};
    
    fprintf(fid, 'Component: CompressedWordCount\n');
    fprintf(fid, '	Resource: bow:%s\n', res_desc);
end

% Fidelity
thresholdRange = linspace(0.1, 1.0, 4); 
for idx = 1:length(bowResources),
    res_desc = bowResources{idx};
    for threshold = thresholdRange,
        fprintf(fid, 'Component: Fidelity\n');
        fprintf(fid, '	Resource: bow:%s\n', res_desc);
        fprintf(fid, '	Resource: value:%f\n', threshold);
    end
end

% Thresholded Feature Counts
thresholdRange = linspace(0.1,0.5,5);
for idx = 1:length(bowResources),
    res_desc = bowResources{idx};
    for threshold = thresholdRange,
        fprintf(fid, 'Component: ThresholdedFeatureCounts\n');
        fprintf(fid, '	Resource: bow:%s\n', res_desc);
        fprintf(fid, '	Resource: value:%f\n', threshold);
    end
end

% encoding costs
fprintf(fid, 'Component: EncodingAccuracy\n');
fprintf(fid, '	Resource: EncodingCosts:rgb\n');

%% Cluster based
bowResources = {'superPixels', 'superPixels/tfidf', 'colorHistogram', 'colorHistogram/tfidf', 'overfeat/classAssignments', 'overfeat/slidingOutputWindowAcrossLayers', 'overfeat/slidingOutputWindow', 'overfeat/slidingInputWindow', 'evaluation'};
qualityMeasures = {'contrasts', 'intensity'};

% Clustered Facility Location
for idx = 1:length(bowResources),
    res_desc_cluster = bowResources{idx};
    for qidx = 1:length(qualityMeasures),
        res_desc_quality = qualityMeasures{qidx};
	for cutoff = 0.5:0.2:1.1,
	    fprintf(fid, 'Component: ClusteredFacilityLocation\n');
	    fprintf(fid, '	Resource: Cluster:%s/cutoff%f\n', res_desc_cluster, cutoff);
	    fprintf(fid, '	Resource: QualityMeasure:%s\n', res_desc_quality);
	 end
    end
end

% Clustered Diversity Reward
for idx = 1:length(bowResources),
    res_desc_cluster = bowResources{idx};
    for qidx = 1:length(qualityMeasures),
        res_desc_quality = qualityMeasures{qidx};
	for cutoff = 0.5:0.2:1.1,
	    for a = linspace(0.1, 0.9, 5),
	        fprintf(fid, 'Component: ClusteredDiversity\n');
	        fprintf(fid, '	Resource: Cluster:%s/cutoff%f\n', res_desc_cluster, cutoff);
	        fprintf(fid, '	Resource: QualityMeasure:%s\n', res_desc_quality);
	        fprintf(fid, '	Resource: Value:%f\n', a);
	    end
	end
    end
end

%% Bias
fprintf(fid, 'Component: Bias\n');

%% Indicator functions
%for i = 1:100,
%   fprintf(fid, 'Component: Indicator\n');
%   fprintf(fid, '	Resource: Value:%d\n', i);
%end
