%% creates an overview of some of the results

%% clean up
clear all;
clc;

%% settings
nrVisualWords = 5000;
nrAverage = 1000;
options.encoder = 'nn'; % nearest neighbor encoder
options.clustering = 'random';
options.normalizeBOW = 'l1';
options.encoderThreshold = 0.1;
useTFIDF = false;

%% run
for dataset = 2:2,
    fprintf(1, '>> Processing dataset %d...\n', dataset);
    
    %% load dataset
    [imagesrgb, imagesgray, imageshsv] = get_images(dataset, false);
    imagescolor = imageshsv;
    imagesgray = double(imagesgray);
    imageshsv = double(imageshsv);
    imagesrgb = double(imagesrgb);
    
    %% load summaries and compute scores
    [summaries, nrSummaries] = load_summaries(dataset);
    
    %% get bow
    %% get codebook (SIFT)
    options = struct();
    options.clustering = 'random';
    options.normalizeBOW = 'l1';
    options.normalizeFeatuers = 'l1';
    options.summaries = summaries;
    options.dataset = dataset;
    options.features = 'phow';
    options.verbose = true;
    
    optionsEncoder = struct();
    optionsEncoder.encoder = 'nn';
    optionsEncoder.logSpaceScaling = true;
    options.encoderOptions = optionsEncoder;
    
    % filename for storing precomputed bow
    fname = sprintf('precomputed/bow_ds_%d_len_%d_enc_%s_clust_%s_norm_%s_feat_%s.mat', ...
        dataset, nrVisualWords, options.encoderOptions.encoder, options.clustering, options.normalizeBOW, ...
        options.features);
    if ~exist(fname, 'file'),
        error('Loading precomputed bag-of-words failed.');
    end
    load(fname);
    bow1 = bow;
    
    %% get codebook (color)
    nrVisualWords = 800;
    options = struct();
    options.clustering = 'random';
    options.normalizeBOW = 'l1';
    options.normalizeFeatuers = 'l1';
    options.summaries = summaries;
    options.dataset = dataset;
    options.features = 'colorSQ';
    options.verbose = true;
    
    optionsEncoder = struct();
    optionsEncoder.encoder = 'nnEMD';
    optionsEncoder.encoderThreshold = 0.80;
    optionsEncoder.logSpaceScaling = true;
    optionsEncoder.sigma = 0.007743;
    options.encoderOptions = optionsEncoder;
    % filename for storing precomputed bow
    fname = sprintf('precomputed/bow_ds_%d_len_%d_enc_%s_clust_%s_norm_%s_feat_%s.mat', ...
        dataset, nrVisualWords, options.encoderOptions.encoder, options.clustering, options.normalizeBOW, ...
        options.features);
    if ~exist(fname, 'file'),
        error('Loading precomputed bag-of-words failed.');
    end
    load(fname);
    bow2 = bow;
    
    %% concatenate words
    bow = vertcat(bow1, bow2);
    for k = 1:size(bow,2),
        nrm = norm(bow(:,k), 1);
        if nrm < 1e-6,
            nrm = 1;
        end
        bow(:,k) = bow(:,k) / nrm;
    end
    
    %% compute MSE
    fprintf(1, 'Computing MSEs...\n');
    MSE = NaN(size(summaries,1),1);
    MSE_SIFT_lambda_0_1 = NaN(size(summaries,1),1);
    MSE_SIFT_lambda_0_01 = NaN(size(summaries,1),1);
    MSE_SIFT_lambda_0_001 = NaN(size(summaries,1),1);
    for sid = 1:size(summaries,1),
        [Rt,At,err] = sparseCoding(bow, bow(:,summaries(sid,:)), 0);
        MSE(sid) = err / size(bow, 2);        
%         [Rt,At,err] = sparseCoding(bow, bow(:,summary(sid,:)), 0.1);
%         MSE_SIFT_lambda_0_1(sid) = err / size(bow, 2);
%         [Rt,At,err] = sparseCoding(bow, bow(:,summary(sid,:)), 0.01);
%         MSE_SIFT_lambda_0_01(sid) = err / size(bow, 2);
%         [Rt,At,err] = sparseCoding(bow, bow(:,summary(sid,:)), 0.001);
%         MSE_SIFT_lambda_0_001(sid) = err / size(bow, 2);
    end
    
    % random summaries
    MSE_rand_l_0_01 = NaN(nrAverage, 1);
    MSE_rand = NaN(nrAverage, 1);
    for i = 1:nrAverage,
        tempSummary = randperm(100);
        tempSummary = tempSummary(1:10);
%         [Rt,At,err] = sparseCoding(bow, bow(:,tempSummary), 0.1);
%         MSE_rand_l_0_01(i) = err / size(bow,2);
        [Rt,At,err] = sparseCoding(bow, bow(:,tempSummary), 0);
        MSE_rand(i) = err / size(bow,2);
    end
    
    %% compute combinatorial n-gram score
    fprintf(1, 'Computing n-gram scores...\n');
    optionsEvaluation = struct();
    evaluation = 'combinatorial';
    optionsEvaluation.k = 1;
    [~,NGRAM_k_1] = summaryEvaluation(summaries, summaries, evaluation, optionsEvaluation);
    optionsEvaluation.k = 2;
    [~,NGRAM_k_2] = summaryEvaluation(summaries, summaries, evaluation, optionsEvaluation);
    optionsEvaluation.k = 5;
    [~,NGRAM_k_5] = summaryEvaluation(summaries, summaries, evaluation, optionsEvaluation);
    
    %% greedily select best summary
    fprintf(1, 'Computing greedy summaries...\n');
    % compute and apply tfidf
    tfidf = get_TFIDF(bow);
    bow = applyTFIDF(bow, tfidf);
    
    BOWsummary = computeBOWSummary(summaries, bow);
    optionsEvaluation.summariesAsBow = BOWsummary;
    optionsEvaluation.bow = bow;
    greedyNGRAM_summaries = NaN(10,10);
    for k = 1:10,
        optionsEvaluation.k = k;
        greedyNGRAM_summaries(k,:) = greedySummary(@(x) summaryEvaluation(x, summaries, 'combinatorial', optionsEvaluation));
    end
    greedyNGRAM_summaries
    %montage(imagescolor, 'Size', [1,10], 'Indices', selected);
    
    %% compute bow
    % random summaries
    optionsEvaluation.k = 2;
    randomSummaries = randomSummary(100, 10, nrAverage);
    NGRAM_rand_k_2 = summaryEvaluation(randomSummaries, summaries, evaluation, optionsEvaluation)*100;
    
    %% compute bow score
    fprintf(1, 'Computing rouge scores...\n');    
    optionsEvaluation = struct();
    optionsEvaluation.bow = bow;
    evaluation = 'rouge';    
    
    [~,BOW] = summaryEvaluation(summaries, summaries, evaluation, optionsEvaluation);
    
    % random summaries
    [~,BOW_rand] = summaryEvaluation(randomSummaries, summaries, evaluation, optionsEvaluation);
   
    %% create html
    fh = fopen(sprintf('html/dataset%d.html', dataset), 'w');
    fprintf(fh, '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01//EN"\n  "http://www.w3.org/TR/html4/strict.dtd"> \n<HTML>\n   <HEAD>\n      <TITLE>Dataset overview</TITLE> \n   </HEAD>\n  <BODY>\n');
    
    % data to show
    toShow = [];
    
    % unsorted summaries
    toShow(end+1).id = 'summaries_unsorted';
    toShow(end).label = 'Human summaries (unsorted)';
    toShow(end).summaries = 1:size(summaries,1);
    
    % summaries sorted by MSE
    toShow(end+1).id = 'summaries_sorted_mse';
    toShow(end).label = 'Summaries (sorted by MSE, l=0, best to worst)';
    [a,b] = sort(MSE);
    toShow(end).summaries = b;
    toShow(end).rand = MSE;
    
    % 10 best by MSE
    toShow(end+1).id = 'summaries_best_mse';
    toShow(end).label = 'Summaries (best 10 by MSE, l=0)';
    [a,b] = sort(MSE);
    b = b(1:10);
    toShow(end).summaries = b;
    
    % 10 worst by MSE
    toShow(end+1).id = 'summaries_worst_mse';
    toShow(end).label = 'Summaries (worst 10 by MSE, l=0)';
    [a,b] = sort(MSE);
    b = b(end-9:end);
    toShow(end).summaries = b;
    
    % summaries sorted by n-gram score
    toShow(end+1).id = 'summaries_sorted_ngram';
    toShow(end).label = 'Summaries (sorted by N-GRAM, k=2, best to worst)';
    [a,b] = sort(NGRAM_k_2, 1, 'descend');
    toShow(end).summaries = b;
    toShow(end).rand = NGRAM_rand_k_2;
    
    % 10 best by n-gram
    toShow(end+1).id = 'summaries_best_ngram';
    toShow(end).label = 'Summaries (best 10 by N-GRAM, k=2)';
    [a,b] = sort(NGRAM_k_2, 1, 'descend');
    b = b(1:10);
    toShow(end).summaries = b;
    
    % 10 worst by n-gram
    toShow(end+1).id = 'summaries_worst_ngram';
    toShow(end).label = 'Summaries (worst 10 by N-GRAM, k=2)';
    [a,b] = sort(NGRAM_k_2, 1, 'descend');
    b = b(end-9:end);
    toShow(end).summaries = b;
    
    % summaries sorted by bow score
    toShow(end+1).id = 'summaries_sorted_bow';
    toShow(end).label = 'Summaries (sorted by rouge (bow), best to worst)';
    [a,b] = sort(BOW, 1, 'descend');
    toShow(end).summaries = b;
    toShow(end).rand = BOW_rand;
    
    % 10 best by bow score
    toShow(end+1).id = 'summaries_best_bow';
    toShow(end).label = 'Summaries (best 10 by rouge (bow))';
    [a,b] = sort(BOW, 1, 'descend');
    b = b(1:10);
    toShow(end).summaries = b;
    
    % 10 worst by bow score
    toShow(end+1).id = 'summaries_worst_bow';
    toShow(end).label = 'Summaries (worst 10 by rouge (bow))';
    [a,b] = sort(BOW, 1, 'descend');
    b = b(end-9:end);
    toShow(end).summaries = b;

    
    % print overview
    fprintf(fh, 'You can find here...<br>\n');
    fprintf(fh, '* <a href="#images">Images in the dataset</a><br>\n');
    fprintf(fh, '* <a href="#images_freq">Images in the dataset sorted by frequency in summaries</a><br>\n');
    fprintf(fh, '* <a href="#images_pairwise_pos">Most common pairwise co-occurences (100)</a><br>\n');
    fprintf(fh, '* <a href="#images_pairwise_neg">Most uncommon pairwise co-occurences (100)</a><br>\n');
    fprintf(fh, '* <a href="#greedy_summaries">Greedy summaries</a><br>\n');
    
    for i = 1:length(toShow),
        fprintf(fh, sprintf('* <a href="#%s">%s</a><br>\n', toShow(i).id, toShow(i).label));
    end

%     fprintf(fh, '* <a href="#summaries_unsorted">Human summaries (unsorted)</a><br>\n');
%     fprintf(fh, '* <a href="#summaries_sorted_mse">Human summaries (sorted by MSE, l=0.01)</a><br>\n');
%     fprintf(fh, '* <a href="#summaries_sorted_ngram">Human summaries (sorted by n-gram score, k=2)</a><br>\n');
%     fprintf(fh, '* <a href="#summaries_bw_mse">5 Best/Worst human summaries (MSE, k=2)</a><br>\n');
%     fprintf(fh, '* <a href="#summaries_bw_ngram">5 Best/Worst human summaries (n-gram score, k=2)</a><br>\n');
    
    % create overview of image
    fprintf(fh, '<a id="images"><h3>Images in this dataset</h3></a>\n');
    
    fprintf(fh, '<table border=0>\n');
    idx = 1;
    for i = 1:10,
        fprintf(fh, '<tr>\n');
        for l = 1:10,
            fprintf(fh, sprintf('<td><img src="Data/set%d/%d.jpg"></td>', dataset, idx));
            idx = idx + 1;
        end
        fprintf(fh, '</tr>\n');
        fprintf(fh, '<tr>\n');
        idx = idx - 10;
        for l = 1:10,
            fprintf(fh, sprintf('<td>%d</td>', idx));
            idx = idx + 1;
        end
        fprintf(fh, '</tr>\n');
    end
    fprintf(fh, '</table>');
    
    % create overview of image
    fprintf(fh, '<a id="images_freq"><h3>Images in this dataset (sorted by frequency)</h3></a>\n');
    f = zeros(100, 1);
    for i = 1:nrSummaries,
        f(summaries(i,:)) = f(summaries(i,:)) + 1;
    end
    f = f / size(summaries,1);
    
    [a,b] = sort(f, 1, 'descend');
    fprintf(fh, '<table border=0>\n');
    idx = 1;
    for i = 1:10,
        fprintf(fh, '<tr>\n');
        for l = 1:10,
            fprintf(fh, sprintf('<td><img src="Data/set%d/%d.jpg"></td>', dataset, b(idx)));
            idx = idx + 1;
        end
        fprintf(fh, '</tr>\n');
        fprintf(fh, '<tr>\n');
        idx = idx - 10;
        for l = 1:10,
            fprintf(fh, sprintf('<td>f=%f</td>', f(b(idx))));
            idx = idx + 1;
        end
        fprintf(fh, '</tr>\n');

    end
    fprintf(fh, '</table>');
    
    % pairwise cooccurences (most common)
    fprintf(fh, '<a id="images_pairwise_pos"><h3>Most common pairwise co-ocurrences (100)</h3></a>\n');
    pairs = nchoosek(1:100,2);
    nrPairs = size(pairs,1);
    f = zeros(nrPairs,1);
    ssummary = sort(summaries, 2);
    for i = 1:nrPairs,
        f(i) = sum(sum(ismembc(ssummary, pairs(i,:)), 2) == 2);
    end
    f = f / nrPairs;
    
    [a,b] = sort(f, 1, 'descend');
    fprintf(fh, '<table border=0>\n');
    idx = 1;
    for idx = 1:100,
        pair = pairs(b(idx),:);
        fprintf(fh, '<tr>\n');
        fprintf(fh, sprintf('<td><img src="Data/set%d/%d.jpg"></td>', dataset, pair(1)));
        fprintf(fh, sprintf('<td><img src="Data/set%d/%d.jpg"></td>', dataset, pair(2)));
        fprintf(fh, sprintf('<td>f=%f</td>', f(b(idx))));
        fprintf(fh, '</tr>\n');
    end
    fprintf(fh, '</table>');
    
    % uncommon
    fprintf(fh, '<a id="images_pairwise_neg"><h3>Most uncommon pairwise co-ocurrences (100)</h3></a>\n');
    
    [a,b] = sort(f, 1, 'ascend');
    fprintf(fh, '<table border=0>\n');
    idx = 1;
    for idx = 1:100,
        pair = pairs(b(idx),:);
        fprintf(fh, '<tr>\n');
        fprintf(fh, sprintf('<td><img src="Data/set%d/%d.jpg"></td>', dataset, pair(1)));
        fprintf(fh, sprintf('<td><img src="Data/set%d/%d.jpg"></td>', dataset, pair(2)));
        fprintf(fh, sprintf('<td>f=%f</td>', f(b(idx))));
        fprintf(fh, '</tr>\n');
    end
    fprintf(fh, '</table>');
    
    % greedy summaries
    fprintf(fh, '<a id="greedy_summaries"><h3>Greedy summaries</h3></a>\n');
    
    fprintf(fh, '<table border=0>\n');
    for k = 1:10,
        summ = greedyNGRAM_summaries(k,:);
        fprintf(fh, 'Combinatorial n-gram-score, k=%d.<br>\n', k);
        fprintf(fh, '<table border=0><tr>\n');
        for l = 1:10,
            fprintf(fh, sprintf('<td><img src="Data/set%d/%d.jpg"></td>', dataset, summ(l)));
        end
        fprintf(fh, '</tr></table>\n');
    end
    fprintf(fh, '</table>');
    
    %    
    for k = 1:length(toShow),
        % show summaries in random order
        fprintf(fh, sprintf('<a id="%s"><h3>%s</h3></a>\n', toShow(k).id, toShow(k).label));
        if isfield(toShow(k), 'rand'),
            fprintf(fh, 'Random summaries: mean=%.3f, std=%.3f<br>\n', mean(toShow(k).rand), std(toShow(k).rand));
        end
        fprintf(fh, '<table style="padding-bottom: 1em;">\n');
        b = toShow(k).summaries;
        summariesToShow = summaries(b,:);
        for i = 1:size(summariesToShow,1),
            fprintf(fh, '<tr>\n');
            fprintf(fh, sprintf('<td valign="top" style="padding-bottom: 1em;">#%d</td>', i));
            for l = 1:10,
                fprintf(fh, sprintf('<td style="padding-bottom: 1em;"><img src="Data/set%d/%d.jpg"></td>', dataset, summariesToShow(i,l)));
            end
            fprintf(fh, '<td valign="top">');
%             fprintf(fh, sprintf('MSE (SIFT, l=%.3f): %.3f (avg: %.3f, std: %.3f)<br>', 0.1, MSE_SIFT_lambda_0_1(b(i)), mean(MSE_SIFT_lambda_0_1), std(MSE_SIFT_lambda_0_1)));
%             fprintf(fh, sprintf('MSE (SIFT, l=%.3f): %.3f (avg: %.3f, std: %.3f)<br>', 0.01, MSE_SIFT_lambda_0_01(b(i)), mean(MSE_SIFT_lambda_0_01), std(MSE_SIFT_lambda_0_01)));
%             fprintf(fh, sprintf('MSE (SIFT, l=%.3f): %.3f (avg: %.3f, std: %.3f)<br>', 0.001, MSE_SIFT_lambda_0_001(b(i)), mean(MSE_SIFT_lambda_0_001), std(MSE_SIFT_lambda_0_001)));
            fprintf(fh, sprintf('MSE (SIFT, l=0): %.3f (avg: %.3f, std: %.3f)<br>', MSE(b(i)), mean(MSE), std(MSE)));
            fprintf(fh, sprintf('N-GRAM (k=%d): %.2f (avg: %.3f, std: %.3f)<br>', 1, NGRAM_k_1(b(i)), mean(NGRAM_k_1), std(NGRAM_k_1)));
            fprintf(fh, sprintf('N-GRAM (k=%d): %.2f (avg: %.3f, std: %.3f)<br>', 2, NGRAM_k_2(b(i)), mean(NGRAM_k_2), std(NGRAM_k_2)));
            fprintf(fh, sprintf('N-GRAM (k=%d): %.2f (avg: %.3f, std: %.3f)<br>', 5, NGRAM_k_5(b(i)), mean(NGRAM_k_5), std(NGRAM_k_5)));
            fprintf(fh, sprintf('ROUGE (BOW, SIFT): %.2f (avg: %.3f, std: %.3f)<br>', BOW(b(i)), mean(BOW), std(BOW)));
            fprintf(fh, '</td>');
            fprintf(fh, '</tr>\n');
        end
        fprintf(fh, '</table>');
    end
    
%     % show summaries in random order
%     fprintf(fh, '<a id="summaries_unsorted"><h3>Summaries (unsorted)</h3></a>\n');
%     fprintf(fh, '<table style="padding-bottom: 1em;">\n');
%     % [a,b] = sort(MSE_SIFT_lambda_0_01);
%     b = 1:size(summary,1);
%     summariesToShow = summary(b,:);
%     for i = 1:nrSummaries,
%         fprintf(fh, '<tr>\n');
%         fprintf(fh, sprintf('<td valign="top" style="padding-bottom: 1em;">#%d</td>', i));
%         for l = 1:10,
%             fprintf(fh, sprintf('<td style="padding-bottom: 1em;"><img src="Data/set%d/%d.jpg"></td>', dataset, summariesToShow(i,l)));
%         end
%         fprintf(fh, '<td valign="top">');
%         fprintf(fh, sprintf('MSE (SIFT, l=%f): %.3f (avg: %.3f, std: %.3f)<br>', 0.1, MSE_SIFT_lambda_0_1(b(i)), mean(MSE_SIFT_lambda_0_1), std(MSE_SIFT_lambda_0_1)));
%         fprintf(fh, sprintf('MSE (SIFT, l=%f): %.3f (avg: %.3f, std: %.3f)<br>', 0.01, MSE_SIFT_lambda_0_01(b(i)), mean(MSE_SIFT_lambda_0_01), std(MSE_SIFT_lambda_0_01)));
%         fprintf(fh, sprintf('MSE (SIFT, l=%f): %.3f (avg: %.3f, std: %.3f)<br>', 0.001, MSE_SIFT_lambda_0_001(b(i)), mean(MSE_SIFT_lambda_0_001), std(MSE_SIFT_lambda_0_001)));
%         fprintf(fh, sprintf('N-GRAM (k=%d): %.2f (avg: %.3f, std: %.3f)<br>', 1, NGRAM_k_1(b(i)), mean(NGRAM_k_1), std(NGRAM_k_1)));
%         fprintf(fh, sprintf('N-GRAM (k=%d): %.2f (avg: %.3f, std: %.3f)<br>', 2, NGRAM_k_2(b(i)), mean(NGRAM_k_2), std(NGRAM_k_2)));
%         fprintf(fh, sprintf('N-GRAM (k=%d): %.2f (avg: %.3f, std: %.3f)<br>', 5, NGRAM_k_5(b(i)), mean(NGRAM_k_5), std(NGRAM_k_5)));
%         fprintf(fh, '</td>');
%         fprintf(fh, '</tr>\n');
%     end
%     fprintf(fh, '</table>');
%     
%     % show summaries sorted by mse
%     fprintf(fh, '<a id="summaries_sorted_mse"><h3>Summaries (sorted by MSE)</h3></a>\n');
%     fprintf(fh, '<table style="padding-bottom: 1em;">\n');
%     [a,b] = sort(MSE_SIFT_lambda_0_01);
%     summariesToShow = summary(b,:);
%     for i = 1:nrSummaries,
%         fprintf(fh, '<tr>\n');
%         fprintf(fh, sprintf('<td valign="top" style="padding-bottom: 1em;">#%d</td>', i));
%         for l = 1:10,
%             fprintf(fh, sprintf('<td style="padding-bottom: 1em;"><img src="Data/set%d/%d.jpg"></td>', dataset, summariesToShow(i,l)));
%         end
%         fprintf(fh, '<td valign="top">');
%         fprintf(fh, sprintf('MSE (SIFT, l=%f): %.3f (avg: %.3f, std: %.3f)<br>', 0.1, MSE_SIFT_lambda_0_1(b(i)), mean(MSE_SIFT_lambda_0_1), std(MSE_SIFT_lambda_0_1)));
%         fprintf(fh, sprintf('MSE (SIFT, l=%f): %.3f (avg: %.3f, std: %.3f)<br>', 0.01, MSE_SIFT_lambda_0_01(b(i)), mean(MSE_SIFT_lambda_0_01), std(MSE_SIFT_lambda_0_01)));
%         fprintf(fh, sprintf('MSE (SIFT, l=%f): %.3f (avg: %.3f, std: %.3f)<br>', 0.001, MSE_SIFT_lambda_0_001(b(i)), mean(MSE_SIFT_lambda_0_001), std(MSE_SIFT_lambda_0_001)));
%         fprintf(fh, sprintf('N-GRAM (k=%d): %.2f (avg: %.3f, std: %.3f)<br>', 1, NGRAM_k_1(b(i)), mean(NGRAM_k_1), std(NGRAM_k_1)));
%         fprintf(fh, sprintf('N-GRAM (k=%d): %.2f (avg: %.3f, std: %.3f)<br>', 2, NGRAM_k_2(b(i)), mean(NGRAM_k_2), std(NGRAM_k_2)));
%         fprintf(fh, sprintf('N-GRAM (k=%d): %.2f (avg: %.3f, std: %.3f)<br>', 5, NGRAM_k_5(b(i)), mean(NGRAM_k_5), std(NGRAM_k_5)));
%         fprintf(fh, '</td>');
%         fprintf(fh, '</tr>\n');
%     end
%     fprintf(fh, '</table>');
%     
%     % show summaries sorted by n-gram score (k=2)
%     fprintf(fh, '<a id="summaries_sorted_ngram"><h3>Summaries (sorted by N-GRAM, k=2)</h3></a>\n');
%     fprintf(fh, '<table style="padding-bottom: 1em;">\n');
%     [a,b] = sort(NGRAM_k_2);
%     summariesToShow = summary(b,:);
%     for i = 1:nrSummaries,
%         fprintf(fh, '<tr>\n');
%         fprintf(fh, sprintf('<td valign="top" style="padding-bottom: 1em;">#%d</td>', i));
%         for l = 1:10,
%             fprintf(fh, sprintf('<td style="padding-bottom: 1em;"><img src="Data/set%d/%d.jpg"></td>', dataset, summariesToShow(i,l)));
%         end
%         fprintf(fh, '<td valign="top">');
%         fprintf(fh, sprintf('MSE (SIFT, l=%f): %.3f (avg: %.3f, std: %.3f)<br>', 0.1, MSE_SIFT_lambda_0_1(b(i)), mean(MSE_SIFT_lambda_0_1), std(MSE_SIFT_lambda_0_1)));
%         fprintf(fh, sprintf('MSE (SIFT, l=%f): %.3f (avg: %.3f, std: %.3f)<br>', 0.01, MSE_SIFT_lambda_0_01(b(i)), mean(MSE_SIFT_lambda_0_01), std(MSE_SIFT_lambda_0_01)));
%         fprintf(fh, sprintf('MSE (SIFT, l=%f): %.3f (avg: %.3f, std: %.3f)<br>', 0.001, MSE_SIFT_lambda_0_001(b(i)), mean(MSE_SIFT_lambda_0_001), std(MSE_SIFT_lambda_0_001)));
%         fprintf(fh, sprintf('N-GRAM (k=%d): %.2f (avg: %.3f, std: %.3f)<br>', 1, NGRAM_k_1(b(i)), mean(NGRAM_k_1), std(NGRAM_k_1)));
%         fprintf(fh, sprintf('N-GRAM (k=%d): %.2f (avg: %.3f, std: %.3f)<br>', 2, NGRAM_k_2(b(i)), mean(NGRAM_k_2), std(NGRAM_k_2)));
%         fprintf(fh, sprintf('N-GRAM (k=%d): %.2f (avg: %.3f, std: %.3f)<br>', 5, NGRAM_k_5(b(i)), mean(NGRAM_k_5), std(NGRAM_k_5)));
%         fprintf(fh, '</td>');
%         fprintf(fh, '</tr>\n');
%     end
%     fprintf(fh, '</table>');
%     
%     % 5 best/worst mse
%     fprintf(fh, '<a id="summaries_bw_mse"><h3>Summaries (5 best/worts by MSE, l=0.01)</h3></a>\n');
%     fprintf(fh, '<table style="padding-bottom: 1em;">\n');
%     [a,b] = sort(MSE_SIFT_lambda_0_01);
%     b = [b(1:5); b(end-4:end)];
%     summariesToShow = summary(b,:);
%     for i = 1:size(summariesToShow, 1),
%         fprintf(fh, '<tr>\n');
%         fprintf(fh, sprintf('<td valign="top" style="padding-bottom: 1em;">#%d</td>', i));
%         for l = 1:10,
%             fprintf(fh, sprintf('<td style="padding-bottom: 1em;"><img src="Data/set%d/%d.jpg"></td>', dataset, summariesToShow(i,l)));
%         end
%         fprintf(fh, '<td valign="top">');
%         fprintf(fh, sprintf('MSE (SIFT, l=%f): %.3f (avg: %.3f, std: %.3f)<br>', 0.1, MSE_SIFT_lambda_0_1(b(i)), mean(MSE_SIFT_lambda_0_1), std(MSE_SIFT_lambda_0_1)));
%         fprintf(fh, sprintf('MSE (SIFT, l=%f): %.3f (avg: %.3f, std: %.3f)<br>', 0.01, MSE_SIFT_lambda_0_01(b(i)), mean(MSE_SIFT_lambda_0_01), std(MSE_SIFT_lambda_0_01)));
%         fprintf(fh, sprintf('MSE (SIFT, l=%f): %.3f (avg: %.3f, std: %.3f)<br>', 0.001, MSE_SIFT_lambda_0_001(b(i)), mean(MSE_SIFT_lambda_0_001), std(MSE_SIFT_lambda_0_001)));
%         fprintf(fh, sprintf('N-GRAM (k=%d): %.2f (avg: %.3f, std: %.3f)<br>', 1, NGRAM_k_1(b(i)), mean(NGRAM_k_1), std(NGRAM_k_1)));
%         fprintf(fh, sprintf('N-GRAM (k=%d): %.2f (avg: %.3f, std: %.3f)<br>', 2, NGRAM_k_2(b(i)), mean(NGRAM_k_2), std(NGRAM_k_2)));
%         fprintf(fh, sprintf('N-GRAM (k=%d): %.2f (avg: %.3f, std: %.3f)<br>', 5, NGRAM_k_5(b(i)), mean(NGRAM_k_5), std(NGRAM_k_5)));
%         fprintf(fh, '</td>');
%         fprintf(fh, '</tr>\n');
%     end
%     fprintf(fh, '</table>');
%     
%     % 5 best/worst ngram, k=2
%     fprintf(fh, '<a id="summaries_bw_ngram"><h3>Summaries (5 best/worts by N-GRAM, k=2)</h3></a>\n');
%     fprintf(fh, '<table style="padding-bottom: 1em;">\n');
%     [a,b] = sort(NGRAM_k_2);
%     b = [b(1:5); b(end-4:end)];
%     summariesToShow = summary(b,:);
%     for i = 1:size(summariesToShow, 1),
%         fprintf(fh, '<tr>\n');
%         fprintf(fh, sprintf('<td valign="top" style="padding-bottom: 1em;">#%d</td>', i));
%         for l = 1:10,
%             fprintf(fh, sprintf('<td style="padding-bottom: 1em;"><img src="Data/set%d/%d.jpg"></td>', dataset, summariesToShow(i,l)));
%         end
%         fprintf(fh, '<td valign="top">');
%         fprintf(fh, sprintf('MSE (SIFT, l=%f): %.3f (avg: %.3f, std: %.3f)<br>', 0.1, MSE_SIFT_lambda_0_1(b(i)), mean(MSE_SIFT_lambda_0_1), std(MSE_SIFT_lambda_0_1)));
%         fprintf(fh, sprintf('MSE (SIFT, l=%f): %.3f (avg: %.3f, std: %.3f)<br>', 0.01, MSE_SIFT_lambda_0_01(b(i)), mean(MSE_SIFT_lambda_0_01), std(MSE_SIFT_lambda_0_01)));
%         fprintf(fh, sprintf('MSE (SIFT, l=%f): %.3f (avg: %.3f, std: %.3f)<br>', 0.001, MSE_SIFT_lambda_0_001(b(i)), mean(MSE_SIFT_lambda_0_001), std(MSE_SIFT_lambda_0_001)));
%         fprintf(fh, sprintf('N-GRAM (k=%d): %.2f (avg: %.3f, std: %.3f)<br>', 1, NGRAM_k_1(b(i)), mean(NGRAM_k_1), std(NGRAM_k_1)));
%         fprintf(fh, sprintf('N-GRAM (k=%d): %.2f (avg: %.3f, std: %.3f)<br>', 2, NGRAM_k_2(b(i)), mean(NGRAM_k_2), std(NGRAM_k_2)));
%         fprintf(fh, sprintf('N-GRAM (k=%d): %.2f (avg: %.3f, std: %.3f)<br>', 5, NGRAM_k_5(b(i)), mean(NGRAM_k_5), std(NGRAM_k_5)));
%         fprintf(fh, '</td>');
%         fprintf(fh, '</tr>\n');
%     end
%     fprintf(fh, '</table>');

    % end html
    fprintf(fh, '</BODY>\n</HTML>\n');
end