% Plots the progress of the pruning algorithms over the iterations and
% displays the removed summaries

clear all;
clc;
warning('on','all');

%% run
for dataset = 1:14,
    fprintf(1, '>> Processing dataset %d...\n', dataset);
    
    %% load dataset
    [imagescolor, images] = get_images(dataset, false);
    
    %% load summaries and compute scores
    [summaries, nrSummaries] = load_summaries(dataset);    
   
    %% create html
    filename = sprintf('%s/html/pruningAlgorithmDataset%d.html', getPreDir(), dataset);
    if exist(filename, 'file'),
        warning('File "%s" already exists, skipping.');
        continue;
    end
    directory = fileparts(filename);
    if ~exist(directory, 'dir'),
        mkdir(directory);
    end
    fh = fopen(filename, 'w');
    fprintf(fh, '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01//EN"\n  "http://www.w3.org/TR/html4/strict.dtd"> \n<HTML>\n   <HEAD>\n      <TITLE>Dataset overview</TITLE> \n   </HEAD>\n  <BODY>\n');
    
    % data to show
    toShow = [];
    
    %% prepare stuff to show
    for addStd = 0:2,
        a = dir(sprintf('%s/html/images/SummaryPruning/set%d/std%d', getPreDir(), dataset, addStd));
        for iter = 0:((length(a)-2)/2)-1,
            fnameImage = sprintf('%s/html/images/SummaryPruning/set%d/std%d/iteration_%d.png', getPreDir(), dataset, addStd, iter);
            fnameSummaries = sprintf('%s/html/images/SummaryPruning/set%d/std%d/iteration_%d.mat', getPreDir(), dataset, addStd, iter);
            
            if ~exist(fnameImage, 'file') || ~exist(fnameSummaries, 'file'),
                error('Expected file does not exist (%s, %s).', fnameImage, fnameSummaries);
            end
            
            load(fnameSummaries);
            
%             toShow(end+1).id = sprintf('std_%d_iter_%d', addStd, iter);
%             toShow(end).label = sprintf('std=%d, Pruning algorithm iteration %d (removed summaries)', addStd, iter);
%             toShow(end).summaries = removedSummaries;
%             
%             if isempty(removedSummaries), % do not display if no summaries were removed
%                 toShow(end) = [];
%             end
%             
%             toShow(end+1).id = sprintf('std_%d_iter_%d_greedyMin', addStd, iter);
%             toShow(end).label = sprintf('std=%d, Pruning algorithm iteration %d (greedy min, score=%f)', addStd, iter, greedyMinScore);
%             toShow(end).summaries = greedyMinSelected;
%             
%             toShow(end+1).id = sprintf('std_%d_iter_%d_greedyMax', addStd, iter);
%             toShow(end).label = sprintf('std=%d, Pruning algorithm iteration %d (greedy max, score=%f)', addStd, iter, greedyMaxScore);
%             toShow(end).summaries = greedyMaxSelected;
        end
        
    end
    
    % print overview
    fprintf(fh, 'You can find here...<br>\n');
    fprintf(fh, '* <a href="#images">Images in the dataset</a><br>\n');
    fprintf(fh, '* <a href="#images_freq">Images in the dataset sorted by frequency in summaries</a><br>\n');
    
    for addStd = 0:2,
        fprintf(fh, sprintf('* <a href="#performance_%d">%s</a><br>\n', addStd, sprintf('std=%d, Pruning algorithm performance', addStd)));
    end
    
    for i = 1:length(toShow),
        fprintf(fh, sprintf('* <a href="#%s">%s</a><br>\n', toShow(i).id, toShow(i).label));
    end
    
    fprintf(fh, '* <a href="#human_summaries_sorted">Human summaries sorted by rouge</a><br>\n');
    
    % create overview of image
    fprintf(fh, '<a id="images"><h3>Images in this dataset</h3></a>\n');
    
    fprintf(fh, '<table border=0>\n');
    idx = 1;
    for i = 1:10,
        fprintf(fh, '<tr>\n');
        for l = 1:10,
            fprintf(fh, sprintf('<td><img src="Data/set%d/%d.jpg"></td>', dataset, idx));
            idx = idx + 1;
        end
        fprintf(fh, '</tr>\n');
        fprintf(fh, '<tr>\n');
        idx = idx - 10;
        for l = 1:10,
            fprintf(fh, sprintf('<td><font size="2">%d</font></td>', idx));
            idx = idx + 1;
        end
        fprintf(fh, '</tr>\n');
    end
    fprintf(fh, '</table>');
    
    % create overview of image
    fprintf(fh, '<a id="images_freq"><h3>Images in this dataset (sorted by frequency)</h3></a>\n');
    f = zeros(100, 1);
    for i = 1:nrSummaries,
        f(summaries(i,:)) = f(summaries(i,:)) + 1;
    end
    f = f / size(summaries,1);
    
    [a,b] = sort(f, 1, 'descend');
    fprintf(fh, '<table border=0>\n');
    idx = 1;
    for i = 1:10,
        fprintf(fh, '<tr>\n');
        for l = 1:10,
            fprintf(fh, sprintf('<td><img src="Data/set%d/%d.jpg"></td>', dataset, b(idx)));
            idx = idx + 1;
        end
        fprintf(fh, '</tr>\n');
        fprintf(fh, '<tr>\n');
        idx = idx - 10;
        for l = 1:10,
            fprintf(fh, sprintf('<td><font size="2">f=%f</font></td>', f(b(idx))));
            idx = idx + 1;
        end
        fprintf(fh, '</tr>\n');

    end
    fprintf(fh, '</table>');
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % performance of pruning algorithm
    
    for addStd = 0:2,
        fprintf(fh, sprintf('<a id="performance_%d"><h3>%s</h3></a>\n', addStd, sprintf('std=%d, Pruning algorithm performance', addStd)));
        a = dir(sprintf('%s/html/images/SummaryPruning/set%d/std%d', getPreDir(), dataset, addStd));
        
        for iter = 0:((length(a)-2)/2)-1,
            fnameImage = sprintf('images/SummaryPruning/set%d/std%d/iteration_%d.png', dataset, addStd, iter);
            fnameSummaries = sprintf('%s/html/images/SummaryPruning/set%d/std%d/iteration_%d.mat', getPreDir(), dataset, addStd, iter);
            
            if  ~exist(fnameSummaries, 'file'),
                error('Expected file does not exist (%s, %s).', fnameImage, fnameSummaries);
            end
            
            fprintf(fh, sprintf('<img src="%s"><br>', fnameImage));
            
            %% load data
            load(fnameSummaries);
            
            %% show removed summaries
            fprintf(fh, '<b>Removed summaries:</b>');
            fprintf(fh, '<table style="padding-bottom: 1em;">\n');
            summariesToShow = removedSummaries;
            for i = 1:size(summariesToShow,1),
                options.textLeft = sprintf('#%d', i);
                options.textRight = sprintf('');
                createHTMLtableOfSummary(fh, dataset, summariesToShow(i,:), options);
            end
            fprintf(fh, '</table>');
            
            %% show greedy max
            fprintf(fh, '<b>Greedy max:</b>');
            fprintf(fh, '<table style="padding-bottom: 1em;">\n');
            options.textLeft = sprintf('#%d', i);
            options.textRight = sprintf('rouge: %f', greedyMaxScore);
            createHTMLtableOfSummary(fh, dataset, greedyMaxSelected, options);
            fprintf(fh, '</table>');
            
            
            %% show best 5 human summaries
            [a,b] = sort(scores, 'descend');
            fprintf(fh, '<b>5 best human summaries:</b>');
            fprintf(fh, '<table style="padding-bottom: 1em;">\n');
            summariesToShow = summaries(b(1:5),:);
            for i = 1:size(summariesToShow,1),
                options.textLeft = sprintf('#%d', i);
                options.textRight = sprintf('rouge: %f', a(i));
                createHTMLtableOfSummary(fh, dataset, summariesToShow(i,:), options);
            end
            fprintf(fh, '</table>');
            
            %% show best 5 random summaries
            [a,b] = sort(randScores, 'descend');
            fprintf(fh, '<b>5 best random summaries:</b>');
            fprintf(fh, '<table style="padding-bottom: 1em;">\n');
            summariesToShow = randomSummaries(b(1:5),:);
            for i = 1:size(summariesToShow,1),
                options.textLeft = sprintf('#%d', i);
                options.textRight = sprintf('rouge: %f', a(i));
                createHTMLtableOfSummary(fh, dataset, summariesToShow(i,:), options);
            end
            fprintf(fh, '</table>');
            
            %% show greedy min
            fprintf(fh, '<b>Greedy min:</b>');
            fprintf(fh, '<table style="padding-bottom: 1em;">\n');
            options.textLeft = sprintf('#%d', i);
            options.textRight = sprintf('rouge: %f', greedyMinScore);
            createHTMLtableOfSummary(fh, dataset, greedyMinSelected, options);
            fprintf(fh, '</table>');
            
            %% show worst 5 human summaries
            [a,b] = sort(scores, 'ascend');
            fprintf(fh, '<b>5 worst human summaries:</b>');
            fprintf(fh, '<table style="padding-bottom: 1em;">\n');
            summariesToShow = summaries(b(1:5),:);
            for i = 1:size(summariesToShow,1),
                options.textLeft = sprintf('#%d', i);
                options.textRight = sprintf('rouge: %f', a(i));
                createHTMLtableOfSummary(fh, dataset, summariesToShow(i,:), options);
            end
            fprintf(fh, '</table>');
            
            %% show worst 5 random summaries
            [a,b] = sort(randScores, 'ascend');
            fprintf(fh, '<b>5 worst random summaries:</b>');
            fprintf(fh, '<table style="padding-bottom: 1em;">\n');
            summariesToShow = randomSummaries(b(1:5),:);
            for i = 1:size(summariesToShow,1),
                options.textLeft = sprintf('#%d', i);
                options.textRight = sprintf('rouge: %f', a(i));
                createHTMLtableOfSummary(fh, dataset, summariesToShow(i,:), options);
            end
            fprintf(fh, '</table>');
            
            
            %toShow(end+1).id = sprintf('std_%d_iter_%d', addStd, iter);
            %toShow(end).label = sprintf('std=%d, Pruning algorithm iteration %d (removed summaries)', addStd, iter);
            %toShow(end).summaries = removedSummaries;
            
            %if isempty(removedSummaries), % do not display if no summaries were removed
            %    toShow(end) = [];
            %end
            
            fprintf(fh, '<hr>');
        end
        
        fprintf(fh, '<hr><hr><hr>\n');
        
    end
    
    %% TODO: Print human summaries sorted by rouge score (which rouge score?)
    fprintf(fh, '* <a id="human_summaries_sorted"><h3>Human summaries sorted by rouge (before pruning)</h3></a><br>\n');
    fnameSummaries = sprintf('%s/html/images/SummaryPruning/set%d/std%d/iteration_%d.mat', getPreDir(), dataset, 0, 0);
    load(fnameSummaries);
    
    [a,b] = sort(scores, 'ascend');
    fprintf(fh, '<table style="padding-bottom: 1em;">\n');
    summariesToShow = summaries(b,:);
    for i = 1:size(summariesToShow,1),
        options.textLeft = sprintf('#%d', i);
        options.textRight = sprintf('rouge: %f', a(i));
        createHTMLtableOfSummary(fh, dataset, summariesToShow(i,:), options);
    end
    fprintf(fh, '</table>');
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % removed summaries
    
%     for k = 1:length(toShow),
%         % show summaries in random order
%         fprintf(fh, sprintf('<a id="%s"><h3>%s</h3></a>\n', toShow(k).id, toShow(k).label));
%         fprintf(fh, '<table style="padding-bottom: 1em;">\n');
%         summariesToShow = toShow(k).summaries;
%         for i = 1:size(summariesToShow,1),
%             options.textLeft = sprintf('#%d', i);
%             createHTMLtableOfSummary(fh, dataset, summariesToShow(i,:), options);
%         end
%         fprintf(fh, '</table>');
%         fprintf(fh, '<p></p>');
%     end
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    % end html
    fprintf(fh, '</BODY>\n</HTML>\n');
end