% Plots the progress of the pruning algorithms over the iterations and
% displays the removed summaries

clear all;
clc;
warning('on','all');

nrDatasets = 14;

%% create images
maxIter = 0;
for dataset = 1:nrDatasets,
    directory = sprintf('%s/html/images/SummaryPruningIterative/set%d', getPreDir(), dataset);
    a = dir(sprintf('%s/*.mat', directory));
    maxIter = max(maxIter, length(a));
end
fprintf(1, 'Algorithm ran for at most %d iterations.\n', maxIter);

% overlap = NaN(nrDatasets, maxIter);
% stopIndices = NaN(nrDatasets, 3);
% for dataset = 2:nrDatasets,
%     fprintf(1, '>> Processing dataset %d...\n', dataset);
%     
%     directory = sprintf('html/images/SummaryPruningIterative/set%d', dataset);
%     a = dir(sprintf('%s/*.mat', directory));
%     for iter = 0:(length(a)-1),
%         clear('scores', 'randScores');
%         filename = sprintf('%s/iteration_%d.mat', directory, iter);
%         load(filename);
%         
%         % compute overlap
%         overlap(dataset, iter+1) = computeGaussianOverlap(mean(scores), mean(randScores), std(scores), std(randScores));
%         
%         for addStd = 0:2,
%             if min(scores) >= mean(randScores) + addStd * std(randScores),
%                 if isnan(stopIndices(dataset, addStd + 1)),
%                     stopIndices(dataset, addStd + 1) = iter;
%                 end
%             end
%         end
%     end
% end
% 
% %% compute stats
% iter = (0:(maxIter-1))*5;
% minOverlap = min(overlap, [], 1);
% maxOverlap = max(overlap, [], 1);
% meanOverlap = mean(overlap, 1);
% 
% clf
% h1 = plot(iter, minOverlap, 'r');
% hold on;
% h2 = plot(iter, maxOverlap, 'b');
% hold on;
% h3 = plot(iter, meanOverlap, 'k');
% xlabel('Number of pruned summaries');
% ylabel('Overlap');
% legend('min', 'max', 'mean');
% ylim([0,0.45]);
% xlim([0, maxIter*5]);
% 
% fnameImage = sprintf('%s/html/images/SummaryPruningIterative/overall.png', getPreDir());
% saveImage(fnameImage, 800, 400);
% 
% %% stop pruning at 0/1/2 std deviations of random mean
% for addStd = 0:2,
%     overlapWithStop = overlap;
%     for dataset = 1:nrDatasets,
%         idx = stopIndices(dataset, addStd+1);
%         overlapWithStop(dataset, (idx+1):end) = overlapWithStop(dataset, idx);
%     end
%     
%     minOverlap = min(overlapWithStop, [], 1);
%     maxOverlap = max(overlapWithStop, [], 1);
%     meanOverlap = mean(overlapWithStop, 1);
%     
%     % plot
%     clf
%     iter = (0:(maxIter-1))*5;
%     h1 = plot(iter, minOverlap, 'r');
%     hold on;
%     h2 = plot(iter, maxOverlap, 'b');
%     hold on;
%     h3 = plot(iter, meanOverlap, 'k');
%     xlabel('Number of pruned summaries');
%     ylabel('Overlap');
%     legend('min', 'max', 'mean');
%     ylim([0,0.45]);
% xlim([0, maxIter*5]);
% 
%     fnameImage = sprintf('%s/html/images/SummaryPruningIterative/overall_stop_at_mean_plus_std_%d.png', getPreDir(), addStd);
%     saveImage(fnameImage, 800, 400);
% end

%% create html
filename = sprintf('%s/html/pruningAlgorithmIterative.html', getPreDir());
% if exist(filename, 'file'),
%     warning('File "%s" already exists, skipping.');
%     continue;
% end
directory = fileparts(filename);
if ~exist(directory, 'dir'),
    mkdir(directory);
end

fh = fopen(filename, 'w');
fprintf(fh, '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01//EN"\n  "http://www.w3.org/TR/html4/strict.dtd"> \n<HTML>\n   <HEAD>\n      <TITLE>Dataset overview</TITLE> \n   </HEAD>\n  <BODY>\n');

% print overview
fprintf(fh, 'You can find here...<br>\n');
%fprintf(fh, '* <a href="#images">Images in the dataset</a><br>\n');
%fprintf(fh, '* <a href="#images_freq">Images in the dataset sorted by frequency in summaries</a><br>\n');
%fprintf(fh, '* <a href="#pruning">Overall performance of pruning algorithm</a><br>\n');
%fprintf(fh, '* Iteration infos (removed summaries, greedy max, ...): ');
%for iter = 0:(maxIter-20),
%    fprintf(fh, '<a href="#iter%d">#%d</a> / ', iter, iter);
%end
for dataset = 1:14,
    fprintf(fh, '<a href="#ds%d">Dataset %d</a><br>\n', dataset, dataset);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% fprintf(fh, '<a id="pruning"><h3>Performance of prunnig algorithm</h3></a>');
% fprintf(fh, '<b>Mean/Min/Max overlap over all 14 datasets over iterations of the algorithm.</b><br>In every iteration the current worst 5 summaries of every image collection are removed and the scores recomputed.<br>');
% fprintf(fh, '<img src="images/SummaryPruningIterative/overall.png"><br>');
% 
% for addStd = 0:2,
%     fprintf(fh, '<b>Mean/Min/Max overlap over all 14 datasets over iterations of the algorithm.</b><br>In every iteration the current worst 5 summaries of every image collection are removed and the scores recomputed. For every image collection individually, the pruning process is stopped, when the score of all human summaries (of the corresponding image collection) is lager than the mean score of the random summaries + %d standard deviations.<br>', addStd);
%     fprintf(fh, '<img src="images/SummaryPruningIterative/overall_stop_at_mean_plus_std_%d.png"><br>', addStd);
% end
    
stopTimes = [3    14     9    14    10     5     5    14     7     7     8    10    29    16];

for dataset = 1:nrDatasets,
    fprintf(fh, '<a id="ds%d"><h2>Dataset %d</h2></a>', dataset, dataset);
    fprintf(fh, 'STOP AFTER %d ITERATIONS (%d SUMMARIES)<br><br>', stopTimes(dataset), stopTimes(dataset)*5);
    
    for iter = 0:(maxIter-20),
    
        fprintf(fh, '<a id="iter%d"><h5>Iteration %d</h5></a>', iter, iter);
    
    
        directory = sprintf('%s/html/images/SummaryPruningIterative/set%d', getPreDir(), dataset);
        clear('scores', 'randScores', 'greedyMinSelected', 'greedyMaxSelected');
        filename = sprintf('%s/iteration_%d.mat', directory, iter);
        load(filename);
        
        if ~exist(filename, 'file'),
            fprintf(fh, '<b>Nothing for dataset %d</b><br>', dataset);
            continue;
        end
        
        summaries = load_summaries(dataset);
        
        %fprintf(fh, '<b>Dataset %d</b><br>', dataset);
        
        %% show removed summaries
        fprintf(fh, '<b>Removed summaries:</b>');
        fprintf(fh, '<table style="padding-bottom: 1em;">\n');
        summariesToShow = removedSummaries;
        for i = 1:size(summariesToShow,1),
            options.textLeft = sprintf('#%d', i);
            options.textRight = sprintf('');
            createHTMLtableOfSummary(fh, dataset, summariesToShow(i,:), options);
        end
        fprintf(fh, '</table>');

        if iter == stopTimes(dataset),
            fprintf(fh, '<br><hr><br>');
            fprintf(fh, '<br><hr><br>');
            fprintf(fh, '<br><hr><br>');
        end
    end


% for iter = 0:(maxIter-20),
%     fprintf(1, '>> Processing iteration %d...\n', iter);
%     
%     fprintf(fh, '<a id="iter%d"><h3>Iteration %d</h3></a>', iter, iter);
%     
%     for dataset = 1:nrDatasets,
%         directory = sprintf('html/images/SummaryPruningIterative/set%d', dataset);
%         clear('scores', 'randScores', 'greedyMinSelected', 'greedyMaxSelected');
%         filename = sprintf('%s/iteration_%d.mat', directory, iter);
%         load(filename);
%         
%         if ~exist(filename, 'file'),
%             fprintf(fh, '<b>Nothing for dataset %d</b><br>', dataset);
%             continue;
%         end
%         
%         summaries = load_summaries(dataset);
%         
%         fprintf(fh, '<b>Dataset %d</b><br>', dataset);
%         
%         %% show removed summaries
%         fprintf(fh, '<b>Removed summaries:</b>');
%         fprintf(fh, '<table style="padding-bottom: 1em;">\n');
%         summariesToShow = removedSummaries;
%         for i = 1:size(summariesToShow,1),
%             options.textLeft = sprintf('#%d', i);
%             options.textRight = sprintf('');
%             createHTMLtableOfSummary(fh, dataset, summariesToShow(i,:), options);
%         end
%         fprintf(fh, '</table>');
% 
%         %% show greedy max
%         fprintf(fh, '<b>Greedy max:</b>');
%         fprintf(fh, '<table style="padding-bottom: 1em;">\n');
%         options.textLeft = sprintf('#%d', i);
%         options.textRight = sprintf('rouge: %f', greedyMaxScore);
%         createHTMLtableOfSummary(fh, dataset, greedyMaxSelected, options);
%         fprintf(fh, '</table>');
% 
% 
%         %% show best 5 human summaries
%         [a,b] = sort(scores, 'descend');
%         fprintf(fh, '<b>5 best human summaries:</b>');
%         fprintf(fh, '<table style="padding-bottom: 1em;">\n');
%         summariesToShow = summaries(b(1:5),:);
%         for i = 1:size(summariesToShow,1),
%             options.textLeft = sprintf('#%d', i);
%             options.textRight = sprintf('rouge: %f', a(i));
%             createHTMLtableOfSummary(fh, dataset, summariesToShow(i,:), options);
%         end
%         fprintf(fh, '</table>');
% 
%         %% show best 5 random summaries
%         [a,b] = sort(randScores, 'descend');
%         fprintf(fh, '<b>5 best random summaries:</b>');
%         fprintf(fh, '<table style="padding-bottom: 1em;">\n');
%         summariesToShow = randomSummaries(b(1:5),:);
%         for i = 1:size(summariesToShow,1),
%             options.textLeft = sprintf('#%d', i);
%             options.textRight = sprintf('rouge: %f', a(i));
%             createHTMLtableOfSummary(fh, dataset, summariesToShow(i,:), options);
%         end
%         fprintf(fh, '</table>');
% 
%         %% show greedy min
%         fprintf(fh, '<b>Greedy min:</b>');
%         fprintf(fh, '<table style="padding-bottom: 1em;">\n');
%         options.textLeft = sprintf('#%d', i);
%         options.textRight = sprintf('rouge: %f', greedyMinScore);
%         createHTMLtableOfSummary(fh, dataset, greedyMinSelected, options);
%         fprintf(fh, '</table>');
% 
%         %% show worst 5 human summaries
%         [a,b] = sort(scores, 'ascend');
%         fprintf(fh, '<b>5 worst human summaries:</b>');
%         fprintf(fh, '<table style="padding-bottom: 1em;">\n');
%         summariesToShow = summaries(b(1:5),:);
%         for i = 1:size(summariesToShow,1),
%             options.textLeft = sprintf('#%d', i);
%             options.textRight = sprintf('rouge: %f', a(i));
%             createHTMLtableOfSummary(fh, dataset, summariesToShow(i,:), options);
%         end
%         fprintf(fh, '</table>');
% 
%         %% show worst 5 random summaries
%         [a,b] = sort(randScores, 'ascend');
%         fprintf(fh, '<b>5 worst random summaries:</b>');
%         fprintf(fh, '<table style="padding-bottom: 1em;">\n');
%         summariesToShow = randomSummaries(b(1:5),:);
%         for i = 1:size(summariesToShow,1),
%             options.textLeft = sprintf('#%d', i);
%             options.textRight = sprintf('rouge: %f', a(i));
%             createHTMLtableOfSummary(fh, dataset, summariesToShow(i,:), options);
%         end
%         fprintf(fh, '</table>');
%     end
    
%%%%%%
        
%     
%     %% prepare stuff to show
%     for addStd = 0:2,
%         a = dir(sprintf('%s/html/images/SummaryPruning/set%d/std%d', getPreDir(), dataset, addStd));
%         for iter = 0:((length(a)-2)/2)-1,
%             fnameImage = sprintf('%s/html/images/SummaryPruning/set%d/std%d/iteration_%d.png', getPreDir(), dataset, addStd, iter);
%             fnameSummaries = sprintf('%s/html/images/SummaryPruning/set%d/std%d/iteration_%d.mat', getPreDir(), dataset, addStd, iter);
%             
%             if ~exist(fnameImage, 'file') || ~exist(fnameSummaries, 'file'),
%                 error('Expected file does not exist (%s, %s).', fnameImage, fnameSummaries);
%             end
%             
%             load(fnameSummaries);
%             
% %             toShow(end+1).id = sprintf('std_%d_iter_%d', addStd, iter);
% %             toShow(end).label = sprintf('std=%d, Pruning algorithm iteration %d (removed summaries)', addStd, iter);
% %             toShow(end).summaries = removedSummaries;
% %             
% %             if isempty(removedSummaries), % do not display if no summaries were removed
% %                 toShow(end) = [];
% %             end
% %             
% %             toShow(end+1).id = sprintf('std_%d_iter_%d_greedyMin', addStd, iter);
% %             toShow(end).label = sprintf('std=%d, Pruning algorithm iteration %d (greedy min, score=%f)', addStd, iter, greedyMinScore);
% %             toShow(end).summaries = greedyMinSelected;
% %             
% %             toShow(end+1).id = sprintf('std_%d_iter_%d_greedyMax', addStd, iter);
% %             toShow(end).label = sprintf('std=%d, Pruning algorithm iteration %d (greedy max, score=%f)', addStd, iter, greedyMaxScore);
% %             toShow(end).summaries = greedyMaxSelected;
%         end
%         
%     end
%     
%     
%     for addStd = 0:2,
%         fprintf(fh, sprintf('* <a href="#performance_%d">%s</a><br>\n', addStd, sprintf('std=%d, Pruning algorithm performance', addStd)));
%     end
%     
%     for i = 1:length(toShow),
%         fprintf(fh, sprintf('* <a href="#%s">%s</a><br>\n', toShow(i).id, toShow(i).label));
%     end
%     
%     %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%     % performance of pruning algorithm
%     
%     for addStd = 0:2,
%         fprintf(fh, sprintf('<a id="performance_%d"><h3>%s</h3></a>\n', addStd, sprintf('std=%d, Pruning algorithm performance', addStd)));
%         a = dir(sprintf('%s/html/images/SummaryPruning/set%d/std%d', getPreDir(), dataset, addStd));
%         
%         for iter = 0:((length(a)-2)/2)-1,
%             fnameImage = sprintf('images/SummaryPruning/set%d/std%d/iteration_%d.png', dataset, addStd, iter);
%             fnameSummaries = sprintf('%s/html/images/SummaryPruning/set%d/std%d/iteration_%d.mat', getPreDir(), dataset, addStd, iter);
%             
%             if  ~exist(fnameSummaries, 'file'),
%                 error('Expected file does not exist (%s, %s).', fnameImage, fnameSummaries);
%             end
%             
%             fprintf(fh, sprintf('<img src="%s"><br>', fnameImage));
%             
%             %% load data
%             load(fnameSummaries);
%             
%             %% show removed summaries
%             fprintf(fh, '<b>Removed summaries:</b>');
%             fprintf(fh, '<table style="padding-bottom: 1em;">\n');
%             summariesToShow = removedSummaries;
%             for i = 1:size(summariesToShow,1),
%                 options.textLeft = sprintf('#%d', i);
%                 options.textRight = sprintf('');
%                 createHTMLtableOfSummary(fh, dataset, summariesToShow(i,:), options);
%             end
%             fprintf(fh, '</table>');
%             
%             %% show greedy max
%             fprintf(fh, '<b>Greedy max:</b>');
%             fprintf(fh, '<table style="padding-bottom: 1em;">\n');
%             options.textLeft = sprintf('#%d', i);
%             options.textRight = sprintf('rouge: %f', greedyMaxScore);
%             createHTMLtableOfSummary(fh, dataset, greedyMaxSelected, options);
%             fprintf(fh, '</table>');
%             
%             
%             %% show best 5 human summaries
%             [a,b] = sort(scores, 'descend');
%             fprintf(fh, '<b>5 best human summaries:</b>');
%             fprintf(fh, '<table style="padding-bottom: 1em;">\n');
%             summariesToShow = summaries(b(1:5),:);
%             for i = 1:size(summariesToShow,1),
%                 options.textLeft = sprintf('#%d', i);
%                 options.textRight = sprintf('rouge: %f', a(i));
%                 createHTMLtableOfSummary(fh, dataset, summariesToShow(i,:), options);
%             end
%             fprintf(fh, '</table>');
%             
%             %% show best 5 random summaries
%             [a,b] = sort(randScores, 'descend');
%             fprintf(fh, '<b>5 best random summaries:</b>');
%             fprintf(fh, '<table style="padding-bottom: 1em;">\n');
%             summariesToShow = randomSummaries(b(1:5),:);
%             for i = 1:size(summariesToShow,1),
%                 options.textLeft = sprintf('#%d', i);
%                 options.textRight = sprintf('rouge: %f', a(i));
%                 createHTMLtableOfSummary(fh, dataset, summariesToShow(i,:), options);
%             end
%             fprintf(fh, '</table>');
%             
%             %% show greedy min
%             fprintf(fh, '<b>Greedy min:</b>');
%             fprintf(fh, '<table style="padding-bottom: 1em;">\n');
%             options.textLeft = sprintf('#%d', i);
%             options.textRight = sprintf('rouge: %f', greedyMinScore);
%             createHTMLtableOfSummary(fh, dataset, greedyMinSelected, options);
%             fprintf(fh, '</table>');
%             
%             %% show worst 5 human summaries
%             [a,b] = sort(scores, 'ascend');
%             fprintf(fh, '<b>5 worst human summaries:</b>');
%             fprintf(fh, '<table style="padding-bottom: 1em;">\n');
%             summariesToShow = summaries(b(1:5),:);
%             for i = 1:size(summariesToShow,1),
%                 options.textLeft = sprintf('#%d', i);
%                 options.textRight = sprintf('rouge: %f', a(i));
%                 createHTMLtableOfSummary(fh, dataset, summariesToShow(i,:), options);
%             end
%             fprintf(fh, '</table>');
%             
%             %% show worst 5 random summaries
%             [a,b] = sort(randScores, 'ascend');
%             fprintf(fh, '<b>5 worst random summaries:</b>');
%             fprintf(fh, '<table style="padding-bottom: 1em;">\n');
%             summariesToShow = randomSummaries(b(1:5),:);
%             for i = 1:size(summariesToShow,1),
%                 options.textLeft = sprintf('#%d', i);
%                 options.textRight = sprintf('rouge: %f', a(i));
%                 createHTMLtableOfSummary(fh, dataset, summariesToShow(i,:), options);
%             end
%             fprintf(fh, '</table>');
%             
%             
%             %toShow(end+1).id = sprintf('std_%d_iter_%d', addStd, iter);
%             %toShow(end).label = sprintf('std=%d, Pruning algorithm iteration %d (removed summaries)', addStd, iter);
%             %toShow(end).summaries = removedSummaries;
%             
%             %if isempty(removedSummaries), % do not display if no summaries were removed
%             %    toShow(end) = [];
%             %end
%             
%             fprintf(fh, '<hr>');
%         end
%         
%         fprintf(fh, '<hr><hr><hr>\n');
%         
%     end
    
    %% TODO: Print human summaries sorted by rouge score (which rouge score?)
%     fprintf(fh, '* <a id="human_summaries_sorted"><h3>Human summaries sorted by rouge (before pruning)</h3></a><br>\n');
%     fnameSummaries = sprintf('%s/html/images/SummaryPruning/set%d/std%d/iteration_%d.mat', getPreDir(), dataset, 0, 0);
%     load(fnameSummaries);
%     
%     [a,b] = sort(scores, 'ascend');
%     fprintf(fh, '<table style="padding-bottom: 1em;">\n');
%     summariesToShow = summaries(b,:);
%     for i = 1:size(summariesToShow,1),
%         options.textLeft = sprintf('#%d', i);
%         options.textRight = sprintf('rouge: %f', a(i));
%         createHTMLtableOfSummary(fh, dataset, summariesToShow(i,:), options);
%     end
%     fprintf(fh, '</table>');
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % removed summaries
    
%     for k = 1:length(toShow),
%         % show summaries in random order
%         fprintf(fh, sprintf('<a id="%s"><h3>%s</h3></a>\n', toShow(k).id, toShow(k).label));
%         fprintf(fh, '<table style="padding-bottom: 1em;">\n');
%         summariesToShow = toShow(k).summaries;
%         for i = 1:size(summariesToShow,1),
%             options.textLeft = sprintf('#%d', i);
%             createHTMLtableOfSummary(fh, dataset, summariesToShow(i,:), options);
%         end
%         fprintf(fh, '</table>');
%         fprintf(fh, '<p></p>');
%     end
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
end
    
% end html
fprintf(fh, '</BODY>\n</HTML>\n');