%% creates an overview of some of the results

%% clear up
clc;
clear all;

%% settings
nrVisualWords = 5000;
nrAverage = 1000;
nrRandomSummaries = 1000;

%% run
for dataset = 2:2,
    fprintf(1, '>> Processing dataset %d...\n', dataset);
    
    %% load images
    [imagesrgb, imagesgray, imageshsv] = get_images(dataset, false);
    
    %% load summaries and compute scores
    [summaries, nrSummaries] = load_summaries(dataset);
    
    %% get codebook (SIFT)
    options = struct();
    options.clustering = 'random';
    options.normalizeBOW = 'l1';
    options.normalizeFeatuers = 'l1';
    options.summaries = summaries;
    options.dataset = dataset;
    options.features = 'phow';
    options.verbose = true;
    
    optionsEncoder = struct();
    optionsEncoder.encoder = 'nn';
    optionsEncoder.logSpaceScaling = true;
    options.encoderOptions = optionsEncoder;
    
    % filename for storing precomputed bow
    fname = sprintf('%s/precomputed/bow_ds_%d_len_%d_enc_%s_clust_%s_norm_%s_feat_%s.mat', ...
        getPreDir(), dataset, nrVisualWords, options.encoderOptions.encoder, options.clustering, options.normalizeBOW, ...
        options.features);
    if ~exist(fname, 'file'),
        error('Loading precomputed bag-of-words failed.');
    end
    load(fname);
    bow1 = bow;
    
    %% get codebook (color)
    nrVisualWords = 800;
    options = struct();
    options.clustering = 'random';
    options.normalizeBOW = 'l1';
    options.normalizeFeatuers = 'l1';
    options.summaries = summaries;
    options.dataset = dataset;
    options.features = 'colorSQ';
    options.verbose = true;
    
    optionsEncoder = struct();
    optionsEncoder.encoder = 'nnEMD';
    optionsEncoder.encoderThreshold = 0.80;
    optionsEncoder.logSpaceScaling = true;
    optionsEncoder.sigma = 0.007743;
    options.encoderOptions = optionsEncoder;
    % filename for storing precomputed bow
    fname = sprintf('precomputed/bow_ds_%d_len_%d_enc_%s_clust_%s_norm_%s_feat_%s.mat', ...
        dataset, nrVisualWords, options.encoderOptions.encoder, options.clustering, options.normalizeBOW, ...
        options.features);
    if ~exist(fname, 'file'),
        error('Loading precomputed bag-of-words failed.');
    end
    load(fname);
    bow2 = bow;
    
    %% concatenate words
    bow = vertcat(bow1, bow2);
    for k = 1:size(bow,2),
        nrm = norm(bow(:,k), 1);
        if nrm < 1e-6,
            nrm = 1;
        end
        bow(:,k) = bow(:,k) / nrm;
    end

    %% compute tfidf
    tfidf = get_TFIDF(bow);
    bow = applyTFIDF(bow, tfidf);
    
    %% create random summaries
    randomSummaries = randomSummary(100, 10, nrRandomSummaries);
    
    %% compute combinatorial n-gram score
    optionsEvaluation = struct();
    optionsEvaluation.k = 2;
    evaluation = 'combinatorial';
    [~,NGRAM_k_2] = summaryEvaluation(randomSummaries, summaries, evaluation, optionsEvaluation);
    [~,NGRAM_ref_k_2] = summaryEvaluation(summaries, summaries, evaluation, optionsEvaluation);
    
    %% compute bow score
    optionsEvaluation = struct();
    evaluation = 'rouge';
    optionsEvaluation.bow = bow;
    
    % precomp bag-of-words representation of summaries
    nrWords = size(bow, 1);
    BOWsummary = computeBOWSummary(summaries, bow);
    
    % evaluate
    [~,BOW] = summaryEvaluation(randomSummaries, summaries, evaluation, optionsEvaluation);
    [~,BOW_ref] = summaryEvaluation(summaries, summaries, evaluation, optionsEvaluation);
    

   
    %% create html
    fh = fopen(sprintf('%s/html/randomSummaryPerformanceDataset%d.html', getPreDir(), dataset), 'w');
    fprintf(fh, '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01//EN"\n  "http://www.w3.org/TR/html4/strict.dtd"> \n<HTML>\n   <HEAD>\n      <TITLE>Dataset overview</TITLE> \n   </HEAD>\n  <BODY>\n');
    
    % data to show
    toShow = [];
    
    % unsorted summaries
    toShow(end+1).id = 'random_summaries_sorted';
    toShow(end).label = 'Random summaries sorted by rouge score';
    [~,b] = sort(BOW,'descend');
    toShow(end).summaries = b;

    
    % print overview
    fprintf(fh, 'You can find here...<br>\n');
    fprintf(fh, '* <a href="#images">Images in the dataset</a><br>\n');
    fprintf(fh, '* <a href="#images_freq">Images in the dataset sorted by frequency in summaries</a><br>\n');
    
    for i = 1:length(toShow),
        fprintf(fh, sprintf('* <a href="#%s">%s</a><br>\n', toShow(i).id, toShow(i).label));
    end
    
    % create overview of image
    fprintf(fh, '<a id="images"><h3>Images in this dataset</h3></a>\n');
    
    fprintf(fh, '<table border=0>\n');
    idx = 1;
    for i = 1:10,
        fprintf(fh, '<tr>\n');
        for l = 1:10,
            fprintf(fh, sprintf('<td><img src="Data/set%d/%d.jpg"></td>', dataset, idx));
            idx = idx + 1;
        end
        fprintf(fh, '</tr>\n');
        fprintf(fh, '<tr>\n');
        idx = idx - 10;
        for l = 1:10,
            fprintf(fh, sprintf('<td>%d</td>', idx));
            idx = idx + 1;
        end
        fprintf(fh, '</tr>\n');
    end
    fprintf(fh, '</table>');
    
    % create overview of image
    fprintf(fh, '<a id="images_freq"><h3>Images in this dataset (sorted by frequency)</h3></a>\n');
    f = zeros(100, 1);
    for i = 1:nrSummaries,
        f(summaries(i,:)) = f(summaries(i,:)) + 1;
    end
    f = f / size(summaries,1);
    
    [a,b] = sort(f, 1, 'descend');
    fprintf(fh, '<table border=0>\n');
    idx = 1;
    for i = 1:10,
        fprintf(fh, '<tr>\n');
        for l = 1:10,
            fprintf(fh, sprintf('<td><img src="Data/set%d/%d.jpg"></td>', dataset, b(idx)));
            idx = idx + 1;
        end
        fprintf(fh, '</tr>\n');
        fprintf(fh, '<tr>\n');
        idx = idx - 10;
        for l = 1:10,
            fprintf(fh, sprintf('<td>f=%f</td>', f(b(idx))));
            idx = idx + 1;
        end
        fprintf(fh, '</tr>\n');

    end
    fprintf(fh, '</table>');
    

	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    for k = 1:length(toShow),
        % show summaries in random order
        fprintf(fh, sprintf('<a id="%s"><h3>%s</h3></a>\n', toShow(k).id, toShow(k).label));
        
        fprintf(fh, '<table style="padding-bottom: 1em;">\n');
        orderOfSummaries = toShow(k).summaries;
        summariesToShow = randomSummaries(orderOfSummaries,:);
        
        options = struct();
        for i = 1:size(summariesToShow,1),
            options.textLeft = sprintf('#%d', i);
            strA = sprintf('N-GRAM (k=%d): %.2f [on HS: avg: %.3f, std: %.3f]<br>', 2, NGRAM_k_2(orderOfSummaries(i)), mean(NGRAM_ref_k_2), std(NGRAM_ref_k_2));
            strB = sprintf('ROUGE (BOW, SIFT): %.2f [on HS: avg: %.3f, std: %.3f]<br>', BOW(orderOfSummaries(i)), mean(BOW_ref), std(BOW_ref));
            options.textRight = strcat(strA, strB);
            
            createHTMLtableOfSummary(fh, dataset, summariesToShow(i,:), options)
        end
        fprintf(fh, '</table>');
    end
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    % end html
    fprintf(fh, '</BODY>\n</HTML>\n');
end