% An evaluation infrastructure.

%% clean up
clc;
%clear all;

%% settings
dataset = 1;
k = 1;

%% structures for accounting
randscorecum = zeros(1, 14);
truescorecum = zeros(1, 14);

%% settings
%evaluation = 'bow';
evaluation = 'combinatorialNew';
nrAverage = 1000;
nrImages = 100;
summarySize = 10;

for dataset = 2:2,
    disp(sprintf('Iteration number %d', dataset));
    summaryfile = sprintf('../Data/set%d/summary.txt', dataset);
    summary = load(summaryfile);
    origSummary = summary;
    
    if strcmp(evaluation, 'bow'),
        % Load images
        [imagescolor, images] = get_images(dataset);
        
        % determine BoWs for every image
        nrWords = 30;
        [BoW,sim] = get_BOW3(imagescolor, images, nrWords, 'phow');
        nrWords = size(BoW, 1);
        tmpSummary = NaN(size(summary, 1), nrWords);
        for k = 1:size(summary, 1),
            tmpSummary(k, :) = sum(BoW(:, summary(k,:)), 2)';
            tmpSummary(k,:) = tmpSummary(k,:) / sum(tmpSummary(k,:));
        end
        summary = tmpSummary;
        
        if any(any(isnan(summary))),
            error('Should never happen.');
        end
    end
    
    %% Random subset
    fprintf(1, 'Testing random summaries...\n');
    randscore = 0;
    for times = 1 : nrAverage,
        rperm = randperm(nrImages);
        rset = rperm(1:summarySize);
        if strcmp(evaluation, 'bow'),
            rset = sum(BoW(:,rset), 2)';
            rset = rset / sum(rset);
        end
        addscore = summaryEvaluation(rset, summary, evaluation, k)*100;
        randscore = randscore + addscore;
    end
    randscorecum(dataset) = randscore/nrAverage;

    %% Actual Summary
    fprintf(1, 'Testing human summaries...\n');
    truescore = NaN(size(summary,1),1);
    for summaryno = 1 : size(summary, 1)
        currsummary = summary(summaryno, :);
        currsummarylist = [summary(1:summaryno - 1, :); summary(summaryno+1:size(summary,1), :)];
        addscore = summaryEvaluation(currsummary, currsummarylist, evaluation, k)*100;
        truescore(summaryno) = addscore;
    end
    truescorecum(dataset) = sum(truescore)/size(summary,1);
end

%% show summary
randscorecum
truescorecum
truescorecum ./ randscorecum

%% plot and save

% m = max(max(randscorecum), max(truescorecum));
% 
% subplot(3,1,1);
% h2 = bar(truescorecum);
% title(sprintf('Scores of human summaries (one-vs-rest) [k=%d]', k));
% xlabel('Dataset');
% ylabel('Score');
% ylim([0, m*1.2]);
% xlim([0,15]);
% 
% subplot(3,1,2);
% h1 = bar(randscorecum);
% title(sprintf('Scores of random summaries (trials = %d) [k=%d]', nrAverage, k));
% xlabel('Dataset');
% ylabel('Score');
% ylim([0, m*1.2]);
% xlim([0,15]);
% 
% subplot(3,1,3);
% h1 = bar(truescorecum ./ randscorecum);
% title(sprintf('Score ratio (human summaries to random summaries) [k=%d]', k));
% xlabel('Dataset');
% ylabel('ratio');
% xlim([0,15]);
% 
% h = line([0,15], [1,1]);
% set(h, 'Color', 'r');
% 
% print(gcf,'-dpdf', sprintf('results/scores_k%d.pdf', k))
