clear all;
clc;

basePath = fullfile('external/Rishabh/results/');
if ~exist(basePath, 'dir'),
    error('Base path ''%s'' does not exist.\n', basePath);
end

%% Perform comparison with competitor algorithms. We cycle through
%% all considered p values, i.e. p=0.1 and p=0.05

addpath external/sfo/

trainingsSetting = cell(2,2); % every row corrsponds to a training setting
% trainingsSetting{1,1} = 'ONEMINUSROUGE';
% trainingsSetting{1,2} = 'SUBSUP';
% trainingsSetting{1,3} = '1-R';
% trainingsSetting{1,4} = 'POSITIVE';
% trainingsSetting{1,5} = 'conic';

trainingsSetting{1,1} = 'ONEMINUSROUGE';
trainingsSetting{1,2} = 'SUBSUP';
trainingsSetting{1,3} = '1-R';
trainingsSetting{1,4} = 'SIMPLEX';
trainingsSetting{1,5} = 'convex';

% trainingsSetting{3,1} = 'COMPLEMENTROUGE';
% trainingsSetting{3,2} = 'RANDOMIZEDGREEDY';
% trainingsSetting{3,3} = 'Compl';
% trainingsSetting{3,4} = 'POSITIVE';
% trainingsSetting{3,5} = 'conic';

trainingsSetting{2,1} = 'COMPLEMENTROUGE';
trainingsSetting{2,2} = 'RANDOMIZEDGREEDY';
trainingsSetting{2,3} = 'Compl';
trainingsSetting{2,4} = 'SIMPLEX';
trainingsSetting{2,5} = 'convex';

% trainingsSetting{5,1} = 'SURROGATE';
% trainingsSetting{5,2} = 'GREEDY';
% trainingsSetting{5,3} = 'Surr';
% trainingsSetting{5,4} = 'POSITIVE';
% trainingsSetting{5,5} = 'conic';

trainingsSetting{3,1} = 'SURROGATE';
trainingsSetting{3,2} = 'GREEDY';
trainingsSetting{3,3} = 'Surr';
trainingsSetting{3,4} = 'SIMPLEX';
trainingsSetting{3,5} = 'convex';

% trainingsSetting{2,1} = 'OneMinusRouge';
% trainingsSetting{2,2} = 'greedy';
% trainingsSetting{2,3} = '1-R';
% trainingsSetting{3,1} = 'SurrogateRougeLoss';
% trainingsSetting{3,2} = 'greedy';
% trainingsSetting{3,3} = 'Surr';

lossFunctions = {'OneMinusRouge', 'SurrogateRougeLoss'};
lossFunctionsShort = {'\textsubscript{subsup}', '\textsubscript{sub}'};

for overlap = [0.10],
    % record results in these structures
    results = {};
    resultsDesc = {}; % first column is short description for table, second column detailed description for legend, third color information
    
    % Initial configurations
    bows = cell(14, 4);
    simBow = cell(14, 6);
    bowEvaluate = cell(14, 1);
    summariesAsBow = cell(14, 1);
    summaries = cell(14, 1);
    rougemax = zeros(1, 14);
    rougemin = zeros(1, 14);
    avghum = zeros(1, 14);
    avgrand = zeros(1, 14);
    resultsDesc{1,1} = '$\diameter$Rnd';
    resultsDesc{1,2} = 'Average random summaries';
    resultsDesc{1,3} = 'yellow';
    resultsDesc{2,1} = '$\diameter$Hmn';
    resultsDesc{2,2} = 'Average human summaries';
    resultsDesc{2,3} = 'yellow';
    resultsDesc{3,1} = 'Max';
    resultsDesc{3,2} = 'Greedy max of v-rouge score';
    resultsDesc{3,3} = 'yellow';
    resultsDesc{4,1} = 'Min';
    resultsDesc{4,2} = 'Min of v-rouge score';
    resultsDesc{4,3} = 'yellow';
    fprintf(1, 'Computing dataset statistics...\n');
    for ds = 1:14;
        % fid = fopen(sprintf('%d-results.txt', dataset), 'w');
        fid = 1;
        
        % remove worst summaries
        [summaries{ds}, prunedSummaries] = pruneSummaries(ds, get_pruning_info(ds, overlap));
        bows{ds, 1} = load_resource('bow', 'superPixels', ds);
        bows{ds, 2} = load_resource('bow', 'superPixels/tfidf', ds);
        
        bows{ds, 3} = load_resource('bow', 'colorHistogram', ds);
        bows{ds, 4} = load_resource('bow', 'colorHistogram/tfidf', ds);
        
        bows{ds, 5} = load_resource('bow', 'evaluation', ds);
        % bows{ds, 6} = load_resource('bow', 'evaluation/tfidf', ds);
        
        simBow{ds, 1} = BOWsimilarity(bows{ds, 1});
        simBow{ds, 2} = BOWsimilarity(bows{ds, 2});
        simBow{ds, 3} = BOWsimilarity(bows{ds, 3});
        simBow{ds, 4} = BOWsimilarity(bows{ds, 4});
        simBow{ds, 5} = BOWsimilarity(bows{ds, 5});
        % simBow{ds, 6} = BOWsimilarity(bows{ds, 6});
        
        %bowEvaluate = [bows{2};bows{4}];
        bowEvaluate{ds, 1} = get_BOW_evaluation(ds);
        
        summariesAsBow{ds, 1} = computeBOWSummary(summaries{ds}, bowEvaluate{ds, 1});
        rand('seed', 123456);
        randomSummaries = randomSummary(100, 10, 300);
        
        % setup functions to use
        optionsEvaluationLoss = struct();
        optionsEvaluationLoss.bow = bowEvaluate{ds, 1};
        optionsEvaluationLoss.summariesAsBow = summariesAsBow{ds, 1};
        
        % do we need to recompute this?
        resFileName = fullfile(getPreDir(), sprintf('tmp/summary_stats_ds_%d.mat', ds));
        if ~exist(resFileName, 'file'),
		[averageHumanScore,a1] = summaryEvaluation(summaries{ds}, summaries{ds}, 'rouge', optionsEvaluationLoss);
		[averageRandomScore,a2] = summaryEvaluation(randomSummaries, summaries{ds}, 'rouge', optionsEvaluationLoss);
		avghum(ds) = averageHumanScore;
		avgrand(ds) = averageRandomScore;
		
		% greedy max
		rougeGreedySum = sfo_greedy_lazy(sfo_fn_wrapper(@(x) summaryEvaluation(x, summaries{ds}, 'rouge', optionsEvaluationLoss)), 1:100, 10);
		rougemax(ds) = summaryEvaluation(rougeGreedySum, summaries{ds}, 'rouge', optionsEvaluationLoss);
		
		% greedy min
		[~,~,greedyMinRouge,~] = mmin(@(x) summaryEvaluation(x, summaries{ds}, 'rouge', optionsEvaluationLoss), 100, [], 'MUB', 10);
		rougemin(ds) = summaryEvaluation(greedyMinRouge, summaries{ds}, 'rouge', optionsEvaluationLoss);
		
		save(resFileName, 'averageRandomScore', 'averageHumanScore', 'rougemax', 'rougemin');  
	else
		disp('Loading stats...');
		load(resFileName);
	end       
        
        %fprintf(fid, '[ds=%2d] Average human score=%f [min=%f,max=%f]\n', ds, mean(b1), min(b1), max(b1));
        %fprintf(fid, '[ds=%2d] Average random score=%f [min=%f,max=%f]\n', ds, mean(b2), min(b2), max(b2));
        
        %[rougeGreedySum] = sfo_greedy_lazy(sfo_fn_wrapper(@(x) summaryEvaluation(x, summaries, 'rouge', optionsEvaluationLoss)), 1:100, 10);
        %fprintf(fid, 'ROUGE SCORE FOR Greedy Rouge: %f\n', summaryEvaluation(rougeGreedySum, summaries, 'rouge', optionsEvaluationLoss));
        
        % Account
        results{ds,1} = mean(averageRandomScore);
        results{ds,2} = mean(averageHumanScore);
        results{ds,3} = rougemax(ds);
        results{ds,4} = rougemin(ds);
    end
    
    resultIdx = 5;
    
    %% >> maximum of mixture (insert separately) >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
    fprintf(1, 'Loading results of mixture...\n');
    for settingIdx = 1:size(trainingsSetting,1),
        lossFunction = trainingsSetting{settingIdx,1};
        lossFunctionShort = trainingsSetting{settingIdx,3};
        algorithm = trainingsSetting{settingIdx,2};
        projection = trainingsSetting{settingIdx,4};
        projectionShort = trainingsSetting{settingIdx,5};
        resultsDesc{resultIdx,1} = sprintf('Mixt$^{\\textrm{%s},\\textrm{%s}}_{\\textrm{%s}}$', lossFunctionShort, projectionShort, algorithm);
        resultsDesc{resultIdx,2} = sprintf('Greedy max of the learned mixture, loss=%s, algo=%s (learning performed on all but one dataset; maximization on the remaining one)', lossFunction, algorithm);
        resultsDesc{resultIdx,3} = 'green';
        for ds = 1:14,
            clear('greedyMax');
            
            % Load precomputed results
            filename = sprintf('result_ds_%d_projection_%s_inference_%s_loss_%s_scenario_REAL.txt', ds, projection, algorithm, lossFunction);
            filename = fullfile(basePath, filename);
            if ~exist(filename, 'file'),
                results{ds,resultIdx} = NaN;
                continue;
            end
            tresults = loadResultsFile(filename);
            greedyMax = tresults.greedyMaxAverageWeights';
            
            % setup evaluation
            [summariesnp, nrSummaries] = load_summaries(ds);
            [summaries{ds}, prunedSummaries] = pruneSummaries(ds, get_pruning_info(ds, overlap));
            bowEvaluate{ds, 1} = get_BOW_evaluation(ds);
            
            summariesAsBow{ds, 1} = computeBOWSummary(summaries{ds}, bowEvaluate{ds, 1});
            optionsEvaluationLoss = struct();
            optionsEvaluationLoss.bow = bowEvaluate{ds, 1};
            optionsEvaluationLoss.summariesAsBow = summariesAsBow{ds, 1};
            
            results{ds,resultIdx} = summaryEvaluation(greedyMax, summaries{ds}, 'rouge', optionsEvaluationLoss);
        end
        resultIdx = resultIdx + 1;
    end
    %% << maximum of mixture <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
    
    %% >> maximum of mixture (insert separately) >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
    fprintf(fid, '>> Max Mix Final (%s)...\n', lossFunction);
    for settingIdx = 1:size(trainingsSetting,1),
        lossFunction = trainingsSetting{settingIdx,1};
        lossFunctionShort = trainingsSetting{settingIdx,3};
        algorithm = trainingsSetting{settingIdx,2};
        projection = trainingsSetting{settingIdx,4};
        projectionShort = trainingsSetting{settingIdx,5};
        resultsDesc{resultIdx,1} = sprintf('Mixt$^{\\textrm{%s,\\textdagger},\\textrm{%s}}_{\\textrm{%s}}$', lossFunctionShort, projectionShort, algorithm);
        resultsDesc{resultIdx,2} = sprintf('Greedy max of the learned mixture, loss=%s, algo=%s (using final weight; learning performed on all but one dataset; maximization on the remaining one)', lossFunction, algorithm);
        resultsDesc{resultIdx,3} = 'green';
        for ds = 1:14,
            clear('greedyMaxFinal');
            
            % Load precomputed results
            filename = sprintf('result_ds_%d_projection_%s_inference_%s_loss_%s_scenario_REAL.txt', ds, projection, algorithm, lossFunction);
            filename = fullfile(basePath, filename);
            if ~exist(filename, 'file'),
                results{ds,resultIdx} = NaN;
                continue;
            end
            tresults = loadResultsFile(filename);
            greedyMaxFinal = tresults.greedyMaxFinalWeights';
            
            if ~exist('greedyMaxFinal', 'var'),
                results{ds,resultIdx} = NaN;
                continue;
            end
            
            % setup evaluation
            [summaries{ds}, prunedSummaries] = pruneSummaries(ds, get_pruning_info(ds, overlap));
            bowEvaluate{ds, 1} = get_BOW_evaluation(ds);
            
            summariesAsBow{ds, 1} = computeBOWSummary(summaries{ds}, bowEvaluate{ds, 1});
            optionsEvaluationLoss = struct();
            optionsEvaluationLoss.bow = bowEvaluate{ds, 1};
            optionsEvaluationLoss.summariesAsBow = summariesAsBow{ds, 1};
            
            results{ds,resultIdx} = summaryEvaluation(greedyMaxFinal, summaries{ds}, 'rouge', optionsEvaluationLoss);
        end
        resultIdx = resultIdx + 1;
    end
    %% << maximum of mixture <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
    
    %% >> Load results from training and testing on the same dataset >>>>>>>>>>>>>>>
    for settingIdx = 1:size(trainingsSetting,1),
        lossFunction = trainingsSetting{settingIdx,1};
        lossFunctionShort = trainingsSetting{settingIdx,3};
        algorithm = trainingsSetting{settingIdx,2};
        projection = trainingsSetting{settingIdx,4};
        projectionShort = trainingsSetting{settingIdx,5};
        resultsDesc{resultIdx,1} = sprintf('Mixt$^{\\textrm{%s,1},\\textrm{%s}}_{\\textrm{%s}}$', lossFunctionShort, projectionShort, algorithm);
        resultsDesc{resultIdx,2} = sprintf('Greedy max of the learned mixture, loss=%s, algo=%s (learning performed on a single dataset; maximization on the same dataset)', lossFunction, algorithm);
        resultsDesc{resultIdx,3} = 'gray';
        for ds = 1:14
            % Load precomputed results
            filename = sprintf('result_ds_%d_projection_%s_inference_%s_loss_%s_scenario_CHEATING.txt', ds, projection, algorithm, lossFunction);
            filename = fullfile(basePath, filename);
            if ~exist(filename, 'file'),
                results{ds,resultIdx} = NaN;
                continue;
            end
            fprintf(1, 'Loading %s\n', filename);
            tresults = loadResultsFile(filename);
            greedyMax = tresults.greedyMaxAverageWeights';
            
            % setup evaluation
            [summariesnp, nrSummaries] = load_summaries(ds);
            [summaries{ds}, prunedSummaries] = pruneSummaries(ds, get_pruning_info(ds, overlap));
            bowEvaluate{ds, 1} = get_BOW_evaluation(ds);
            
            summariesAsBow{ds, 1} = computeBOWSummary(summaries{ds}, bowEvaluate{ds, 1});
            optionsEvaluationLoss = struct();
            optionsEvaluationLoss.bow = bowEvaluate{ds, 1};
            optionsEvaluationLoss.summariesAsBow = summariesAsBow{ds, 1};
            
            results{ds,resultIdx} = summaryEvaluation(greedyMax, summaries{ds}, 'rouge', optionsEvaluationLoss);
        end
        resultIdx = resultIdx + 1;
    end
    %% << Max Single <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
    
    %% >> maximum of single (insert separately) >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
    fprintf(fid, '>> Max Single Final...\n');
    for settingIdx = 1:size(trainingsSetting,1),
        lossFunction = trainingsSetting{settingIdx,1};
        lossFunctionShort = trainingsSetting{settingIdx,3};
        algorithm = trainingsSetting{settingIdx,2};
        projection = trainingsSetting{settingIdx,4};
        projectionShort = trainingsSetting{settingIdx,5};
        resultsDesc{resultIdx,1} = sprintf('Mixt$^{\\textrm{%s,\\textdagger,1},\\textrm{%s}}_{\\textrm{%s}}$', lossFunctionShort, projectionShort, algorithm);
        resultsDesc{resultIdx,2} = sprintf('Greedy max of the learned mixture, loss=%s, algo=%s (using final weight; learning performed on a single dataset; maximization on the same dataset)', lossFunction, algorithm);
        resultsDesc{resultIdx,3} = 'gray';
        for ds = 1:14,
            clear('greedyMaxFinal');
            
            % Load precomputed results
            filename = sprintf('result_ds_%d_projection_%s_inference_%s_loss_%s_scenario_CHEATING.txt', ds, projection, algorithm, lossFunction);
            filename = fullfile(basePath, filename);
            if ~exist(filename, 'file'),
                results{ds,resultIdx} = NaN;
                continue;
            end
            tresults = loadResultsFile(filename);
            greedyMaxFinal = tresults.greedyMaxFinalWeights';
            
            % setup evaluation
            [summaries{ds}, prunedSummaries] = pruneSummaries(ds, get_pruning_info(ds, overlap));
            bowEvaluate{ds, 1} = get_BOW_evaluation(ds);
            
            summariesAsBow{ds, 1} = computeBOWSummary(summaries{ds}, bowEvaluate{ds, 1});
            optionsEvaluationLoss = struct();
            optionsEvaluationLoss.bow = bowEvaluate{ds, 1};
            optionsEvaluationLoss.summariesAsBow = summariesAsBow{ds, 1};
            
            results{ds,resultIdx} = summaryEvaluation(greedyMaxFinal, summaries{ds}, 'rouge', optionsEvaluationLoss);
        end
        resultIdx = resultIdx + 1;
    end
    %% << maximum of single <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
    
    
    %% >> Facility location >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
    fprintf(fid, '>> Facility location...\n');
    resultsDesc{resultIdx,1} = 'FL';
    resultsDesc{resultIdx,2} = 'Greedy max of facility location (using the BoWs used for evaluation)';
    resultsDesc{resultIdx,3} = 'red';
    n = 100;
    sumsize = 10;
    options = {'sift', 'sifttfidf', 'color', 'colortfidf', 'siftcolor', 'siftcolortfidf'};
    facLocRouge = zeros(1, 14);
    for ds = 1:14
        % load correct BoWs
        bow = bowEvaluate{ds, 1};
        sim = BOWsimilarity(bow);
        
        optionsEvaluationLoss = struct();
        optionsEvaluationLoss.bow = bowEvaluate{ds, 1};
        optionsEvaluationLoss.summariesAsBow = summariesAsBow{ds, 1};
        
        % compute score
        Fcov = sfo_facility_coverage(sim);
        A = sfo_greedy_lazy(Fcov,1:n,sumsize);
        facLocRouge(1, ds) = summaryEvaluation(A, summaries{ds}, 'rouge', optionsEvaluationLoss);
        
        % bookkeeping
        results{ds,resultIdx} = facLocRouge(1,ds);
    end
    resultIdx = resultIdx + 1;
    %% << Facility Location <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
    
    %% >> Facility Location with penalty >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
    fprintf(fid, '>> Facility location with penalty...\n');
    resultsDesc{resultIdx,1} = 'FL\textsubscript{pen}\textsuperscript{*}';
    resultsDesc{resultIdx,2} = 'Greedy max of facility location + penalty (optimal penalty "weighting" selected on other datasets)';
    resultsDesc{resultIdx,3} = 'red';
    n = 100;
    sumsize = 10;
    
    lambdaRange = 0:0.1:1.0;
    tempScores = zeros(14, length(lambdaRange));
    for ds = 1:14,
        bow = bowEvaluate{ds, 1};
        sim = BOWsimilarity(bow);
        
        optionsEvaluationLoss = struct();
        optionsEvaluationLoss.bow = bowEvaluate{ds, 1};
        optionsEvaluationLoss.summariesAsBow = summariesAsBow{ds, 1};
        
        for lambdaIdx = 1:length(lambdaRange),
            lambda = lambdaRange(lambdaIdx);
            
            Fcov = sfo_facility_coverage(sim);
            Fdisp = sfo_penalty_disparity(sim);
            Fdisplambda = ntimes(Fdisp, lambda);
            Fsum = sfo_sum(Fcov, Fdisplambda);
            A = sfo_greedy_lazy(Fsum, 1:n, sumsize);
            tempScores(ds, lambdaIdx)  = summaryEvaluation(A, summaries{ds}, 'rouge', optionsEvaluationLoss);
        end
    end
    
    
    facdispRouge = zeros(1, 14);
    for ds = 1:14
        % load correct BoWs
        bow = bowEvaluate{ds, 1};
        sim = BOWsimilarity(bow);
        
        optionsEvaluationLoss = struct();
        optionsEvaluationLoss.bow = bowEvaluate{ds, 1};
        optionsEvaluationLoss.summariesAsBow = summariesAsBow{ds, 1};
        
        % select optimal lambda
        setd = setdiff(1:14,ds);
        [~,optLambdaIdx] = max(sum(tempScores(setd,:), 1));
        optLambda = lambdaRange(optLambdaIdx);
        
        Fcov = sfo_facility_coverage(sim);
        Fdisp = sfo_penalty_disparity(sim);
        Fdisplambda = ntimes(Fdisp, optLambda);
        Fsum = sfo_sum(Fcov, Fdisplambda);
        A = sfo_greedy_lazy(Fsum, 1:n, sumsize);
        facdispRouge(1, ds) = summaryEvaluation(A, summaries{ds}, 'rouge', optionsEvaluationLoss);
        
        results{ds,resultIdx} = facdispRouge(1,ds);
    end
    resultIdx = resultIdx + 1;
    %% << Facility Location with penalty <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
    
    %% >> MMR >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
    %fprintf(fid, '>> MMR...\n');
    %resultsDesc{resultIdx,1} = 'MMR';
    %resultsDesc{resultIdx,2} = '?';
    %n = 100;
    %sumsize = 10;
    %options = {'sift', 'sifttfidf', 'color', 'colortfidf', 'siftcolor', 'siftcolortfidf'};
    %MMRRouge = zeros(1, 14);
    %for ds = 1:14
    %    % load correct BoWs
    %    bow = bowEvaluate{ds, 1};
    %    sim = BOWsimilarity(bow);
    %
    %    optionsEvaluationLoss = struct();
    %    optionsEvaluationLoss.bow = bowEvaluate{ds, 1};
    %    optionsEvaluationLoss.summariesAsBow = summariesAsBow{ds, 1};
    %
    %    A = mmr_summarize(sim, n, sumsize);
    %    MMRRouge(1, ds)  = summaryEvaluation(A, summaries{ds}, 'rouge', optionsEvaluationLoss);
    %
    %    results{ds,resultIdx} = MMRRouge(1,ds);
    %end
    %resultIdx = resultIdx + 1;
    %% << MMR <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
    
    %% >> MMR++ (find optimal lambda on other datasets) >>>>>>>>>>>>>>>>>>>>>>>>>>>>
    fprintf(fid, '>> MMR...\n');
    resultsDesc{resultIdx,1} = 'MMR\textsuperscript{*}';
    resultsDesc{resultIdx,2} = 'MMR with optimal lambda selected on remaining datasets';
    resultsDesc{resultIdx,3} = 'red';
    n = 100;
    sumsize = 10;
    
    lambdaRange = 0:0.1:1.0;
    tempScores = zeros(14, length(lambdaRange));
    for ds = 1:14,
        bow = bowEvaluate{ds, 1};
        sim = BOWsimilarity(bow);
        
        optionsEvaluationLoss = struct();
        optionsEvaluationLoss.bow = bowEvaluate{ds, 1};
        optionsEvaluationLoss.summariesAsBow = summariesAsBow{ds, 1};
        
        for lambdaIdx = 1:length(lambdaRange),
            lambda = lambdaRange(lambdaIdx);
            A = mmr_summarize(sim, n, sumsize, lambda);
            tempScores(ds, lambdaIdx)  = summaryEvaluation(A, summaries{ds}, 'rouge', optionsEvaluationLoss);
        end
    end
    
    MMRRougePP = zeros(1, 14);
    for ds = 1:14
        % load correct BoWs
        bow = bowEvaluate{ds, 1};
        sim = BOWsimilarity(bow);
        
        optionsEvaluationLoss = struct();
        optionsEvaluationLoss.bow = bowEvaluate{ds, 1};
        optionsEvaluationLoss.summariesAsBow = summariesAsBow{ds, 1};
        
        setd = setdiff(1:14,ds);
        [~,optLambdaIdx] = max(sum(tempScores(setd,:), 1));
        optLambda = lambdaRange(optLambdaIdx);
        
        A = mmr_summarize(sim, n, sumsize, optLambda);
        MMRRougePP(1, ds)  = summaryEvaluation(A, summaries{ds}, 'rouge', optionsEvaluationLoss);
        
        results{ds,resultIdx} = MMRRougePP(1,ds);
    end
    resultIdx = resultIdx + 1;
    %% << MMR++ <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
    
    %% MMR like - graph-cut and penalty >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
    fprintf(fid, '>> Graph cut with penalty...\n');
    resultsDesc{resultIdx,1} = 'GC\textsubscript{pen}\textsuperscript{*}';
    resultsDesc{resultIdx,2} = 'Greedy max of graph cut + penalty (optimal penalty "weighting" selected on remaining datasets)';
    resultsDesc{resultIdx,3} = 'red';
    n = 100;
    sumsize = 10;
    
    lambdaRange = 0:0.1:1.0;
    tempScores = zeros(14, length(lambdaRange));
    for ds = 1:14,
        bow = bowEvaluate{ds, 1};
        sim = BOWsimilarity(bow);
        
        optionsEvaluationLoss = struct();
        optionsEvaluationLoss.bow = bowEvaluate{ds, 1};
        optionsEvaluationLoss.summariesAsBow = summariesAsBow{ds, 1};
        
        for lambdaIdx = 1:length(lambdaRange),
            lambda = lambdaRange(lambdaIdx);
            
            Fcov = sfo_cut_coverage(sim);
            Fdisp = sfo_penalty_disparity(sim);
            Fdisplambda = ntimes(Fdisp, lambda);
            Fsum = sfo_sum(Fcov, Fdisplambda);
            A = sfo_greedy_lazy(Fsum, 1:n, sumsize);
            
            tempScores(ds, lambdaIdx) = summaryEvaluation(A, summaries{ds}, 'rouge', optionsEvaluationLoss);
        end
    end
    
    gcdispRouge = zeros(1, 14);
    for ds = 1:14
        % load correct BoWs
        bow = bowEvaluate{ds, 1};
        sim = BOWsimilarity(bow);
        
        optionsEvaluationLoss = struct();
        optionsEvaluationLoss.bow = bowEvaluate{ds, 1};
        optionsEvaluationLoss.summariesAsBow = summariesAsBow{ds, 1};
        
        % select optimal lambda
        setd = setdiff(1:14,ds);
        [~,optLambdaIdx] = max(sum(tempScores(setd,:), 1));
        optLambda = lambdaRange(optLambdaIdx);
        
        Fcov = sfo_cut_coverage(sim);
        Fdisp = sfo_penalty_disparity(sim);
        Fdisplambda = ntimes(Fdisp, optLambda);
        Fsum = sfo_sum(Fcov, Fdisplambda);
        A = sfo_greedy_lazy(Fsum, 1:n, sumsize);
        gcdispRouge(1, ds) = summaryEvaluation(A, summaries{ds}, 'rouge', optionsEvaluationLoss);
        
        results{ds,resultIdx} = gcdispRouge(1,ds);
    end
    resultIdx = resultIdx + 1;
    %% << MMR like - graph-cut and penalty <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
    
    
    %% >> KMeans >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
    fprintf(fid, '>> K-Means...\n');
    resultsDesc{resultIdx,1} = 'KMeans';
    resultsDesc{resultIdx,2} = 'KMeans';
    resultsDesc{resultIdx,3} = 'red';
    rand('seed', 123456');
    n = 100;
    sumsize = 10;
    options = {'sift', 'sifttfidf', 'color', 'colortfidf', 'siftcolor', 'siftcolortfidf'};
    kmeansRouge = zeros(1, 14);
    for ds = 1:14
        % load correct BoWs
        bow = bowEvaluate{ds, 1};
        sim = BOWsimilarity(bow);
        
        optionsEvaluationLoss = struct();
        optionsEvaluationLoss.bow = bowEvaluate{ds, 1};
        optionsEvaluationLoss.summariesAsBow = summariesAsBow{ds, 1};
        
        A = kmeans_clusteringheuristic(bow', sim, sumsize);
        kmeansRouge(1, ds)  = summaryEvaluation(A, summaries{ds}, 'rouge', optionsEvaluationLoss);
        
        results{ds,resultIdx} = kmeansRouge(1,ds);
    end
    resultIdx = resultIdx + 1;
    %% << KMeans <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
    
    %% >> Dictionary learning according to Yang et al >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
    fprintf(fid, '>> Yangs'' method...\n');
    resultsDesc{resultIdx,1} = 'Yang';
    resultsDesc{resultIdx,2} = 'Yang';
    resultsDesc{resultIdx,3} = 'red';
    for ds = 1:14
        % TODO: Wrap code in function
        
        results{ds,resultIdx} = nan;
    end
    resultIdx = resultIdx + 1;
    %% << Yang <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
    
    
    %% >> Create LaTeX Table >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
    texFile = sprintf('../results/experiment_competitors_eval_overlap_%1.2f.tex', overlap);
    if ~exist(fileparts(texFile), 'dir'),
        fprintf(1, 'Creating directory ''%s''...\n', fileparts(texFile));
        mkdir(fileparts(texFile));
    end
    fid = 1;
    fid = fopen(texFile, 'w');
    fprintf(fid, '\\documentclass{article}\n');
    fprintf(fid, '\\usepackage{booktabs}\n');
    fprintf(fid, '\\usepackage{subscript}\n');
    fprintf(fid, '\\usepackage{wasysym}\n');
    fprintf(fid, '\\usepackage[table]{xcolor}\n');
    fprintf(fid, '\\usepackage[a0paper,top=1cm,bottom=1cm,left=1cm,right=1cm,landscape]{geometry}\n');
    
    fprintf(fid, '\\begin{document}\n');
    
    fprintf(fid, '\\section*{Overlap %1.2f}\n', overlap);
    
    fprintf(fid, '\\begin{table}[!h]\n');
    fprintf(fid, '\\caption{Comparsion with various baseline methods. \\emph{Max Single} refers to training and testing of the mixture on the same dataset.}\n');
    
    fprintf(fid, '\\begin{tabular}{l');
    for i = 1:size(resultsDesc,1),
        fprintf(fid, 'c');
    end
    fprintf(fid, '}\n');
    
    fprintf(fid, '\\toprule');
    fprintf(fid, [' & \\multicolumn{', int2str(length(resultsDesc)), '}{c}{Method}\\\\\\cmidrule{2-', int2str(1+length(resultsDesc)), '}\n']);
    
    % print header
    fprintf(fid, ' & ');
    for i = 1:size(resultsDesc,1),
        if ~isempty(resultsDesc{i,3}),
            fprintf(fid, '\\cellcolor{%s!25}', resultsDesc{i,3});
        end
        fprintf(fid, '%s', resultsDesc{i});
        if i < size(resultsDesc,1),
            fprintf(fid, ' & ');
        end
    end
    fprintf(fid, '\\\\\\midrule\n');
    
    % print data
    avgRndIdx = 1;
    avgHmnIdx = 2;
    for ds = 1:size(results, 1),
        fprintf(fid, ['\\textbf{DS', int2str(ds), '} & ']);
        for i = 1:size(resultsDesc,1),
            if ~isempty(resultsDesc{i,3}),
                fprintf(fid, '\\cellcolor{%s!25}', resultsDesc{i,3});
            end
            fprintf(fid, '%1.2f (%1.2f)', results{ds,i}, (results{ds,i} - results{ds,avgRndIdx}) / (results{ds,avgHmnIdx} - results{ds,avgRndIdx}));
            if i < size(resultsDesc,1),
                fprintf(fid, '& ');
            end
        end
        
        if ds == size(results,1),
            fprintf(fid, '\\\\[0.2cm]\n');
        else
            fprintf(fid, '\\\\\n');
        end
    end
    
    % compute aggregate score
    avgRndIdx = 1;
    avgHmnIdx = 2;
    score = zeros(1, size(results,2));
    for i = 1:size(results,2),
        for ds = 1:size(results,1),
            nom = (results{ds,i} - results{ds,avgRndIdx});
            denom = (results{ds,avgHmnIdx} - results{ds,avgRndIdx});
            score(i) = score(i) + nom / denom;
        end
        score(i) = score(i) / 14;
    end
    
    % print aggregate score
    fprintf(fid, [' & ']);
    for i = 1:size(resultsDesc,1),
        if ~isempty(resultsDesc{i,3}),
            fprintf(fid, '\\cellcolor{%s!25}', resultsDesc{i,3});
        end
        fprintf(fid, '%1.3f', score(i));
        if i < size(resultsDesc,1),
            fprintf(fid, '& ');
        end
    end
    fprintf(fid, '\\\\\n');
    
    % end table
    fprintf(fid, '\\bottomrule\n');
    
    fprintf(fid, '\\end{tabular}\n');
    fprintf(fid, '\\end{table}\n');
    
    % show legend
    %fprintf(fid, '\\begin{itemize}\n');
    for i = 1:size(resultsDesc,1),
        fprintf(fid, '\\noindent\\textbf{%s.} %s\\\\\n', resultsDesc{i,1}, resultsDesc{i,2});
    end
    %fprintf(fid, '\\end{itemize}\n');
    
    % print config
    fprintf(fid, '\\section*{Configuration}\n');
    newline = sprintf('\n');
%     configStr = evalc('config');
    configStr = strrep(tresults.config, newline, '\\');
    %fprintf(fid, '%s\n', configStr);
    
%     fprintf(fid, '\\section*{Configuration Optimizer}\n');
%     newline = sprintf('\n');
%     configStr = evalc('config.optimizer');
%     configStr = strrep(configStr, newline, '\\');
%     fprintf(fid, '%s\n', configStr);
    
    fprintf(fid, '\\end{document}\n');
    
    fclose(fid);
    [status, result] = system(sprintf('cd ../results; pdflatex %s', texFile));
    if status ~= 0,
        warning('Creation of pdf file failed: %s\n', result);
    end
    
    fprintf(1, 'Output written to ''%s''...\n', texFile);
end


