% sanity check for java implementation of certain functions

clear all;
clear java;
clc;

javaaddpath(fullfile(pwd, 'Java/SubmodularFunctions/bin'));
javaaddpath(fullfile(pwd, 'Java/Cuda/jcuda.jar'));
%javaaddpath('/srv/stschia/tmp/JCuda/jcuda.jar');
addpath('external/sfo');
addpath('SubmodularComponents');
addpath('Java/MatlabInterface');
setupVLFEAT

%% >> Settings >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
nrChecks = 100;
tol = 1e-4;
ds = 9;
overlap = 0.1;
format long
%% << Settings <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

%% >> Facility Location >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
tic
for i = 1:nrChecks,
    bow = rand(1000,100);
    fctMatlab = @(S) sum(sqrt(sum(bow(:,S),2))) / sum(sqrt(sum(bow,2)));
    
    %simJava = MatrixJ(sim); 
    fctJava = CompressedWordCountJ(MatrixJ(bow));
    
    S = randperm(90);
    S = S(1:max(S(1),2));
    candidates = setdiff(1:100, S);
    
    valMatlab = zeros(length(candidates), 1);
    for c = 1:length(candidates),
        valMatlab(c) = fctMatlab([S, candidates(c)]);
    end
    valJava = java2matArray(fctJava.eval(S, candidates));
    
    if any(abs(valMatlab(:) - valJava(:)) > tol),
        valMatlab
        valJava
        error('Function values do not match.');
    end
end
fprintf(1, 'OK\n');
return

%% >> Greedy >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
for i = 1:nrChecks,
    sim = rand(100);
    fctMatlab = @(S) sum(sum(max(sim(:,S),[],2))) / 100;
    
    fctJavaLinComb = LinCombFunctionJ();
    fctJava = FacilityLocationJ(MatrixJ(sim));
    fctJavaLinComb.addFunction(fctJava);
    fctJavaLinComb.setWeights(1.0);
    greedy = GreedyJ(fctJavaLinComb);
    SmaxJava = greedy.maximizeadv(100, 10);
    SmaxJava = java2matArray(SmaxJava);
    SmaxMatlab = greedySummary(fctMatlab, 10);
    
    if any(sort(SmaxJava(:)) ~= sort(SmaxMatlab(:))),
        error('Function values do not match.');
    end
end
fprintf(1, 'Greedy: OK\n');
fprintf(1, 'Performing timing...\n');
nrTries = 1e2;
nrFunctions = 1e3;
fctJavaLinComb = LinCombFunctionJ();
for i = 1:nrFunctions,
    fctJava = SumCoverageJ(simJava);
    fctJavaLinComb.addFunction(fctJava);
end
fctJavaLinComb.setWeights(rand(1, nrFunctions));
greedy = GreedyJ(fctJavaLinComb);
tic; for i = 1:nrTries; greedy.maximizep(100, 10); end; ttt = toc; fprintf(1, 'Java Greedy Parallel: %f seconds\n', ttt);
tic; for i = 1:nrTries; greedy.maximizeadv(100, 10); end; ttt = toc; fprintf(1, 'Java Greedy Advanced: %f seconds\n', ttt);
tic; for i = 1:nrTries; greedy.maximize(100, 10); end; ttt = toc; fprintf(1, 'Java Greedy: %f seconds\n', ttt);
%tic; for i = 1:nrTries; greedySummary(@(S) fctJavaLinComb.eval(S), 10); end; ttt = toc; fprintf(1, 'Matlab Greedy: %f seconds\n', ttt);
%% << Greedy <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
return

%% >> Facility Location >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
tic
sim = rand(100);
fctMatlab = @(S) sum(sum(max(sim(:,S),[],2))) / 100;
fctJava = FacilityLocationJ(MatrixJ(simJava));
S = randperm(90);
S = S(1:S(1));
candidates = setdiff(1:100, S);
tic;
for i = 1:nrChecks,
    valMatlab = zeros(length(candidates), 1);
    for c = 1:length(candidates),
        valMatlab(c) = fctMatlab([S, candidates(c)]);
    end
end
timeMatlab = toc;
tic;
for i = 1:nrChecks,
    valJava = fctJava.eval(S, candidates);
end
timeJava = toc;
timeMatlab
timeJava

% timing
sim = rand(100);
fctMatlab = @(S) sum(sum(max(sim(:,S),[],2))) / 100;
simJava = MatrixJ(sim); 
fctJava = FacilityLocationJ(simJava);
S = randperm(100);
S = int32(S(1:S(1)));
candidates = int32(setdiff(1:100, S));
tic
for i = 1:1e4,
    valMatlab = zeros(length(candidates), 1);
    for c = 1:length(candidates),
        valMatlab(c) = fctMatlab([S, candidates(c)]);
    end
end
toc

tic
for i = 1:1e4,
    valJava = fctJava.eval(S, candidates);
end
toc

fprintf(1, 'Facility Location Parallel: OK\n');
%% << Facility Location <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
return


%% >> Facility LocationCuda >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
tic
for i = 1:nrChecks,
    sim = rand(100);
    fctMatlab = @(S) sum(sum(max(sim(:,S),[],2))) / 100;
    
    simJava = MatrixJ(sim); 
    fctJava = FacilityLocationCudaJ(simJava);
    
    S = randperm(100);
    S = S(1:S(1));
    
    valMatlab = fctMatlab(S);
    valJava = fctJava.eval(S);
    
    if abs(valMatlab - valJava) > tol,
        valMatlab
        valJava
        error('Function values do not match.');
    end
end
fprintf(1, 'Facility Location Cuda: OK\n');
%% << Facility Location <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
return

%% >> Rouge Loss >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
tic
for i = 1:nrChecks,
    ds = 1;
    bow = rand(1000,100);
    summaries = randomSummary(100, 10, 20);
    summariesAsBow = computeBOWSummary(summaries, bow);
    fctMatlab = @(S) rougeLoss(S, bow, summariesAsBow, ds);
    
    fctJava = SurrogateLoss(MatrixJ(bow), MatrixJ(summariesAsBow));
    
    S = randperm(100);
    S = S(1:S(1));
    
    valMatlab = fctMatlab(S);
    valJava = fctJava.eval(S);
    
    if abs(valMatlab - valJava) > tol,
        valMatlab
        valJava
        error('Function values do not match.');
    end
end
fprintf(1, 'Rouge Loss: OK\n');
%% << Rouge Loss <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
return

%% >> Greedy Lazy >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
% for i = 1:nrChecks,
%     sim = rand(100);
%     fctMatlab = @(S) sum(sum(max(sim(:,S),[],2))) / 100;
%     
%     fctJavaLinComb = LinCombFunction();
%     fctJava = FacilityLocation(sim);
%     fctJavaLinComb.addFunction(fctJava);
%     fctJavaLinComb.setWeights(1);
%     greedy = GreedyLazy(fctJavaLinComb);
%     SmaxJava = greedy.maximize(100, 10);
%     SmaxMatlab = greedySummary(fctMatlab, 10);
%     
%     if any(sort(SmaxJava(:)) ~= sort(SmaxMatlab(:))),
%         error('Function values do not match.');
%     end
% end
% %tic; for i = 1:1e3; greedy.maximize(100, 10); end; toc
% %tic; for i = 1:1e3; greedy.maximizep(100, 10); end; toc
% %tic; for i = 1:1e3; greedySummary(fctMatlab, 10); end; toc
% fprintf(1, 'Greedy Lazy: OK\n');
%% << Greedy Lazy <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<


% rand('seed', 123456);
% nrComponents = 2;
% fct1 = LinCombFunction();
% fct1Matlab = sfo_fn_wrapper(@(S) 0);
% weights = rand(1,nrComponents);
% for i = 1:nrComponents,
%     sim = rand(100);
%     fct1.addFunction(FacilityLocation(sim));
%     fct1Matlab = sfo_fn_lincomb({fct1Matlab, sfo_fn_wrapper(@(S) sum(sum(max(sim(:,S),[],2))) / 100)}, [1,weights(i)]);
% end
% fct1.setWeights(weights);
% sim = rand(100);
% fct2 = OneMinusRouge([], rand(1000,100), 5*rand(1000,300));
% F = @(S) fct1.eval(S);
% G = @(S) 10*fct2.eval(S);
% V = 1:100;
% nrChecks = 2;
% finalFct = LinCombFunction();
% finalFct.addFunction(fct1);
% finalFct.addFunction(fct2);
% finalFct.setWeights([1,-1]);
% foo = Greedy(finalFct);
% Xinit = foo.maximize(100,10);
% tic; 
% for i=1:nrChecks, 
%     disp('with init'); 
%     y1 = sfo_supsubimproveducNew(sfo_fn_wrapper(F), sfo_fn_wrapper(G), V, 2, fct1, fct2, Xinit); 
%     disp('without init');
%     y1 = sfo_supsubimproveducNew(sfo_fn_wrapper(F), sfo_fn_wrapper(G), V, 2, fct1, fct2); 
% end; 
% toc
% tic; 
% for i=1:nrChecks, 
%     disp('with init'); 
%     y2 = sfo_supsubimproveduc(fct1Matlab, sfo_fn_wrapper(G), V, 2, Xinit'); 
%     disp('without init');
%     y2 = sfo_supsubimproveduc(fct1Matlab, sfo_fn_wrapper(G), V, 2); 
% end; 
% toc
% tic; 
% for i=1:nrChecks, 
%     disp('with init'); 
%     y2 = sfo_supsubimproveduc(sfo_fn_wrapper(F), sfo_fn_wrapper(G), V, 2, Xinit'); 
%     disp('without init');
%     y2 = sfo_supsubimproveduc(sfo_fn_wrapper(F), sfo_fn_wrapper(G), V, 2); 
% end; 
% toc
% if any(y1(:) ~= y2(:)),
%    error('Results do not match.');
% end
% return

% %% >> Greedy >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
% for i = 1:1,
%     sim = rand(100);
%     fctMatlab = @(S) sum(sum(max(sim(:,S),[],2))) / 100;
%     
%     fctJavaLinComb = LinCombFunction();
%     fctJava = FacilityLocation(sim);
%     fctJavaLinComb.addFunction(fctJava);
%     fctJavaLinComb.setWeights(1);
%     greedy = Greedy(fctJavaLinComb);
%     greedyNew = GreedyNew(fctJavaLinComb);
%     SmaxJava = greedy.maximizep(100, 10);
%     SmaxMatlab = greedySummary(fctMatlab, 10);
%     SmaxJavaNew = greedyNew.maximizep(100, 10);
%     
%     if any(sort(SmaxJava(:)) ~= sort(SmaxMatlab(:))),
%         error('Function values do not match.');
%     end
%     
%     if any(sort(SmaxJavaNew(:)) ~= sort(SmaxMatlab(:))),
%         error('Function values do not match.');
%     end
% end
% disp('maximize ');
% tic; for i = 1:1e3; greedy.maximize(100, 10); end; toc
% disp('maximizep ');
% tic; for i = 1:1e3; greedy = Greedy(fctJavaLinComb); greedy.maximizep(100, 10); end; toc
% disp('maximizep new ');
% tic; for i = 1:1e3; greedyNew.maximizep(100, 10); end; toc
% tic; for i = 1:1e3; greedySummary(fctMatlab, 10); end; toc
% % fprintf(1, 'Greedy: OK\n');
% %% << Greedy <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
% return



%% >> Facility Location >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
tic
for i = 1:nrChecks,
    sim = rand(100);
    fctMatlab = @(S) sum(sum(max(sim(:,S),[],2))) / 100;
    
    simJava = MatrixJ(sim); 
    fctJava = FacilityLocationJ(simJava);
    
    S = randperm(100);
    S = S(1:S(1));
    
    valMatlab = fctMatlab(S);
    valJava = fctJava.eval(S);
    
    if abs(valMatlab - valJava) > tol,
        valMatlab
        valJava
        error('Function values do not match.');
    end
end
fprintf(1, 'Facility Location: OK\n');
%% << Facility Location <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

%% >> Clustered Diversity Component >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
for i = 1:nrChecks,
    a = rand(1);
    reward = rand(100,1);
    clusterAssignment = ceil(rand(100,1)*10);
    fctMatlab = @(S) diversityComponent(S, clusterAssignment, reward, a);
    fctJava = ClusteredDiversityJ(clusterAssignment, reward, a);
    
    S = randperm(100);
    S = S(1:S(1));
    
    valMatlab = fctMatlab(S);
    valJava = fctJava.eval(S);
    
    if abs(valMatlab - valJava) > tol,
        valMatlab
        valJava
        error('Function values do not match.');
    end
end
fprintf(1, 'Clustered Diversity Component: OK\n');
%% << Clustered Diversity Component <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

%% >> Clustered Facility Location >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
for i = 1:nrChecks,
    reward = rand(100,1);
    clusterAssignment = ceil(rand(100,1)*10);
    fctMatlab = @(S) facilityLocation(S, clusterAssignment, reward);
    fctJava = ClusteredFacilityLocationJ(clusterAssignment, reward);
    
    S = randperm(100);
    S = S(1:S(1));
    
    valMatlab = fctMatlab(S);
    valJava = fctJava.eval(S);
    
    if abs(valMatlab - valJava) > tol,
        valMatlab
        valJava
        error('Function values do not match.');
    end
end
fprintf(1, 'Clustered Facility Location: OK\n');
%% << Clustered Facility Location <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

%% >> Thresholded Feature Counts >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
for i = 1:nrChecks,
    bow = rand(1000, 100);
    threshold = rand(1);
    fctMatlab = @(S) 1/(size(bow,1)*threshold) * sum(min(sum(bow(:,S),2) ./ sum(bow,2), threshold*ones(size(bow,1),1)));
    bowJava = MatrixJ(bow);
    fctJava = ThresholdedFeatureCountsJ(bowJava, threshold);
    
    S = randperm(100);
    S = S(1:S(1));
    
    valMatlab = fctMatlab(S);
    valJava = fctJava.eval(S);
    
    if abs(valMatlab - valJava) > tol,
        valMatlab
        valJava
        error('Function values do not match.');
    end
end
fprintf(1, 'Thresholded Feature Counts: OK\n');
%% << Thresholded Feature Counts <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

%% >> Cosine Similartiy >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
for i = 1:nrChecks,
    bow = rand(1000, 100);
    threshold = rand(1);
    fctMatlab = @(S) cosineSimilarity(S, bow, threshold);
    fctJava = CosineSimilarityJ(MatrixJ(bow'*bow), threshold);
    
    S = randperm(100);
    S = S(1:S(1));
    
    valMatlab = fctMatlab(S);
    valJava = fctJava.eval(S);
    
    if abs(valMatlab - valJava) > tol,
        valMatlab
        valJava
        error('Function values do not match.');
    end
end
fprintf(1, 'Cosine Similartiy: OK\n');
%% << Cosine Similartiy <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

%% >> Indicator Function >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
for i = 1:nrChecks,
    idx = randperm(100);
    idx = idx(1);
    fctMatlab = @(S) any(S == idx);
    fctJava = IndicatorJ(idx);
    
    S = randperm(100);
    S = S(1:S(1));
    
    valMatlab = fctMatlab(S);
    valJava = fctJava.eval(S);
    
    if abs(valMatlab - valJava) > tol,
        valMatlab
        valJava
        error('Function values do not match.');
    end
end
fprintf(1, 'Indicator Function: OK\n');
%% << Indicator Function <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

%% >> Compressed Word Count >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
for i = 1:nrChecks,
    bow = rand(1000,100);
    bowJava = MatrixJ(bow);
    fctMatlab = @(S) sum(sqrt(sum(bow(:,S),2))) / sum(sqrt(sum(bow,2)));
    fctJava = CompressedWordCountJ(bowJava);
    
    S = randperm(100);
    S = S(1:S(1));
    
    valMatlab = fctMatlab(S);
    valJava = fctJava.eval(S);
    
    if abs(valMatlab - valJava) > tol,
        valMatlab
        valJava
        error('Function values do not match.');
    end
end
fprintf(1, 'Compressed Word Count: OK\n');
%% << Compressed Word Count <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<


%% >> Greedy >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
for i = 1:nrChecks,
    sim = rand(100);
    fctMatlab = @(S) sum(sum(max(sim(:,S),[],2))) / 100;
    
    fctJavaLinComb = LinCombFunctionJ();
    fctJava = FacilityLocationJ(MatrixJ(sim));
    fctJavaLinComb.addFunction(fctJava);
    fctJavaLinComb.setWeights(1.0);
    greedy = GreedyJ(fctJavaLinComb);
    SmaxJava = greedy.maximize(100, 10);
    SmaxJava = java2matArray(SmaxJava);
    SmaxMatlab = greedySummary(fctMatlab, 10);
    
    if any(sort(SmaxJava(:)) ~= sort(SmaxMatlab(:))),
        error('Function values do not match.');
    end
end
%tic; for i = 1:1e3; greedy.maximize(100, 10); end; toc
%tic; for i = 1:1e3; greedy.maximizep(100, 10); end; toc
%tic; for i = 1:1e3; greedySummary(fctMatlab, 10); end; toc
fprintf(1, 'Greedy: OK\n');
%% << Greedy <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

%% >> Sum Coverage >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
for i = 1:nrChecks,
    sim = rand(100);
    simJava = MatrixJ(sim);
    fctMatlab = @(S) sum_coverage(S,sim);
    fctJava = SumCoverageJ(simJava);
    
    S = randperm(100);
    S = S(1:S(1));
    
    valMatlab = fctMatlab(S);
    valJava = fctJava.eval(S);
    
    if abs(valMatlab - valJava) > tol,
        valMatlab
        valJava
        error('Function values do not match.');
    end
end
fprintf(1, 'Sum Coverage: OK\n');
%% << Sum Coverage <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

%% >> Cut Coverage >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
for i = 1:nrChecks,
    sim = rand(100);
    simJava = MatrixJ(sim);
    fctMatlab = @(S) cut_coverage(S,sim) / 100;
    fctJava = CutCoverageJ(simJava);
    
    S = randperm(100);
    S = sort(S(1:S(1)));
    
    valMatlab = fctMatlab(S);
    valJava = fctJava.eval(S);
    
    if abs(valMatlab - valJava) > tol,
        valMatlab
        valJava
        error('Function values do not match.');
    end
end
fprintf(1, 'Cut Coverage: OK\n');
%% << Sum Coverage <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

%% >> Encoding Accuracy >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
for i = 1:nrChecks,
    encodingCosts = rand(1000,100);
    encodingCostsJava = MatrixJ(encodingCosts);
    fctMatlab = @(S) encodingAccuracy(S, encodingCosts);
    fctJava = EncodingAccuracyJ(encodingCostsJava);
    
    S = randperm(100);
    S = S(1:S(1));
    
    valMatlab = fctMatlab(S);
    valJava = fctJava.eval(S);
    
    if abs(valMatlab - valJava) > tol,
        valMatlab
        valJava
        error('Function values do not match.');
    end
end
fprintf(1, 'Encoding Accuracy: OK\n');
%% << Encoding Accuracy <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

%% >> Bias >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
for i = 1:nrChecks,
    fctMatlab = @(S) 1;
    fctJava = BiasJ();
    
    S = randperm(100);
    S = S(1:S(1));
    
    valMatlab = fctMatlab(S);
    valJava = fctJava.eval(S);
    
    if abs(valMatlab - valJava) > tol,
        valMatlab
        valJava
        error('Function values do not match.');
    end
end
fprintf(1, 'Bias: OK\n');
%% << Bias <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

%% >> Fidelity >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
for i = 1:nrChecks,
    bow = rand(1000,100);
    bowJava = MatrixJ(bow);
    threshold = 0.9;
    fctMatlab = @(S) fidelity(S, bow, threshold);;
    fctJava = FidelityJ(bowJava, threshold);
    
    S = randperm(100);
    S = S(1:S(1));
    
    valMatlab = fctMatlab(S);
    valJava = fctJava.eval(S);
    
    if abs(valMatlab - valJava) > tol,
        valMatlab
        valJava
        error('Function values do not match.');
    end
end

fprintf(1, 'Fidelity: OK\n');
%% << Fidelity <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

%% >> OneMinusRouge >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
for ds = 1:14,
    summaries = pruneSummaries(ds, get_pruning_info(ds, overlap));
    bowLoss = get_BOW_evaluation(ds);
    bowSummaryLoss = computeBOWSummary(summaries, bowLoss);
    
    optionsEvaluationLoss = struct();
    optionsEvaluationLoss.bow = bowLoss;
    optionsEvaluationLoss.summariesAsBow = bowSummaryLoss;

    fctMatlab = @(S) 1 - summaryEvaluation(S, summaries, 'rouge', optionsEvaluationLoss);
    fctJava = OneMinusRougeJ(MatrixJ(summaries), MatrixJ(bowLoss), MatrixJ(bowSummaryLoss'));
    
    S = randperm(100);
    S = S(1:S(1));
    
    valMatlab = fctMatlab(S);
    valJava = fctJava.eval(S);
    
    if abs(valMatlab - valJava) > tol,
        valMatlab
        valJava
        error('Function values do not match.');
    end
end
fprintf(1, 'OneMinusRouge: OK\n');
%% << OneMinusRouge <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<


%% >> Foo>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
clear java
nrChecks = 100;
tol = 1e-6;

for i = 1:nrChecks,
    encodingCosts = rand(1000,100);
    fctMatlab = @(S) encodingAccuracy(S, encodingCosts);
    fctJava = EncodingAccuracyJ(MatrixJ(encodingCosts));
    
    S = randperm(100);
    S = S(1:S(1));
    
    valMatlab = fctMatlab(S);
    valJava = fctJava.eval(S);
    
    if abs(valMatlab - valJava) > tol,
        valMatlab
        valJava
        error('Function values do not match.');
    end
end
fprintf(1, 'Encoding Accuracy: OK\n');
%% << Foo <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<


