function [] = experiment_losses()

clc

javaaddpath(fullfile(pwd, 'Java/SubmodularFunctions/bin'));
%javaaddpath(fullfile(pwd, 'Java/Cuda/jcuda.jar'));
%javaaddpath('/srv/stschia/tmp/JCuda/jcuda.jar');
addpath('external/sfo');
addpath('SubmodularComponents');
addpath('Java/MatlabInterface');
%setupVLFEAT

if vl_isoctave(),
    page_output_immediately(1);
    page_screen_output(0);
    pkg load java
end

rand('seed', 12345);

% matlabpool(6)

meanOverlap = 0;
datasetRange = 1:14;
for ds = datasetRange,
    fprintf(1, '>> DS %2d <<<<<<\n', ds);
    
    overlap = 0.05;
    
    summaries = pruneSummaries(ds, get_pruning_info(ds, overlap));
    nrSummaries = size(summaries, 1);
    nrRandomSummaries = nrSummaries;
    
    
    randomSummaries = randomSummary(100, 10, nrRandomSummaries);
    
    bowLoss = get_BOW_evaluation(ds);
    nrWordsT = size(bowLoss, 1);
    
    allWords = randperm(nrWordsT);
    allOffset = 0;
    allWeights = zeros(nrWordsT, 1);
    
    bowSummaryLoss = computeBOWSummary(summaries, bowLoss);
    optionsEvaluationLoss = struct();
    optionsEvaluationLoss.bow = bowLoss;
    optionsEvaluationLoss.summariesAsBow = computeBOWSummary(summaries, optionsEvaluationLoss.bow);
    [~,humanScores] = summaryEvaluation(summaries, summaries, 'rouge', optionsEvaluationLoss);
    [~,randomScores] = summaryEvaluation(randomSummaries, summaries, 'rouge', optionsEvaluationLoss);
    
    % overlap of one-minus-rouge
    randomLosses = 1-randomScores;
    humanLosses = 1-humanScores;
    overlapEmp = computeOverlap(humanLosses, randomLosses, 'loss');
    humanLossesOrig = 1-humanScores;
    allLosses = [humanLosses; randomLosses];
    allSummaries = [summaries; randomSummaries];
    
    while ~isempty(allWords),
    
    fprintf(1, 'Remaining words: %d\n', length(allWords));
    
    currentWords = allWords(1:min(30,length(allWords)));
    allWords(1:min(30,length(allWords))) = [];
    bowLoss = get_BOW_evaluation(ds);
    size(bowLoss)
    bowLoss = bowLoss(currentWords, :);
    size(bowLoss)
    nrWords = size(bowLoss, 1);

    sumAsBow = computeBOWSummary(allSummaries, bowLoss);
    
    fprintf(1, 'Overlap One-Minus-Rouge: %2.2f\n', overlapEmp);
    fprintf(1, 'Mean Human: %f, mean random: %f\n', mean(humanLosses), mean(randomLosses));
    
    weights = zeros(nrWords, 1);
    
    % overlap of surrogate rouge loss
    tic;
    % humanLosses = NaN(size(summaries,1), 1);

    % humanLosses = rougeLossNew(summaries, bowLoss, optionsEvaluationLoss.summariesAsBow, ds, exp(weights));
    
    options = optimset('display', 'iter');
    %options = optimset(options, 'GradObj', 'on');
    options = optimset(options, 'Algorithm', 'interior-point');
    options = optimset(options, 'Hessian', {'lbfgs', 3});
    options = optimset(options, 'MaxFunEvals', 10000);
    %options = optimset(options, 'MaxIter', 200);
    %options = optimset(options, 'DerivativeCheck', 'on');
    
    tweights = [-5; 0*ones(size(weights))];
    
    sl = SurrogateLossJ(MatrixJ(bowLoss), MatrixJ(optionsEvaluationLoss.summariesAsBow));
    nr = 10;
    slA = cell(nr,1);
    for i = 1:nr,
        slA(i) = SurrogateLossJ(MatrixJ(bowLoss), MatrixJ(optionsEvaluationLoss.summariesAsBow));;
    end

    initTh = -0*ones(size(weights));
    weightsNew = fmincon(@(w) obj3(sl, w, allLosses, sumAsBow, nrWords,slA),[-2; initTh],[],[],[],[],-10*ones(1*length(weights)+1,1),inf*ones(1*length(weights)+1,1),[],options);

    
    
    %humanLosses = exp(weightsNew(1)) + rougeLossNew(summaries, bowLoss, optionsEvaluationLoss.summariesAsBow, ds, ones(nrWords, 1), exp(weightsNew(2:end-nrWords)));
    sl.setOffset(exp(weightsNew(1)));
    sl.setThresholds(exp(weightsNew(2:end)));
    humanLosses = sl.evalSumBow(computeBOWSummary(summaries, bowLoss));
    
    %randomLosses = exp(weightsNew(1)) + rougeLossNew(randomSummaries, bowLoss, optionsEvaluationLoss.summariesAsBow, ds, ones(nrWords, 1), exp(weightsNew(2:end-nrWords)));
    randomLosses = sl.evalSumBow(computeBOWSummary(randomSummaries, bowLoss));
    
    ttt = toc;
    %%
    fprintf(1, 'Computing losses took %.2f seconds.\n', ttt);
    overlapEmp = computeOverlap(humanLosses, randomLosses, 'loss');
    fprintf(1, 'Overlap Surrogate Rouge Loss: %2.2f\n', overlapEmp);
    fprintf(1, 'Mean Human: %f, mean random: %f\n', mean(humanLosses), mean(randomLosses));
    
    % store weighs
    allWeights(currentWords) = weightsNew(2:end);
    allOffset = 0.5*(allOffset + weightsNew(1));
    
    norm(allWeights)
    end
    
    
    % final training
    bowLoss = get_BOW_evaluation(ds);
    sumAsBow = computeBOWSummary(allSummaries, bowLoss);
        bowSummaryLoss = computeBOWSummary(summaries, bowLoss);
    optionsEvaluationLoss = struct();
    optionsEvaluationLoss.bow = bowLoss;
    optionsEvaluationLoss.summariesAsBow = computeBOWSummary(summaries, optionsEvaluationLoss.bow);
    [~,humanScores] = summaryEvaluation(summaries, summaries, 'rouge', optionsEvaluationLoss);
    [~,randomScores] = summaryEvaluation(randomSummaries, summaries, 'rouge', optionsEvaluationLoss);
        sl = SurrogateLossJ(MatrixJ(bowLoss), MatrixJ(optionsEvaluationLoss.summariesAsBow));
    nrWords = size(bowLoss,1);
    weightsNew = fmincon(@(w) obj3(sl, w, allLosses, sumAsBow, nrWords,slA),[allOffset; allWeights],[],[],[],[],-10*ones(length(allWeights)+1,1),inf*ones(1*length(allWeights)+1,1),[],options);
  
     randomLosses = sl.evalSumBow(computeBOWSummary(randomSummaries, bowLoss));
    
    ttt = toc;
    %%
    fprintf(1, 'Computing losses took %.2f seconds.\n', ttt);
    overlapEmpN = computeOverlap(humanLosses, randomLosses, 'loss');
    fprintf(1, 'Overlap Surrogate Rouge Loss: %2.2f\n', overlapEmpN);
    fprintf(1, 'Mean Human: %f, mean random: %f\n', mean(humanLosses), mean(randomLosses));
           
    
    meanOverlap = meanOverlap + overlapEmp;
    
    fprintf(1, '\n');
    
    save(sprintf('weights_%d.mat', ds), 'allOffset', 'allWeights');
    
    %% plot
    clf
    bins = linspace(min(randomLosses)*0.5, max(randomLosses)*1.5, 100);
    bar(hist(randomLosses, bins), 'r');
    hold on
    bar(hist(humanLosses, bins), 'g');
    
    % keyboard
end

fprintf(1, 'Mean overlap over all datasets: %f\n', meanOverlap / length(datasetRange));    

end

function [finalVal, grad] = obj3(sl, w, allLosses, summariesAsBow, nrWords, slA)
    sl.setOffset(exp(w(1)));
    sl.setThresholds(exp(w(2:(1+nrWords))));
    %t = exp(w((2+nrWords):end));
    %sl.setWeights(t);
    
    val = sl.evalSumBow(summariesAsBow);
    
    finalVal = norm(allLosses - val, 1);
    
    if nargout == 2,
        % compute gradient
        disp('Computing gradient.');
%         grad = zeros(length(w),1);
%         sgn = -sign(allLosses - val);
%         
%         for s = 1:size(summariesAsBow, 1),
%             t = java2matArray(sl.computeGradient(summariesAsBow(s,:)));
%             grad = grad(:) + sgn(s)*t(:);
%         end
%         
%         grad = grad .* exp(w)

        num = length(w);
        delta = 1e-6;
        grad = zeros(num, 1);
        idx = 1:num;
        idx = 1 + mod(idx, length(slA));
        val2 = sl.evalSumBow(summariesAsBow);
        val2 = norm(allLosses - val2, 1);
        for i = 1:num,
            w1 = w;
            w2 = w;
            w1(i) = w1(i) + delta;
            %w2(i) = w2(i) - delta;
            
            tsl = slA{idx(i)};
            tsl.setOffset(exp(w1(1)));
            tsl.setThresholds(exp(w1(2:end)));
            val1 = tsl.evalSumBow(summariesAsBow);
            val1 = norm(allLosses - val1, 1);
            
            %tsl.setOffset(exp(w2(1)));
            %tsl.setThresholds(exp(w2(2:end)));
            %val2 = tsl.evalSumBow(summariesAsBow);
            %val2 = norm(allLosses - val2, 1);
            
            grad(i) = (val1 - val2) / (delta);
        end
    end
end

function [finalVal, grad] = obj2(sl, w, allLosses, allSummaries, nrWords)
    sl.setOffset(exp(w(1)));
    sl.setThresholds(exp(w(2:(1+nrWords))));
    t = exp(w((2+nrWords):end));
    sl.setWeights(t);
    
    val = zeros(size(allSummaries,1),1);
    for s = 1:size(allSummaries, 1),
        val(s) = sl.eval(allSummaries(s,:));
    end
    
    finalVal = norm(allLosses - val, 1);
    
    if nargout == 2,
        % compute gradient
        grad = zeros(length(w),1);
        sgn = -sign(allLosses - val);
        
        for s = 1:size(allSummaries, 1),
            t = java2matArray(sl.computeGradient(allSummaries(s,:)));
            grad = grad(:) + sgn(s)*t(:);
        end
        
        grad = grad .* exp(w)
    end
end

function [valFinal, grad] = obj(w, allLosses, sumAsBow, bow, summariesAsBow)
    nrSamples = length(allLosses);
    nrSummaries = size(summariesAsBow, 1);
    nrWords = size(summariesAsBow, 2);

    offset = exp(w(1));
    expw = exp(w(1:(1+nrWords)));
    weights = exp(w((2+nrWords):end));
    %weights = ones(nrWords,1);
    reg = 0;

    % compute thresholds; size: nrSummaries x nrWords
    normalization = nrSummaries*sum(expw(2:end) .* weights);
    thresholds = repmat(expw(2:end)', nrSummaries, 1) - summariesAsBow;
    thresholds(thresholds < 0) = 0;
    %weightsMatrix = repmat(weights(:)', nrSummaries, 1);
    % normalization = sum(sum(thresholds));

    val = zeros(nrSamples, 1);
    nom = zeros(nrSamples, 1);
    denom = zeros(nrSamples, 1);

    for i = 1:nrSamples,
        asBow = sumAsBow(i,:);
        % nom(i) = sum(weights(:)' .* sum(min(thresholds, repmat(asBow, nrSummaries, 1)),1));
        nom(i) = sum(weights(:)' .* sum(min(thresholds, repmat(asBow, nrSummaries, 1)),1));
        denom(i) = normalization;
        val(i) = offset + nom(i) / denom(i);
    end

    partVal = norm(allLosses - val,1);
    % valFinal = partVal^2 + 0.5+reg*norm(w)^2;
    valFinal = partVal + 0.5*reg*norm(w(2:end))^2;

    %% gradient
    if nargout == 2,
        grad = zeros(size(w));
        for i = 1:nrSamples,
            sgn = sign(allLosses(i) - val(i));
            grad(1) = grad(1) + sgn*(-offset);
        end

        for i = 1:nrSamples,
            asBow = sumAsBow(i,:); % sum(bow(:,S(i,:)), 2)';
            sgn = -sign(allLosses(i) - val(i));

            for j = 1:nrWords,

                grd = -nrSummaries * nom(i) * weights(j) * expw(1+j) / (denom(i)^2);
                grad(1+j) = grad(1+j) + sgn*(grd);

                for k = 1:nrSummaries,
                    if asBow(j) > thresholds(k,j),
                        grd = weights(j)*expw(1+j) / denom(i);
                    else
                        grd = 0;
                    end
                    %grd
                    grad(1+j) = grad(1+j) + sgn*(grd);
                end
            end
        end
        
        % derivative with respect to weights
        idxOffset = 1+nrWords;
        for i = 1:nrSamples,
            asBow = sumAsBow(i,:); % sum(bow(:,S(i,:)), 2)';
            sgn = -sign(allLosses(i) - val(i));

            for j = 1:nrWords,

                grd = -nrSummaries * nom(i) * weights(j) * expw(1+j) / (denom(i)^2);
                grad(idxOffset+j) = grad(idxOffset+j) + sgn*(grd);

                for k = 1:nrSummaries,
                    if asBow(j) > thresholds(k,j),
                        tval = thresholds(k,j);
                    else
                        tval = asBow(j);
                    end
                    grd = weights(j)*tval/ denom(i);
                    %grd
                    grad(idxOffset+j) = grad(idxOffset+j) + sgn*(grd);
                end
            end
        end
        grad(2:end) = grad(2:end) + reg*w(2:end);
    end
    
    
    %% gradient
%     allsign = sign(allLosses - val);
%     if nargout == 2,
%         grad = zeros(size(w));
%         grad(1) = grad(1) - sum(allsign.*offset);
% 
%         for i = 1:nrSamples,
%             asBow = sumAsBow(i,:);
%             sgn = -allsign(i);
% 
%             mask = repmat(asBow(:)', nrSummaries, 1) >= thresholds;
%             grad(2:end) = grad(2:end) -  sgn.*nrSummaries * nom(i) * expw(2:end) / (denom(i)^2);
%             grad(2:end) = grad(2:end) + sum(mask,1)' .* expw(2:end) / denom(i);
%         end
% 
%         %grad = grad * partVal;
%         grad = grad + reg*w;
%     end
end
