clear all;
clc

javaaddpath(fullfile(pwd, 'Java/SubmodularFunctions/bin'));
%javaaddpath(fullfile(pwd, 'Java/Cuda/jcuda.jar'));
%javaaddpath('/srv/stschia/tmp/JCuda/jcuda.jar');
addpath('external/sfo');
addpath('SubmodularComponents');
addpath('Java/MatlabInterface');
%setupVLFEAT

if vl_isoctave(),
    page_output_immediately(1);
    page_screen_output(0);
    pkg load java
end

rand('seed', 12345);

% matlabpool(6)

meanOverlap = 0;
datasetRange = 1:14;
for ds = datasetRange,
    fprintf(1, '>> DS %2d <<<<<<\n', ds);
    
    overlap = 0.05;
    
    summaries = pruneSummaries(ds, get_pruning_info(ds, overlap));
    nrSummaries = size(summaries, 1);
    nrRandomSummaries = nrSummaries;
    
    
    randomSummaries = randomSummary(100, 10, nrRandomSummaries);
    
    bowLoss = load_resource('bow', 'overfeat/slidingInputWindowSparseFiltering', ds);    
    bowSummaryLoss = computeBOWSummary(summaries, bowLoss);
    optionsEvaluationLoss = struct();
    optionsEvaluationLoss.bow = bowLoss;
    optionsEvaluationLoss.summariesAsBow = computeBOWSummary(summaries, optionsEvaluationLoss.bow);
    [~,humanScores] = summaryEvaluation(summaries, summaries, 'rouge', optionsEvaluationLoss);
    [~,randomScores] = summaryEvaluation(randomSummaries, summaries, 'rouge', optionsEvaluationLoss);
    
    % overlap of one-minus-rouge
    overlapEmp = computeOverlapModified(humanScores, randomScores);
    humanLossesOrig = 1-humanScores;

    fprintf(1, 'Overfeat new: %f\n', overlapEmp);
    
    %%%%%%%%%%%%%%
    
    bowLoss = load_resource('bow', 'overfeat/slidingInputWindow', ds);    
    bowSummaryLoss = computeBOWSummary(summaries, bowLoss);
    optionsEvaluationLoss = struct();
    optionsEvaluationLoss.bow = bowLoss;
    optionsEvaluationLoss.summariesAsBow = computeBOWSummary(summaries, optionsEvaluationLoss.bow);
    [~,humanScores] = summaryEvaluation(summaries, summaries, 'rouge', optionsEvaluationLoss);
    [~,randomScores] = summaryEvaluation(randomSummaries, summaries, 'rouge', optionsEvaluationLoss);
    
    % overlap of one-minus-rouge
    overlapEmp = computeOverlapModified(humanScores, randomScores);

    fprintf(1, 'Overfeat old: %f\n', overlapEmp);
    
    %%%%%%%%%%%%%%
    
    bowLoss = load_resource('bow', 'superPixels', ds);    
    bowSummaryLoss = computeBOWSummary(summaries, bowLoss);
    optionsEvaluationLoss = struct();
    optionsEvaluationLoss.bow = bowLoss;
    optionsEvaluationLoss.summariesAsBow = computeBOWSummary(summaries, optionsEvaluationLoss.bow);
    [~,humanScores] = summaryEvaluation(summaries, summaries, 'rouge', optionsEvaluationLoss);
    [~,randomScores] = summaryEvaluation(randomSummaries, summaries, 'rouge', optionsEvaluationLoss);
    
    % overlap of one-minus-rouge
    overlapEmp = computeOverlapModified(humanScores, randomScores);
    humanLossesOrig = 1-humanScores;

    fprintf(1, 'SuperPixels old: %f\n', overlapEmp);
    
    %%%%%%%%%%%%%%
    
    bowLoss = eye(100);   
    bowSummaryLoss = computeBOWSummary(summaries, bowLoss);
    optionsEvaluationLoss = struct();
    optionsEvaluationLoss.bow = bowLoss;
    optionsEvaluationLoss.summariesAsBow = computeBOWSummary(summaries, optionsEvaluationLoss.bow);
    [~,humanScores] = summaryEvaluation(summaries, summaries, 'rouge', optionsEvaluationLoss);
    [~,randomScores] = summaryEvaluation(randomSummaries, summaries, 'rouge', optionsEvaluationLoss);
    
    % overlap of one-minus-rouge
    overlapEmp = computeOverlapModified(humanScores, randomScores);
    humanLossesOrig = 1-humanScores;

    fprintf(1, 'Diagonal BoW: %f\n', overlapEmp);
  
    
    
    fprintf(1, '\n');
end

fprintf(1, 'Mean overlap over all datasets: %f\n', meanOverlap / length(datasetRange));    



