%% compares inference by modmod and by greedy

clear all;
clear fun;
clc;

addpath('external/sfo');
javaaddpath('Java/SubmodularFunctions/bin');

rand('seed', 12345);

ds = 14;
overlap = 0.05;
nrRandomSummaries = 1e3;

randomSummaries = randomSummary(100, 10, nrRandomSummaries);

summaries = pruneSummaries(ds, get_pruning_info(ds, overlap));
bowLoss = get_BOW_evaluation(ds);
bowSummaryLoss = computeBOWSummary(summaries, bowLoss);
optionsEvaluationLoss = struct();
optionsEvaluationLoss.bow = bowLoss;
optionsEvaluationLoss.summariesAsBow = computeBOWSummary(summaries, optionsEvaluationLoss.bow);
[~,humanScores] = summaryEvaluation(summaries, summaries, 'rouge', optionsEvaluationLoss);
[~,randomScores] = summaryEvaluation(randomSummaries, summaries, 'rouge', optionsEvaluationLoss);

%% setup function facility location + 1-rouge
sim = BOWsimilarity(bowLoss);
tfl = FacilityLocation(sim);
FL = @(S) tfl.eval(S);
% FL = @(S) sum(sum(max(sim(:,S),[],2))) / 100;
tloss = OneMinusRouge(summaries, bowLoss, bowSummaryLoss');
loss = @(S) tloss.eval(S);
% loss = @(S) 1 - summaryEvaluation(S, summaries, 'rouge', optionsEvaluationLoss);
fct = @(S) FL(S) + loss(S);
sfofct = sfo_fn_wrapper(fct);
%% maximize using greedy
maxGreedy = greedySummary(fct, 10);

%% maximmize using modmod
%maxModMod = sfo_modmodimproved(sfo_fn_wrapper(FL), sfo_fn_wrapper(@(S) -loss(S)), 1:100, 10, 2);
%maxModMod1 = sfo_modmodimproved(sfo_fn_wrapper(FL), sfo_fn_wrapper(@(S) -loss(S)), 1:100, 10, 1);
%maxModModv1 = sfo_modmodimprovedv1(sfo_fn_wrapper(FL), sfo_fn_wrapper(@(S) -loss(S)), 1:100, 10, 2);
%maxModModv11 = sfo_modmodimproved(sfo_fn_wrapper(FL), sfo_fn_wrapper(@(S) -loss(S)), 1:100, 10, 1);
% maxModModv2 = sfo_modmodimprovedv2(sfo_fn_wrapper(FL), sfo_fn_wrapper(@(S) -loss(S)), 1:100, 10);
maxModMod = sfo_subsupimproveduc(sfo_fn_wrapper(FL), sfo_fn_wrapper(@(S) -loss(S)), 1:100);
maxModMod2 = sfo_modmodimproveduc(sfo_fn_wrapper(FL), sfo_fn_wrapper(@(S) -loss(S)), 1:100);
fct(maxGreedy)
fct(maxModMod)
fct(maxModMod2)



%% maximmize using modmod with initialization
%maxModModInit = sfo_modmodimproved(sfo_fn_wrapper(FL), sfo_fn_wrapper(@(S) -loss(S)), 1:100, 10, 2, maxGreedy);
%maxModModInit1 = sfo_modmodimproved(sfo_fn_wrapper(FL), sfo_fn_wrapper(@(S) -loss(S)), 1:100, 10, 1, maxGreedy);
%maxModModInitv1 = sfo_modmodimprovedv1(sfo_fn_wrapper(FL), sfo_fn_wrapper(@(S) -loss(S)), 1:100, 10, 2, maxGreedy);
%maxModModInitv11 = sfo_modmodimprovedv1(sfo_fn_wrapper(FL), sfo_fn_wrapper(@(S) -loss(S)), 1:100, 10, 1, maxGreedy);
% maxModModInitv2 = sfo_modmodimprovedv2(sfo_fn_wrapper(FL), sfo_fn_wrapper(@(S) -loss(S)), 1:100, 10, maxGreedy);


%% score
fprintf(1, 'Max of greedy: %f\n', fct(maxGreedy));
fprintf(1, 'Max of modmod: %f\n', fct(maxModMod));
fprintf(1, 'Max of modmod with mode=1: %f\n', fct(maxModMod1));
fprintf(1, 'Max of modmodv1: %f\n', fct(maxModModv1));
fprintf(1, 'Max of modmodv1 with mode=1: %f\n', fct(maxModModv1));
fprintf(1, 'Max of modmodv2: %f\n', fct(maxModModv2));

fprintf(1, 'Max of modmod with initialization: %f\n', fct(maxModModInit));
fprintf(1, 'Max of modmod with initialization and mode=1: %f\n', fct(maxModModInit1));
fprintf(1, 'Max of modmodv1 with initialization: %f\n', fct(maxModModInitv1));
fprintf(1, 'Max of modmodv1 with initialization and mode=1: %f\n', fct(maxModModInitv11));
fprintf(1, 'Max of modmodv2 with initialization: %f\n', fct(maxModModInitv2));

%% Unconstrained case
addpath Codes_on_Web_UAI2012/
maxModMod = sfo_modmodimproveduc(sfo_fn_wrapper(FL), sfo_fn_wrapper(@(S) -loss(S)), 1:100);
length(maxModMod)
sfofct = sfo_fn_wrapper(fct);
greedsol = sfo_greedy_lazy(sfofct, V, inf);
fct(greedsol)

%% Beating the greedy with constraints - this is a simple hack which beats greedy atleast in this example
addpath Codes_on_Web_UAI2012/
maxModMod = sfo_modmodimproveduc(sfo_fn_wrapper(FL), sfo_fn_wrapper(@(S) -loss(S) - 0.002*length(S)), 1:100);
% This one is a tuned \lambda for this function (in general, this one would
% need to be found using binary search. It gives a solution of size 6.
% To get a size 10 solution, just initialize the constrained mod-mod to this solution
maxModModInit = sfo_modmodimproved(sfo_fn_wrapper(FL), sfo_fn_wrapper(@(S) -loss(S)), 1:100, 10, 2, maxModMod);
% This beats greedy atleast in this example!
