%% Computes the fraction of saturated words at different pruning levels
clear all
clc

thresholds = 0.5:-0.05:0.05;

saturationFrac = zeros(14, length(thresholds));
for ds = 1:14,
for thIdx = 1:length(thresholds),
    th = thresholds(thIdx);
    
    % setup loss
    summaries = pruneSummaries(ds, get_pruning_info(ds, th));

    % setup evaluation
    bowLoss = get_BOW_evaluation(ds);
    summariesAsBOW = computeBOWSummary(summaries, bowLoss);
    
    % compute fraction of words that saturate
    saturationCount = 0;
    testSummaries = summaries;
    %testSummaries = randomSummary(100, 10, 1000);
    testSummariesAsBOW = computeBOWSummary(testSummaries, bowLoss);
    for i = 1:size(testSummaries,1),
        tbow = testSummariesAsBOW(i,:);
        map = min(repmat(tbow(:)', size(summariesAsBOW,1), 1), summariesAsBOW) ./ summariesAsBOW;
        saturationCount = saturationCount + sum(sum(repmat(tbow(:)', size(summariesAsBOW,1), 1) > summariesAsBOW + 1e-5));
    end
    saturationCount = saturationCount / (numel(summariesAsBOW) * size(testSummariesAsBOW, 1));
    saturationFrac(ds,thIdx) =saturationCount; 
end
end

%% print table
if true,
fid = 1;
fprintf(fid, '      ');
for i = 1:length(thresholds);
  fprintf(fid, 'th=%1.2f ', thresholds(i));
end
fprintf(fid, '\n');

for ds = 1:14,
    fprintf(fid, 'DS%2d  ', ds);
    for i = 1:length(thresholds),
        fprintf(fid, '%1.4f  ', saturationFrac(ds, i));
    end
    fprintf(fid, '\n');
end

end
