clear all;
clc

javaaddpath(fullfile(pwd, 'Java/SubmodularFunctions/bin'));
%javaaddpath(fullfile(pwd, 'Java/Cuda/jcuda.jar'));
%javaaddpath('/srv/stschia/tmp/JCuda/jcuda.jar');
addpath('external/sfo');
addpath('SubmodularComponents');
addpath('Java/MatlabInterface');
%setupVLFEAT

if vl_isoctave(),
    page_output_immediately(1);
    page_screen_output(0);
    pkg load java
end

rand('seed', 12345);

% matlabpool(6)

meanOverlap = 0;
datasetRange = 1:14;
for ds = datasetRange,
    fprintf(1, '>> DS %2d <<<<<<\n', ds);
    
    overlap = 0.1;
    
    % load/generate summaries
    summaries = pruneSummaries(ds, get_pruning_info(ds, overlap));
    nrSummaries = size(summaries, 1);
    nrRandomSummaries = nrSummaries;
    randomSummaries = randomSummary(100, 10, nrRandomSummaries);
    
    % bowLoss = load_resource('bow', 'evaluation', ds);    
    bowLoss = get_BOW_evaluation(ds);    
    bowSummaryLoss = computeBOWSummary(summaries, bowLoss);
    optionsEvaluationLoss = struct();
    optionsEvaluationLoss.bow = bowLoss;
    optionsEvaluationLoss.summariesAsBow = computeBOWSummary(summaries, optionsEvaluationLoss.bow);
    [~,humanScores] = summaryEvaluation(summaries, summaries, 'rouge', optionsEvaluationLoss);
    [~,randomScores] = summaryEvaluation(randomSummaries, summaries, 'rouge', optionsEvaluationLoss);
    
    humanLossesNew = surrogateLoss( summaries, bowLoss, bowSummaryLoss );
    randomLossesNew = surrogateLoss( randomSummaries, bowLoss, bowSummaryLoss );
    
    % complment losses
    complementHumanSummaries = zeros(size(summaries, 1), 90);
    complementRandomSummaries = zeros(size(randomSummaries, 1), 90);
    for s = 1:size(summaries,1),
        complementHumanSummaries(s,:) = setdiff(1:100, summaries(s,:));
    end
    for s = 1:size(summaries,1),
        complementRandomSummaries(s,:) = setdiff(1:100, randomSummaries(s,:));
    end
    [~,humanLosses] = summaryEvaluation(complementHumanSummaries, summaries, 'rouge', optionsEvaluationLoss);
    [~,randomLosses] = summaryEvaluation(complementRandomSummaries, summaries, 'rouge', optionsEvaluationLoss);
    
    
    % overlap of one-minus-rouge
    overlapEmp = computeOverlap(humanScores, randomScores);
    overlapEmpMod = computeOverlapModified(humanScores, randomScores);
    overlapEmpLoss = computeOverlap(humanLosses, randomLosses, 'loss');
    overlapEmpModLoss = computeOverlapModified(humanLosses, randomLosses, 'loss');
    humanLossesOrig = 1-humanScores;

    fprintf(1, 'SCORE Overlap (hard): %f\n', overlapEmp);
    fprintf(1, 'SCORE Overlap (soft): %f\n', overlapEmpMod);
    fprintf(1, ' LOSS Overlap (hard): %f\n', overlapEmpLoss);
    fprintf(1, ' LOSS Overlap (soft): %f\n', overlapEmpModLoss);
    
    %% compute correlation
    data1 = [1-humanScores(:), humanLosses];
    data2 = [1-randomScores(:), randomLosses];
    data = [data1; data2];
    rComplement = corr(data);
    rComplement = rComplement(1,2);
    fprintf(1, 'Correlation coefficient (complement rouge): %f\n', rComplement);
    
    %% compute correlation
    data1 = [1-humanScores(:), humanLossesNew];
    data2 = [1-randomScores(:), randomLossesNew];
    data = [data1; data2];
    rNew = corr(data);
    rNew = rNew(1,2);
    fprintf(1, 'Correlation coefficient (new rouge): %f\n', rNew);
  
    %% plot
    clf;
    subplot(3,1,1);
    minx = min(min(1-humanScores), min(1-randomScores));
    maxx = max(max(1-humanScores), max(1-randomScores));
    rg = linspace(minx, maxx, 50);
    [a,b] = hist((1-humanScores), rg);
    bar(b,a, 'g');
    hold on;
    [a,b] = hist((1-randomScores), rg);
    bar(b,a, 'r');
    xlim([minx, maxx]);
    legend('human', 'random');
    title('one minus rouge');
    
    subplot(3,1,2);
    minx = min(min(humanLosses), min(randomLosses));
    maxx = max(max(humanLosses), max(randomLosses));
    rg = linspace(minx, maxx, 50);
    [a,b] = hist(humanLosses, rg);
    bar(b,a, 'g');
    hold on;
    [a,b] = hist(randomLosses, rg);
    bar(b,a, 'r');
    xlim([minx, maxx]);
    legend('human', 'random');
    title(sprintf('complement vrouge -- corr. coeff. with 1-R: %f', rComplement));
    
    subplot(3,1,3);
    minx = min(min(humanLossesNew), min(randomLossesNew));
    maxx = max(max(humanLossesNew), max(randomLossesNew));
    rg = linspace(minx, maxx, 50);
    [a,b] = hist(humanLossesNew, rg);
    bar(b,a, 'g');
    hold on;
    [a,b] = hist(randomLossesNew, rg);
    bar(b,a, 'r');
    xlim([minx, maxx]);
    legend('human', 'random');
    title(sprintf('surrogate loss (eq. 19) -- corr. coeff with 1-R: %f', rNew));
    
    % add overall title
    suptitle(sprintf('Dataset %d -- Loss comparisons', ds));
    
    print(sprintf('../results/OneMinusRougeVSSurrogate_ds_%d.pdf', ds), '-dpdf', '-r300');
    
    fprintf(1, '\n');
end

fprintf(1, 'Mean overlap over all datasets: %f\n', meanOverlap / length(datasetRange));    



