%% evaluate execution time of greedy algorithm

clear all;
clear java;
clc;

rand('seed', 12345);

ds = 12;
overlap = 0.05;
nrRandomSummaries = 1e3;

randomSummaries = randomSummary(100, 10, nrRandomSummaries);

summaries = pruneSummaries(ds, get_pruning_info(ds, overlap));
bowLoss = get_BOW_evaluation(ds);
bowSummaryLoss = computeBOWSummary(summaries, bowLoss);
optionsEvaluationLoss = struct();
optionsEvaluationLoss.bow = bowLoss;
optionsEvaluationLoss.summariesAsBow = computeBOWSummary(summaries, optionsEvaluationLoss.bow);

%%
clear java;
javaaddpath('Java/SubmodularFunctions/bin');

sim1 = randn(100);
sim2 = randn(100);
FL1 = FacilityLocation(sim1);
FL2 = FacilityLocation(sim2);
jfct = LinCombFunction();
jfct.addFunction(FL1);
jfct.addFunction(FL2);
jfct.setWeights([0.1,1]);
S = int32([1,2,3]);
eval(sprintf('fct = @(S) 0.1*sum(sum(max(sim1(:,S),[],2))) / 100 + sum(sum(max(sim2(:,S),[],2))) / 100;'));

%% setup fcts
nrComponents = 100;
scoreFunctionJava = LinCombFunction();
fcts = cell(nrComponents, 1);
sims = cell(1,nrComponents);
for i = 1:nrComponents,
    sim = rand(100);
    eval(sprintf('fct = @(S) sum(sum(max(sim(:,S),[],2))) / 100;'));
    FL = FacilityLocation(sim);
    scoreFunctionJava.addFunction(FL)
    fcts{i} = fct;
end

w = rand(size(fcts));
scores = zeros(nrComponents);
eval(sprintf('scoreFct = %s;', constructScoreFunction(w, 'fcts')));
scoreFunctionJava.setWeights(w);

%%
nrTries = 5;
wAll = rand(nrTries, length(fcts));
resMatlab = zeros(nrTries, 10);
resJava = zeros(nrTries, 10);

tic;
for i = 1:nrTries,
    w = wAll(i,:);
    eval(sprintf('scoreFct = %s;', constructScoreFunction(w, 'fcts')));
    resMatlab(i,:) = greedySummary(@(S) scoreFct(S), 10);
end
t = toc;
fprintf(1, 'MATLAB: Computation took %.2f seconds.\n', t);

tic;
for i = 1:nrTries,
    w = wAll(i,:);
    scoreFunctionJava.setWeights(w);
    resJava(i,:) = greedySummary(@(S) scoreFunctionJava.eval(S), 10);
end
t = toc;
fprintf(1, 'JAVA: Computation took %.2f seconds.\n', t);

tic;
for i = 1:nrTries,
    w = wAll(i,:);
    scoreFunctionJava.setWeights(w);
    resJava(i,:) = greedySummary(@(S) scoreFunctionJava.evalp(S), 10);
end
t = toc;
fprintf(1, 'JAVA Parallel: Computation took %.2f seconds.\n', t);

if all(all(sort(resMatlab,2) == sort(resJava,2))),
    fprintf(1, 'All computations are ok.\n');
else
    error('Computations not consistent.');
end

% without parallel computation: Computation took 0.82 minutes.
% with multiple workers: Computation took 0.58 minutes.