/* 
 * File:   Helper.h
 * Author: stschia
 *
 * Created on May 21, 2014, 11:32 AM
 */

#ifndef HELPER_H
#define	HELPER_H

#include <algorithm>
#include "ComponentDescriptor.h"

#include <iostream>
#include <fstream>
#include <sstream>
#include <vector>
#include <sys/stat.h>
#include <sys/types.h>
#include <set>
#include <unordered_set>
#include <algorithm>

#include "stdio.h"
#include "stdlib.h"
#include "string.h"
#include "math.h"
#include "time.h"
#include <stdarg.h>
#include <assert.h>
#include <queue>
#include <stdexcept>
using namespace std;

// Utilities
#include "Utils/error.h"
#include "Utils/sort.h"
#include "Utils/utils.h"
#include "Utils/fileHandling.h"
#include "Utils/totalOrder.h"

//Submodular Functions
#include "Functions/setFunctions.h"
#include "Functions/submodularFunctions.h"
#include "Functions/modularFunctions.h"
#include "Functions/featureBasedFunctions.h"
#include "Functions/facilityLocation.h"
#include "Functions/graphCutFunctions.h"
#include "Functions/saturatedCoverage.h"
#include "Functions/complexityFunctions.h"
#include "Functions/confusabilityFunctions.h"
#include "Functions/combineSubmodularFunctions.h"
#include "Functions/scaleSubmodularFunctions.h"
#include "Functions/combineSubmodularModularFunctions.h"
#include "Functions/differenceSubmodularFunctions.h"
#include "Functions/complementSubmodularFunctions.h"
#include "Functions/truncateSubmodularFunctions.h"
#include "Functions/clusteredSubmodularFunctions.h"

#include <stdio.h>

enum projections { SIMPLEX, POSITIVE, NONE };
enum optimizers { SGD, GRADIENTSCALING };
enum inferenceT { GREEDY, SUBSUP, SUPSUB, RANDOMIZEDGREEDY };
enum lossType { SURROGATE, ONEMINUSROUGE, COMPLEMENTROUGE };
enum scenarioType { CHEATING, REAL };

struct configT {
	int testDataset;
	scenarioType scenario = CHEATING;
	std::string componentsConfigurationFile;
	projections projectionType = SIMPLEX;
	optimizers optimizer = SGD;
	inferenceT inference = SUPSUB;
	lossType loss = ONEMINUSROUGE;
	double etaInitial = 1e-1;
	double l2reg = 1e-2;
	int nrIter = 1000;
	int logEvery = 1;
	bool initializeWithGreedy = true;
	std::string resultFilename;
};

class Helper {
private:
	static const std::string preDir;
   
public:
    static std::list<ComponentDescriptor> readComponentsFile(std::string filename);
    static void dumpComponentDescriptorList(std::list<ComponentDescriptor> componentDescriptorList);
    static char* getFilenameForResource(std::string resource, int ds);
    static std::vector<std::vector<unordered_set<int> > > loadSummaries();
    static std::vector<std::vector<double> > readMatrixBinary(std::string filename, int x);
    static std::vector<std::vector<double> > readMatrixBinary(std::string filename, int x, int y);
    static std::vector<std::vector<int> > readMatrixBinaryInteger(std::string filename, int x);
    static double getValueForResource(std::string resource);
    static void timeFunctions(std::list<ComponentDescriptor> componentDescriptorList, std::vector<submodularFunctions*> componentList);
    static void printSummary(std::unordered_set<int> &sset);
    static configT parseConfig(std::string filename);
    
    static facilityLocation* createFacilityLocationComponent(ComponentDescriptor descriptor, std::unordered_set<int> &groundSet, int ds);
    static modularFunctions* createSumCoverageComponent(ComponentDescriptor descriptor, std::unordered_set<int> &groundSet, int ds);
    static graphCutFunctions* createCutCoverageComponent(ComponentDescriptor descriptor, std::unordered_set<int> &groundSet, int ds);
    static submodularFunctions* createCosineSimilarityComponent(ComponentDescriptor descriptor, std::unordered_set<int> &groundSet, int ds);
    static submodularFunctions* createCompressedWordCountComponent(ComponentDescriptor descriptor, std::unordered_set<int> &groundSet, int ds);
    static submodularFunctions* createFidelityComponent(ComponentDescriptor descriptor, std::unordered_set<int> &groundSet, int ds);
    static submodularFunctions* createThresholdedFeatureCountsComponent(ComponentDescriptor descriptor, std::unordered_set<int> &groundSet, int ds);
    static submodularFunctions* createClusteredFacilityLocationComponent(ComponentDescriptor descriptor, std::unordered_set<int> &groundSet, int ds);
    static submodularFunctions* createClusteredDiversityComponent(ComponentDescriptor descriptor, std::unordered_set<int> &groundSet, int ds);
    static submodularFunctions* createHingeLossComponent(ComponentDescriptor descriptor, std::unordered_set<int> &groundSet, int ds);

    static submodularFunctions* createSurrogateLoss(std::unordered_set<int> &groundSet, int ds, std::vector<std::unordered_set<int>> summaries);
    static submodularFunctions* createRouge(std::unordered_set<int> &groundSet, int ds, std::vector<std::unordered_set<int>>);
    static submodularFunctions* createComplementRouge(std::unordered_set<int> &groundSet, int ds, std::vector<std::unordered_set<int>> summaries);
    static submodularFunctions* createOneMinusRouge(std::unordered_set<int> &groundSet, int ds, std::vector<std::unordered_set<int>> summaries);
 
    
// trim from start
static inline std::string &ltrim(std::string &s) {
        s.erase(s.begin(), std::find_if(s.begin(), s.end(), std::not1(std::ptr_fun<int, int>(std::isspace))));
        return s;
}

// trim from end
static inline std::string &rtrim(std::string &s) {
        s.erase(std::find_if(s.rbegin(), s.rend(), std::not1(std::ptr_fun<int, int>(std::isspace))).base(), s.end());
        return s;
}

// trim from both ends
static inline std::string &trim(std::string &s) {
        return ltrim(rtrim(s));
}
};

#endif	/* HELPER_H */

