#ifndef PROJECTIONONTOSIMPLEX_H
#define PROJECTIONONTOSIMPLEX_H

#include <algorithm>
#include <vector>

class projectionOntoSimplex : public projection {
public:
	projectionOntoSimplex(int n) : projection(n) {} ;
	void project(std::vector<double> &w);
};

inline void projectionOntoSimplex::project(std::vector<double> &w) {
    double z = 1.0;
	
    // n = (1:length(v))';
    // mu = sort(v(:), 1, 'descend');
    std::vector<double> mu(w);
    std::sort(mu.begin(), mu.end());
    std::reverse(mu.begin(), mu.end());
    
    // t = cumsum(mu);
    std::vector<double> t(n);
    t[0] = mu[0];
    for (int i = 1;i < n; i++) {
        t[i] = t[i-1] + mu[i];
    }
    
    // tc = t;
    std::vector<double> tc(t);
    
    // t = mu - 1./n .* (t - z);
    for (int i = 0; i < n; i++) {
        t[i] = mu[i] - 1.0 / ((double) (i+1)) * (t[i] - z);
    }
    
    // rho = find(t > 0, 1, 'last');
    int rho = 0;
    for (int i = 0; i < n; i++) {
        if (t[i] > 0.0) {
            rho = i + 1;
        }
    }
    
    // theta = 1/rho * (tc(rho) - z);
    double tval = 1.0 / ((double) rho);
    std::vector<double> theta(n);
    for (int i = 0; i < n; i++) {
         theta[i] = tval * (tc[rho-1] - z);
    }

    // w = max(v - theta, 0);
    for (int i = 0; i < n; i++) {
    	 double max = 0.0;
    	 if (w[i] - theta[i] > 0.0) {
    	     max = w[i] - theta[i];
    	 }
         w[i] = max;
    }
}

#endif
