/*
 *	Combining two submodular functions h(X) = \sum_{i = 1}^k \lambda_i f_i(X), for two submodular functions f and g.
	Melodi Lab, University of Washington, Seattle
 *
 */

#ifndef COMBINESUBMODULARFUNCTIONS_H
#define COMBINESUBMODULARFUNCTIONS_H

class combineSubmodularFunctions: public submodularFunctions{
	protected:
	vector<submodularFunctions*> fvec;
	vector<double> lambda;
	int nrFunctions;
	
	public:
	// Functions
	combineSubmodularFunctions(unordered_set<int>& groundSet, vector<submodularFunctions*>& fvec,vector<double> lambda);
	double eval(const unordered_set<int>& sset);
	double evalFast(const unordered_set<int>& sset, bool safe);
	double evalGainsadd(const unordered_set<int>& sset, int item);
	double evalGainsremove(const unordered_set<int>& sset, int item);
	double evalGainsaddFast(const unordered_set<int>& sset, int item, bool safe);
	double evalGainsremoveFast(const unordered_set<int>& sset, int item, bool safe);
	void updateStatisticsAdd(const unordered_set<int>& sset, int item, bool safe);
	void updateStatisticsRemove(const unordered_set<int>& sset, int item, bool safe);
	void setpreCompute(const unordered_set<int>& sset); // Set preCompute for removing elements.
	void clearpreCompute();
	
	void updateWeights(std::vector<double> lambda);
};

inline combineSubmodularFunctions::combineSubmodularFunctions(unordered_set<int>& groundSet, vector<submodularFunctions*>& fvec, vector<double> lambda): submodularFunctions(groundSet), fvec(fvec), lambda(lambda), nrFunctions(fvec.size()) {}

inline void combineSubmodularFunctions::updateWeights(std::vector<double> lambda) {
	if (lambda.size() != this->lambda.size()) 
		throw std::runtime_error("New weights have invalid size.");
	this->lambda.assign(lambda.begin(), lambda.end());
}

inline double combineSubmodularFunctions::eval(const unordered_set<int>& sset){
	double sumtot = 0;
	for (int i = 0; i < nrFunctions; i++){
		sumtot += lambda[i]*fvec[i]->eval(sset);
	}
	return sumtot;
}

inline double combineSubmodularFunctions::evalFast(const unordered_set<int>& sset, bool safe = 0){
	double sumtot = 0;
	for (int i = 0; i < nrFunctions; i++){
		sumtot += lambda[i]*fvec[i]->evalFast(sset, safe);
	}
	return sumtot;
}

inline double combineSubmodularFunctions::evalGainsadd(const unordered_set<int>& sset, int item){
	double gainsadd = 0;
	for (int i = 0; i < nrFunctions; i++){
		gainsadd += lambda[i]*fvec[i]->evalGainsadd(sset, item);
	}
	return gainsadd;
}

inline double combineSubmodularFunctions::evalGainsremove(const unordered_set<int>& sset, int item){
	double gainsremove = 0;
	for (int i = 0; i < nrFunctions; i++){
		gainsremove += lambda[i]*fvec[i]->evalGainsremove(sset, item);
	}
	return gainsremove;
}

inline double combineSubmodularFunctions::evalGainsaddFast(const unordered_set<int>& sset, int item, bool safe = 0){
	double gainsadd = 0;

	for (int i = 0; i < nrFunctions; i++){
		double tmp = lambda[i]*fvec[i]->evalGainsaddFast(sset, item, safe);
		gainsadd += tmp;
	}
	
	return gainsadd;
}

inline double combineSubmodularFunctions::evalGainsremoveFast(const unordered_set<int>& sset, int item, bool safe = 0){
	double gainsadd = 0;
	for (int i = 0; i < nrFunctions; i++){
		gainsadd += lambda[i]*fvec[i]->evalGainsremoveFast(sset, item, safe);
	}
	return gainsadd;
}

inline void combineSubmodularFunctions::updateStatisticsAdd(const unordered_set<int>& sset, int item, bool safe = 0){
	for (int i = 0; i < nrFunctions; i++){
		fvec[i]->updateStatisticsAdd(sset, item, safe);
	}
}

inline void combineSubmodularFunctions::updateStatisticsRemove(const unordered_set<int>& sset, int item, bool safe = 0){
	for (int i = 0; i < nrFunctions; i++){
		fvec[i]->updateStatisticsRemove(sset, item, safe);
	}
}

inline void combineSubmodularFunctions::clearpreCompute(){
	for (int i = 0; i < nrFunctions; i++){
		fvec[i]->clearpreCompute();
	}
}

inline void combineSubmodularFunctions::setpreCompute(const unordered_set<int>& sset){
	for (int i = 0; i < nrFunctions; i++){
		fvec[i]->setpreCompute(sset);
	}
}

#endif