/*
 *	Header file for defining the facility location function: f(X) = \sum_{i \in V} \max_{j \in X} s_{ij}.
	Assumes that the kernel S is symmetric.
	Author: Rishabh Iyer.
	Email: rkiyer@uw.edu
	Melodi Lab, University of Washington, Seattle
 *
 */

#ifndef CONSTANTFUNCTION_H
#define CONSTANTFUNCTION_H

class constantFunction: public submodularFunctions{
	protected:
		double value;
	
	public:
	// Functions
	constantFunction(std::unordered_set<int>& groundSet, double value);
	double eval(const unordered_set<int>& sset);
	double evalFast(const unordered_set<int>& sset, bool safe);
	double evalGainsadd(const unordered_set<int>& sset, int item);
	double evalGainsremove(const unordered_set<int>& sset, int item);
	double evalGainsaddFast(const unordered_set<int>& sset, int item, bool safe);
	double evalGainsremoveFast(const unordered_set<int>& sset, int item, bool safe);
	void updateStatisticsAdd(const unordered_set<int>& sset, int item, bool safe);
	void updateStatisticsRemove(const unordered_set<int>& sset, int item, bool safe);
	void setpreCompute(const unordered_set<int>& ssett);
	void clearpreCompute();
};

inline constantFunction::constantFunction(std::unordered_set<int>& groundSet, double value): submodularFunctions(groundSet), value(value) {
}


inline double constantFunction::eval(const unordered_set<int>& sset){
	return value;
}

inline double constantFunction::evalFast(const unordered_set<int>& sset, bool safe = 0){
	return value;
}

inline double constantFunction::evalGainsadd(const unordered_set<int>& sset, int item){
	return 0.0;
}

inline double constantFunction::evalGainsremove(const unordered_set<int>& sset, int item){
	return 0.0;
}

inline double constantFunction::evalGainsaddFast(const unordered_set<int>& sset, int item, bool safe = 0){
// Fast evaluation of Adding gains using the precomputed statistics. This is used, for example, in the implementation of the forward greedy algorithm.
// The safety parameter 'safe' controls the safety and speed of this procedure. If not sure how to use it, set it to it's default value, viz. 1
// If safe = 0, (for the sake of speed) we do not check if item does not belong to the subset. We also do not check if the preComputed statistics actually points to the set sset. You would need to check both these.
	return 0.0;	
}

inline double constantFunction::evalGainsremoveFast(const unordered_set<int>& sset, int item, bool safe = 0){
// Fast evaluation of Removing gains using the precomputed statistics. This is used, for example, in the implementation of the reverse greedy algorithm.
// The safety parameter 'safe' controls the safety and speed of this procedure. If not sure how to use it, set it to it's default value, viz. 1
// If safe = 0, (for the sake of speed) we do not check if item belong to the subset. We also do not check if the preComputed statistics actually points to the set sset. You would need to check both these.
	return 0.0;	
}

inline void constantFunction::updateStatisticsAdd(const unordered_set<int>& sset, int item, bool safe = 0){
// Update statistics for algorithms sequentially adding elements (for example, the greedy algorithm).
// The safety parameter 'safe' controls the safety and speed of this procedure. If not sure how to use it, set it to it's default value, viz. 1
// If safe = 0, (for the sake of speed) we do not check if item does not belong to the subset. We also do not check if the preComputed statistics actually points to the set sset. You would need to check both these.


}

inline void constantFunction::updateStatisticsRemove(const unordered_set<int>& sset, int item, bool safe = 0){
// Update statistics for algorithms sequentially removing elements (for example, the reverse greedy algorithm).
// The safety parameter 'safe' controls the safety and speed of this procedure. If not sure how to use it, set it to it's default value, viz. 1
// If safe = 0, (for the sake of speed) we do not check if item belong to the subset. We also do not check if the preComputed statistics actually points to the set sset. You would need to check both these.


}

inline void constantFunction::clearpreCompute(){

	
}

inline void constantFunction::setpreCompute(const unordered_set<int>& sset){

	
}

#endif