/*
 *	A tutorial for HSSFOTK.
	Author: Rishabh Iyer
	Email: rkiyer@uw.edu
	Melodi Lab, University of Washington, Seattle
 *
 */

#include <iostream>
#include <fstream>
#include <sstream>
#include <vector>
#include <sys/stat.h>
#include <sys/types.h>
#include <set>
#include <unordered_set>
#include <algorithm>

#include "stdio.h"
#include "stdlib.h"
#include "string.h"
#include "math.h"
#include "time.h"
#include <stdarg.h>
#include <assert.h>
#include <queue>
using namespace std;


char* strInputMat=NULL;
double per = 0;
int nUtt = 0;
char* strOutput=NULL;
char* costFile=NULL;
char* help=NULL;
double alpha=1;
double theta=5;
int unitsize=4;

// Utilities
#include "Utils/error.h"
#include "Utils/sort.h"
#include "Utils/utils.h"
#include "Utils/fileHandling.h"
#include "Utils/totalOrder.h"

//Submodular Functions
#include "Functions/setFunctions.h"
#include "Functions/submodularFunctions.h"
#include "Functions/modularFunctions.h"
#include "Functions/featureBasedFunctions.h"
#include "Functions/facilityLocation.h"
#include "Functions/graphCutFunctions.h"
#include "Functions/saturatedCoverage.h"
#include "Functions/complexityFunctions.h"
#include "Functions/combineSubmodularFunctions.h"
#include "Functions/differenceSubmodularFunctions.h"
#include "Functions/complementSubmodularFunctions.h"

//Algorithms
#include "Algorithms/lazyGreedyMax.h"
#include "Algorithms/lazyGreedySC.h"
#include "Algorithms/revlazyGreedyMax.h"
#include "Algorithms/localSearchMax.h"
#include "Algorithms/minNormPoint.h"
#include "Algorithms/naiveGreedyMax.h"
#include "Algorithms/mminCC.h"
#include "Algorithms/mminSCSC.h"
#include "Algorithms/mminSCSK.h"
#include "Algorithms/mminUC.h"
#include "Algorithms/mminSubSup.h"
#include "Algorithms/mminModMod.h"
#include "Algorithms/mminSupSub.h"
#define SMALLEST_NUMBER -1e70
#define LARGEST_NUMBER 1e70


int main(int argc, char** argv){
	char* similarityFile1 = "TestData/testsimilarity";
	char* featureFile = "/s0/rkiyer/LVCC/data/feats.noid.txt";
	char* transcriptFile = "TestData/testtranscript";
	char* scoreFile = "TestData/testscore";
	nUtt = 191698;
	int n = nUtt;
	vector<double> defCosts(n, 1);
	std::unordered_set<int> groundSet = unordered_set<int>();	
	for (int i = 0; i < n; ++i) 
		groundSet.insert(i);
	// The Lazy Greedy Algorithm for Submodular Maximization.
	complexityFunctions f (groundSet, transcriptFile, scoreFile);
	featureBasedFunctions g (groundSet, 1, featureFile);
	double lambda = (f.eval(groundSet))/(10*g.eval(groundSet));
	// Modular-Modular Procedure
	unordered_set<int> mminModModSet = mminModMod(f, g, lambda, 1, 1);
	cout<<"ModMod is f(X) = "<< f.eval(mminModModSet)<<" and g(X) = " << g.eval(mminModModSet)<<"\n";

	// Supermodular-Submodular Procedure
	unordered_set<int> mminSupSubSet = mminSupSub(f, g, lambda, 1, 0);
	cout<<"SubSup is f(X) = "<< f.eval(mminSupSubSet)<<" and g(X) = " << g.eval(mminSupSubSet)<<"\n";
	
	// The naive greedy algorithm
	combineSubmodularFunctions h (&g, &f, lambda, -1);
	unordered_set<int> GrSet = lazyGreedymax(h, defCosts, LARGEST_NUMBER, unordered_set<int>(), 0);
	cout<<"h(X) = "<<h.eval(GrSet)<<"\n";
	cout<<"Gr is f(X) = "<< f.eval(GrSet)<<" and g(X) = " << g.eval(GrSet)<<"\n";
	

	// Submodular-Supermodular Procedure
	unordered_set<int> mminSubSupSet = mminSubSup(f, g, lambda, 1, 1);
	cout<<"SubSup is f(X) = "<< f.eval(mminSubSupSet)<<" and g(X) = " << g.eval(mminSubSupSet)<<"\n";
	

	//submodularFunctions* h = new featureBasedFunctions(groundSet, 3, strInputMat2);
	//submodularFunctions* c = new combineSubmodularFunctions(f, g, 1, 1);
	//vector<double> costList = readCostfunction(costFile, maxCost);
	//double budget = 0.001*maxCost;
	//cout<<budget<<"\n";
	/*double* weights = new double[n];
	fill_n(weights, n, 1.0);
	submodularFunctions* m = new modularFunctions(weights, 0, groundSet);
	time_t start, end;
	double timetaken;*/
	/* Lazy Greedy
	start = time(0);
	std::unordered_set<int> greedySetl = lazyGreedymax(f, costList, budget);
	cout<<"Greedy Max with Lazy is "<< f->evalFast(greedySetl)<<"\n";
	end = time(0);
	timetaken = difftime(end, start);
	cout<<"Time taken by Greedy Lazy is "<<timetaken<<" seconds.\n";*/
	/*	
	// Naive Greedy
	start = time(0);
	std::unordered_set<int> greedySetn = naiveGreedymax(c, costList, budget);
	cout<<"Greedy Max with naive is "<< c->eval(greedySetn)<<"\n";
	end = time(0);
	timetaken = difftime(end, start);
	cout<<"Time taken by Naive Greedy is "<<timetaken<<" seconds.\n";*/	
	//double lambda = f->eval(groundSet)/n;
	//cout<<"SIze of the total vocabulary is "<< f->eval(groundSet)<<"\n";
	//submodularFunctions* c = new combineSubmodularFunctions(f, m, 1, -0.009);

	// SCSK
	std::unordered_set<int> mminscskSet = mminSCSK(f, g, f.eval(mminSubSupSet));
	cout<<"SCSK is f(X) = "<< f.eval(mminscskSet)<<" and g(X) = " << g.eval(mminscskSet)<<"\n";
	
	// SCSC
	std::unordered_set<int> mminscscSet = mminSCSC(f, g, g.eval(mminSubSupSet));
	cout<<"SCSC is f(X) = "<< f.eval(mminscscSet)<<" and g(X) = " << g.eval(mminscscSet)<<"\n";
	//CC
	/*
	//MN
	unordered_set<int> minSolSet;
	unordered_set<int> maxSolSet; 
	std::vector<double> xstar;
	minNormPoint(c, minSolSet, maxSolSet, xstar);
	cout<<"minSolSet has a value "<< f->eval(minSolSet)<<" and size "<<minSolSet.size()<<".\n";
	cout<<"maxSolSet has a value "<< f->eval(maxSolSet)<<" and size "<<maxSolSet.size()<<".\n";*/
	// Constrained Submodular Minimization:
	/*int k = 20;
	//submodularFunctions* f = new facilityLocation(groundSet, strInputMat);
	start = time(0);
	std::unordered_set<int> mminSet = mminCardinality(f, k);
	cout<<"MMin is "<< f->eval(mminSet)<<" with k = "<< k<<"\n";
	end = time(0);
	timetaken = difftime(end, start);
	cout<<"Time taken for MMin is "<<timetaken<<" seconds.\n";
	

	double* weights = new double[n];
	fill_n(weights, n, 1.0);
	submodularFunctions* m = new modularFunctions(weights, 0, groundSet);
	std::unordered_set<int> mminscskSet = mminSubKnapsack(f, m, 114);
	cout<<"SCSK is f(X) = "<< f->eval(mminscskSet)<<" and g(X) = " << m->eval(mminscskSet)<<"\n";
	cout<<"Number of utterances is "<<mminscskSet.size()<<"\n";
	//cout<<k<<"\n";
	
	// Minimum norm point
	submodularFunctions* c = new combineSubmodularFunctions(f, m, 1, -0.01);
	unordered_set<int> minSolSet;
	unordered_set<int> maxSolSet; 
	std::vector<double> xstar;
	minNormPoint(c, minSolSet, maxSolSet, xstar);
	cout<<"minSolSet has a value "<< f->eval(minSolSet)<<" and size "<<minSolSet.size()<<".\n";
	cout<<"maxSolSet has a value "<< f->eval(maxSolSet)<<" and size "<<maxSolSet.size()<<".\n";*/
	//complexityFunctions cc(groundSet, transcriptFile, scoreFile);
	/*for(unordered_set<int>::iterator it = mminscskSet.begin(); it != mminscskSet.end(); it++){
		cout<<cc.mUttrMap[*it]<<"\n";
	}*/
	// Random Sets
	/*srand(time(NULL));
	for (int times = 0; times < 10; times++){
    		std::unordered_set<int> randomSet;
		for (int tt = 0; tt < budget; tt++){
			int randel = rand() % n;
			randomSet.insert(randomSet.end(), randel);
		}
		cout<<"Random Set value is "<< f->eval(randomSet)<<"\n";
	}*/
}

/* Random sets debug:

*/
