clear all;
clc;

for ds = 1:14,
overlap = 0.1;
nrSamples = 50;

addpath('external/sfo');

% load summaries and compute average scores
[summaries, prunedSummaries] = pruneSummaries(ds, get_pruning_info(ds, overlap));
bow = load_resource('bow', 'evaluation', ds);
bowEvaluate = get_BOW_evaluation(ds);
rand('seed', 123456);
randomSummaries = randomSummary(100, 10, 300);

summariesAsBow = computeBOWSummary(summaries, bowEvaluate);
optionsEvaluationLoss = struct();
optionsEvaluationLoss.bow = bowEvaluate;
optionsEvaluationLoss.summariesAsBow = summariesAsBow;

% compute rouge min/max
scoreFunction = @(S) summaryEvaluation(S, summaries, 'rouge', optionsEvaluationLoss);
[averageHumanScore,a1] = summaryEvaluation(summaries, summaries, 'rouge', optionsEvaluationLoss);
[averageRandomScore,a2] = summaryEvaluation(randomSummaries, summaries, 'rouge', optionsEvaluationLoss);

% try to sample
rougeGreedySum = sfo_greedy_lazy(sfo_fn_wrapper(@(x) summaryEvaluation(x, summaries, 'rouge', optionsEvaluationLoss)), 1:100, 10);
rougemax = scoreFunction(rougeGreedySum);

[~,~,greedyMinRouge,~] = mmin(@(x) summaryEvaluation(x, summaries, 'rouge', optionsEvaluationLoss), 100, [], 'MUB', 10);
rougemin = scoreFunction(greedyMinRouge);

% sample scores
averageRandomScore
std(a2)
rougemin
scoresLow = [];
listSummaries = [];
while size(listSummaries,1) < nrSamples,
    fprintf(1, 'We have %d/%d summaries.\n', size(listSummaries,1), nrSamples);
    bowNew = bow .* (0.9 + 0.2*randn(size(bow)));
    
    summariesAsBowNew = computeBOWSummary(summaries, bowNew);
    optionsEvaluationLossNew = struct();
    optionsEvaluationLossNew.bow = bowNew;
    optionsEvaluationLossNew.summariesAsBow = summariesAsBowNew;
    
    [~,~,greedyMinRouge,~] = mmin(@(x) summaryEvaluation(x, summaries, 'rouge', optionsEvaluationLossNew), 100, [], 'MUB', 10);
    greedyMinRouge = sort(greedyMinRouge);
    if size(listSummaries,1) > 0,
        cmp = listSummaries == repmat(greedyMinRouge, size(listSummaries,1), 1);
        if any(all(cmp,2)),
            fprintf(1, 'Not new.\n');
            continue;
        end   
    end
    
    scoresLow(end+1) = scoreFunction(greedyMinRouge);
    %scoresLow(end)
    listSummaries(end+1,:) = greedyMinRouge;
end

summariesLowScores = listSummaries;

%% sample scores
averageHumanScore
rougemax
scoresHigh = [];
listSummaries = [];
while size(listSummaries,1) < nrSamples,
    fprintf(1, 'We have %d/%d summaries.\n', size(listSummaries,1), nrSamples);
    bowNew = bow .* (0.9 + 0.2*randn(size(bow)));
    
    summariesAsBowNew = computeBOWSummary(summaries, bowNew);
    optionsEvaluationLossNew = struct();
    optionsEvaluationLossNew.bow = bowNew;
    optionsEvaluationLossNew.summariesAsBow = summariesAsBowNew;
    
    rougeGreedySum = sfo_greedy_lazy(sfo_fn_wrapper(@(x) summaryEvaluation(x, summaries, 'rouge', optionsEvaluationLossNew)), 1:100, 10);
%     rougeGreedySum = sfo_greedy_lazy(sfo_fn_wrapper(@(x) summaryEvaluation(x, summaries, 'rouge', optionsEvaluationLoss)), 1:100, 10);
%     t = randperm(10);
%     t2 = randperm(90);
%     s = setdiff(1:100, rougeGreedySum);
%     rougeGreedySum(t(1)) = s(t2(1));
    rougeGreedySum = sort(rougeGreedySum);
    
    % is this a new summary?
    if size(listSummaries,1) > 0,
        cmp = listSummaries == repmat(rougeGreedySum, size(listSummaries,1), 1);
        if any(all(cmp,2)),
            fprintf(1, 'Not new.\n');
            continue; 
        end  
    end
    
    scoresHigh(end+1) = scoreFunction(rougeGreedySum);
    scoresHigh(end)
    listSummaries(end+1,:) = rougeGreedySum;
end

summariesHighScores = listSummaries;

%%
t = randperm(size(summaries, 1));
summariesHuman = summaries(t(1:nrSamples),:);
t = randperm(size(randomSummaries, 1));
summariesRandom = randomSummaries(t(1:nrSamples),:);

%% plot 2
scoresHuman = zeros(50,1);
scoresRandom = zeros(50,1);
scoresLow = zeros(50,1);
scoresHigh = zeros(50,1);
for i = 1:nrSamples,
    scoresHuman(i) = scoreFunction(summariesHuman(i,:));
    scoresRandom(i) = scoreFunction(summariesRandom(i,:));
    scoresLow(i) = scoreFunction(summariesLowScores(i,:));
    scoresHigh(i) = scoreFunction(summariesHighScores(i,:));
end

ymax = 1;
centers = linspace(rougemin, rougemax, 50);
clf
hold all;

% random
[a,b] = hist(scoresRandom, centers);
bar(b,a/sum(a), 'r');

% sampled low scores
[a,b] = hist(scoresLow, centers);
bar(b,a/sum(a));

% human
[a,b] = hist(scoresHuman, centers);
bar(b,a/sum(a), 'g');

% sampled high scores
[a,b] = hist(scoresHigh, centers);
bar(b,a/sum(a));

line([rougemax, rougemax], [0,ymax])
line([rougemin, rougemin], [0,ymax])
%xlim([rougemin-0.05,rougemax+0.05])
%hist(a1, zeros(size(a1)));
title('Scores of selected summaries');
xlabel('v-rouge score');
ylabel('normalized histogram');

% save plot
print('-dpdf', '-r300', sprintf('correlation/scoreDistribution_ds_%d.pdf', ds));

allSummaries = vertcat(summariesLowScores, summariesHighScores, summariesHuman, summariesRandom);
scores = zeros(size(allSummaries,1),1);
for i = 1:size(allSummaries,1),
    scores(i) = scoreFunction(allSummaries(i,:));
end

% save summaries to file
save(sprintf('correlation/selected_summaries_ds_%d.mat', ds), 'allSummaries', 'scores');

% save in our format
fh = fopen(sprintf('correlation/selected_summaries_ds_%d.txt', ds), 'w');
for s = 1:size(allSummaries, 1),
    fprintf(fh, 'ds%ds%d: ', ds, s);
    fprintf(fh, '%d ', allSummaries(s,:));
    fprintf(fh, '\n');
end
fclose(fh);
end

return
