function [h] = get_colorHistogram(images, options)
% options.colorspace: 'rgb', 'hue'
% options.bins: number of bins

nOptionsArg = 2;
if nargin < nOptionsArg || ~isstruct(options) || ~isfield(options, 'colorspace'),
    options.colorspace = 'rgb';
    warning('Colorspace not defined. Setting to "%s".\n', options.colorspace);
end
if nargin < nOptionsArg || ~isstruct(options) || ~isfield(options, 'bins'),
    options.bins = 16;
    warning('Number of bins not defined. Setting to "%d".\n', options.bins);
end

if ~strcmp(options.colorspace, 'rgb'),
    error('No support for other colorspaces than rgb yet.');
end

h = zeros(3*options.bins, size(images, 4));
bins = linspace(0, 255, options.bins);
for img = 1:size(images, 4),
    ch1 = images(:,:,1,img);
    ch2 = images(:,:,2,img);
    ch3 = images(:,:,3,img);
    h(1:options.bins,img) = histc(ch1(:), bins);% / numel(ch1);
    h((options.bins+1):(2*options.bins),img) = histc(ch2(:), bins); % / numel(ch1);
    h((2*options.bins+1):(3*options.bins),img) = histc(ch3(:), bins);% / numel(ch1);
end


end