function [h] = get_colorHistogramPatch(patch, options)
% extracts the colorhistogram of a patch
% options.bins: number of bins (array with 3 nuumbers, one for every
% color component)

nOptionsArg = 2;
if nargin < nOptionsArg || ~isstruct(options) || ~isfield(options, 'bins'),
    options.bins = [4,4,4];
    warning('Number of bins not defined. Setting to default.\n');
end

if nargin < nOptionsArg || ~isstruct(options) || ~isfield(options, 'histType'),
    options.histType = 'SQ';
    warning('Quantizer for histogram computation not specified. Using "%s".', options.histType);
end

% sanity check
if any(any((patch < 0) | (patch > 1))),
    error('Invalid data range');
end

bins1 = linspace(0, 1 + 1e-6, options.bins(1) + 1);
bins2 = linspace(0, 1 + 1e-6, options.bins(2) + 1);
bins3 = linspace(0, 1 + 1e-6, options.bins(3) + 1);

if size(patch, 3) > 1,
    t1 = patch(:,:,1);
    t2 = patch(:,:,2);
    t3 = patch(:,:,3);
else
    t1 = patch(:,1);
    t2 = patch(:,2);
    t3 = patch(:,3);
end
[~,h1] = histc(t1(:), bins1);
[~,h2] = histc(t2(:), bins2);
[~,h3] = histc(t3(:), bins3);

switch options.histType,
    case 'SQ'
        h1 = accumarray(h1, ones(size(h1,1),1));
        h1 = vertcat(h1, zeros(options.bins(1) - length(h1),1));
        h2 = accumarray(h2, ones(size(h2,1),1));
        h2 = vertcat(h2, zeros(options.bins(2) - length(h2),1));
        h3 = accumarray(h3, ones(size(h3,1),1));
        h3 = vertcat(h3, zeros(options.bins(3) - length(h3),1));
        
        h = vertcat(h1,h2,h3);
    case 'VQ'
        h = zeros(prod(options.bins), 1);
        for i = 1:length(h1),
            idx = sub2ind(options.bins, h1(i), h2(i), h3(i));
            h(idx) = h(idx) + 1;
        end
    otherwise
        error('Invalid type of quantizer.');
end


end