function [ colors ] = get_frequentColors( iimage , display)
%GET_FREQUENTCOLORS Summary of this function goes here
%   Detailed explanation goes here

blockSizeX = 10;
blockSizeY = 10;
simThreshold = 1e-3;

nrBlocksX = 100 / blockSizeX;
nrBlocksY = 100 / blockSizeY;

if nrBlocksX ~= ceil(nrBlocksX) || nrBlocksY ~= ceil(nrBlocksY),
    error('Invalid block size chosen.');
end

colors = NaN(nrBlocksX * nrBlocksY, 3);
idx = 1;
for x = 1:nrBlocksX,
    for y = 1:nrBlocksY,
        % extract block
        xmin = (x-1)*blockSizeX + 1;
        xmax = xmin + blockSizeX - 1;
        ymin = (y-1)*blockSizeY + 1;
        ymax = ymin + blockSizeY - 1;
        
        block = iimage(xmin:xmax, ymin:ymax, :);
        block = reshape(block, blockSizeX*blockSizeY, 3);
        
        % find most common color in block
        frequencies = NaN(blockSizeX*blockSizeY, 1);
        for i = 1:size(block, 1),
            tmp = sum((block - repmat(block(i,:), size(block,1), 1)).^2,2);
            frequencies(i) = sum(tmp < simThreshold);
        end
        if ~any(frequencies > 4), % pick random color
            perm = randperm(size(block,1));
            colors(idx,:) = block(perm(1), :);
        else
            [~,cidx] = max(frequencies);
            colors(idx,:) = block(cidx, :);
        end
            
        idx = idx + 1;
    end
end

if nargin == 2, % display found colors
    img = zeros(nrBlocksX*10, nrBlocksY*10, 3);
    idx = 1;
    for x = 1:nrBlocksX,
        for y = 1:nrBlocksY,
            % extract block
            xmin = (x-1)*blockSizeX + 1;
            xmax = xmin + blockSizeX - 1;
            ymin = (y-1)*blockSizeY + 1;
            ymax = ymin + blockSizeY - 1;

            img(xmin:xmax, ymin:ymax, :) = reshape(repmat(colors(idx,:), 100, 1), 10, 10, 3);

            idx = idx + 1;
        end
    end
    image(img)
end

end

