function [imagescolor, imagesgray] = get_images(dataset, display, colorsp)
% getImages   Loads images from dataset.
%    The function get_images(dataset) loads the images of the collection
%    specified by dataset (path of the images '../Data/setX/images.mat'). 
%    If get_images(dataset, display) is called with the display argument 
%    set to true, the image collection is also displayed.
%    
%    [imagesrgb, imagesgray, imageshsv] = get_images(dataset, display),
%    where imagesrgb are the images in rgb colorspace (normalized to
%    [0,1]), imagesgray are the images in grayscale (normalized to [0,255])
%    and imageshsv are the images in hsv colorspace (normalized to [0,1]).

if nargin < 2, % do not display by default
    display = false;
end

if nargin < 3,
    colorsp = 'rgb';
end

preDir = getPreDir();
%% check if data file exists
filename = sprintf('%s/Data/set%d/images.mat', preDir, dataset);
if ~exist(filename, 'file'),
    error('File "%s" does not exist. Maybe you have to run "convertImages" first.');
end
keyboard

%% load images
switch colorsp,
    case 'rgb'
        load(filename, 'imagesgray', 'imagesrgb');
        imagescolor = imagesrgb;
    case 'hsv'
        load(filename, 'imagesgray', 'imageshsv');
        imagescolor = imageshsv;
    case 'lab'
        load(filename, 'imagesgray', 'imageslab');
        imagescolor = imageslab;
    otherwise
        error('Invalid color space.');
end

%% if desired, display image collection
if display,
    montage(imagesrgb, 'size',[10,10]);
    set(gca,'position',[0 0 1 1],'units','normalized')
end

end