%% The script to get the summaries
disp = 1;
addpath sfo/

% Initial configurations
dataset = 3;
    % fid = fopen(sprintf('%d-results.txt', dataset), 'w');
    fid = 1;
    nrSummariesToPrune = 100;

    [imagescolor, imagesgray] = get_images(dataset, false);
    [summaries, nrSummaries] = load_summaries(dataset);

    % remove 100 worst summaries
    [summaries, prunedSummaries] = pruneSummaries(dataset, nrSummariesToPrune);

    fname = sprintf('%s/precomputed/bow_superpixels_ds_%d_dictSize_%d.mat', getPreDir(), dataset, 300);
    load(fname);
    bowSIFT = bow;
    %bowSIFT = get_BOW(dataset);
    bowSIFTTFIDF = applyTFIDF(bowSIFT, get_TFIDF(bowSIFT));
    bowSIFTTFIDF = bsxfun(@times, bowSIFTTFIDF, 1./sum(bowSIFTTFIDF,1));
    simBowSIFT = BOWsimilarity(bowSIFT);
    simBowSIFTTFIDF = BOWsimilarity(bowSIFTTFIDF);

    fname = sprintf('%s/precomputed/colorHistogram_ds_%d.mat', getPreDir(), dataset);
    load(fname);
    bowColor = bow;
    bowColorTFIDF = applyTFIDF(bowColor, get_TFIDF(bowColor));
    simBowColor = BOWsimilarity(bowColor);
    simBowColorTFIDF = BOWsimilarity(bowColorTFIDF);


    bowCombined = [bowSIFT; bowColor];
    bowCombinedTFIDF = applyTFIDF(bowCombined, get_TFIDF(bowCombined));
    simBowCombined = BOWsimilarity(bowCombined);
    simBowCombinedTFIDF = BOWsimilarity(bowCombinedTFIDF);

    %[gist,simGist] = get_gist(dataset);

    % bowEvaluate = bowCombined;
    bowEvaluate = bowSIFTTFIDF;
    % bowB = bow;
    % fname = sprintf('%s/precomputed/colorHistogram_ds_%d.mat', getPreDir(), dataset);
    % load(fname);
    % bowColor = bow;

    %bowTFIDF = applyTFIDF(bow, get_TFIDF(bow));
    %bow = bowTFIDF;


    summariesAsBow = computeBOWSummary(summaries, bowEvaluate);
    rand('seed', 123456);
    randomSummaries = randomSummary(100, 10, 1000);

    % setup functions to use
    optionsEvaluationLoss = struct();
    optionsEvaluationLoss.bow = bowEvaluate;
    optionsEvaluationLoss.summariesAsBow = summariesAsBow;

    [a1,b1] = summaryEvaluation(summaries, summaries, 'rouge', optionsEvaluationLoss);
    [a2,b2] = summaryEvaluation(randomSummaries, summaries, 'rouge', optionsEvaluationLoss);
    fprintf(fid, 'Average human score=%f [min=%f,max=%f]\n', mean(b1), min(b1), max(b1));
    fprintf(fid, 'Average random score=%f [min=%f,max=%f]\n', mean(b2), min(b2), max(b2));
    fprintf(fid, 'Overlap=%f\n', computeGaussianOverlap(mean(b1), mean(b2), std(b1), std(b2)));

    optionsEvaluationLossNGRAM = struct();
    optionsEvaluationLossNGRAM.k = 5;

    [rougeGreedySum] = sfo_greedy_lazy(sfo_fn_wrapper(@(x) summaryEvaluation(x, summaries, 'rouge', optionsEvaluationLoss)), 1:100, 10);
    if disp == 1 
        montage(imagescolor, 'Indices', rougeGreedySum, 'Size',[1,10]);
    end
    fprintf(fid, 'ROUGE SCORE FOR Greedy Rouge: %f\n', summaryEvaluation(rougeGreedySum, summaries, 'rouge', optionsEvaluationLoss));
    fprintf(fid, 'NGRAM SCORE FOR Greedy Rouge: %f\n', summaryEvaluation(rougeGreedySum, summaries, 'combinatorial', optionsEvaluationLossNGRAM));


    [ngramGreedySum] = greedySummary(@(x) summaryEvaluation(x, summaries, 'combinatorial', optionsEvaluationLossNGRAM), 10); % sfo_greedy_lazy(sfo_fn_wrapper(@(x) summaryEvaluation(x, summaries, 'combinatorial', optionsEvaluationLossNGRAM)), 1:100, 10);
    if disp == 1
        figure; montage(imagescolor, 'Indices', ngramGreedySum, 'Size',[1,10]);
    end
    fprintf(fid, 'ROUGE SCORE FOR Greedy NGRAM: %f\n', summaryEvaluation(ngramGreedySum, summaries, 'rouge', optionsEvaluationLoss));
    fprintf(fid, 'NGRAM SCORE FOR Greedy NGRAM: %f\n', summaryEvaluation(ngramGreedySum, summaries, 'combinatorial', optionsEvaluationLossNGRAM));


    %% Initialization
    % Assumes that the similarity matrix has been created. Then using different
    % objectves we find the summaries.
    n = size(simBowCombined,1);
    sumsize = 10;

    % features = bow';
    %features = gist';

    %% Get the clustering for the reward terms using Facility Location
    %clear clusters;
    numiterations = 1;
    numclusters = 20;
    sim = simBowCombined;
    Fcov = sfo_facility_coverage(simBowCombined);
    cidx = sfo_greedy_lazy(Fcov,1:n,numclusters);
    S  = sim(cidx,:);
    [vals,IDX] = max(S,[],1);
    for j = 1 : numiterations
        for i = 1 : numclusters
            A = find(IDX == i);
            clusters(i, j) = struct('elements',A);
            figure; montage(imagescolor, 'indices',clusters(i).elements);
        end
    end

    %% The graph cut objective alone
    options = {'simBowSIFT', 'simBowSIFTTFIDF', 'simBowColor', 'simBowColorTFIDF', 'simBowCombined', 'simBowCombinedTFIDF'};%, 'simGist'};
    for k = 1:length(options);
        eval(sprintf('sim = %s;', options{k}));
        Fcov = sfo_cut_coverage(sim);
        A = sfo_greedy_lazy(Fcov,1:n,sumsize);
        if disp == 1
            figure, montage(imagescolor, 'Indices',A,'Size',[1,10]);
            title(sprintf('Graph Cut, sim=%s', options{k}));
        end
        fprintf(fid, 'ROUGE SCORE FOR Graph Cut using %s: %f\n', options{k}, summaryEvaluation(A, summaries, 'rouge', optionsEvaluationLoss)); 
        set(gca, 'Position', [0 0 1 1])
    end

    % get_analysis;
    % Result analysis
    %% The graph cut objective with penalty term

    %axis tight;
    %set(gca, 'Position', [0 0 1 1])
    % imdisp(imagescolor(:,:,:,A), 'Size', [10, 1]);
    % get_analysis;
    %summaryEvaluation(A, summaries, 'rouge', optionsEvaluationLoss)

    options = {'simBowSIFT', 'simBowSIFTTFIDF', 'simBowColor', 'simBowColorTFIDF', 'simBowCombined', 'simBowCombinedTFIDF'};%, 'simGist'};
    for k = 1:length(options);
        eval(sprintf('sim = %s;', options{k}));
        lambdaRange = 1:2:20;
        bestScore = -inf;
        bestA = [];
        bestLambda = [];
        for lambda = lambdaRange,
            Fcov = sfo_cut_coverage(sim);
            Fdisp = sfo_penalty_disparity(sim);
            Fdisplambda = ntimes(Fdisp, lambda);
            Fsum = sfo_sum(Fcov, Fdisplambda);
            A = sfo_greedy_lazy(Fsum, 1:n, sumsize);
            score = summaryEvaluation(A, summaries, 'rouge', optionsEvaluationLoss);
            if score > bestScore,
                bestA = A;
                bestScore = score;
                bestLambda = lambda;
            end
        end
        A = bestA;
        if disp == 1
            figure; montage(imagescolor, 'Indices', A, 'Size',[1,10]);
            title(sprintf('Graph Cut with penalty, sim=%s, lambda=%f', options{k}, bestLambda));
        end
        fprintf(fid, 'ROUGE SCORE FOR Graph Cut with penalty using %s: %f\n', options{k}, summaryEvaluation(A, summaries, 'rouge', optionsEvaluationLoss)); 
        set(gca, 'Position', [0 0 1 1])
        drawnow;
    end
    fprintf('\n')
    %% The graph-cut objective and the disparity reward term

    %figure; montage(imagescolor, 'Indices', A, 'Size',[1,10]);
    %summaryEvaluation(A, summaries, 'rouge', optionsEvaluationLoss)

    options = {'simBowSIFT', 'simBowSIFTTFIDF', 'simBowColor', 'simBowColorTFIDF', 'simBowCombined', 'simBowCombinedTFIDF'}; %, 'simGist'};
    %load('clusteringByHand.mat');
    for k = 1:length(options);
        eval(sprintf('sim = %s;', options{k}));
        lambdaRange = 1:2:20;
        bestScore = -inf;
        bestA = [];
        bestLambda = [];
        for lambda = lambdaRange,
            Fcov = sfo_cut_coverage(sim);
            Fdisp = sfo_reward_disparity(sim, clusters);
            Fdisplambda = ntimes(Fdisp, lambda);
            Fsum = sfo_sum(Fcov, Fdisplambda);
            A = sfo_greedy_lazy(Fsum, 1:n, sumsize);
            score = summaryEvaluation(A, summaries, 'rouge', optionsEvaluationLoss);
            if score > bestScore,
                bestA = A;
                bestScore = score;
                bestLambda = lambda;
            end
        end
        A = bestA;
        if disp == 1
            figure; montage(imagescolor, 'Indices', A, 'Size',[1,10]);
            title(sprintf('Graph Cut with diversity reward, sim=%s, lambda=%f', options{k}, bestLambda));

        end
        fprintf(fid, 'ROUGE SCORE FOR Graph Cut with diversity using %s: %f\n', options{k}, summaryEvaluation(A, summaries, 'rouge', optionsEvaluationLoss)); 
        set(gca, 'Position', [0 0 1 1])
        drawnow;
    end
    fprintf( '\n')

    %% The facility location objective alone
    options = {'simBowSIFT', 'simBowSIFTTFIDF', 'simBowColor', 'simBowColorTFIDF', 'simBowCombined', 'simBowCombinedTFIDF'};%, 'simGist'};
    for k = 1:length(options);
        eval(sprintf('sim = %s;', options{k}));
        Fcov = sfo_facility_coverage(sim);
        A = sfo_greedy_lazy(Fcov,1:n,sumsize);
        if disp == 1
            figure, montage(imagescolor, 'Indices',A,'Size',[1,10]);
            title(sprintf('Facility Location, sim=%s', options{k}));

        end
        fprintf(fid, 'ROUGE SCORE FOR Facility Location using %s: %f\n', options{k}, summaryEvaluation(A, summaries, 'rouge', optionsEvaluationLoss)); 
        set(gca, 'Position', [0 0 1 1])
    end
    fprintf('\n')
    
        %% Feature based functions
    options = {'bowSIFT', 'bowSIFTTFIDF', 'bowColor', 'bowColorTFIDF', 'bowCombinedTFIDF'}; %, 'bowCombined'};
    for k = 1:length(options);
        eval(sprintf('bow = %s;', options{k}));
        Fcov = sfo_feature_based_sqrt(bow);
        A = sfo_greedy_lazy(Fcov,1:n,sumsize);
        if disp == 1
            figure, montage(imagescolor, 'Indices',A,'Size',[1,10]);
            title(sprintf('Feature based functions, sim=%s', options{k}));
        end
        fprintf(fid, 'ROUGE SCORE FOR Feature based functions using %s: %f\n', options{k}, summaryEvaluation(A, summaries, 'rouge', optionsEvaluationLoss)); 
        set(gca, 'Position', [0 0 1 1])
    end
    fprintf('\n')
    %% The facility location objective and penalty term
    options = {'simBowSIFT', 'simBowSIFTTFIDF', 'simBowColor', 'simBowColorTFIDF', 'simBowCombined', 'simBowCombinedTFIDF'};%, 'simGist'};
    %load('clusteringByHand.mat');
    for k = 1:length(options);
        eval(sprintf('sim = %s;', options{k}));
        lambdaRange = logspace(-2,0,10);
        bestScore = -inf;
        bestA = [];
        bestLambda = [];
        for lambda = lambdaRange,
            Fcov = sfo_facility_coverage(sim);
            Fdisp = sfo_penalty_disparity(sim);
            Fdisplambda = ntimes(Fdisp, lambda);
            Fsum = sfo_sum(Fcov, Fdisplambda);
            A = sfo_greedy_lazy(Fsum, 1:n, sumsize);
            score = summaryEvaluation(A, summaries, 'rouge', optionsEvaluationLoss);
            if score > bestScore,
                bestA = A;
                bestScore = score;
                bestLambda = lambda;
            end
        end
        A = bestA;
        if disp == 1
            figure; montage(imagescolor, 'Indices', A, 'Size',[1,10]);
            title(sprintf('Facility Location with diversity penalty, sim=%s, lambda=%f', options{k}, bestLambda));
        end
        fprintf(fid, 'ROUGE SCORE FOR Facility Location with diversity penalty using %s: %f\n', options{k}, summaryEvaluation(A, summaries, 'rouge', optionsEvaluationLoss)); 
        set(gca, 'Position', [0 0 1 1])
        drawnow;
    end
    fprintf('\n')


    %% The facility location objective with diversity reward term 
    options = {'simBowSIFT', 'simBowSIFTTFIDF', 'simBowColor', 'simBowColorTFIDF', 'simBowCombined', 'simBowCombinedTFIDF'};%, 'simGist'};
    %load('clusteringByHand.mat');
    for k = 1:length(options);
        eval(sprintf('sim = %s;', options{k}));
        lambdaRange = logspace(-2,0,10);
        bestScore = -inf;
        bestA = [];
        bestLambda = [];
        for lambda = lambdaRange,
            Fcov = sfo_facility_coverage(sim);
            Fdisp = sfo_reward_disparity(sim, clusters);
            Fdisplambda = ntimes(Fdisp, lambda);
            Fsum = sfo_sum(Fcov, Fdisplambda);
            A = sfo_greedy_lazy(Fsum, 1:n, sumsize);
            score = summaryEvaluation(A, summaries, 'rouge', optionsEvaluationLoss);
            if score > bestScore,
                bestA = A;
                bestScore = score;
                bestLambda = lambda;
            end
        end
        A = bestA;
        if disp == 1
            figure; montage(imagescolor, 'Indices', A, 'Size',[1,10]);
            title(sprintf('Facility Location with diversity reward, sim=%s, lambda=%f', options{k}, bestLambda));
        end
        fprintf(fid, 'ROUGE SCORE FOR Facility Location with diversity using %s: %f\n', options{k}, summaryEvaluation(A, summaries, 'rouge', optionsEvaluationLoss)); 
        set(gca, 'Position', [0 0 1 1])
        drawnow;
    end
    fprintf('\n')
    fclose(fid)
%end
% %% Get the clustering for the reward terms using kmeans
% clear clusters;
% numiterations = 1;
% numclusters = 10;
% IDX = kmeans(features, numclusters, 'replicates', 2);
% for j = 1 : numiterations
%     for i = 1 : numclusters
%         A = find(IDX == i);
%         clusters(i, j) = struct('elements',A);
%         figure; montage(imagescolor, 'indices',clusters(i).elements);
%     end
% end
% 
% %% Get the clustering for the reward terms using kmediods
% clear clusters;
% numiterations = 1;
% numclusters = 4;
% IDX = kmedioids(1 - sim, numclusters);
% for j = 1 : numiterations
%     for i = 1 : numclusters
%         A = find(IDX == i);
%         clusters(i, j) = struct('elements',A);
%         figure; montage(imagescolor, 'indices',clusters(i).elements);
%     end
% end
% 
% %% Fit in the ground-truth clusters
% clear clusters
% for i = 1 : numclusters
%     clusters(i,1) = struct('elements', gt(i):(gt(i+1)-1));
% end
% 
% %% The thresholded sum objective alone
% Fcov = sfo_threshsum_coverage(sim);
% A = sfo_greedy_lazy(Fcov,1:n,sumsize);
% figure; montage(imagescolor, 'Indices',A,'Size',[1,10]);
% % get_analysis;
% 
% %% The thresholded sum objective with penalty term
% 
% %% The threshold sum objective and the disparity reward term
% lambda = 4;
% thresh = 0.1;
% Fcov = sfo_threshsum_coverage(sim, thresh);
% Fdisp = sfo_reward_disparity(sim, clusters);
% Fdisplambda = ntimes(Fdisp, lambda);
% Fsum = sfo_sum(Fcov, Fdisplambda);
% A = sfo_greedy_lazy(Fsum, 1:n, sumsize);
% figure; montage(imagescolor, 'Indices', A, 'Size',[1,10]);
% summaryEvaluation(A, summaries, 'rouge', optionsEvaluationLoss)
% 
% %% Clustering objective
% clear clusters;
% numclusters = 10;
% IDX = kmeans(features, numclusters);
% B = ones(1, 10);
%     for i = 1 : numclusters
%         A = find(IDX == i);
%         minsum = 1000;
%         for j = 1 : length(A)
%             if(sum(sim(A(j), A')) < minsum)
%                 minsum =  sum(sim(A(j), A'));
%                 B(i) = A(j);
%             end
%         end
%     end
%  figure; montage(imagescolor, 'Indices', B, 'Size',[10,1]);