function [ summary, score, gains ] = greedySummary( fct, len )
%GREEDYSUMMARY Computes a summary in a greedy way.
%   The function
%
%      [ summary, score, gains ] = greedySummary( fct, len )
%
%  computes the summary "summary" of length "len" in a greedy way.
%  Candidate summaries are evaluated using the anonymous function "fct".
%  The score "score" of the final summary as well as the gains "gains"
%  obtained in every step are returned. If "len" is not specified, a
%  summary of length 10 is returned.

if nargin < 2,
    len = 10;
end

selected = [];
curScore = 0;
gains = zeros(len,1);
for i = 1:len,
    candidates = setdiff(1:100, selected);
    curgains = NaN(length(candidates), 1);
    for l = 1:length(candidates),
        curgains(l) = fct([selected, candidates(l)]) - curScore;
    end
    [a,b] = max(curgains);
    
    % if multiple components achieve same gain. select at random
    idx = find(curgains == a);
    if length(idx) == 0,
        error('Invalid gains...');
    end
    if length(idx) == 1,
    	b = idx;
    else
        b = idx(ceil(rand(1)*length(idx)));
    end
    
    if length(idx) > 1,
        fprintf(1, 'multiple elements with same gain (%d).\n', length(idx));
    end
    
    curScore = curScore + a;
    gains(i) = a;
    selected = horzcat(selected, candidates(b));
end

summary = selected;
score = curScore;

end

