function [ yht ] = inference(type, yt, ds, weights, scoreFunctionsJava, laiFunctionsJava, lossesJava, rouge, options)
%INFERENCE Summary of this function goes here
%   Detailed explanation goes here

if nargin < 9,
    options = struct();
    options.initializeWithGreedy = false;
end

lenOutput = 10;
sampleScore = rouge{ds}(yt);

switch lower(type)
    case 'greedy' % use greedy algorithm
        % perform greedy algorithm
        % Update weights of score function
        laiJava = laiFunctionsJava{ds};
        laiJava.setWeights([weights; 1]);

        % perform inference
        greedy = GreedyJ(laiJava);
        yht = java2matArray(greedy.maximizeadv(100, lenOutput));
    case 'subsupplus' % sub sup with negative weights
        scoreFunctionJava = scoreFunctionsJava{ds};
        scoreFunctionJava.setWeights(weights);
        laiJava = laiFunctionsJava{ds};
        laiJava.setWeights([weights; 1]);

        posFunctions = LinCombFunctionJ();
        negFunctions = LinCombFunctionJ();
        posIndices = weights >= 0;
        negIndices = weights < 0;
        for f = 1:length(weights),
            if weights(f) >= 0,
                posFunctions.addFunction(fctsJava{ds,f});
            else
                negFunctions.addFunction(fctsJava{ds,f});
            end
        end
        posFunctions.setWeights(weights(posIndices));
        if isempty(weights(negIndices)),
            negFunctions.setWeights([]);
        else
            negFunctions.setWeights(-weights(negIndices));
        end
        fprintf(1, '#pos: %d / #neg: %d\n', sum(posIndices), sum(negIndices));

        sampleScore = rouge{ds}(yt);

        % perform inference
        greedy = GreedyJ(laiJava);
        yinit = java2matArray(greedy.maximizeadv(100, lenOutput));
        % keyboard
        yht = sfo_supsubimproveduc(sfo_fn_wrapper(@(S) posFunctions.eval(S)), sfo_fn_wrapper(@(S) negFunctions.eval(S) + min(1, (1-lossesJava{ds}.eval(S))/sampleScore)-1), 1:100, 1, yinit);

        c = scoreFunctionJava.eval(yinit);
        d = -(min(1, (1-lossesJava{ds}.eval(yinit))/sampleScore)-1);
        fprintf(1, 'GREEDY MIXTURE: %f // LOSS: %f => SUM: %f\n', c, d, c+d);
    case 'modmod' % use submodular-supermodular procedure
        % INITIALIZE
        laiJava = laiFunctionsJava{ds};
        laiJava.setWeights([weights; 1]);

        % perform inference
        if options.initializeWithGreedy,
            greedy = GreedyJ(laiJava);
            yinit = java2matArray(greedy.maximizeadv(100, lenOutput));
        else
            yinit = [];
        end

        tic;
        scoreFunctionJava = scoreFunctionsJava{ds};
        scoreFunctionJava.setWeights(weights);
        % yht = sfo_modmodimprovedv2(sfo_fn_wrapper(@(S) scoreFunctionJava.eval(S)), sfo_fn_wrapper(@(S) -lossesJava{ds}.eval(S)), 1:100, 10);
        % yht = sfo_modmodimproveduc(sfo_fn_wrapper(@(S) scoreFunctionJava.eval(S)), sfo_fn_wrapper(@(S) -lossesJava{ds}.eval(S)), 1:100);

        %keyboard
        % fprintf(1, 'USING SUBSUP ONLY...\n');
        constV = lossesJava{ds}.eval(yt);
        sampleScore = rouge{ds}(yt);
        %loss = @(S) min(1, (1-lossesJava{ds}.eval(S))/sampleScore)-1;
        loss = @(S) -lossesJava{ds}.eval(S);
        yht = sfo_supsubimproveduc(sfo_fn_wrapper(@(S) scoreFunctionJava.eval(S)), sfo_fn_wrapper(loss), 1:100, 1, yinit);
        %yht = sfo_subsupinterface(sfo_fn_wrapper(@(S) scoreFunctionJava.eval(S)), sfo_fn_wrapper(@(S) -lossesJava{ds}.eval(S)), 1:100, scoreFunctionJava, lossesJava{ds});

        % stats on summary lengths
        %lstats(end+1) = length(yht)
        %[aaa,bbb] = hist(lstats, 1:20)

        % keyboard

        %yht = sfo_subsupimproveduc(sfo_fn_wrapper(@(S) scoreFunctionJava.eval(S)), sfo_fn_wrapper(@(S) -20*max(0,lossesJava{ds}.eval(S)-constV)), 1:100);
        % yht = sfo_subsupimproveduc(sfo_fn_wrapper(@(S) scoreFunctionJava.eval(S)), sfo_fn_wrapper(@(S) -lossesJava{ds}.eval(S)), 1:100);
        lossesJavaTmp = LinCombFunctionJ();
        lossesJavaTmp.addFunction(lossesJava{ds});
        lossesJavaTmp.setWeights(-1); 
        %yht = sfo_supsubimproveducNew(sfo_fn_wrapper(@(S) scoreFunctionJava.eval(S)), sfo_fn_wrapper(@(S) -lossesJava{ds}.eval(S)), 1:100, 2, scoreFunctionJava, lossesJavaTmp, yinit);
        ttt = toc;
        fprintf(1, 'JAVA MODMOD: %f seconds\n', ttt);

        %fprintf(1, 'Score greedy: %1.4f\n', lai(yinit));
        %fprintf(1, 'Score modmod: %1.4f\n', lai(yht));
        c = scoreFunctionJava.eval(yinit);
        d = loss(yinit);

        fprintf(1, 'GREEDY MIXTURE: %f // LOSS: %f => SUM: %f\n', c, d, c+d);
    otherwise
        error('Invalid method for inference.');
end

end

