function [results] = loadResultsFile(filename),

results = struct();

fh = fopen(filename, 'r');

%% number of components
expected = 'Number of components';
line = fgetl(fh);
if strcmp(line, expected) ~= 1,
    fprintf(1, 'Expected: %s\nRead: %s\n', expected, line);
    error('Did not read expected content.');
end
line = fgetl(fh);
results.nrComponents = str2num(line);
line = fgetl(fh);

%% final weights
expected = 'Final weights';
line = fgetl(fh);
if strcmp(line, expected) ~= 1,
    fprintf(1, 'Expected: %s\nRead: %s\n', expected, line);
    error('Did not read expected content.');
end
line = fgetl(fh);
[results.finalWeights, count] = sscanf(line, '%f');
if count ~= results.nrComponents,
    error('Could not read expected number of numbers.');
end
line = fgetl(fh);

%% average weights
expected = 'Average weights';
line = fgetl(fh);
if strcmp(line, expected) ~= 1,
    fprintf(1, 'Expected: %s\nRead: %s\n', expected, line);
    error('Did not read expected content.');
end
line = fgetl(fh);
[results.averageWeights, count] = sscanf(line, '%f');
if count ~= results.nrComponents,
    error('Could not read expected number of numbers.');
end
line = fgetl(fh);

%% all weights
expected = 'All weights';
line = fgetl(fh);
if strcmp(line, expected) ~= 1,
    fprintf(1, 'Expected: %s\nRead: %s\n', expected, line);
    error('Did not read expected content.');
end
line = fgetl(fh);
nrWeights = str2num(line);
allWeights = zeros(results.nrComponents,nrWeights);
for i = 1:nrWeights,
    line = fgetl(fh);
    [allWeights(:,i), count] = sscanf(line, '%f');
    if count ~= results.nrComponents,
        error('Could not read expected number of numbers.');
    end
end
results.allWeights = allWeights;
line = fgetl(fh);

%% greedy max final weights
expected = 'Greedy Max Final Weights';
line = fgetl(fh);
if strcmp(line, expected) ~= 1,
    fprintf(1, 'Expected: %s\nRead: %s\n', expected, line);
    error('Did not read expected content.');
end
line = fgetl(fh);
[results.greedyMaxFinalWeights, count] = sscanf(line, '%d');
if count ~= 10,
    error('Could not read expected number of numbers.');
end
results.greedyMaxFinalWeights = results.greedyMaxFinalWeights + 1;
line = fgetl(fh);

%% greedy max average weights
expected = 'Greedy Max Average Weights';
line = fgetl(fh);
if strcmp(line, expected) ~= 1,
    fprintf(1, 'Expected: %s\nRead: %s\n', expected, line);
    error('Did not read expected content.');
end
line = fgetl(fh);
[results.greedyMaxAverageWeights, count] = sscanf(line, '%d');
if count ~= 10,
    error('Could not read expected number of numbers.');
end
results.greedyMaxAverageWeights = results.greedyMaxAverageWeights + 1;
line = fgetl(fh);

%% config
expected = 'Config';
line = fgetl(fh);
if strcmp(line, expected) ~= 1,
    fprintf(1, 'Expected: %s\nRead: %s\n', expected, line);
    error('Did not read expected content.');
end
config = '';
while 1,
    line = fgets(fh);
    if ~ischar(line),
        break;
    end
    config = [config, line];
end
results.config = config;


fclose(fh);

end