function [ varargout ] = load_resource(type, desc, ds, dry)
%LOAD_RESOURCE Summary of this function goes here
%   Detailed explanation goes here
%  dry ... only return filename

if nargin < 4,
    dry = 0;
end

verbose = 0;

switch lower(type),
    case 'similarity'
        if verbose,
            fprintf(1, 'Loading similiarity: %s\n', desc); 
        end
        fnameSIM = fullfile(getPreDir(), 'resources', 'similarity', desc, sprintf('%d.mat', ds));
        if dry,
            varargout{1} = fnameSIM;
            return;
        end
        
        load(fnameSIM, 'sim');
        
        if ~exist('sim', 'var'),
            error('File "%s" does not contain variable "sim".\n', fnameSIM);
        end
        
        varargout{1} = sim;
    case 'bow'
        if verbose,
            fprintf(1, 'Loading bag-of-words: %s\n', desc); 
        end
        fnameBOW = fullfile(getPreDir(), 'resources', 'bow', desc, sprintf('%d.mat', ds));
        if dry,
            varargout{1} = fnameBOW;
            return;
        end
        
        load(fnameBOW, 'bow');
        
        if ~exist('bow', 'var'),
            error('File "%s" does not contain variable "bow".\n', fnameBOW);
        end
        
        varargout{1} = bow;
    case 'qualitymeasure'
        if verbose,
            fprintf(1, 'Loading quality measure: %s\n', desc); 
        end
        fnameQUALITY = fullfile(getPreDir(), 'resources', 'quality', desc, sprintf('%d.mat', ds));
        if dry,
            varargout{1} = fnameQUALITY;
            return;
        end
        
        load(fnameQUALITY, 'quality');
        
        if ~exist('quality', 'var'),
            error('File "%s" does not contain variable "qualtiy".\n', fnameQUALITY);
        end
        
        varargout{1} = quality;
    case 'cluster'
        if verbose,
            fprintf(1, 'Loading cluster: %s\n', desc); 
        end
        fnameCLUSTER = fullfile(getPreDir(), 'resources', 'cluster', desc, sprintf('%d.mat', ds));
        if dry,
            varargout{1} = fnameCLUSTER;
            return;
        end
        
        load(fnameCLUSTER, 'cluster');
        
        if ~exist('cluster', 'var'),
            error('File "%s" does not contain variable "cluster".\n', fnameCLUSTER);
        end
        
        varargout{1} = cluster;
    case 'encodingcosts'
        if verbose,
            fprintf(1, 'Loading encoding costs: %s\n', desc); 
        end
        fnameENCCOSTS = fullfile(getPreDir(), 'resources', 'encodingCosts', desc, sprintf('%d.mat', ds));
        if dry,
            varargout{1} = fnameENCCOSTS;
            return;
        end
        
        load(fnameENCCOSTS, 'encodingCosts');
        
        if ~exist('encodingCosts', 'var'),
            error('File "%s" does not contain variable "encodingCosts".\n', fnameENCCOSTS);
        end
        
        varargout{1} = encodingCosts;
    case 'value'
    	val = str2num(desc);
    	varargout{1} = val;
    otherwise
        error('Invalid resource type.');
end


end

