function [BoW, sim] = get_BOW2(imagescolor, images, numberOfWords, featureType, options)
% Returns a bag of visual word representation for every image. Currently,
% supported featureTypes are 'sift', 'denseSift', 'phow', ...

%% settings
histBins = 16;
histPatchSize = 8; % patch Size for color histogram

if nargin < 5,
    if strcmp(featureType, 'dsift'),
        options = {'Step', 2, 'Size', 4};
    elseif strcmp(featureType, 'phow'),
        % options = {'Step', 2};
        options = {'Sizes', [2,4,8,16], 'Step',2, 'Color', 'gray'};
    else
        options = {};
    end
end

%% retrieve gist
clear param
param.imageSize = [256 256]; % set a normalized image size
param.orientationsPerScale = [8 8 8 8]; % number of orientations per scale (from HF to LF)
param.numberBlocks = 4;
param.fc_prefilt = 4;

% Pre-allocate gist:
Nfeatures = sum(param.orientationsPerScale)*param.numberBlocks^2;
gist = zeros([size(images,3) Nfeatures]); 

% Load first image and compute gist:
[gist(1, :), param] = LMgist(imagescolor(:,:,:,1), '', param); % first call
% Loop:
for l = 2:size(images,3)
   gist(l, :) = LMgist(imagescolor(:,:,:,l), '', param); % the next calls will be faster
end

%% general stuff
nrImages = size(images, 3);
dictSize = numberOfWords;

features = {};
frames = {};
nrFeatureVectors = 0;

%% compute sift descriptors for all images
for img = 1:nrImages,
    if strcmp(featureType, 'dsift'),
        [F, features{img}] = vl_dsift(single(images(:,:,img)), options{:}); % vl_sift(single(images(:,:,img))); % vl_dsift(single(images(:,:,img)), 'Step', 4);
        features{img} = single(features{img});
    elseif strcmp(featureType, 'sift'),
        [F, features{img}] = vl_sift(single(images(:,:,img)), options{:}); % vl_sift(single(images(:,:,img))); % vl_dsift(single(images(:,:,img)), 'Step', 4);
        features{img} = single(features{img});
    elseif strcmp(featureType, 'phow'),
        % [F, features{img}] = vl_phow(single(images(:,:,img)), options{:}); % vl_sift(single(images(:,:,img))); % vl_dsift(single(images(:,:,img)), 'Step', 4);
        options = {};
        [F, features{img}] = vl_sift(single(images(:,:,img)), options{:}); % vl_sift(single(images(:,:,img))); % vl_dsift(single(images(:,:,img)), 'Step', 4);
        features{img} = single(features{img});
        frames{img} = F;
        % F is an 4xnrFeatures Matrix, where the first two rows correspond
        % to the x and y coordinate
%         nrFrames = size(F, 2);
%         hsvimage = rgb2hsv(imagescolor(:,:,:,img));
%         imSizeX = size(hsvimage, 1);
%         imSizeY = size(hsvimage, 2);
%         hsvfeat = zeros(histBins, nrFrames);
%         tsize = size(features{img}, 1);
%         for f = 1:nrFrames,
%             x = F(1,f);
%             y = F(2,f);
%             xmin = max(1, x - histPatchSize/2);
%             xmax = min(imSizeX, x + histPatchSize/2);
%             ymin = max(1, y - histPatchSize/2);
%             ymax = min(imSizeY, y + histPatchSize/2);
%             area = (xmax - xmin) * (ymax - ymin);
%             patch = hsvimage(xmin:xmax, ymin:ymax, 1);
%             tmp = hist(patch(:), linspace(0,1,histBins));
%             hsvfeat(:, f) = tmp * histPatchSize*histPatchSize / area * tsize / histBins / 3;
%         end
%         features{img} = vertcat(features{img}, hsvfeat);
%         features{img} = vertcat(features{img}, repmat(gist(img,:)', 1, nrFrames));
    elseif strcmp(featureType, 'surf'),
        Options.upright=true;
        Options.tresh=0.00005;
        Options.init_sample = 1;
        Ipts=OpenSurf(single(images(:,:,img)),Options);
        features{img} = horzcat(Ipts.descriptor);
    else
        error('Unsupported feature type.');
    end
    nrFeatureVectors = nrFeatureVectors + size(features{img}, 2);
end

fprintf(1, 'Computed %d feature descriptors.\n', nrFeatureVectors);

%% concatenate feature vectors and perform dictionary computation 
%% by k-means clustering
featuresM = zeros(size(features{1}, 1), nrFeatureVectors);
idx = 1;
for img = 1:nrImages,
    nrDesc = size(features{img}, 2);
    featuresM(:, idx:(idx+nrDesc-1)) = features{img};
    idx = idx + nrDesc;
end

fprintf(1, 'Performing k-means...\n');
%clust = KMedoids(single(featuresM)', dictSize, {});

dictionary = vl_kmeans(single(featuresM), dictSize, 'verbose', 'algorithm', 'elkan', 'MaxNumIterations', 100);
%perm = randperm(size(featuresM, 2));
%dictionary = featuresM(:,perm(1:dictSize));

% params = struct();
% params.c = dictSize;
% params.vis = false;
% data = struct();
% data.X = single(featuresM');
% result = kmedoid(data, params);
% dictionary = result.cluster.v';

fprintf(1, 'Computing BoWs...\n');
bow = []; %zeros(dictSize, nrImages);
kdtree = vl_kdtreebuild(dictionary);
%meandist = vl_alldist(single(dictionary), single(featuresM));
%meandist = mean(meandist, 2);

model = struct();
model.numSpatialX = [2 4];
model.numSpatialY = [2 4];
numWords = dictSize;

for img = 1:nrImages,
    %dist = vl_alldist(single(dictionary), single(features{img}));
    %[~, binsa] = min(dist, [], 1);
    %[dist,bt] = sort(dist,1);
    %for i = 1:size(dist,2), % first dim: dict size, second dim: number of interest points
    %    if dist(1,i) < 0.5*meandist(bt(i)), % 2*dist(2,i),
    %        bow(bt(1,i), img) = bow(bt(1,i), img) + 1;
    %    end
    %end
    
    % bow(binsa, img) = bow(binsa, img) + 1;
    
    
   
     binsa = double(vl_kdtreequery(kdtree, single(dictionary), ...
                                   single(features{img}), ...
                                   'MaxComparisons', 50)) ;
%      tmp = accumarray(binsa(:), ones(length(binsa), 1));
%      tmp = vertcat(tmp, zeros(dictSize-length(tmp),1));
%      bow(:,img) = tmp;



    hists = {};
    for i = 1:length(model.numSpatialX)
      binsx = vl_binsearch(linspace(1,100,model.numSpatialX(i)+1), frames{img}(1,:)) ;
      binsy = vl_binsearch(linspace(1,100,model.numSpatialY(i)+1), frames{img}(2,:)) ;

      % combined quantization
      bins = sub2ind([model.numSpatialY(i), model.numSpatialX(i), numWords], ...
                     binsy,binsx,binsa) ;
      hist = zeros(model.numSpatialY(i) * model.numSpatialX(i) * numWords, 1) ;
      hist = vl_binsum(hist, ones(size(bins)), bins) ;
      hists{i} = single(hist / sum(hist)) ;
    end
    hist = cat(1,hists{:}) ;
    hist = hist / sum(hist) ;
    bow(:,img ) = hist;
end

BoW = bow;
for k = 1:size(BoW, 2),
    BoW(:,k) = BoW(:,k);% / norm(BoW(:,k), 1); %size(features{k},2);% norm(BoW(:,k), 1);
end

% options = zeros(1,14);
% options(1) = 1; % display
% options(2) = 1;
% options(3) = 0.1; % precision
% options(5) = 1; % initialization
% options(14) = 100; % maximum iterations
% 
% centers = zeros(dictSize, size(sift, 1));
% 
% % run kmeans
% fprintf('\nRunning k-means\n');
% dictionary = sp_kmeans(centers, sift', options);

% %% compute BOWs
% % perform vector quantization
% disp('Performing vector quantization...');
% sift = zeros(dictSize, nrImages);
% dictionary = dictionary';
% for i = 1:nrImages,
%     for j = 1:size(sift_all{i}, 2),
%         tmp = zeros(dictSize, 1);
%         for k = 1:size(dictionary, 1),
%             tmp(k) = norm(dictionary(k,:) - single(sift_all{i}(:,j))');
%         end
%         %[~,idx] = min(tmp);
%         %sift(idx, i) = sift(idx, i) + 1;
%         [a,b] = sort(tmp);
%         idx = b(1);
%         sift(idx, i) = sift(idx, i) + 1;
%     end
% end

for i = 1:size(bow, 2),
    bow(:,i) = bow(:,i) / norm(bow(:,i));
end

%% Get the similarity matrix
sim = NaN(nrImages, nrImages);
for i = 1 : nrImages,
    for j = 1 : nrImages,
        sim(i, j) = bow(:,i)'*bow(:,j);
    end
end

if any(any(isnan(sim))),
    error('There are NaNs in the similarity matrix.');
end

imagesc(sim);

end