function [BoW,FDESC] = get_BOW4(imagescolor, images, numberOfWords, featureType, encoder, options)
% Returns a bag of visual word representation for every image. Currently,
% supported featureTypes are 'sift', 'denseSift', 'phow', ...

%% settings
clustering = 'random';
% clustering = 'kmeans';
if nargin < 6,
    if strcmp(featureType, 'dsift'),
        options = {'Step', 2, 'Size', 4};
    elseif strcmp(featureType, 'phow'),
        % options = {'Step', 2};
        options = {'Sizes', [8,16,32], 'Step',2, 'Color', 'gray'};
    else
        options = {};
    end
end

%% general stuff
nrImages = size(images, 3);
dictSize = numberOfWords;

desc = [];;
features = {};
nrFeatureVectors = 0;

%% compute sift descriptors for all images
for img = 1:nrImages,
    if strcmp(featureType, 'dsift'),
        [F, features{img}] = vl_dsift(single(images(:,:,img)), options{:}); % vl_sift(single(images(:,:,img))); % vl_dsift(single(images(:,:,img)), 'Step', 4);
        features{img} = single(features{img});
    elseif strcmp(featureType, 'sift'),
        [F, features{img}] = vl_sift(single(images(:,:,img)), options{:}); % vl_sift(single(images(:,:,img))); % vl_dsift(single(images(:,:,img)), 'Step', 4);
        features{img} = single(features{img});
    elseif strcmp(featureType, 'phow'),
        [F, features{img}] = vl_phow(single(images(:,:,img)), options{:}); % vl_sift(single(images(:,:,img))); % vl_dsift(single(images(:,:,img)), 'Step', 4);
        features{img} = single(features{img});
        desc = horzcat(desc, F);
    elseif strcmp(featureType, 'surf'),
        Options.upright=true;
        Options.tresh=0.00005;
        Options.init_sample = 1;
        Ipts=OpenSurf(single(images(:,:,img)),Options);
        features{img} = horzcat(Ipts.descriptor);
    else
        error('Unsupported feature type.');
    end
    nrFeatureVectors = nrFeatureVectors + size(features{img}, 2);
end

% normalize sift
for img = 1:nrImages,
    ft = features{img};
    for k = 1:size(ft,2),
        ft(:,k) = ft(:,k) / norm(ft(:,k));
    end
    features{img} = ft;
end

fprintf(1, 'Computed %d feature descriptors.\n', nrFeatureVectors);

%% concatenate feature vectors and perform dictionary computation 
%% by k-means clustering
featuresM = zeros(size(features{1}, 1), nrFeatureVectors);
idx = 1;
for img = 1:nrImages,
    nrDesc = size(features{img}, 2);
    featuresM(:, idx:(idx+nrDesc-1)) = features{img};
    idx = idx + nrDesc;
end
featuresM = single(featuresM);

if strcmp(clustering, 'kmeans'),
    fprintf(1, 'Performing k-means...\n');
    dictionary = vl_kmeans(single(featuresM), dictSize, 'verbose', 'algorithm', 'elkan', 'MaxNumIterations', 100);
    fvecids = 1:dictSize; % not true, just for making the code run
elseif strcmp(clustering, 'random'),
    perm = randperm(size(featuresM,2));
    dictionary = featuresM(:,perm(1:dictSize));
    fvecids = sort(perm(1:dictSize));
else
    error('Invalid clustering.');
end
    
% set FDESC
FDESC = cell(dictSize, 1);
for i =  1:length(fvecids),
    data.sift = featuresM(:,fvecids(i));
    data.desc = desc(:,fvecids(i));
    data.img = ceil(fvecids(i) / size(features{1}, 2));
    FDESC{i} = data;
end

%% compute bow
bow = [];
for img = 1:nrImages,
    bow(:,img) = encode(single(features{img}), dictionary, encoder, 0.9);
end

% normalize
%for img = 1:nrImages,
%    bow(:,img) = bow(:,img) / norm(bow(:,img));
%end

BoW = bow;
