% A script to get the bag of words descriptors of a collection of images...

% Example of how to use the BuildPyramid function
% set image_dir and data_dir to your actual directories
image_dir = '../Data/set2'; 
data_dir = sprintf('%s/data', image_dir);
addpath Spatial' Pyramid'/
% for other parameters, see BuildPyramid

fnames = dir(fullfile(image_dir, '*.jpg'));
num_files = size(fnames,1);
filenames = cell(num_files,1);

for f = 1:num_files
	filenames{f} = sprintf('%d.jpg', f);
end

% Sebastian, 2013-09-18
nrImages = size(filenames, 1);
params = struct();
params.dictionarySize = nrImages*10;
params.pyramidLevels = 3;
params.gridSpacing = 1;
params.numTextonImages = nrImages;
params.saveSift = 1;
params.canSkip = 0;
params.patchSize = 8;

% return pyramid descriptors for all files in filenames
[pyramid_all dictionary histogram] = BuildPyramid(filenames,image_dir,data_dir, params);

% Sebastian, 2013-09-19
% create dictionary by picking some sift clusters from every image
disp('Creating dictionary...');
nrPerPicture = params.dictionarySize / nrImages;
dictionary = [];
for i = 1:nrImages,
    load(sprintf('../Data/set2/data/%d_sift.mat', i));
    sift = features.data;
    
    %% perform clustering
    options = zeros(1,14);
    options(1) = 1; % display
    options(2) = 1;
    options(3) = 0.1; % precision
    options(5) = 1; % initialization
    options(14) = 100; % maximum iterations

    centers = zeros(nrPerPicture, size(sift,2));

    %% run kmeans
    fprintf('\nRunning k-means\n');
    ldictionary = sp_kmeans(centers, sift, options);
    dictionary = vertcat(dictionary, ldictionary);
end

% Sebastian, 2013-09-18
% perform vector quantization
disp('Performing vector quantization...');
sift = zeros(params.dictionarySize, nrImages);
for i = 1:nrImages,
    load(sprintf('../Data/set2/data/%d_sift.mat', i));
    for j = 1:size(features.data, 1),
        tmp = zeros(params.dictionarySize, 1);
        for k = 1:size(dictionary, 1),
            tmp(k) = norm(dictionary(k,:) - features.data(j,:));
        end
        %[~,idx] = min(tmp);
        %sift(idx, i) = sift(idx, i) + 1;
        [a,b] = sort(tmp);
        if a(1)*1.2 < a(2), % good match for first entry
            idx = b(1);
            sift(idx, i) = sift(idx, i) + 1;
        end
    end
end
for i = 1:size(sift, 2),
    sift(:,i) = sift(:,i) / norm(sift(:,i));
end

%% Get the similarity matrix
for i = 1 : nrImages,
    for j = 1 : nrImages,
        sim(i, j) = sift(:,i)'*sift(:,j);
    end
end
%return

% compute histogram intersection kernel
sim = hist_isect(pyramid_all, pyramid_all); 

% for faster performance, compile and use hist_isect_c:
% K = hist_isect_c(pyramid_all, pyramid_all);