% select sift centers in a greedy manner such that the resulting BoW
% reprsentations yiel good scores

%% settings
dictSize = 200;
dataset = 2;
evaluation = 'bow';
nrAverage = 1000;
nrImages = size(images, 3);
nrMaxAccept = 1;
summarySize = 10;

%% (1) compute SIFT features
% options = {'Step', 16, 'Size', 8};
options = {'Sizes', [4,8,16], 'Step', 4, 'Color', 'gray'};
nrFeatureVectors = 0;
frames = [];
for img = 1:nrImages,
    [F, features{img}] = vl_phow(single(images(:,:,img)), options{:}); % vl_sift(single(images(:,:,img))); % vl_dsift(single(images(:,:,img)), 'Step', 4);
    features{img} = single(features{img});
    nrFeatureVectors = nrFeatureVectors + size(features{img}, 2);
    frames = horzcat(frames, F);
end
featuresM = zeros(size(features{1}, 1), nrFeatureVectors);
idx = 1;
for img = 1:nrImages,
    nrDesc = size(features{img}, 2);
    featuresM(:, idx:(idx+nrDesc-1)) = features{img};
    idx = idx + nrDesc;
end
featuresM = single(featuresM);

% normalize features
for k = 1:size(featuresM,2),
    featuresM(:,k) = featuresM(:,k) / norm(featuresM(:,k));
end

fprintf(1, 'Computed %d feature descriptors.\n', nrFeatureVectors);

%% (2) load summaries
summaryfile = sprintf('../Data/set%d/summary.txt', dataset);
summary = load(summaryfile);
origSummary = summary;

%% (3) select SIFT centers in a greedy manner
centers = [];
centers(end+1) = 1;

% compute initial bow
dictionary = featuresM(:,centers);
tmp = dictionary' * featuresM;
alphaThreshold = 0.75;
tmp = tmp - alphaThreshold;
tmp = max(zeros(size(tmp)), tmp);

idx = 1;
bow2 = [];
for ii = 1:nrImages,
    bow2(:,ii) = max(tmp(:,idx:(idx+size(features{ii},2)-1)), [], 2);
    idx = idx + size(features{ii},2);
end
bowContainer = bow2;

imgIdx = 1;
while length(centers) < dictSize,
    st = (imgIdx-1) * size(features{img}, 2) + 1;
    en = imgIdx * size(features{img}, 2);
    % candidates = setdiff(1:nrFeatureVectors, centers);
    candidates = setdiff(st:en, centers);
    imgIdx = imgIdx + 1;
    disp(sprintf('Iteration number %d (%d candidates)', dataset, length(candidates)));
    
    scores = zeros(length(candidates), 1);
    randscores = zeros(length(candidates), 1);
    centerLists = horzcat(repmat(centers, length(candidates), 1), candidates(:));
    for i = 1:length(candidates),
        % current set of centers
        %tmpCenters = horzcat(centers, candidates(i));
        tmpCenters = centerLists(i,:);
        
        % compute BoW representation
        dictionary = featuresM(:,centerLists(i,:));

        %fprintf(1, 'Computing BoWs...\n');
%         bow = zeros(length(tmpCenters), nrImages);
%         tdist = vl_alldist(single(dictionary(:,end)), featuresM);
%         me = mean(tdist);
%         mi = min(tdist);
%         th = mean([me, mi]);
%         bow = zeros(nrImages, 1);
        
        % use soft threshold for encoding
        tmp = dictionary' * featuresM;
        tmp = tmp - alphaThreshold;
        tmp = max(zeros(size(tmp)), tmp);
        
        idx = 1;
        bow2 = [];
        for ii = 1:nrImages,
            bow2(:,ii) = max(tmp(:,idx:(idx+size(features{ii},2)-1)), [], 2);
            idx = idx + size(features{ii},2);
        end
        bow = bow2;
        
%         count = 0;
%         for img = 1:nrImages,
%             dist = vl_alldist(single(dictionary(:,end)), single(features{img}));
%             bow(img) = sum(dist < th);
%         end
        % bow = vertcat(bowContainer, bow(:)');
        
%         for img = 1:nrImages,
%             dist = vl_alldist(single(dictionary), features{img});
%             % [~, binsa] = min(dist, [], 1);
%             [dist,bt] = sort(dist,1);
%             for l = 1:size(dist,2),
%                 if dist(1,l)*1.5 < dist(2,l),
%                     bow(bt(1,l), img) = bow(bt(1,l), img) + 1;
%                 end
%             end
% %             tmp = accumarray(binsa(:), ones(length(binsa), 1));
% %             tmp = vertcat(tmp, zeros(length(tmpCenters)-length(tmp),1));
% %             bow(:,img) = tmp;
%         end
        
        tmpSummary = NaN(size(origSummary, 1), length(tmpCenters));
        for k = 1:size(origSummary, 1),
            tmpSummary(k, :) = max(bow(:, origSummary(k,:)), [], 2)';
        end
        summary = tmpSummary;

        if any(any(isnan(summary))),
            error('Should never happen.');
        end

        %% Actual Summary
        %fprintf(1, 'Testing human summaries...\n');
        truescore = 0;
        for summaryno = 1 : size(summary, 1)
            currsummary = summary(summaryno, :);
            currsummarylist = [summary(1:summaryno - 1, :); summary(summaryno+1:size(summary,1), :)];
            addscore = summaryEvaluation(currsummary, currsummarylist, evaluation, k)*100;
            truescore = truescore + addscore;
        end
        scores(i) = truescore/size(summary,1);
        
        %% compute random score
        randscore = 0;
        for times = 1 : nrAverage,
            rperm = randperm(nrImages);
            rset = rperm(1:summarySize);
            if strcmp(evaluation, 'bow'),
                rset = max(bow(:,rset), [], 2)';
            end
            addscore = summaryEvaluation(rset, summary, evaluation, k)*100;
            randscore = randscore + addscore;
        end
        randscore =  randscore/nrAverage;
        
        % scores shall be discriminative, i.e. we penalize by the number of
        % nonzero BoW entries
        gamma = 5; % weight.
        scores(i) = -gamma*(sum(sum(bow > 0)));
        
        % pay if dictionaries show the same content
        addscore = 0;
        foo = nchoosek(1:size(summary,1), 2);
        for k = 1:size(foo, 1),
            cmp = foo(k,:);
            addscore = addscore + norm(summary(cmp(1),:) - summary(cmp(2),:));
        end
        scores(i) = scores(i) - addscore;
        
        %scores(i) = scores(i) - 0.5*randscore;

        %% output progress
        if mod(i, 100) == 0,
            fprintf(1, 'Round %d/%d...\n', i, length(candidates));
            drawnow;
        end
    end
    
%     % select best scorer
%     [a,b] = max(scores);
%     fprintf(1, 'Feature %d (candidate %d) has best score of %f (random score is %f; ratio %f).\n', candidates(b), b, a, randscores(b), a / randscores(b));
%     centers = horzcat(centers, candidates(b));

    %% select nrMaxAccept best scorers
    [a,b] = sort(scores, 1, 'descend'); 
%     if max(scores) < 0,
%         fprintf(1, 'Rejected to accept any feature (all are not robust).\n');
%         continue;
%     end
    sim = featuresM(:,candidates(b))' * featuresM(:,candidates(b));
    centers = horzcat(centers, candidates(b(1)));
    centers
    fprintf(1, 'Accepted %d\n', candidates(b(1)));
    lastcenter = 1;
    for k = 2:nrMaxAccept,
        % find local minimum in similarity matrix
        abort = false;
        for l = (lastcenter+1):size(sim,1),
            if l > size(sim,1) || (l+1) > size(sim,1),
                abort = true;
                break;
            end
            if sim(lastcenter, l)  < sim(lastcenter, l+1),
                break;
            end
        end
        if abort,
            break;
        end
        
        % fprintf(1, 'Feature %d (candidate %d) has best score of %f (random score is %f; ratio %f).\n', candidates(b), b, a, randscores(b), a / randscores(b));
        centers = horzcat(centers, candidates(b(l)));
        lastcenter = l;
        fprintf(1, 'Accepted %d.\n', lastcenter);
    end
    
    %% recompute score
    % compute BoW representation
    dictionary = featuresM(:,centers);
    
    tmp = dictionary' * featuresM;
    tmp = tmp - alphaThreshold;
    tmp = max(zeros(size(tmp)), tmp);
    
    idx = 1;
    bow2 = [];
    for ii = 1:nrImages,
        bow2(:,ii) = max(tmp(:,idx:(idx+size(features{ii},2)-1)), [], 2);
        idx = idx + size(features{ii},2);
    end
    bow = bow2;
    
    bowContainer = bow;
    
    disp('BoW entries sparsity...');
    sum(sum(bowContainer > 0)) / prod(size(bowContainer))

%     %fprintf(1, 'Computing BoWs...\n');
%     bow = zeros(length(centers), nrImages);
%     kdtree = vl_kdtreebuild(dictionary);
%     for img = 1:nrImages,
%         dist = vl_alldist(single(dictionary), single(features{img}));
%         % [~, binsa] = min(dist, [], 1);
%         [dist,bt] = sort(dist,1);
%         for l = 1:size(dist,2),
%             if dist(1,l)*1.5 < dist(2,l),s
%                 bow(bt(1,l), img) = bow(bt(1,l), img) + 1;
%             end
%         end
% %             tmp = accumarray(binsa(:), ones(length(binsa), 1));
% %             tmp = vertcat(tmp, zeros(length(tmpCenters)-length(tmp),1));
% %             bow(:,img) = tmp;
%     end

    tmpSummary = NaN(size(origSummary, 1), length(centers));
    for k = 1:size(origSummary, 1),
        tmpSummary(k, :) = max(bow(:, origSummary(k,:)), [], 2)';
    end
    summary = tmpSummary;

    if any(any(isnan(summary))),
        error('Should never happen.');
    end

    %% Actual Summary
    %fprintf(1, 'Testing human summaries...\n');
    truescore = 0;
    for summaryno = 1 : size(summary, 1)
        currsummary = summary(summaryno, :);
        currsummarylist = [summary(1:summaryno - 1, :); summary(summaryno+1:size(summary,1), :)];
        addscore = summaryEvaluation(currsummary, currsummarylist, evaluation, k)*100;
        truescore = truescore + addscore;
    end
    score = truescore/size(summary,1);

    %% compute random score
    randscore = 0;
    for times = 1 : nrAverage,
        rperm = randperm(nrImages);
        rset = rperm(1:summarySize);
        if strcmp(evaluation, 'bow'),
            rset = max(bow(:,rset), [], 2)';
        end
        addscore = summaryEvaluation(rset, summary, evaluation, k)*100;
        randscore = randscore + addscore;
    end
    randscore =  randscore/nrAverage;
    
    randscore
    score
    score / randscore
    
    %%
    subplot 211;
    imagesc(bow'*bow);
    
    subplot 212;
    image(imagescolor(:,:,:,imgIdx))
    % draw selected sift features of this figure
    si = intersect(st:en, centers);
    h1 = vl_plotframe(frames(:,si)) ;
    set(h1,'color','y','linewidth',2) ;
    drawnow;
    
    if imgIdx == nrImages,
        imgIdx = 1;
    end
end
    