function [ weights, allWeights, scoreMax, sc ] = mm_subgrad( fcts, loss, input, output, lambda, testInput, testOutput, rouge, desc)
%MM_SUBGRAD Summary of this function goes here
%   Detailed explanation goes here

options = struct();
options.verbose = true;
% options.lambdaLearningRate = 1e1;
options.lambdaLearningRate = length(fcts) / length(fcts) * sqrt((2*(1+size(input,1)))/size(input,1));
options.lambdaLearningRate


%% sanity checks
if length(fcts) < 1,
    error('At least one function must be given.');
end

if size(input,1) ~= size(output,1),
    error('Input and output must have same number of samples');
end

%% extract infos
nrComponents = length(fcts);
nrSamples = size(input, 1);


if options.verbose,
    fprintf(1, 'Performing training with %d components.\n', nrComponents);
end

%% initialize weights
currWeights = zeros(nrComponents, 1);

%% cycle through samples
allWeights = NaN(nrComponents, nrSamples);
eta = 1;
useLazyA = true;
for t = 1:nrSamples,
    fprintf(1, '>> Performing iteration %d/%d.\n', t, nrSamples);
    
    lenOutput = size(output,2);
    xt = input(t,:);
    yt = output(t,:);
    
    if t > 1,
        useLazy = useLazyA;
    else
        useLazy = false;
    end
    
%     %% create overall anonymous function
%     str = 'currWeights(1)*fcts{1}(x)';
%     for f = 2:length(fcts),
%         str = strcat(str, sprintf(' + currWeights(%d)*fcts{%d}(x)', f, f));
%     end
%     str = strcat(str, ' + loss(x)');
%     eval(sprintf('lai = @(x) %s;', str));
%     if options.verbose,
%         fprintf(1, 'Current function used for optimization:\n');
%         lai
%     end
    %% create overall anonymous function
    tfcts = fcts;
    tfcts{end+1} = loss;
    eval(sprintf('lai = %s;', constructScoreFunction([currWeights; 1], 'tfcts')));
    if options.verbose,
        fprintf(1, 'Current function used for optimization:\n');
        lai
    end
    
    %% compute learning rate
    eta = 2/(options.lambdaLearningRate*t);
    if options.verbose,
        fprintf(1, 'Current learning rate is %f.\n', eta);
    end
    
    %% perform LAI
    if ~useLazy,
        [yht, score, gains] = greedySummary(lai, lenOutput);
    else
        [yht, scores] = sfo_greedy_lazy(sfo_fn_wrapper(@(x) lai(x)), 1:100, 10);
    end
    if length(yht) ~= 10,
        fprintf(1, 'LAI didn''t return length 10 summary.');
        [yht, score, gains] = greedySummary(lai, lenOutput);
    end
    yht = sort(yht);
    fprintf(1, 'Strongest competitor...');
    yht
    %yht
    %score
    
    %% compute subgradient
    grad = lambda * currWeights;
    for f = 1:length(fcts),
        grad(f) = grad(f) + fcts{f}(yht) - fcts{f}(yt);
    end
    %grad
    
    %% perform weight updates with projection
    oWeights = currWeights;
    currWeights = max(zeros(size(currWeights)), currWeights - eta*grad);
    lambda
    for k = 1:length(currWeights),
        fprintf(1, '[%3d] val=%1.4f (grad=%1.4f; reg=%1.4f): %s\n', k, currWeights(k), grad(k), lambda*oWeights(k), desc{k});
    end
    
    %% store weights
    allWeights(:,t) = currWeights;
    
    %% score fct
    tWeights = mean(allWeights(:,1:t), 2);
    eval(sprintf('scoreFct = %s;', constructScoreFunction(tWeights, 'fcts')));
    [yht, scores] = sfo_greedy_lazy(sfo_fn_wrapper(@(x) scoreFct(x)), 1:100, 10);
    if length(yht) < 10,
        [yht, score] = greedySummary(scoreFct, lenOutput);
    end
    fprintf('Current best summary (average pars):');
    yht
    fprintf('Score of current best summary: %f\n', rouge(yht));
    
    eval(sprintf('scoreFct = %s;', constructScoreFunction(currWeights, 'fcts')));
    [yht, scores] = sfo_greedy_lazy(sfo_fn_wrapper(@(x) scoreFct(x)), 1:100, 10);
    if length(yht) < 10,
        [yht, score] = greedySummary(scoreFct, lenOutput);
    end
    fprintf('Current best summary (current pars):');
    yht
    fprintf('Score of current best summary: %f\n', rouge(yht));
    
%     %% compute objective
%     obj = 0;
%     [yht, score, gains] = sfo_greedy_lazy(sfo_fn_wrapper(@(x) lai(x)), 1:100, lenOutput);
%     yht
%     for k = 1:nrSamples,
%         obj = obj + lai(yht);
%         for l = 1:length(fcts),
%             obj = obj - currWeights(l)*fcts{l}(output(k,:));
%         end
%     end
%     obj = obj / nrSamples;
%     obj = obj + lambda/2 * norm(currWeights);
%     
%     fprintf(1, '[%2d] objective = %f\n', t, obj);
    
    %% compute average loss on test data
%     if nargin == 7,
%         avgLoss = 0;
%         for l = 1:size(testInput, 1),
%             avgLoss = avgLoss + loss(testOutput(l,:));
%         end
%         avgLoss = avgLoss / size(testInput, 1);
%         
%         fprintf(1, '*** Average loss: %f\n\n\n', avgLoss);
%     end
end

%% average weights and return
weights = mean(allWeights, 2);

%% determine score maximizer
eval(sprintf('scoreFct = %s;', constructScoreFunction(weights, 'fcts')));

fprintf(1, 'Final result.\n');
[yht, score, gains] = greedySummary(scoreFct, lenOutput);
yht
score
gains
scoreMax = yht;
sc = score;

end

