%clear all;
clc;

addpath('sfo');

%% settings
rand('seed', 123456');
dataset = 10;
perTrain = 0.2; % percentage of samples used for training

% if dataset ~= 2 && dataset ~= 1 && dataset ~= 10 && dataset ~= 5 && dataset ~= 3,
%     error('Need to precompute');
% end
% 
% load(sprintf('%s/precomp_ds_%d.mat', getPreDir(), dataset));

%% load data
[imagesrgb, imagesgray] = get_images(dataset, false);
[summaries, nrSummaries] = load_summaries(dataset);

%% remove 100 worst summaries
nrPrune = 300;
summaries = pruneSummaries(dataset, nrPrune);

%% shuffle summaries and extract training and test set
nrSamples = size(summaries, 1);
nrTrain = ceil(nrSamples*perTrain);
nrTest = nrSamples - nrTrain;

summaries = summaries(randperm(nrSamples), :);
trainingData = summaries(1:nrTrain, :);
testData = summaries((nrTrain+1):end, :);

fprintf(1, 'There are %d samples for training and %d samples for testing.\n', nrTrain, nrTest);

fname = sprintf('%s/precomputed/colorHistogram_ds_%d.mat', getPreDir(), dataset);
if ~exist(fname, 'file'),
    error('Color histogram cluster file "%s" does not exists. Skipping.', fname);
end
%load(fname);
%bowColor = bow;
bowColor = [];

%% get bag-of-words
%bow = get_BOW(dataset);
% bow = vertcat(bow, bowColor);
% bowTFIDF = applyTFIDF(bow, get_TFIDF(bow));

%% setup functions to use
bowTFIDF = applyTFIDF(bow, get_TFIDF(bow));
summariesAsBow = computeBOWSummary(summaries, bow);
optionsEvaluationLoss = struct();
optionsEvaluationLoss.bow = bow;
optionsEvaluationLoss.summariesAsBow = summariesAsBow;
%loss = @(x) 1 - summaryEvaluation(x, summaries, 'rouge', optionsEvaluationLoss);
loss = @(x) rougeLoss(x, bow, computeBOWSummary(trainingData, bow));
% optionsEvaluationLoss.k = 3;
% loss = @(x) 1 - summaryEvaluation(x, summaries, 'combinatorial', optionsEvaluationLoss);

%% SETUP SHELLS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fcts = {};
desc = {};

% for th = 0.25:0.25:1.75,
%     fname = sprintf('%s/precomputed/bow_ds_%d_nr_%d_nnTh_%f.mat', getPreDir(), dataset, 1000, th);
%     load(fname);
%     imagesc(BOWsimilarity(applyTFIDF(bow, get_TFIDF(bow))));
%     pause
% end



%% diversity functions based on gist
clusterSizes = 5:5:10; % 5:5:20
idx = 1;
% bowS = {'bowSIFT', 'bowColorHSV', 'bowColorLAB', 'bowColorRGB', 'bowColorHistogramHSV', 'bowColorHistogramLAB', 'bowColorHistogramRGB'};
% distS = {'sqEuclidean', 'cityblock', 'cosine'};
% for clusterSize = clusterSizes,
%     for nrClusters = 1:5,
%         for d = 1:length(distS),
%             dist = distS{d};
%             for i = 1:length(bowS),
%                 eval(sprintf('bow = %s;', bowS{i}));
% 
%                 % cluster
%                 [clusterAssignment, clusterCenters] = kmeans(bow', clusterSize, 'Distance', dist, 'EmptyAction', 'singleton');
%                 clusterCenters = clusterCenters';
%                 distanceToClusterCenters = vl_alldist(clusterCenters, bow);
%                 
%                 reward = NaN(100, 1);
%                 for img = 1:100,
%                     ass = clusterAssignment(img);
%                     reward(img) = exp(-distanceToClusterCenters(ass, img)/5);
%                 end
% 
%                 for a = 0.25:0.25:0.75,
%                     % add components
%                     eval(sprintf('clusterAssignment%d = clusterAssignment;', idx));
%                     eval(sprintf('reward%d = reward;', idx));
%                     eval(sprintf('a%d = a;', idx));
% 
%                     eval(sprintf('fcts{end+1} = @(x) diversityComponent(x, clusterAssignment%d, reward%d, a%d);', idx, idx, idx));
%                     desc{idx} = sprintf('clustered diversity component, bow=%s, CS=%d, exp=%f', bowS{i}, clusterSize, a);
%                     %fcts{end+1} = @(x) diversityComponent(x, clusterAssignment, dist, a);
%                     idx = idx + 1;
%                 end
% 
%                 %% facility location
%                 eval(sprintf('clusterAssignment%d = clusterAssignment;', idx));
%                 eval(sprintf('reward%d = reward;', idx));
% 
%                 eval(sprintf('fcts{end+1} = @(x) facilityLocation(x, clusterAssignment%d, reward%d);', idx, idx));
%                 desc{idx} = sprintf('facility location component, bow=%s, CS=%d', bowS{i}, clusterSize);
%                 %fcts{end+1} = @(x) diversityComponent(x, clusterAssignment, dist, a);
%                 idx = idx + 1; 
%             end
%         end
%     end
% end

% for clusterSize = clusterSizes,
%     fprintf('Clustersize: %d\n', clusterSize);
% 
%     fname = sprintf('%s/precomputed/gist_cluster_ds_%d_size_%d.mat', getPreDir(), dataset, clusterSize);
%     if ~exist(fname, 'file'),
%         error('GIST cluster file "%s" does not exists. Skipping.', fname);
%     end
% 
%     %% load
%     load(fname);
%     
%     % normalize distances to [0,1]
%     distanceToClusterCenters = distanceToClusterCenters / max(distanceToClusterCenters(:));
%     
%     nrClusterCenters = size(clusterCenters, 2);
%     
%     % compute normalization
%     reward = NaN(100, 1);
%     for img = 1:100,
%         ass = clusterAssignment(img);
%         reward(img) = 1 - distanceToClusterCenters(ass, img);
%     end
%     normalization = diversityComponent(1:100, clusterAssignment, reward, a);
%     
%     eval(sprintf('clusterAssignment%d = clusterAssignment;', idx));
%     eval(sprintf('reward%d = reward;', idx));
%     eval(sprintf('a%d = a;', idx));
%    
%     eval(sprintf('fcts{end+1} = @(x) diversityComponent(x, clusterAssignment%d, reward%d, a%d);', idx, idx, idx));
%     %fcts{end+1} = @(x) diversityComponent(x, clusterAssignment, dist, a);
%     idx = idx + 1;
%     
%     %% facility location
%     eval(sprintf('clusterAssignment%d = clusterAssignment;', idx));
%     eval(sprintf('reward%d = reward;', idx));
%    
%     eval(sprintf('fcts{end+1} = @(x) facilityLocation(x, clusterAssignment%d, reward%d);', idx, idx));
%     %fcts{end+1} = @(x) diversityComponent(x, clusterAssignment, dist, a);
%     idx = idx + 1; 
% end

% for clusterSize = clusterSizes,
%     fprintf('Clustersize: %d\n', clusterSize);
% 
%     fname = sprintf('%s/precomputed/colorHistogram_cluster_ds_%d_size_%d.mat', getPreDir(), dataset, clusterSize);
%     if ~exist(fname, 'file'),
%         error('Color histogram cluster file "%s" does not exists. Skipping.', fname);
%     end
% 
%     %% load
%     load(fname);
%     
%     % normalize distances to [0,1]
%     distanceToClusterCenters = distanceToClusterCenters / max(distanceToClusterCenters(:));
%     
%     nrClusterCenters = size(clusterCenters, 2);
%     
%     % compute normalization
%     reward = NaN(100, 1);
%     for img = 1:100,
%         ass = clusterAssignment(img);
%         reward(img) = 1 - distanceToClusterCenters(ass, img);
%     end
%     normalization = diversityComponent(1:100, clusterAssignment, reward, a);
%     
%     eval(sprintf('clussummariesAsBow = computeBOWSummary(summaries, bow);terAssignment%d = clusterAssignment;', idx));
%     eval(sprintf('reward%d = reward;', idx));
%     eval(sprintf('a%d = a;', idx));
%    
%     eval(sprintf('fcts{end+1} = @(x) diversityComponent(x, clusterAssignment%d, reward%d, a%d);', idx, idx, idx));
%     %fcts{end+1} = @(x) diversityComponent(x, clusterAssignment, dist, a);
%     idx = idx + 1;
%     
%     %% facility location
%     eval(sprintf('clusterAssignment%d = clusterAssignment;', idx));
%     eval(sprintf('reward%d = reward;', idx));
%    
%     eval(sprintf('fcts{end+1} = @(x) facilityLocation(x, clusterAssignment%d, reward%d);', idx, idx));
%     %fcts{end+1} = @(x) diversityComponent(x, clusterAssignment, dist, a);
%     idx = idx + 1; 
% end

% %% similarity based on color words
% bowB = bow;
% fname = sprintf('%s/precomputed/colorHistogram_ds_%d.mat', getPreDir(), dataset);
% load(fname);
% bowColor = bow;
% bow = bowB;
% colsim = BOWsimilarity(bowColor);
colsim = sim;
fcts{end+1} = @(S) sum(max(colsim(:,S),[],2)) / 100;
desc{idx} = 'facility location, sim=bowColor';
idx = idx +1;

fcts{end+1} = @(S) sum_coverage(S, colsim);
desc{idx} = 'sum coverage, sim=bowColor';
idx = idx + 1;

fcts{end+1} = @(S) cut_coverage(S, colsim);
desc{idx} = 'cut coverage, sim=bowColor';
idx = idx + 1;

% fcts{end+1} = @(S) penalty_disparity(S, colsim);
% idx = idx + 1;


for clusterSize = clusterSizes,
    fprintf('Clustersize: %d\n', clusterSize);

%     fname = sprintf('%s/precomputed/bow_cluster_ds_%d_size_%d.mat', getPreDir(), dataset, clusterSize);
%     if ~exist(fname, 'file'),
%         error('Bag-of-words cluster file "%s" does not exists. Skipping.', fname);
%     end
%     [clusterCenters, clusterAssignment] = vl_kmeans(bow, clusterSize);
%     clusterAssignment = clusterAssignment';
%     distanceToClusterCenters = vl_alldist(clusterCenters, bow);
% 
%     %% load
%     %load(fname);
% 
%     % normalize distances to [0,1]
%     distanceToClusterCenters = distanceToClusterCenters / max(distanceToClusterCenters(:));
% 
%     nrClusterCenters = size(clusterCenters, 2);
    nrClusterCenters = 3;
    clusterAssignment = ones(1,100)';
    rewards = zeros(size(clusterAssignment))';
    rewards(35:39) = 1;
    clusterAssignment([35,36]) = 2;
    clusterAssignment([37:39]) = 3;

    % compute normalization
    %reward = ones(size(reward));
    %normalization = diversityComponent(1:100, clusterAssignment, reward, a);

    for aa = 0.25:0.25:0.75,
        a = zeros(nrClusterCenters,1);
%         reward = NaN(100, 1);
%         for img = 1:100,
%             ass = clusterAssignment(img);
%             reward(img) = 1 - distanceToClusterCenters(ass, img);
%         end
        for c = 1:nrClusterCenters,
            i = find(clusterAssignment == c);
            a(c) = 1 - (1-aa)*(min(min(sim(i,i))));
        end
        
        eval(sprintf('clusterAssignment%d = clusterAssignment;', idx));
        eval(sprintf('reward%d = reward;', idx));
        eval(sprintf('a%d = a;', idx));

        eval(sprintf('fcts{end+1} = @(x) diversityComponent(x, clusterAssignment%d, reward%d, a%d);', idx, idx, idx));
        desc{idx} = sprintf('clustered diversity component, CS=%d, exp=%f', clusterSize, a);
        %fcts{end+1} = @(x) diversityComponent(x, clusterAssignment, dist, a);
        idx = idx + 1;
    end

    %% facility location
    eval(sprintf('clusterAssignment%d = clusterAssignment;', idx));
    eval(sprintf('reward%d = reward;', idx));

    eval(sprintf('fcts{end+1} = @(x) facilityLocation(x, clusterAssignment%d, reward%d);', idx, idx));
    desc{idx} = sprintf('clustered facility location, CS=%d', clusterSize);
    %fcts{end+1} = @(x) diversityComponent(x, clusterAssignment, dist, a);
    idx = idx + 1; 
end


% %% fidelity component
for threshold = linspace(0.05, 1.0, 20),
    eval(sprintf('bow%d = bow;', idx));
    eval(sprintf('threshold%d = threshold;', idx));

    eval(sprintf('fcts{end+1} = @(x) fidelity(x, bow%d, threshold%d);', idx, idx));
    desc{idx} = 'thresholded fidelity';
    idx = idx + 1; 
end

for threshold = linspace(0.05, 1.0, 20),
    eval(sprintf('bowTFIDF%d = bowTFIDF;', idx));
    eval(sprintf('threshold%d = threshold;', idx));

    eval(sprintf('fcts{end+1} = @(x) fidelity(x, bowTFIDF%d, threshold%d);', idx, idx));
    desc{idx} = 'thresholded fidelity, TFIDF';
    idx = idx + 1; 
end

%% cosine similarity component
% for threshold = linspace(0.05, 1.0, 10),
%     eval(sprintf('bow%d = bow;', idx));
%     eval(sprintf('threshold%d = threshold;', idx));
% 
%     eval(sprintf('fcts{end+1} = @(x) cosineSimilarity(x, bow%d, threshold%d);', idx, idx));
%     idx = idx + 1; 
% end3

% % facility location bow
% bowS = {'bowSIFT', 'bowColorHSV', 'bowColorLAB', 'bowColorRGB', 'bowColorHistogramHSV', 'bowColorHistogramLAB', 'bowColorHistogramRGB'};
% for i = 1:length(bowS),
%     eval(sprintf('bowsimMM%d = BOWsimilarity(%s);', i, bowS{i}));
%     eval(sprintf('fcts{end+1} = @(S) sum(max(bowsimMM%d(:,S),[],2)) / 100;', i));
%     desc{idx} = sprintf('facility location, sim=%s', bowS{i});
%     idx = idx +1;
% 
%     eval(sprintf('fcts{end+1} = @(S) sum_coverage(S, bowsimMM%d);', i));
%     desc{idx} = sprintf('sum coverage, sim=%s;', bowS{i});
%     idx = idx + 1;
% 
%     eval(sprintf('fcts{end+1} = @(S) cut_coverage(S, bowsimMM%d);', i));
%     desc{idx} = sprintf('cut coverage, sim=%s;', bowS{i});
%     idx = idx + 1;
% end

% for k = 1:10,
%     eval(sprintf('t = randperm(5000); bow%d = bow(t(1:500),:);', idx));
%     eval(sprintf('bowsim%d = BOWsimilarity(bow%d);', idx, idx));
%     eval(sprintf('fcts{end+1} = @(S) sum(max(bowsim%d(:,S),[],2)) / 100;', idx));
%     idx = idx +1;
% end

% %%
bowsim = BOWsimilarity([bow;]);
bowsim2 = BOWsimilarity([bow; bowColor]);
bowsim3 = BOWsimilarity([bowColor]);
% fcts{end+1} = @(S) sum(max(bowsim2(:,S),[],2)) / 100;
% idx = idx +1;

 
% fcts{end+1} = @(S) sum_coverage(S, bowsim2);
% idx = idx + 1;
% 
% fcts{end+1} = @(S) cut_coverage(S, bowsim2);
% desc{idx} = 'cut coverage, bowsim2';
% idx = idx + 1;
% 
% for lambda = logspace(-2, 1, 10),
%     eval(sprintf('lambda_%d = lambda;', idx));
%     eval(sprintf('fcts{end+1} = @(S) cut_coverage(S, bowsim2) + lambda_%d*penalty_disparity(S, bowsim2);', idx));
%     desc{idx} = sprintf('cut coverage with penalty, SIFT+COLOR, lambda=%f', lambda);
%     idx = idx + 1;
% end
% 
% fcts{end+1} = @(S) cut_coverage(S, bowsim);
% desc{idx} = 'cut coverage, bowsim';
% idx = idx + 1;
% 
% for lambda = logspace(-2, 1, 10),
%     eval(sprintf('lambda_%d = lambda;', idx));
%     eval(sprintf('fcts{end+1} = @(S) cut_coverage(S, bowsim) + lambda_%d*penalty_disparity(S, bowsim);', idx));
%     desc{idx} = sprintf('cut coverage with penalty, SIFT, lambda=%f', lambda);
%     idx = idx + 1;
% end
% 
% fcts{end+1} = @(S) cut_coverage(S, bowsim3);
% desc{idx} = 'cut coverage, bowsim';
% idx = idx + 1;
% 
% for lambda = logspace(-2, 1, 10),
%     eval(sprintf('lambda_%d = lambda;', idx));
%     eval(sprintf('fcts{end+1} = @(S) cut_coverage(S, bowsim3) + lambda_%d*penalty_disparity(S, bowsim3);', idx));
%     desc{idx} = sprintf('cut coverage with penalty, COLOR, lambda=%f', lambda);
%     idx = idx + 1;
% end


% 
% compressed word count
fcts{end+1} = @(S) sum(sqrt(sum(bow(:,S),2))) / sum(sqrt(sum(bow,2)));
desc{idx} = 'compressed word count, compression=sqrt';
idx = idx + 1;
% 
% 
% 
% % compressed word count
% fcts{end+1} = @(S) sum(sqrt(sum(bowTFIDF(:,S),2))) / sum(sqrt(sum(bowTFIDF,2)));
% desc{idx} = 'compressed word count, TFIDF, compression=sqrt';
% idx = idx + 1;

% compressed word count
fcts{end+1} = @(S) sum(sqrt(sum(bow(:,S) > 0,2))) / sum(sqrt(sum(bow > 0,2)));
desc{idx} = 'compressed word existence count, compression=sqrt';
idx = idx + 1;

%%%%%%%%%%%%%%
clusters = {};
% clusters{1} = 1:11;
% clusters{2} = 12:20;
% clusters{3} = 21:23;
% clusters{4} = 24:29;
% clusters{5} = 30:32;
% clusters{6} = 33:34;
% clusters{7} = 35:36;
% clusters{8} = 37:39;
% clusters{9} = 40:43;
% clusters{10} = 44:58;
% clusters{11} = 59:90;
% clusters{12} = setdiff(91:100, 95);
% clusters{13} = 95;
% ass = kmeans([bowColor;bow]', 20);
% a = Kmedoids([bowColor;bow]', 20);
% ass = a.clustInds;
% for c = 1:max(ass),
%     clusters{c} = find(ass == c);
% end

%% DS 2
% clusters = {};
% clusters{1} = 1:2;
% clusters{2} = 10:11;
% clusters{3} = 23:25;
% clusters{4} = 41:42;
% clusters{5} = 53:54;
% clusters{6} = 63:64;
% clusters{7} = 96:97;


% bowS = {'bow', 'bowSIFT', 'bowColorHSV', 'bowColorLAB', 'bowColorRGB', 'bowColorHistogramHSV', 'bowColorHistogramLAB', 'bowColorHistogramRGB'};
% clusters = {};
% for i = 1:length(bowS),
%     eval(sprintf('sim = BOWsimilarity(%s);', bowS{i}));
%     candidates = 1:100;
%     %sim = BOWsimilarity(bowColor);
%     simE = sim - eye(100);
%     while ~isempty(candidates),
%         c = candidates(1);
%         idx = find(sim(c,:) > mean(simE(:)) + 2*std(simE(:)));
%         if length(idx) > 1,
%             clusters{end+1} = idx;
%         end
%         if isempty(idx),
%             idx = c;
%         end
%         candidates = setdiff(candidates, idx);
%     end
% end

%%

%idx = 1;
% fcts = {};
% desc = {};

% compute contrasts as rewards
reward = ones(100,1);
% for img = 1:100,
%     t = imagesgray(:,:,img);
%     c = imageslab(:,:,1,img);
%     ma = max(t(:));
%     mi = min(t(:));
%     %size(vl_sift(single(imagesgray(:,:,img))), 2);
%     reward(img) = sum(sum((vl_imgrad(imagesgray(:,:,img)))));
% end
% reward = reward + abs(min(reward)) + mean(abs(reward));
% reward = reward / max(reward);
% for c = 1:length(clusters),
%     reward(clusters{c}) = reward(clusters{c}) / max(reward(clusters{c}));
% end
% reward'
% 
% for l = 1:length(clusters),
%     clusterAssignment = ones(100,1);
%     clusterAssignment(clusters{l}) = 2;
%     a = [1;0.75];
%     
% %     normTerm = accumarray(clusterAssignment, reward);
% %     normTerm = sum(normTerm.^a);
% %     
% %     eval(sprintf('clusterAssignment%d = clusterAssignment;', idx));
% %     eval(sprintf('reward%d = reward;', idx));
% %     eval(sprintf('a%d = a;', idx));
% %     eval(sprintf('normTerm%d = normTerm;', idx));
% % 
% %     eval(sprintf('fcts{end+1} = @(x) diversityComponent(x, clusterAssignment%d, reward%d, a%d, normTerm%d);', idx, idx, idx, idx));
% %     desc{idx} = sprintf('clustered diversity component');
% %     idx = idx + 1;
%     
%     th = zeros(2, 1);
%     for k = 1:length(th),
%         th(k) = max((sum(clusterAssignment == k) / 20), 1);
%     end
%     eval(sprintf('clusterAssignment%d = clusterAssignment;', idx));
%     eval(sprintf('reward%d = reward;', idx));
%     eval(sprintf('th%d = th;', idx));
%     eval(sprintf('fcts{end+1} = @(x) facilityLocationMod(x, clusterAssignment%d, reward%d, th%d);', idx, idx, idx));
%     %eval(sprintf('fcts{end+1} = @(x) facilityLocation(x, clusterAssignment%d, reward%d);', idx, idx));
%     desc{idx} = sprintf('clustered facility location');
%     idx = idx + 1;
% end
% 
% clusterAssignment = ones(100,1);
% reward = ones(100,1);
% a = 0.75*ones(length(clusters),1);
% th = zeros(length(clusters), 1);
% for k = 1:length(clusters),
%     clusterAssignment(clusters{k}) = k;
%     th(k) = ceil(length(clusters{k}) / 10);
% end


% normalize clusterwise
% for c = 1:length(clusters),
%     reward(clusters{c}) = reward(clusters{c}) / max(reward(clusters{c}));
% end
% 
% normTerm = accumarray(clusterAssignment, reward);
% normTerm = sum(normTerm.^a);
%
% eval(sprintf('clusterAssignment%d = clusterAssignment;', idx));
% eval(sprintf('reward%d = reward;', idx));
% eval(sprintf('a%d = a;', idx));
% eval(sprintf('normTerm%d = normTerm;', idx));
% 
% eval(sprintf('fcts{end+1} = @(x) diversityComponent(x, clusterAssignment%d, reward%d, a%d, normTerm%d);', idx, idx, idx, idx));
% desc{idx} = sprintf('clustered diversity component');
% idx = idx + 1;

% eval(sprintf('clusterAssignment%d = clusterAssignment;', idx));
% eval(sprintf('reward%d = reward;', idx));
% eval(sprintf('th%d = th;', idx));
% eval(sprintf('fcts{end+1} = @(x) facilityLocation(x, clusterAssignment%d, reward%d);', idx, idx));
% desc{idx} = sprintf('clustered facility location');
% idx = idx + 1;

%%%%%%%%%

% thresholded feature counts
for alpha = linspace(0.1,1,10),
    eval(sprintf('alpha%d = alpha;', idx));
    
    eval(sprintf('fcts{end+1} = @(S) 1/(size(bow,1)*alpha%d) * sum(min(sum(bow(:,S),2) ./ sum(bow,2), alpha%d*ones(size(bow,1),1)));', idx, idx));
    desc{idx} = sprintf('thresholded features counts, thresh=%f', alpha);
    idx = idx + 1;
end

% 
% fcts{end+1} = @(S) penalty_disparity(S, bowsim);
% idx = idx + 1;

% add rouge score
% eval(sprintf('fcts{end+1} = @(x) summaryEvaluation(x, summaries, ''rouge'', optionsEvaluationLoss);'));
% idx = idx + 1; 

%%
% fcts = {};
% 
% for i = 1:100,
%     eval(sprintf('fcts{i} = @(x) any(x == %d);', i));
%     desc{i} = sprintf('indicator of image %d', i);
% end
% 
% 
% fcts
%return

% %
% f = zeros(100, 1);
% for i = 1:nrTrain,
%     f(trainingData(i,:)) = f(trainingData(i,:)) + 1;
% end
% f = f / size(nrTrain,1);
%     
% fcts = {};
% fcts{1} = @(x) sum(f(x));


%% perform training
%bow = get_BOW(dataset);
%bow = [bow; bowColor];
%bowTFIDF = applyTFIDF(bow, get_TFIDF(bow));
optionsEvaluationLossTest = struct();
optionsEvaluationLossTest.bow = bow;
optionsEvaluationLossTest.summariesAsBow = computeBOWSummary(testData, bow);
[ weights, allWeights, scoreMax, sc] = mm_subgrad(fcts, loss, repmat(1:100, nrTrain, 1), trainingData, 1e-1 / length(fcts), repmat(1:100, nrTest, 1), testData, @(x) summaryEvaluation(x, testData, 'rouge', optionsEvaluationLossTest), desc);

%% score test data
% TODO: USE TEST DATA!
eval(sprintf('scoreFct = %s;', constructScoreFunction(weights, 'fcts')));

hScores = NaN(size(summaries,1),1);
rScores = NaN(size(randomSummaries,1),1);
for k = 1:size(summaries,1),
    hScores(k) = scoreFct(summaries(k,:));
end
for k = 1:size(randomSummaries,1),
    rScores(k) = scoreFct(randomSummaries(k,:));
end

[a1,b1] = summaryEvaluation(summaries, summaries, 'rouge', optionsEvaluationLoss);
[a2,b2] = summaryEvaluation(randomSummaries, summaries, 'rouge', optionsEvaluationLoss);
greedyMaxRouge = sfo_greedy_lazy(sfo_fn_wrapper(@(x) summaryEvaluation(x, summaries, 'rouge', optionsEvaluationLoss)), 1:100, 10);
fprintf(1, 'Average human score=%f (rouge=%f, rouge max=%f, rouge greedy max=%f).\n', mean(hScores), a1, max(b1), summaryEvaluation(greedyMaxRouge, summaries, 'rouge', optionsEvaluationLoss));
fprintf(1, 'Average random score=%f (rouge=%f, rouge max=%f).\n', mean(rScores), a2, max(b2));
greedyMax = sfo_greedy_lazy(sfo_fn_wrapper(@(x) scoreFct(x)), 1:100, 10);
fprintf(1, 'Score of greedy max = %f\n', summaryEvaluation(greedyMax, summaries, 'rouge', optionsEvaluationLoss));
fprintf(1, 'Overlap = %f (rouge=%f)\n', computeGaussianOverlap(mean(hScores), mean(rScores), std(hScores), std(rScores)), computeGaussianOverlap(mean(b1), mean(b2), std(b1), std(b2)));

xmin = min([min(hScores), min(rScores)]) - 0.2;
xmax = max([max(hScores), max(hScores)]) + 0.2;
subplot(311)
hist(hScores, 20);
xlim([xmin, xmax]);
title('human scores');
subplot(312)
hist(rScores, 20)
xlim([xmin, xmax]);
title('random scores');

%% show result
subplot(313); montage(imagesrgb, 'Size', [1,10], 'Indices', scoreMax);

return

%% 
% kernelsize = 4;
% ratio = 0.5;
% maxdist = 20;
% imagescolor = imageshsv;
% imgseg = zeros(size(imagescolor));
% chist = cell(100,1);
% optionsHist = struct();
% optionsHist.bins = [32,16,16];
% optionsHist.histType = 'SQ';
% for img = 1:100,
%     [map,gaps] = vl_quickshift(imagescolor(:,:,:,img), kernelsize, maxdist);
%     c1 = imagescolor(:,:,1,img);
%     c2 = imagescolor(:,:,2,img);
%     c3 = imagescolor(:,:,3,img);
%     [labels,clusters] = vl_flatmap(map);
%     nrClusters = max(clusters);
%     chist{img} = cell(nrClusters,1);
%     for c = 1:nrClusters,
%         chist{img}{c} = get_colorHistogramPatch([c1(clusters == c), c2(clusters == c), c3(clusters == c)], optionsHist);
%     end
% end
% 
% %
% nrObj = 0;
% nrObjImg = zeros(100,1);
% vec = [];
% for img = 1:100,
%     for c = 1:length(chist{img}),
%         chist{img}{c} = chist{img}{c} / sum(chist{img}{c});
%         vec = horzcat(vec, chist{img}{c});
%         nrObj = nrObj + 1;
%     end
%     nrObjImg(img) = length(chist{img});
% end
% 
% %%
% enc = zeros(nrObj, 100);
% sim = zeros(100);
% idx = 1;
% for img = 1:100,
%     idxE = idx + nrObjImg(img) - 1;
%     
%     idx2 = 1;
%     img
%     for img2 = 1:100,
%         idx2E = idx2 + nrObjImg(img2) - 1;
%         
%         t = pdist2(vec(:,idx:idxE)', vec(:,idx2:idx2E)', 'L1');
%         %t(t == 0) = inf;
%         %[cut,cost] = Hungarian(t);
%         %assigned = sum(cut,2);
%         sim(img,img2) = sum(min(t, [], 2)) / size(t,2);
%         
%         idx2 = idx2 + nrObjImg(img2);
%     end
%     
%     %tdist = pdist2(vec(:,idx:idxE)', vec', 'emd');
%     %%tdist(tdist > 2) = 10;
%     %enc(:,img) = exp(-min(tdist,[],1)'/1);
%     
%     idx = idx + nrObjImg(img);
% end
% imagesc(BOWsimilarity(enc))
% 
% %%
% for img = 1:100,
%     imgseg(:,:,:,img) = vl_quickseg(imagesrgb(:,:,:,img), ratio, kernelsize, maxdist);
% end
% montage(imgseg, 'Size', [10,10]);
% 
% %%
% optionsHist = struct();
% optionsHist.bins = [16,8,8];
% optionsHist.histType = 'SQ';
% if strcmp(optionsHist.histType, 'SQ'),
%     hists = zeros(sum(optionsHist.bins), 100);
% else
%     hists = zeros(prod(optionsHist.bins), 100);
% end
% for img = 1:100,
%     hists(:,img) = get_colorHistogramPatch(imageslab(:,:,:,img), optionsHist);
%     hists(:,img) = hists(:,img) / sum(hists(:,img));
% end
% 
% imagesc(BOWsimilarity(hists, 'emd', optionsHist))

%% try to find robust clusters
% sim1 = BOWsimilarity(hists, 'emd', optionsHist);
% sim2 = BOWsimilarity(bow);
% %%
% th1 = mean(sim1(:)) + 1*std(sim1(:));
% th2 = mean(sim2(:)) + 1*std(sim2(:));
% th1
% th2
% cl = cell(100,1);
% for img = 1:100,
%     cand = find(sim1(img,:) > th1);
%     cand2 = find(sim2(img,:) > th2);
%     ok = intersect(cand, cand2)
%     cl{img} = ok;
% end
% 
% %%
% l = cell(100,1);
% for img = 1:100,
%     list = cl{img};
%     for k = 1:length(cl{img})
%         list = intersect(list, cl{cl{img}(k)});
%     end
%     l{img} = list;
%     list
% end
% 
% 
% %%
% for img = 1:100,
%     list = l{img};
%     for k = 1:length(l{img})
%         list = union(list, l{l{img}(k)});
%     end
%     l{img} = list;
%     list
% end

%%
% cellSize = 8;
% hog = vl_hog(single(imagesrgb(:,:,:,20)), cellSize, 'verbose');
% imhog = vl_hog('render', hog, 'verbose') ;
% clf ; imagesc(imhog) ; colormap gray ;
% %%
% sim = zeros(100);
% for i1 = 1:100,
%     i1
%     hog1 = vl_hog(single(imagesrgb(:,:,:,i1)), cellSize);
%     t1 = hog1(:);
%     for i2 = 1:100,
%         hog2 = vl_hog(single(imagesrgb(:,:,:,i2)), cellSize);
%         d = zeros(25,25);
%         for a  = 1:25,
%             for b = 1:25,
%                 f = hog1(a,b,:);
%                 g = hog2(a,b,:);
%                 d(a,b) = norm(f(:) - g(:),2);
%             end
%         end
%         t2 = hog2(:);
%         sim(i1,i2) = exp(-norm(d,1)/50);
%     end
% end
% 
% %%
% d = zeros(9,100);
% for img = 1:100,
%     i = 1;
%     for ch = 1:3,
%         t = imageslab(:,:,ch,img);
%         d(i,img) = mean(t(:));
%         d(i+1,img) = var(t(:));
%         d(i+2,img) = median(t(:));
%         i = i + 3;
%     end
% end
% 
% dist = vl_alldist(d,d,'l1');
% imagesc(exp(-dist));
