clear all;
clc;

%% settings
dataset = 10;

%% extract patches
[images,imagesgray] = get_images(dataset, false);
dimX = size(imagesgray,1);
dimY = dimX;

if ~exist('patches.mat', 'file'),
    patchSize = 15;
    patches = zeros((dimX/patchSize)^2*100,patchSize^2, 'single');
    idx = 1;
    for img = 1:100,
        startX = 1;
        startY = 1;
        for row = 1:(dimX/patchSize),
            for col = 1:(dimY/patchSize),
                endX = startX + patchSize - 1;
                endY = startY + patchSize - 1;
                
                tpatch = imagesgray(startX:endX,startY:endY,img);
                patches(idx,:) = tpatch(:)';
                
                startX = startX + patchSize;
                if startX > dimX,
                    startX = 1;
                    startY = startY + patchSize;
                end
                idx = idx + 1;
            end
        end 
    end
end

%% perform sparse filtering
% Remove DC
tperm = randperm(size(patches,1));
%patches = patches(tperm(1:5000),:);
patches = bsxfun(@minus, patches, mean(patches));

% Train Layer 1
L1_size = 512;  % Increase this for more features
L1 = sparseFiltering(L1_size, double(patches'));

%% display
% Show Layer 1 Bases
displayData(L1);

%% Feed-forward Layer 1
data1 = feedForwardSF(L1, patches');
datatemp = data1;
data1 = bsxfun(@minus, data1, mean(data1));

%% Train Layer 2
L2_size = 256;
L2 = sparseFiltering(L2_size, data1);

% Visualize Layer 2
figure;

% Number of L2 units to visualize
num_viz = 10; 

% Visualize different units
offset = 1;

% Plot the units
for i = 1:num_viz
    j = offset+i;
    
    % Find the sign of the unit with the maximum absolute values
    [a, b] = max(abs(L2(j, :)));
    sgn    = sign(L2(j, b)); 
    
    % Sort and plot units in those direction
    [a, b] = sort(sgn*(L2(j, :)), 'descend');
    
    % Plot
    subplot(1, num_viz, i, 'align'); 
    displayData(L1(b(1:10), :), [] , 1);
end

%% transcribe images
