function [ weights, allWeights, scoreMax, sc, stats ] = mm_subgradAcrossDatasets( trainingDatasets, testDatasets, fcts, loss, fctsJava, lossesJava, input, output, lambda, testInput, testOutput, rouge, desc, preScores, options)
%MM_SUBGRAD Summary of this function goes here
%   Detailed explanation goes here

stats = struct(); % for accounting
stats.scoresAverageWeights = [];
stats.scoresFinalWeights = [];
stats.iter = [];

%% sanity checks
if size(fcts,2) < 1,
    error('At least one function must be given.');
end

%% extract infos
nrComponents = size(fctsJava, 2);

nrSamples = zeros(length(output), 1);
for i = 1:length(output),
    nrSamples(i) = size(output{i}, 1);
end
options.nrIter = max(nrSamples);
fprintf(1, 'Samples in datasets:\n');
nrSamples(:)'
options.nrIter = ceil(10*max(nrSamples(trainingDatasets)));
%options.nrIter = 2;

if options.verbose,
    fprintf(1, '\n\nPerforming training with %d components.\n', nrComponents);
end

%% initialize weights
currWeights = 2*options.pruneThreshold*ones(nrComponents, 1);
currGradient = zeros(nrComponents, 1);

% initalize weights to "generative" solution
fprintf(1, 'Precomputing weights.\n');
for ds = trainingDatasets,
    fprintf(1, '[DS%2d] ', ds);
    nrSamplest = size(output{ds}, 1);
    for k = 1:nrSamplest,
	yt = output{ds}(k,:);
	
	for f = 1:size(fctsJava,2),
            currWeights(f) = currWeights(f) + fctsJava{ds,f}.eval(yt);
        end
        
        if mod(k, floor(nrSamplest/10)) == 0,
            fprintf(1, '%3.0f%% ', k / nrSamplest * 100);
        end
    end
    fprintf(1, '\n');
end
currWeights(1:end-1) = projectOntoSimplex(currWeights(1:end-1));

% Find index of hinge -- if available
hingeIndex = [];
hingeIndicator = cellfun(@(x) strcmp(x, 'HingeLoss'), desc);
if any(hingeIndicator),
    hingeIndex = find(hingeIndicator);
    fprintf(1, 'Found hinge at index %d.\n', hingeIndex);
end 
fprintf(1, 'WARNING: SETTING WEIGHT FOR HINGE!!!\n');
currWeights(hingeIndex) = options.hingeWeight;

if options.loadWeights,
    fprintf(1, 'Loading precomputed weights...\n');
    
    saveResultsPath ='/clusterFS/home/user/stschia/ECCV2014/BatchSystem/experiments/TwentyPasses/results';
    filenameResult = fullfile(saveResultsPath, sprintf('results_dataset_%d_overlap_%1.2f_loss_%s_algorithm_%s.mat', testDatasets, 0.1, 'OneMinusRouge', 'greedy'));
    
    filenameResult
    load(filenameResult, 'allWeights');
    currWeights = mean(allWeights, 2);
    clear('allWeights');
end

%% construct functions for lai
laiFunctionsJava = cell(14,1);
for ds = 1:14,
    laiFunctionJava = LinCombFunctionJ();
    for f = 1:size(fctsJava, 2),
        laiFunctionJava.addFunction(fctsJava{ds,f});
    end
    laiFunctionJava.addFunction(lossesJava{ds});
    laiFunctionJava.setPruneThreshold(options.pruneThreshold);
    % laiFunctionJava.timeFunctions();
    laiFunctionsJava{ds} = laiFunctionJava;
end

%% construct score functions
scoreFunctionsJava = cell(14,1);
for ds = 1:14,
    scoreFunctionJava = LinCombFunctionJ();
    for f = 1:size(fctsJava, 2),
        scoreFunctionJava.addFunction(fctsJava{ds,f});
    end
    % scoreFunctionJava.timeFunctions();
    scoreFunctionJava.setPruneThreshold(options.pruneThreshold);
    scoreFunctionsJava{ds} = scoreFunctionJava;
end

lstats = [];

%% cycle through samples
allWeights = NaN(nrComponents, options.nrIter);
allGradients = NaN(nrComponents, options.nrIter);
eta = options.etaInitial;
useLazyA = false;

%% start server and add work
[~,host] = system('hostname');
host
if strcmp(strtrim(host), 'Sebastians-MacBook-Pro.local')
    disp('local');
    client = javaObject('Client', '127.0.0.1',9999);
else
    client = javaObject('Client', 'bayes.spsc.tugraz.at',9999);
end

fprintf(1, 'Adding work... ');
client.setWeights(options.hash, currWeights);
client.clearQueues(options.hash);
t = 0;
for k = 1:options.nrIter,    
    for ds = trainingDatasets,
        t = t + 1;        
        sampleIndex = 1 + mod(k, nrSamples(ds));
        yt = output{ds}(sampleIndex,:);
        
        %work = javaObject('Work', options.inference, yt, ds);
        %server.addWork(work);
        client.addWorkList(options.hash, options.inference, yt, ds);
    end
end
client.addWork(options.hash);
while client.isWorkDone(options.hash),
    options.hash
    client.isWorkDone(options.hash)
    fprintf(1, 'Waiting for work to be added.');
    pause(1);
end
fprintf(1, 'DONE\n');

client.setWeights(options.hash, currWeights);

t = 0;
tic;
timeStart = now();
quitNow = false;
for k = 1:options.nrIter,
    fprintf(1, '>> Performing iteration %d/%d.\n', k, options.nrIter);
    timeNow = now();
    timePassed = (timeNow - timeStart) * (3600*24);
    fprintf(1, '(Approximate time remaining: %f hours)\n', timePassed/k*(options.nrIter - k)/ 3600);
    
    % show 5 strongest components
    if options.verbose,
        nrC = 5;
        tWeights = mean(allWeights(:,1:t), 2);
        [a,b] = sort(abs(tWeights), 'descend');
        b = b(1:nrC);
        fprintf(1, 'List of %d strongest components (lambda=%f):\n', nrC, lambda);
        for kk = b(:)',
            fprintf(1, '    [%3d] val=%e (curr=%e): %s\n', kk, tWeights(kk), currWeights(kk), desc{kk});
        end
        fprintf(1, '    HINGE val=%e (curr=%e): %s\n', tWeights(end), currWeights(end), desc{end});
    end
    
    for ds = trainingDatasets,
        fprintf(1, '(Passed through the data %.2f times)\n', k / nrSamples(ds));
        t = t + 1;
        xt = 1:100;
        
        lenOutput = size(yt,2);
        
        tic;
        workResult = [];
        done = false;
        while (isempty(workResult)),
            workResult = client.getWorkResult(options.hash);
            if isempty(workResult) && client.isWorkDone(options.hash)
                done = true;
                break;
            end
            if isempty(workResult),
                %fprintf(1, 'Waiting for gradient...\n');
                pause(1);
            end
        end
        if (done)
            % truncate allWeights matrix
            allWeights = allWeights(:,1:(t-1));
            quitNow = true;
            break;
        end
        yht = java2matArray(workResult.getYht())';
        yt = java2matArray(workResult.getYt())';
        grad = java2matArray(workResult.getGrad());
        ds = workResult.getDataset();
        ttt = toc;

        sampleScore = rouge{ds}(yt);
        competitorScore = rouge{ds}(yht);
        fprintf(1, '>>            Sample: [');
        fprintf(1, '%3d ', sort(yt));
        fprintf(1, '] Score = %1.4f (avg hum = %1.4f)\n', sampleScore, preScores(ds,1));
        fprintf(1, 'Strongest competitor: [');
        fprintf(1, '%3d ', yht);
        fprintf(1, '] Score = %1.4f (avg rnd = %1.4f)\n', competitorScore, preScores(ds,2));
        %fprintf(1, 'Strongest competitor...');
        %yht
        
        %% perform weight updates with projection
        oldWeights = currWeights;
        
        %% SGD update
        if strcmp(options.SGDalgorithm, 'classic'),
            grad = grad + lambda * currWeights;
            allGradients(:,t) = grad;
            if norm(grad) > 1e-9,
                grad = grad / norm(grad);
                fprintf(1, 'Gradient normalized.\n');
            end
            
            eta = options.etaInitial/sqrt(t);
            newWeights = currWeights - eta*grad;
        elseif strcmp(options.SGDalgorithm, 'adagrad'),
            % Update according to John Duchi
            delta = 0.1;
            allGradients(:,t) = grad;
            
            %keyboard
            newWeights = adagradl2(eta, lambda, delta, allGradients(:,1:t));
            % newWeights = adagradl1(eta, lambda, delta, allGradients(:,1:t));
        elseif strcmp(options.SGDalgorithm, 'gradientScaling'),
            fprintf(1, 'Gradient scaling.\n');
            
            %weight = (sampleScore - preScores(ds,5)) / (preScores(ds,6) - preScores(ds,5));
            %weight = 1 - (competitorScore - preScores(ds,3)) / (preScores(ds,6) - preScores(ds,3));
            %if competitorScore > preScores(ds,8),
            %   weight = 0;
            %else
            %   weight = 1;
            %end
            %weight
            
            %weight = (sampleScore - preScores(ds,5)) / (preScores(ds,6) - preScores(ds,5));
            weight = normpdf(sampleScore, preScores(ds,7),preScores(ds,8)) / normpdf(preScores(ds,7), preScores(ds,7),preScores(ds,8));
            fprintf(1, 'Weight of sample: %f\n', weight);
            
            grad = weight*(grad + lambda * currWeights);
            allGradients(:,t) = grad;
            delta = 1e-1;
            h = zeros(nrComponents, 1);
            for i = 1:nrComponents,
                h(i) = delta + norm(allGradients(i,1:t));
            end
            
            ngrad = eta*grad./h;
            fprintf(1, 'HINGE GRAD: %f (eta=%f, h = %f)\n', ngrad(end), eta, h(end));
            newWeights = currWeights - ngrad;
        else
            error('Invalid SGD method.');
        end
        
        % project to non-negative orthant
        switch lower(options.project)
            case 'positive'
                newWeightsProjected = max(zeros(size(currWeights)), newWeights);
            case 'none'
                newWeightsProjected = newWeights;
            case 'simplex'
                fprintf(1, 'Projecting onto simplex...\n');
                newWeightsProjected = projectOntoSimplex(newWeights);
                
                % sanity check
                if any(newWeightsProjected < 0) || abs(sum(newWeightsProjected) - 1) > 1e-3,
                    fprintf(1, 'Projection onto simplex failed.\n');
                    keyboard
                    error('Projection onto simplex failed.\n');
                end
            otherwise
                error('Invalid projection selected.');
        end
        currWeights = newWeightsProjected;
        currWeights(hingeIndex) = options.hingeWeight;
        %if any(newWeightsProjected < 0),
        %    newWeightsProjected(:)'
        %    error('Projection to non-negative orthant failed.');
        %end
        fprintf(1, '||Weight update||=%f, ||grad||=%f\n', norm(oldWeights-currWeights, 1), norm(grad));
        fprintf(1, '||Weights||=%f\n', norm(currWeights, 1));
        
        client.setWeights(options.hash, currWeights);
        
        fprintf(1, '# of nonzero weights: %d (%2.2f%%) \n', sum(currWeights > 0), sum(currWeights > 0) / length(currWeights)*100);
        
        %% store weights
        allWeights(:,t) = currWeights;
        
        %% score fct
        if options.logProgress && mod(k, options.logEvery) == 0,
            %% JAVA %%%%%%%%%%%%%%%%
            tic
            tstart = cputime;
            scoreFunctionJava = scoreFunctionsJava{ds};
            tWeights = mean(allWeights(:,1:t), 2);
            scoreFunctionJava.setWeights(tWeights);
            greedy = GreedyJ(scoreFunctionJava);
            yht = java2matArray(greedy.maximizeadv(100, lenOutput));
            yht = yht(:)';
            %yht = sfo_greedy_lazy(sfo_fn_wrapper(@(S) scoreFunctionJava.eval(S)), 1:100, lenOutput);
            scoreAverageWeights = rouge{ds}(yht);
            fprintf('[ds=%2d] JAVA AVG WGHT. Score of current best summary: %f (average human=%f, average random=%f)\n', ds, scoreAverageWeights, preScores(ds,1), preScores(ds,2));
            stats.scoresAverageWeights(end+1) = scoreAverageWeights;
            
            tWeights = allWeights(:,t);
            scoreFunctionJava.setWeights(tWeights);
            greedy = GreedyJ(scoreFunctionJava);
            yht = java2matArray(greedy.maximizeadv(100, lenOutput));
            yht = yht(:)';
            %yht = sfo_greedy_lazy(sfo_fn_wrapper(@(S) scoreFunctionJava.eval(S)), 1:100, lenOutput);
            scoreFinalWeights = rouge{ds}(yht);
            fprintf('[ds=%2d] JAVA FIN WGHT. Score of current best summary: %f (average human=%f, average random=%f)\n', ds, scoreFinalWeights, preScores(ds,1), preScores(ds,2));
            stats.scoresFinalWeights(end+1) = scoreFinalWeights;
            ttt = toc;
            tend = cputime;
            fprintf(1, 'JAVA Scoring took %.2f seconds (CPU time: %f).\n', ttt, tend-tstart);
            stats.iter(end+1) = k;
            %% JAVA %%%%%%%%%%%%%%%%
            
            % plot
            x = (options.logEvery*(1:length(stats.scoresAverageWeights))) / nrSamples(ds);
            ymin = preScores(ds,3);
            ymax = preScores(ds,4);
            plot(x, stats.scoresAverageWeights, 'g');
            hold on;
            plot(x, stats.scoresFinalWeights, 'b');
            hold on
            legend('avg weights', 'final weights');
            line([min(x),max(x)], [preScores(ds,1), preScores(ds,1)], 'Color', 'k');
            hold on
            line([min(x),max(x)], [preScores(ds,2), preScores(ds,2)], 'Color', 'k');
            hold on
            line([min(x),max(x)], [preScores(ds,3), preScores(ds,3)], 'Color', 'r');
            hold on
            line([min(x),max(x)], [preScores(ds,4), preScores(ds,4)], 'Color', 'r');
            title(sprintf('Dataset %d (learning rate %g)', testDatasets, options.etaInitial));
            ylim([ymin-0.05,ymax+0.05]);
            xlabel('passes through dataset');
            ylabel('v-rouge');
            drawnow
        end
    end
    
    if quitNow,
        break;
    end
end
%server.stopServer();

%% save lstats
save('lengthStatsInteface5times.mat', 'lstats');

%% save plot
gcf;
t = 1;
fname = sprintf('/clusterFS/home/user/stschia/progress_ds_%d_try_%d.pdf', testDatasets, t);
while exist(fname, 'file'),
    t = t + 1;
    fname = sprintf('/clusterFS/home/user/stschia/progress_ds_%d_try_%d.pdf', testDatasets, t);
end
print('-dpdf','-r300',fname);

%% average weights and return
weights = mean(allWeights, 2);

%% determine score maximizer
eval(sprintf('scoreFct = %s;', constructScoreFunction(weights, 'fcts')));

fprintf(1, 'Final result.\n');
% [yht, score, gains] = greedySummary(scoreFct, 10);
% yht
% score
% gains
% scoreMax = yht;
% sc = score;
scoreMax = NaN;
sc = NaN;

end

function [x] = adagradl1(lambda, delta, allGradients)
H = zeros(size(allGradients,1));
for i = 1:size(allGradients,1),
    H(i,i) = delta + norm(allGradients(i,1:t),2);
end
avgGradient = mean(allGradients(:,1:t), 2);

x = sign(-avgGradient)*eta*t ./ diag(H) .* max(0, abs(avgGradient) - lambda);
end

function [x] = adagradl2(eta, lambda, delta, allGradients) % Figure 4 in his paper
useOnlyDiagonal = true;
if useOnlyDiagonal,
    H = zeros(size(allGradients,1));
    for i = 1:size(allGradients,1),
        H(i,i) = delta + norm(allGradients(i,:),2);
    end
    Hinv = diag(1./diag(H));
else
    G = zeros(size(allGradients,1));
    for t = 1:size(allGradients,2),
        g = allGradients(:,t);
        G = G + g*g';
    end
    H = delta*eye(size(allGradients,1)) + real(sqrtm(G));
    
    Hinv = inv(H);
end


avgGradient = mean(allGradients, 2);
singV = svd(H);
sigmaMax = max(singV);
sigmaMin = min(singV);
u = eta*size(allGradients,2)*avgGradient; % avgGradient; % allGradients(:,end);

if norm(u, 2) <= lambda,
    x = zeros(size(allGradients,1), 1);
    return;
end

v = Hinv*u;
thetaMax = norm(v,2) / lambda - 1/sigmaMax;
thetaMin = norm(v,2) / lambda - 1/sigmaMin;

eps = 1e-6;
theta = (thetaMax + thetaMin) / 2;
malpha = -inv(Hinv + theta*eye(size(H,1)))*v;
while thetaMax - thetaMin > eps,
    theta = (thetaMax + thetaMin) / 2;
    malpha = -inv(Hinv + theta*eye(size(H,1)))*v;
    
    if norm(malpha,2) > lambda,
        thetaMin = theta;
    else
        thetaMax = theta;
    end
    
    % todo: remove
    %x = -Hinv*(u + malpha);
    %val = u'*x + 0.5*x'*(H*x) + lambda*norm(x,2);
    %theta
    %fprintf(1, 'Val: %f\n', val);
end
x = -Hinv*(u + malpha);
end

