function A = mmr_summarize(sim, n, k, lambda)
% Find the summary based on MMR rule
if nargin < 4,
    lambda = 0.5;
end

A = [];
V = 1:n;
for i = 1:k,
    candidates = setdiff(V, A);
    
    % assign scores to candidates
    score = zeros(1, n);
    for candidate = candidates,
        score(candidate) = lambda*rel(candidate, A, sim) - (1-lambda)*maxSim(candidate, A, sim);
    end
    [mscore, idx] = max(score(candidates));
    
    A = [A, candidates(idx)];
end
end

function [val] = maxSim(cand, summary, sim)
    val = sum(max(sim(cand, summary)));
end

function [val] = rel(cand, summary, sim)
    % facility location (outer sum is to avoid problems with empty set)
    val = sum(sum(sim(:,[cand,summary])));
end