%% Collects all information and prepares for plotting. Further, generates a
%% LaTeX table summarizing all results

clear all;
clc;

%% settings
datasets = 1:14;
nrRandom = 1000; % number of random summaries
overlap = 0.1;

%% setup
cd('..');
addpath('');
addpath('external/sfo');
tableFile = 'results.tex';

%% convert
scaledScores = [];
methods = {}; % cell array containing descriptors of the methods
results = []; % matrix for results -- contents must match the methods cell array
for dataset = datasets,
    fprintf(1, '>> Processing dataset %d...\n', dataset);
    randomSummaries = randomSummary(100,10,nrRandom);
    
    %% NOT-PRUNED
    humanSummaries = load_summaries(dataset);
    
    bow = get_BOW_evaluation(dataset);

    optionsEvaluation = struct();
    optionsEvaluation.bow = bow;
    optionsEvaluation.summariesAsBow = computeBOWSummary(humanSummaries, bow);
    
    %% scores of human summaries
    [avgScoreHuman,scoresHuman] = summaryEvaluation(humanSummaries, humanSummaries, 'rouge', optionsEvaluation);

    % compute best/worst/medium summaries
    [~,a] = sort(scoresHuman, 'descend');
    bestSummariesBefore = humanSummaries(a(1:3), :);
    mediumSummariesBefore  = humanSummaries(a(floor(length(a)/2):(floor(length(a)/2)+2)), :);
    worstSummariesBefore = humanSummaries(a((end-2):end),:);
    
    
    %% Pruned
    % humanSummaries = pruneSummaries(dataset, get_pruning_info(dataset, overlap));
    humanSummaries = load_summaries(dataset);

    bow = get_BOW_evaluation(dataset);

    optionsEvaluation = struct();
    optionsEvaluation.bow = bow;
    optionsEvaluation.summariesAsBow = computeBOWSummary(humanSummaries, bow);
    
    %% scores of human summaries
    [avgScoreHuman,scoresHuman] = summaryEvaluation(humanSummaries, humanSummaries, 'rouge', optionsEvaluation);
    [avgScoreRandom,scoresRandom] = summaryEvaluation(randomSummaries, humanSummaries, 'rouge', optionsEvaluation);
    [~,tmp] = sort(scoresHuman);
    avgHumanSummary = humanSummaries(tmp(ceil(length(tmp)/2)),:);
    [~,tmp] = sort(scoresRandom);
    avgRandomSummary = randomSummaries(tmp(ceil(length(tmp)/2)),:);

    % compute best/worst/medium summaries
    [~,a] = sort(scoresHuman, 'descend');
    bestSummaries = humanSummaries(a(1:3), :);
    mediumSummaries = humanSummaries(a(floor(length(a)/2):(floor(length(a)/2)+2)), :);
    worstSummaries = humanSummaries(a((end-2):end),:);

    %% summaries 
    summaries = struct();
    summaries(1).score = avgScoreRandom;
    summaries(1).summary = avgRandomSummary - 1;
    summaries(1).description = 'Average Random';
    summaries(1).position = 'top';
    methods{1} = 'Avg Rnd';
    results(dataset,1) = summaries(1).score;

    summaries(2).score = avgScoreHuman;
    summaries(2).summary = avgHumanSummary - 1;
    summaries(2).description = 'Average Human';
    summaries(2).position = 'top';
    methods{2} = 'Avg Hum';
    results(dataset,2) = summaries(2).score;
    
    %% greedy max
    greedyMaxRouge = sfo_greedy_lazy(sfo_fn_wrapper(@(x) summaryEvaluation(x, humanSummaries, 'rouge', optionsEvaluation)), 1:100, 10);
    summaries(3).score = summaryEvaluation(greedyMaxRouge, humanSummaries, 'rouge', optionsEvaluation);
    summaries(3).summary = greedyMaxRouge - 1;
    summaries(3).description = 'Greedy Max';
    summaries(3).position = 'bottom';
    methods{3} = 'Greedy Max';
    results(dataset,3) = summaries(3).score;
    
    %% greedy min
    [~,~,greedyMinRouge,~] = mmin(@(x) summaryEvaluation(x, humanSummaries, 'rouge', optionsEvaluation), 100, [], 'MUB', 10);
    summaries(4).score = summaryEvaluation(greedyMinRouge, humanSummaries, 'rouge', optionsEvaluation);
    summaries(4).summary = greedyMinRouge - 1;
    summaries(4).description = 'Submod Min';
    summaries(4).position = 'bottom';
    methods{4} = 'Submod Min';
    results(dataset,4) = summaries(4).score;
    
    %% scores from learning accross datasets
    filename = sprintf('result_ds_%d_projection_%s_inference_%s_loss_%s_scenario_CHEATING.txt', dataset, 'SIMPLEX', 'SUBSUP', 'ONEMINUSROUGE');    
    filename = fullfile('external/Rishabh/results/', filename);
    filename
    disp('Trying to load...');
    if ~exist(filename, 'file'),
        warning('File does not exist...');
        continue;
    end
    fprintf(1, 'Loading %s\n', filename);
    tresults = loadResultsFile(filename);
    greedyMax = tresults.greedyMaxAverageWeights';
    greedyMax
    
    
    %fname = sprintf('../BatchSystem/results/results_dataset_%d.mat', dataset);
    %load(fname, 'greedyMax');
    
    
%     scoreLearntMixture = summaryEvaluation(greedyMax, humanSummaries, 'rouge', optionsEvaluation);
%     summaries(5).score = scoreLearntMixture;
%     summaries(5).summary = greedyMax - 1;
%     summaries(5).description = 'Max of Learned Mixture';
%     summaries(5).position = 'bottom';
%     methods{5} = 'Max Mixt';
%     results(dataset,5) = summaries(5).score;
    
    %% Account score
%    scaledScores(end+1) = (scoreLearntMixture - avgScoreRandom) / (avgScoreHuman - avgScoreRandom);
    
    %% sort struct according to score (for nice plotting)
    scores = zeros(length(summaries), 1);
    for t = 1:length(summaries),
        scores(t) = summaries(t).score;
    end
    [~,perm] = sort(scores);
    tsummaries = summaries;
    for t = 1:length(summaries),
        summaries(t) = tsummaries(perm(t));
    end

    %fprintf(1, 'Score mixture: %f; Score human: %f; score random: %f\n', scoreLearntMixture, mean(scoresHuman), mean(scoresRandom));

    fname = sprintf('plotting/dataset%d.mat', dataset);
    save(fname, 'dataset', 'greedyMinRouge', 'greedyMaxRouge', 'avgHumanSummary', 'bestSummaries', 'mediumSummaries', 'worstSummaries', 'bestSummariesBefore', 'mediumSummariesBefore', 'worstSummariesBefore', 'summaries', 'scoresHuman', 'scoresRandom');  
end
fprintf(1, 'Score of current results: %f (+/- %f)\n', mean(scaledScores), std(scaledScores));

cd('plotting');

%% generate results as LaTeX table
strDataset = []; % string describing the results for this dataset
strHeader = []; % string describing the header

fh = fopen(tableFile, 'w');
fprintf(fh, '\\documentclass{article}\n\\usepackage{booktabs}\n\\begin{document}\n');

celltypes = 'l';
for i = 1:length(methods),
    celltypes = [celltypes, 'c'];
end
fprintf(fh, '\\begin{tabular}{%s}\n', celltypes);

% print header
fprintf(fh, '\\toprule ');
for m = 1:length(methods),
    fprintf(fh, ' & \\textbf{%s} ', methods{m});
end
fprintf(fh, ' \\\\ \\midrule \n');

% print results
for dataset = datasets,
    fprintf(fh, 'Dataset %d ', dataset);
    
    for m = 1:length(methods),
        fprintf(fh, ' & %.2f ', results(dataset,m));
    end
    fprintf(fh, '\\\\\n');
end
fprintf(fh, '\\bottomrule');

fprintf(fh, '\\end{tabular}\n');

fprintf(fh, '\\end{document}\n');
fclose(fh);

% try to compile
system(sprintf('pdflatex -halt-on-error %s', tableFile));

