#!/usr/bin/python

import sys 				# for command line arguments
import pylab as py
import numpy as np
import matplotlib.pyplot as plt
from matplotlib import rc
from scipy.optimize import curve_fit
import matplotlib.image as mpimg
import os
import scipy.io as sio

# check command line parameters
if len(sys.argv) != 2:
   print("You must provide one command line argument (the path of a .mat file).")
   sys.exit(-1)
matFile = sys.argv[1]

# set path for tex on Mac
os.environ['PATH'] = os.environ['PATH'] + ':/opt/local/bin/latex'

# load .mat file and convert to internal structure
tmp = sio.loadmat(matFile, squeeze_me=True)
tmp = tmp["summaries"]
summaries = []
for s in range(0, len(tmp)):
   summaries.append({'summary': tmp[s]["summary"], 'score': tmp[s]["score"], 'description': tmp[s]["description"], 'position': tmp[s]["position"]})

tmp = sio.loadmat(matFile)
scoresHuman = tmp["scoresHuman"]
scoresRandom = tmp["scoresRandom"]
dataset = tmp["dataset"]
bestSummaries = tmp["bestSummaries"]
mediumSummaries = tmp["mediumSummaries"]
worstSummaries = tmp["worstSummaries"]
bestSummariesBefore = tmp["bestSummariesBefore"]
mediumSummariesBefore = tmp["mediumSummariesBefore"]
worstSummariesBefore = tmp["worstSummariesBefore"]
greedyMinRouge = tmp["greedyMinRouge"]
greedyMaxRouge = tmp["greedyMaxRouge"]
print(dataset)

rc('text', usetex=True)
plt.rc('font', family='times')

# Define model function to be used to fit to the data above:
def gauss(x, *p):
    A, mu, sigma = p
    return A*np.exp(-(x-mu)**2/(2.*sigma**2))

# Function for plotting a summary above some point
def plotSummary(summary, score, label="Summary", starty=1.2, position='top'):
    plt.plot([score, score], [0, starty], color="k", linewidth=2)
    idx = 0
    startx = score - 2.5*imgSize
    if position == 'top':
    	valignment='bottom'
    	starty = starty + imgSize/2
    	textpos = starty+2*imgSize+imgSize/4
    else:
    	valignment='top'
    	starty = starty - 2*imgSize - imgSize/2
    	print(starty)
    	textpos = starty-imgSize/4
    ax.text(score, textpos, label, horizontalalignment="center", verticalalignment=valignment)
    for r in range(0,2):
        for c in range(0,5):
        	img = images[summary[idx]]
        	plt.imshow(img, extent=[startx+imgSize*c,startx+imgSize*(c+1),starty+imgSize*(r+1),starty+imgSize*(r)])
        	idx = idx + 1

# generate scores
#scoresRandom = 0.5+0.1*np.random.standard_normal(10000)
#scoresHuman = 0.7+0.2*np.random.standard_normal(10000)

histRandom, bin_edges = np.histogram(scoresRandom, bins=np.linspace(0,1,100), density=True)
histHuman, bin_edges  = np.histogram(scoresHuman, bins=np.linspace(0,1,100), density=True)

scale = 0.3
imgSize = scale / 10
normFactor = np.max([histRandom.max(), histHuman.max()])
histRandom = histRandom / normFactor * scale
histHuman = histHuman / normFactor * scale
bin_centres = (bin_edges[:-1] + bin_edges[1:])/2
p0 = [histRandom.max(), scoresRandom.mean(), scoresRandom.std()]
coeffRandom, varRandom = curve_fit(gauss, bin_centres, histRandom, p0=p0)
p0 = [histHuman.max(), scoresHuman.mean(), scoresHuman.std()]
coeffHuman,  varHuman  = curve_fit(gauss, bin_centres, histHuman, p0=p0)

# plot axes and histograms
fig = plt.figure(figsize=(15,7))

# show images 
ax1 = plt.subplot2grid((1,3), (0,0), colspan=1)
ax1.set_frame_on(False)
idx = 1
ax1.set_axis_off()
ax1.set_xlim((0,10))
ax1.set_ylim((0,10))
ax1.set_xticklabels([])
ax1.set_yticklabels([])
images = []
for r in range(0,10):
  for c in range(0,10):
    fname = format('../../Data/set%d/%d.jpg' % (dataset, idx))
    idx = idx + 1
    img = mpimg.imread(fname)
    images.append(img)
    plt.imshow(img, extent=[c,(c+1),10-(r),10-(r+1)])

#
ax = plt.subplot2grid((1,3), (0,1), colspan=2)

#img=mpimg.imread('/clusterFS/home/user/stschia/ECCV2014/Data/set1/1.jpg')

print("Hello")
#data = py.hist(scoresRandom, bins = 100)
#x = [0.5 * (data[1][i] + data[1][i+1]) for i in xrange(len(data[1])-1)]
#scoresRandom = data[0]
#plt.plot(x, scoresRandom, lw=4, color="r")
#plt.show()
print("FOO")

# Get the fitted curve
a = 0.3
width = bin_edges[1] - bin_edges[0]
histFitRandom = gauss(bin_centres, *coeffRandom)
histFitHuman = gauss(bin_centres, *coeffHuman)
plt.plot(bin_centres, histFitRandom, color="b", linewidth=2, label='random summaries')
plt.bar(bin_centres, histRandom, width, color="b", alpha=a)
plt.plot(bin_centres, histFitHuman, color="r", linewidth=2, label='human summaries')
plt.bar(bin_centres, histHuman, width, color="r", alpha=a)
#plt.imshow(img, extent=[0.5,0.6,0.1,0.2])

# set x-axis
ax.set_xlim((-0.1,1.1))
ax.set_ylim((-8*imgSize,scale+8*imgSize))
ax.spines['bottom'].set_position(('data',0))
ax.set_xticks([0,1])
ax.set_xticklabels(["$f(\emptyset)=0$", "$f(V)=1$"])
ax.set_frame_on(False)
ax.get_xaxis().tick_bottom()
ax.get_yaxis().set_visible(False)
ax.axhline(linewidth=1, color='k')

# show average random summary
avgHumanSummary = 1+np.array(range(0,10))
avgHumanScore = scoresHuman.mean()
avgRandomSummary = 15+np.array(range(0,10))
avgRandomScore = scoresRandom.mean()

# To replace by real data -- dictionary of results to plot
#summaries = []
#summaries.append({'summary': avgHumanSummary, 'score': avgHumanScore, 'description': r"Average Pruned Human", 'position': 'top'})
#summaries.append({'summary': avgRandomSummary, 'score': avgRandomScore, 'description': r"Average Pruned Random", 'position': 'top'})
#summaries.append({'summary': avgRandomSummary, 'score': 0.1, 'description': r"Constrained Submodular Min", 'position': 'top'})
#summaries.append({'summary': avgRandomSummary, 'score': 0.25, 'description': r"Constrained Submodular Min", 'position': 'bottom'})

idx = 1
bottomidx = 1
topidx = 1
for summary in summaries:
   if summary['position'] == 'top':
   		topidx = topidx + 1
   		y = imgSize + scale + (topidx % 2)*4*imgSize
   else:
   		bottomidx = bottomidx + 1
   		y = -imgSize - (bottomidx % 2)*4*imgSize
   plotSummary(summary['summary'], summary['score'], summary['description'], starty=y, position=summary['position'])
   idx = idx + 1

#plotSummary(avgHumanSummary, avgHumanScore, r"Average Pruned Human", starty=1.2)
#plotSummary(avgRandomSummary, avgRandomScore, r"Average Pruned Random", starty=1.1)

fig.tight_layout(w_pad=1, h_pad=2)
fname = format("images/dataset%d.pdf" % (dataset))
plt.savefig(fname, dpi=150, format="pdf")

# plot best/worst/medium summaries
for i in range(0,6):
   if i == 0:
       summaries = bestSummaries
       fname = format("images/dataset%d_best.jpg" % (dataset))
   elif i == 1:
       summaries = mediumSummaries
       fname = format("images/dataset%d_medium.jpg" % (dataset))
   elif i == 2:
       summaries = worstSummaries
       fname = format("images/dataset%d_worst.jpg" % (dataset))
   elif i == 3:
       summaries = bestSummariesBefore
       fname = format("images/dataset%d_best_before.jpg" % (dataset))
   elif i == 4:
       summaries = mediumSummariesBefore
       fname = format("images/dataset%d_medium_before.jpg" % (dataset))
   elif i == 5:
       summaries = worstSummariesBefore
       fname = format("images/dataset%d_worst_before.jpg" % (dataset))
   print(fname)
   
   fig.clear()
   fig = plt.figure(figsize=(10,3))
   fig.add_axes([0,0,1,1])
   plt.xlim((0,10))
   plt.ylim((0,3))
   plt.axis('off')
   for r in range(0,3):
       for l in range(0,10):
            plt.imshow(images[summaries[r,l]-1], extent=[l,(l+1),r+1,r], interpolation='none')
   plt.savefig(fname, dpi=150, format="jpg", bbox_inches='tight')

# plot best/worst/medium summaries
for i in range(0,3):
   if i == 0:
       summaries = greedyMaxRouge
       fname = format("images/dataset%d_rouge_max.jpg" % (dataset))
   elif i == 1:
       summaries = tmp["avgHumanSummary"]
       fname = format("images/dataset%d_rouge_mediocre.jpg" % (dataset))
   elif i == 2:
       summaries = greedyMinRouge
       fname = format("images/dataset%d_rouge_min.jpg" % (dataset))
   print(fname)
   print summaries
   
   fig.clear()
   fig = plt.figure(figsize=(10,1))
   fig.add_axes([0,0,1,1])
   plt.xlim((0,10))
   plt.ylim((0,1))
   plt.axis('off')
   for r in range(0,1):
       for l in range(0,10):
            plt.imshow(images[summaries[r,l]-1], extent=[l,(l+1),r+1,r], interpolation='none')
   plt.savefig(fname, dpi=150, format="jpg", bbox_inches='tight')

        

