%% Collects all information and prepares for plotting. Further, generates a
%% LaTeX table summarizing all results

clear all;
clc;

%% settings
datasets = 1:14;
nrRandom = 1000; % number of random summaries
overlap = 0.1;

%% setup
cd('..');
addpath('');
addpath('external/sfo');

%% convert
scaledScores = [];
methods = {}; % cell array containing descriptors of the methods
results = []; % matrix for results -- contents must match the methods cell array
for dataset = datasets,
    fprintf(1, '>> Processing dataset %d...\n', dataset);
    
    humanSummaries = pruneSummaries(dataset, get_pruning_info(dataset, overlap));
    randomSummaries = randomSummary(100,10,nrRandom);
    
    bow = get_BOW_evaluation(dataset);

    optionsEvaluation = struct();
    optionsEvaluation.bow = bow;
    optionsEvaluation.summariesAsBow = computeBOWSummary(humanSummaries, bow);
    
    %% scores of human summaries
    [avgScoreHuman,scoresHuman] = summaryEvaluation(humanSummaries, humanSummaries, 'rouge', optionsEvaluation);
    [avgScoreRandom,scoresRandom] = summaryEvaluation(randomSummaries, humanSummaries, 'rouge', optionsEvaluation);

    % compute best/worst/medium summaries
    [~,a] = sort(scoresHuman, 'descend');
    bestSummaries = humanSummaries(a(1:3), :);
    mediumSummaries = humanSummaries(a(floor(length(a)/2):(floor(length(a)/2)+2)), :);
    worstSummaries = humanSummaries(a((end-2):end),:);

    %% summaries 
    summaries = struct();
    sidx = 1;
    
    nrHumanSummariesToShow = 3;
    [a,b] = sort(scoresHuman);
    indicesHumanSummaries = ceil(linspace(1, length(b), nrHumanSummariesToShow));
    for idx = 1:nrHumanSummariesToShow,
        summaryIndex = b(indicesHumanSummaries(idx));
        summaries(sidx).score = a(indicesHumanSummaries(idx));
        summaries(sidx).summary = humanSummaries(summaryIndex,:) - 1;
        summaries(sidx).description = 'Hum Summ';
        summaries(sidx).position = 'top';  
        
        sidx = sidx + 1;
    end
    
    nrRandomSummariesToShow = 3;
    [a,b] = sort(scoresRandom);
    indicesRandomSummaries = ceil(linspace(1, length(b), nrRandomSummariesToShow));
    for idx = 1:nrRandomSummariesToShow,
        summaryIndex = b(indicesRandomSummaries(idx));
        summaries(sidx).score = a(indicesRandomSummaries(idx));
        summaries(sidx).summary = randomSummaries(summaryIndex,:) - 1;
        summaries(sidx).description = 'Rnd Summ';
        summaries(sidx).position = 'bottom';  
        
        sidx = sidx + 1;
    end
    

    %% sort struct according to score (for nice plotting)
    %scores = zeros(length(summaries), 1);
    %for t = 1:length(summaries),
    %    scores(t) = summaries(t).score;
    %end
    %[~,perm] = sort(scores);
    %tsummaries = summaries;
    %for t = 1:length(summaries),
    %    summaries(t) = tsummaries(perm(t));
    %end

    fprintf(1, 'Score human: %f; score random: %f\n', mean(scoresHuman), mean(scoresRandom));

    fname = sprintf('plotting2/dataset%d.mat', dataset);
    save(fname, 'dataset', 'summaries', 'scoresHuman', 'scoresRandom');  
end

cd('plotting2');

