function [] = precomputeCache(configComponentsFile)

%% setup
nrDatasets = 14

addpath('Java/MatlabInterface');
addpath('SubmodularComponents');
javaaddpath('Java/SubmodularFunctions/bin');

if ~exist(configComponentsFile, 'file'),
    error('Configuration file for components not found.');
end

if ~exist('cache', 'dir'),
    mkdir('cache');
end

fprintf(1, 'Setting up cache...\n');
cacheIndex = cell(nrDatasets,0);
cache = {};
components = readComponentsFile(configComponentsFile);
hash = DataHash(components);
cacheFile = strcat('cache/', hash, '.mat');
cacheFile

idx = 1;
for compIdx = 1:length(components)
	if isfield(components(compIdx), 'resource')
	   % TODO: Make better description
	   desc{compIdx} = sprintf('%s: %s', components(compIdx).type, components(compIdx).resources(1).type);
	else
	   desc{compIdx} = components(compIdx).type;
	end
	for ds = 1:nrDatasets,
	    fprintf(1, '[Component %3d] %s\n', compIdx, components(compIdx).type);
	    
	    % command to construct component
	    createStr = sprintf('comp = %sJ(', components(compIdx).type);
	    
	    % load resources
	    nrResources = length(components(compIdx).resources);
	    for resIdx = 1:nrResources,
		if length(fieldnames(components(compIdx).resources)) == 0,
		    break;
		end
		resource = components(compIdx).resources(resIdx);
		res_desc_full = strcat(resource.type, ':', resource.url);
		cacheHit = cellfun(@(x) strcmpi(x, res_desc_full), cacheIndex(ds,:));
		
		if any(cacheHit)
		   fprintf(1, '    Found "%s" in cache (index %d)\n', res_desc_full, find(cacheHit));
		   chidx = find(cacheHit);
		else
		   fprintf(1, '    Loading resource "%s"\n', res_desc_full);
		   tmp = load_resource(resource.type, resource.url, ds);
		   chidx = length(cacheIndex)+1;
		   cacheIndex{ds,chidx} = res_desc_full;
		   
		   if strcmpi(resource.type, 'value') || strcmpi(resource.type, 'QualityMeasure') || strcmpi(resource.type, 'Cluster')
			cache{ds,chidx} = tmp;
		   else
			cache{ds,chidx} = MatrixJ(tmp);
		   end
		end
	    end
	end
	idx = idx + 1;
end

save(cacheFile, 'cache', 'cacheIndex');

end

function [components] = readComponentsFile(configComponentsFile)

components = struct();
fid = fopen(configComponentsFile, 'r');
tline = fgetl(fid);
compIdx = 1;
while ischar(tline)
    if all(isspace(tline)), % skip empty lines
       tline = fgetl(fid);
       continue;
    end

    tline = strtrim(tline);
    if length(tline) < 10 || ~strcmpi('Component:', tline(1:10)),
        error('Invalid configuration: "%s"\n', tline);
    end
    
    tline = strrep(tline, 'Component:', '');
    tline = strtrim(tline);
    
    components(compIdx).type = tline;
    
    % Read in resources (if any)
    resIdx = 1;
    components(compIdx).resources = struct();
    tline = fgetl(fid);
    tline = strtrim(tline);
    while ischar(tline) && length(tline) >= 9 && strcmpi('Resource:', tline(1:9)),
        tline = strtrim(tline);
    	tline = strrep(tline, 'Resource:', '');
        tline = strtrim(tline);
        
        [tok,remainder] = strtok(tline, ':');
        res_type = tok;
        res_url = strtrim(remainder(2:end)); % skip ':'
        components(compIdx).resources(resIdx).type = res_type;
        components(compIdx).resources(resIdx).url = res_url;
        
        resIdx = resIdx + 1;
        
        tline = strtrim(fgetl(fid));
    end
    compIdx = compIdx + 1;
end
fclose(fid);

end

