% precomputes features from all layers using overfeat

clear all
clc


for dataset = 1:14,
    fprintf(1, '>> Dataset %d...\n', dataset);
    clear('labels');

    for image = 1:100,
        fprintf(1, '.', image);
        
        % path to image
        imageFile = sprintf('../Data/large/set%d/%d.jpg', dataset, image);
        if ~exist(imageFile, 'file'),
            error('File ''%s'' does not exist.', imageFile);
        end
        
        % path to features file
        featuresFile = sprintf('../precomputed/features/overfeatClasses/set%d/%d.mat', dataset, image);
        if exist(featuresFile, 'file'),
            fprintf(1, 'skip\n');
            continue;
        end

            % prepare command string
	    classesFile = sprintf('%s.classes', imageFile);
            fh = fopen(classesFile, 'r');
            values = zeros(1001,1);
            readLabels = false;
            if ~exist('labels', 'var')
                labels = cell(1001,1);
                readLabels = true;
            end
            currLabels = cell(1001,1);
            idx = 1;
            while ~feof(fh),
               line = fgetl(fh);
               k = strfind(line, ' ');
               clstr = line(1:k(end));
               valstr = line(k(end):end);
               val = str2num(valstr); 
               values(idx) = val;
               if readLabels,
                   labels{idx} = clstr;
               end
               currLabels{idx} = clstr;
               idx = idx + 1;
            end
	    if readLabels,
	       labels = unique(labels);
	    end

            % map labels to indices
            labelRef = zeros(1001,1);
            for i = 1:length(currLabels),
               ind = find(ismember(labels,currLabels{i}));
	       if isempty(ind),
	          error('index not found');
	       end
	       labelRef(i) = ind;
            end

        features = values(labelRef);
        
        % save features
        direc = fileparts(featuresFile);
        if ~exist(direc, 'dir'),
            mkdir(direc);
        end
        save(featuresFile, 'features');
    end
end
