% precomputes features from all layers using overfeat

clear all
clc

layersRange = 0:21;

for dataset = 1:14,
    fprintf(1, '>> Dataset %d...\n', dataset);
    
    for image = 1:100,
        fprintf(1, '[%3d] ', image);
        
        % path to image
        imageFile = sprintf('../Data/large/set%d/%d.jpg', dataset, image);
        if ~exist(imageFile, 'file'),
            error('File ''%s'' does not exist.', imageFile);
        end
        
        % path to features file
        featuresFile = sprintf('../precomputed/features/overfeat/set%d/%d.mat', dataset, image);
        if exist(featuresFile, 'file'),
            fprintf(1, 'skip\n');
            continue;
        end

        features = cell(length(layersRange), 1);
        idx = 1;
        for layer = layersRange,
            fprintf(1, '.');
            
            % prepare command string
            cmdString = sprintf('LD_LIBRARY_PATH=/usr/lib external/overfeat/src/overfeat -L %d %s', layer, imageFile);
            [~,b] = system(cmdString);
            tmp = strsplit(b, '\n');
            dimsStr = tmp{1};
            weightsStr = tmp{2};
            dims = str2num(dimsStr);
            weights = str2num(weightsStr);
            
            features{idx} = reshape(weights, dims);
            idx = idx + 1;
        end
        fprintf(1, '\n');
        
        % save features
        direc = fileparts(featuresFile);
        if ~exist(direc, 'dir'),
            mkdir(direc);
        end
        save(featuresFile, 'features');
    end
end