% We perform summary pruning. In every step, the worst summary is
% pruned. We continue until there are no more summaries left. In every
% iteration, we record the removed summaries, the scores of the random and
% the human summaries.

clc;
clear all;
addpath('external/sfo');

%% settings
nrRemoveWorst = 1; % number of worst scoring summaries to remove in every iteration
nrAverage = 1000;  % number of random summaries
nrImages = 100;
summaryLength = 10;

for dataset = 1:14,
    fprintf(1, '>> Processing dataset %d...\n', dataset);
    
    %% load summaries and compute scores
    [summaries, nrSummaries] = load_summaries(dataset);
    
    %% compute random summaries
    % rng(123456);
    rand('seed', 123456);
    randomSummaries = randomSummary(nrImages, summaryLength, nrAverage);    
    
    %% get bag-of-words
    bow = get_BOW_evaluation(dataset);    
    optionsEvaluation = struct();
    optionsEvaluation.bow = bow;
    
    %% Run algorithm to get rid of bad human summaries   
    filename = sprintf('%s/summary_pruning_set%d', getPreDir(), dataset);
    if exist(filename, 'file'),
        %warning('File "%s" already exists, skipping.\n=> Skipping this dataset.\n', dirname);
        %continue;
    end

    tSummaries = summaries;
    converged = false;
    iterIdx = 1;
    pruningInformation = cell(size(tSummaries, 1), 1);
    removedSummaries = []; % all removed summaries
    while ~isempty(tSummaries),
        clear('removeSummariesIter', 'greedyMinScore', 'greedyMinSummary', 'greedyMaxSummary', 'greedyMaxScore', 'randScores', 'humanScores');
        pruningInformation{iterIdx} = struct();
        
        if mod(iterIdx, 10) == 1,
            fprintf(1, '%2.2f %%\n', iterIdx / size(summaries,1) * 100);
        end
        
        % compute bag-of-words of human and random summaries
        summariesAsBOW = computeBOWSummary(tSummaries, bow);
        optionsEvaluation.summariesAsBow = summariesAsBOW;

        [randScore, randScores] = summaryEvaluation(randomSummaries, tSummaries, 'rouge', optionsEvaluation);
        [sumScore, humanScores] = summaryEvaluation(tSummaries, tSummaries, 'rouge', optionsEvaluation);

        %% compute submodular min
        %[~,greedyMinScore,greedyMinSummary,~] = mmin(@(x) summaryEvaluation(x, tSummaries, 'rouge', optionsEvaluation), 100, [], 'MUB', 10);
        %[greedyMaxSummary, greedyMaxScore, ~] = sfo_greedy_lazy(sfo_fn_wrapper(@(x) summaryEvaluation(x, tSummaries, 'rouge', optionsEvaluation)), 1:100, 10);

        %% remove summaries
        [a,b] = sort(humanScores, 'ascend');
        toRemove = b(1:min(nrRemoveWorst,length(b)));
        removedSummariesIter = tSummaries(toRemove,:);
        removedSummaries = vertcat(removedSummaries, removedSummariesIter);
        removedSummariesScores = summaryEvaluation(removedSummaries, tSummaries, 'rouge', optionsEvaluation);
        tSummaries(toRemove,:) = [];

        %% print status info
        % scoresStr = sprintf('Human summaries/Random summaries (overlap): %f/%f (%f)\n', mean(humanScores), mean(randScores), computeGaussianOverlap(mean(humanScores), mean(randScores), std(humanScores), std(randScores)));
        %fprintf(1, scoresStr); 
        %fprintf(1, 'Removing elements...');
        %toRemove

        %% Save to file
        pruningInformation{iterIdx}.summaries = tSummaries;
        pruningInformation{iterIdx}.removedSummaries = removedSummaries;
        pruningInformation{iterIdx}.removedSummariesScores = removedSummariesScores;
        pruningInformation{iterIdx}.removedSummariersIter = removedSummaries; 
        %pruningInformation{iterIdx}.greedyMinScore = greedyMinScore;
        %pruningInformation{iterIdx}.greedyMinSummary = greedyMinSummary;
        %pruningInformation{iterIdx}.greedyMaxScore = greedyMaxScore;
        %pruningInformation{iterIdx}.greedyMaxSummary = greedyMaxSummary;
        pruningInformation{iterIdx}.randScores = randScores;
        pruningInformation{iterIdx}.humanScores = humanScores;

        iterIdx = iterIdx + 1;
   
    end
    
    %% save results
    save(filename, 'pruningInformation', 'randomSummaries');
end
