function [] = precompute_BOW(datasets)

setupVLFEAT

if nargin < 1,
    datasets = 1:14;
end

%% setup path
if ~exist('external/gist', 'dir'),
    error('Please download GIST from http://people.csail.mit.edu/torralba/code/spatialenvelope/ to ''external/gist''.');
end
addpath('external/gist');

%%%%% CLUSTER DATA %%%%%%%
% TODO: FIX THIS
fprintf(1, 'Clusterings based on bag-of-words: ');
bowResources = {'superPixels', 'superPixels/tfidf', 'colorHistogram', 'colorHistogram/tfidf', 'overfeat/classAssignments', 'overfeat/slidingOutputWindowAcrossLayers', 'overfeat/slidingOutputWindow', 'overfeat/slidingInputWindow', 'evaluation'};
for bidx = 1:length(bowResources),
    for cutoff = 0.5:0.2:1.1,
	fprintf(1, '.');
	for ds = datasets,
	    % clusters:bow/cutoff%.2f
	    fname = load_resource('cluster', sprintf('%s/cutoff%f', bowResources{bidx}, cutoff), ds, true);
	    if exist(fname, 'file'),
	        fprintf(1, 'Skipping "%s"...\n', fname);
	        continue;
	    end
	    
	    % (1) cluster
	    clusterAssingment = clusterdata(load_resource('bow', bowResources{bidx}, ds), 'distance', 'city', 'cutoff', cutoff);
	    cluster = clusterAssingment;
	    
	    % save
	    if ~exist(fileparts(fname), 'dir'),
		mkdir(fileparts(fname));
	    end
	    save(fname, 'cluster');

	     clear('sim', 'clusters');
	 end
    end
end
fprintf(1, '\n');
%%%%%%%%%%%%

%% similarities for bag-of-words
fprintf(1, 'Similarities for bag-of-words...\n');
bowResources = {'superPixels', 'superPixels/tfidf', 'colorHistogram', 'colorHistogram/tfidf', 'overfeat/classAssignments', 'overfeat/slidingOutputWindowAcrossLayers', 'overfeat/slidingOutputWindow', 'overfeat/slidingInputWindow', 'evaluation'};
for bidx = 1:length(bowResources),
    res_desc = bowResources{bidx};
    for ds = datasets,
	fname = load_resource('similarity', strcat(res_desc, '/histIntersection'), ds, true);
	if exist(fname, 'file') 
	    fprintf(1, 'Skipping "%s"...\n', fname);
	    continue;
	end
	
	%% compute
	bow = load_resource('bow', res_desc, ds);
	sim = BOWsimilarity(bow);
	
	% save
	if ~exist(fileparts(fname), 'dir')
	    mkdir(fileparts(fname));
        end
        save(fname, 'sim');
    end
end

bowResources = {'superPixels', 'superPixels/tfidf', 'colorHistogram', 'colorHistogram/tfidf', 'overfeat/classAssignments', 'overfeat/slidingOutputWindowAcrossLayers', 'overfeat/slidingOutputWindow', 'overfeat/slidingInputWindow', 'evaluation'};
for bidx = 1:length(bowResources),
    res_desc = bowResources{bidx};
    for ds = datasets,
	fname = load_resource('similarity', strcat(res_desc, '/ip'), ds, true);
	if exist(fname, 'file') 
	    fprintf(1, 'Skipping "%s"...\n', fname);
	    continue;
	end
	
	%% compute
	bow = load_resource('bow', res_desc, ds);
	sim = bow'*bow;
	
	% save
	if ~exist(fileparts(fname), 'dir')
	    mkdir(fileparts(fname));
        end
        save(fname, 'sim');
    end
end

%% Bag-of-words used for evaluation
fprintf(1, 'BOW used for evaluation...\n');
for ds = datasets,
    fname = load_resource('bow', 'evaluation', ds, true);
    if exist(fname)
        fprintf(1, 'Skipping computation of "%s"..\n', fname);
        continue;
    end
    
    bow = get_BOW_evaluation(ds);
    
    fprintf(1, 'Saving to "%s"...\n', fname);
    if ~exist(fileparts(fname), 'dir')
	mkdir(fileparts(fname));
    end
    save(fname, 'bow');
end

%% Computing TFIDF variants (all bag-of-words should be computed by now)
bowResources = {'superPixels', 'colorHistogram'};
for i = 1:length(bowResources),
    bowDesc = bowResources{i};
    
    for ds = datasets,
        fname = load_resource('bow', strcat(bowDesc, '/tfidf'), ds, true);
        if exist(fname)
            fprintf(1, 'Skipping computation of "%s"..\n', fname);
            continue;
        end
        
        bow = load_resource('bow', bowDesc, ds);
        bow = applyTFIDF(bow, get_TFIDF(bow));
        
        fprintf(1, 'Saving to "%s"...\n', fname);
        if ~exist(fileparts(fname), 'dir')
            mkdir(fileparts(fname));
        end
        save(fname, 'bow');
    end
end

%% >> Image contrasts and image intensities >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
fprintf(1, 'Computing image quality measures...\n');
for ds = datasets,
    [~, imagesgray] = get_images(ds, false);
    [imageshsv] = get_images(ds, false, 'hsv'); 
    
    contrastsFile = load_resource('qualityMeasure', 'contrasts', ds, true);
    if exist(contrastsFile, 'file'),
        fprintf(1, 'Skipping recompoutation of "%s".\n', contrastsFile);
    else
        if ~exist(fileparts(contrastsFile)),
            mkdir(fileparts(contrastsFile));
        end
        
        contrasts = ones(100,1);
        for img = 1:100,
            contrasts(img) = sum(sum(abs(vl_imgrad(imagesgray(:,:,img)))));
        end
        contrasts = contrasts + abs(min(contrasts)) + mean(abs(contrasts));
        contrasts = contrasts / max(contrasts);
        
        quality = contrasts;
        save(contrastsFile, 'quality');
    end
    
    intesityFile = load_resource('qualityMeasure', 'intensity', ds, true);
    if exist(intesityFile, 'file'),
        fprintf(1, 'Skipping recompoutation of "%s".\n', intesityFile);
    else
        if ~exist(fileparts(intesityFile)),
            mkdir(fileparts(intesityFile));
        end
        
        intensities = zeros(100,1);
        for img = 1:100,
           intensities(img) = sum(sum(squeeze(imageshsv(:,:,3,img))));
        end
        intensities = intensities / max(intensities);
        
        quality = intensities;
        save(intesityFile, 'quality');
    end
    clear('contrasts', 'intensities', 'imagesrgb', 'imagesgray', 'imageshsv', 'imageslab');
end
%% << Image contrasts and image intensities <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

%% GIST
fprintf(1, '>> Precomputing GIST based features...\n');
%precompute_BOW_gist(datasets);

%% SuperPixels
fprintf(1, '>> Precomputing super pixels...\n');
%precompute_BOW_superpixels(datasets);

%% Encoding costs
fprintf(1, '>> Precomputing encoding costs...\n');
%precompute_encodingsCosts(datasets);

%% >> NN activations of image slices using SPARSE FILTERING >>>>>>>>>>>>>>>
% run precompute_overfeat_features.py first
addpath('external/sparseFiltering');
addpath('external/sparseFiltering/common');
nrWords = 1000;
for dataset = datasets,
    fnameBOW = load_resource('bow', 'overfeat/slidingInputWindowSparseFiltering', dataset, true);
    
    % determine matrix dimensions
    %filename = fullfile(getPreDir(), sprintf('features/overfeat/set%d/temp/image_%d.mat', dataset, 1));
    %tmp = dlmread(filename);
    filename = fullfile(getPreDir(), sprintf('features/overfeat/set%d/temp/image_%d.conv.mat', dataset, 1));
    load(filename);
    tmp = data;
    featuresPerImage = size(tmp,1);
    nrFeatures = size(tmp,2);
    clear('tmp');
    
    features = zeros(100*featuresPerImage, nrFeatures);
    idx = 1;
    fprintf(1, '[Loading] ');
    for img = 1:100,
    	fprintf(1, '.');
        % filename = fullfile(getPreDir(), sprintf('features/overfeat/set%d/temp/image_%d.mat', dataset, img));
        filename = fullfile(getPreDir(), sprintf('features/overfeat/set%d/temp/image_%d.conv.mat', dataset, img));
        if ~exist(filename, 'file'),
            filename
            error('File not found.');
        end
        try
            load(filename);
            % features(idx:(idx+featuresPerImage -1),:) = dlmread(filename);
            features(idx:(idx+featuresPerImage -1),:) = data;
        catch ME
            filename
            %keyboard
            error('Error in file found.');
        end
        idx = idx + featuresPerImage;
    end
    fprintf(1, '\n');
    size(features)
    
    % perform sparse filtering
    %%
    % Remove DC
    data = bsxfun(@minus, features', mean(features'));

    % Train Layer 1
    L1_size = nrWords;  % Increase this for more features
    L1 = sparseFiltering(L1_size, data);


    % Feed-forward Layer 1
    data1 = feedForwardSF(L1, data);
    % data1 = bsxfun(@minus, data1, mean(data1));
    
    %%
    %keyboard
    
    %%
    encoderOptions = struct();
    % encoderOptions.encoder = 'nnTh';
    encoderOptions.encoder = 'exp';
    encoderOptions.logSpaceScaling = false;
    encoderOptions.encoderThreshold = 2.0;
    bow = zeros(nrWords, 100);
    fprintf(1, 'Encoding: ');
    idx = 1;
    for img = 1:100,
        fprintf(1, '.');
        iend = idx + featuresPerImage - 1;
        bow(:,img) = sum(data1(:,idx:iend), 2);
        idx = idx + featuresPerImage;
    end
    fprintf(1, '\n');
    
    % (4) normalize bow
    for img = 1:100,
        nrm = norm(bow(:,img), 1);
        if nrm < 1e-6,
            nrm = 1;
        end
        bow(:,img) = bow(:,img) / nrm;
    end
    
    %% save
    if ~exist(fileparts(fnameBOW)),
        mkdir(fileparts(fnameBOW));
    end
    save(fnameBOW, 'bow');
end

%% >> NN activations of image slices >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
% run precompute_overfeat_features.py first
nrWords = 300;
for dataset = datasets,
    fnameBOW = load_resource('bow', 'overfeat/slidingInputWindow', dataset, true);
    
    % determine matrix dimensions
    %filename = fullfile(getPreDir(), sprintf('features/overfeat/set%d/temp/image_%d.mat', dataset, 1));
    %tmp = dlmread(filename);
    filename = fullfile(getPreDir(), sprintf('features/overfeat/set%d/temp/image_%d.conv.mat', dataset, 1));
    load(filename);
    tmp = data;
    featuresPerImage = size(tmp,1);
    nrFeatures = size(tmp,2);
    clear('tmp');
    
    features = zeros(100*featuresPerImage, nrFeatures);
    idx = 1;
    fprintf(1, '[Loading] ');
    for img = 1:100,
    	fprintf(1, '.');
        % filename = fullfile(getPreDir(), sprintf('features/overfeat/set%d/temp/image_%d.mat', dataset, img));
        filename = fullfile(getPreDir(), sprintf('features/overfeat/set%d/temp/image_%d.conv.mat', dataset, img));
        if ~exist(filename, 'file'),
            filename
            error('File not found.');
        end
        try
            load(filename);
            % features(idx:(idx+featuresPerImage -1),:) = dlmread(filename);
            features(idx:(idx+featuresPerImage -1),:) = data;
        catch ME
            filename
            %keyboard
            error('Error in file found.');
        end
        idx = idx + featuresPerImage;
    end
    fprintf(1, '\n');
    size(features)
    
    % normalize activations
    for i = 1:size(features, 1),
        nrm = norm(features(i,:), 1);
        if nrm < 1e-6,
            nrm = 1;
        end
        % features(i,:) = features(i,:) / nrm;
    end
    
    % (3) kmeans + encoding
    features = features';
    fprintf(1, 'Performing kmeans...\n');
    % [centers] = vl_kmeans(features, nrWords, 'algorithm', 'elkan');
    t = size(features);
    t = randperm(t(1));
    centers = features(:,t(1:nrWords));
    
    %%
    encoderOptions = struct();
    % encoderOptions.encoder = 'nnTh';
    encoderOptions.encoder = 'exp';
    encoderOptions.logSpaceScaling = false;
    encoderOptions.encoderThreshold = 2.0;
    bow = zeros(nrWords, 100);
    fprintf(1, 'Encoding: ');
    idx = 1;
    for img = 1:100,
        fprintf(1, '.');
        iend = idx + featuresPerImage - 1;
        bow(:,img) = encode(features(:,idx:iend), centers, encoderOptions.encoder, encoderOptions);
        idx = idx + featuresPerImage;
    end
    fprintf(1, '\n');
    
    % (4) normalize bow
    for img = 1:100,
        nrm = norm(bow(:,img));
        if nrm < 1e-6,
            nrm = 1;
        end
        bow(:,img) = bow(:,img) / nrm;
    end
    
    %% save
    if ~exist(fileparts(fnameBOW)),
        mkdir(fileparts(fnameBOW));
    end
    save(fnameBOW, 'bow');
end

%% << NN activations of image slices <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
return

%% NN activations to BOW (patchwise) %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
layer = 16;
nrClustersDesired = 20;
nrPatchesDesired = 200;
nrWords = 2000;
for dataset = datasets,
    fprintf(1, '[%2d]\n', dataset);
    fnameBOW = load_resource('bow', 'overfeat/slidingOutputWindow', dataset, true);
    
    % load features of images
    fprintf(1, 'Loading features: ');
    allFeatures = cell(100,1);
    for image = 1:100,
        fprintf(1, '.');
        filename = sprintf('../precomputed/features/overfeat/set%d/%d.mat', dataset, image);
        load(filename);
        allFeatures{image} = features{layer}(:);
    end
    fprintf(1, '\n');
    
    %% determine patches for encoding
    % (1) densly extract patches
    patchWidth = 32;
    patchOffset = 8;
    dim = sqrt(length(allFeatures{1}));
    nrSteps = (dim - patchWidth) / patchOffset;
    
    patches = zeros(patchWidth*patchWidth, (nrSteps+1)*(nrSteps+1)*100);
    idx = 1;
    fprintf(1, 'Extracting patches: ');
    for img = 1:100,
        fprintf(1, '.');
        ft = reshape(allFeatures{img}, dim, dim);
        for xoffset = 0:nrSteps,
            for yoffset = 0:nrSteps,
                xstart = 1 + xoffset*patchOffset;
                ystart = 1 + yoffset*patchOffset;
                xend = xstart + patchWidth - 1;
                yend = ystart + patchWidth - 1;
                patches(:,idx) = reshape(ft(xstart:xend, ystart:yend), patchWidth*patchWidth, 1);
                nrm = norm(patches(:,idx));
                if nrm < 1e-6,
                    nrm = 1;
                end
                patches(:,idx) = patches(:,idx) / nrm;
                idx = idx + 1;
            end
        end
    end
    fprintf(1, '\n');
    %%
    
    % (3) kmeans + encoding
    fprintf(1, 'Performing kmeans...\n');
    [centers] = vl_kmeans(patches, nrWords, 'algorithm', 'elkan');
    
    %%
    encoderOptions = struct();
    encoderOptions.encoder = 'nnTh';
    encoderOptions.logSpaceScaling = false;
    encoderOptions.encoderThreshold = 0.75;
    bow = zeros(nrWords, 100);
    fprintf(1, 'Encoding: ');
    for img = 1:100,
        fprintf(1, '.');
        istart = 1 + (nrSteps+1)*(nrSteps+1)*(img-1);
        iend = (nrSteps+1)*(nrSteps+1)*img;
        bow(:,img) = encode(patches(:,istart:iend), centers, encoderOptions.encoder, encoderOptions);
    end
    fprintf(1, '\n');
    
    % (4) normalize bow
    for img = 1:100,
        nrm = norm(bow(:,img), 1);
        if nrm < 1e-6,
            nrm = 1;
        end
        bow(:,img) = bow(:,img) / nrm;
    end
    
    % save
    if ~exist(fileparts(fnameBOW)),
        mkdir(fileparts(fnameBOW));
    end
    save(fnameBOW, 'bow');
    
    % cleanup
    clear('patches', 'allFeatures');
end
return

%% NN activations to BOW %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
layer = 16;
nrClustersDesired = 20;
nrPatchesDesired = 200;
for dataset = datasets,
    fprintf(1, '[%2d]\n', dataset);
    
    % load features of images
    fprintf(1, 'Loading features: ');
    allFeatures = cell(100,1);
    for image = 1:100,
        fprintf(1, '.');
        filename = sprintf('../precomputed/features/overfeat/set%d/%d.mat', dataset, image);
        load(filename);
        allFeatures{image} = features{layer}(:);
    end
    fprintf(1, '\n');
    
    %% determine patches for encoding
    % (1) cluster samples
    fprintf(1, 'Clustering samples.\n');
    ft = zeros(length(allFeatures{1}), 100);
    for img = 1:100,
        ft(:,img) = allFeatures{img};
    end
    cidx = clusterdata(ft', 'maxclust', nrClustersDesired, 'linkage','complete');
    nrClusters = max(cidx);
      
    % (2) cluster features
    fprintf(1, 'Clustering features and extracting patches.\n');
    patches = cell(1, nrClusters*nrPatchesDesired);
    tidx = 1;
    for cl = 1:nrClusters,
        clidx = find(cidx == cl);
        % pidx = clusterdata(ft(:,clidx), 'maxclust', nrPatchesDesired, 'savememory','on');
        pidx = kmeans(ft(:,clidx), nrPatchesDesired, 'emptyaction', 'drop');
        patchIndices = unique(pidx)';
        for p = patchIndices,
            patches{tidx} = find(pidx == p);
            tidx = tidx + 1;
        end
    end
    patches(tidx:end) = [];
    
    % (3) quantize with respect to these patches
    keyboard
    
    %% compute similarties
    sim = NaN(100);
    for i = 1:100,
        for j = 1:i,
            sim(i,j) = exp(-norm(allFeatures{i}/norm(allFeatures{i}) - allFeatures{j}/norm(allFeatures{j}), 2) / 10);
            sim(j,i) = sim(i,j);
        end
    end
    fprintf(1, '\n');
    
    fname = sprintf('%s/features/overfeat_sim_ds_%d.mat', getPreDir(), dataset);
    save(fname, 'sim');
end
return

%% NN activations similarity %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
layer = 20;
for dataset = datasets,
    fprintf(1, '[%2d]\n', dataset);
    
    % load features of images
    fprintf(1, 'Loading features: ');
    allFeatures = cell(100,1);
    for image = 1:100,
        fprintf(1, '.');
        filename = sprintf('../precomputed/features/overfeat/set%d/%d.mat', dataset, image);
        load(filename);
        allFeatures{image} = features{layer}(:);
    end
    fprintf(1, '\n');
    
    %% compute similarties
    sim = NaN(100);
    for i = 1:100,
        for j = 1:i,
            sim(i,j) = exp(-norm(allFeatures{i}/norm(allFeatures{i}) - allFeatures{j}/norm(allFeatures{j}), 2) / 10);
            sim(j,i) = sim(i,j);
        end
    end
    fprintf(1, '\n');
    
    fname = sprintf('%s/features/overfeat_sim_ds_%d.mat', getPreDir(), dataset);
    save(fname, 'sim');
end
return


%% NN classess %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for dataset = datasets,
    fprintf(1, '[%2d]\n', dataset);
    fnameBOW = load_resource('bow', 'overfeat/classAssignments', dataset, true);
    
    % load features of images
    fprintf(1, 'Loading features: ');
    allFeatures = cell(100,1);
    % TODO: SHOULD THIS BE l1 normalzied???
    for image = 1:100,
        fprintf(1, '.');
        filename = sprintf('../precomputed/features/overfeatClasses/set%d/%d.mat', dataset, image);
        load(filename);
        allFeatures{image} = features / norm(features,2);
    end
    fprintf(1, '\n');
    
    %% select subset of activations
    fprintf(1, 'Computing bag-of-words: ');
    bow = NaN(length(allFeatures{1}), 100);
    for image = 1:100,
       bow(:,image) = allFeatures{image};
    end
    
    % save
    if ~exist(fileparts(fnameBOW)),
        mkdir(fileparts(fnameBOW));
    end
    save(fnameBOW, 'bow');
end


%% NN activations %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
nrWords = 500;
layers = 15:19;
nrPercentPerLayer = 0.2; % percent of activations of every layer to use
for dataset = datasets,
    fprintf(1, '[%2d]\n', dataset);
    
    fnameBOW = load_resource('bow', 'overfeat/slidingOutputWindowAcrossLayers', dataset, true);
    
    nrLayers = length(layers);
    nrPerLayer = ceil(nrWords / nrLayers);
    
    % load features of images
    fprintf(1, 'Loading features: ');
    allFeatures = cell(100,1);
    for image = 1:100,
        fprintf(1, '.');
        filename = sprintf('../precomputed/features/overfeat/set%d/%d.mat', dataset, image);
        load(filename);
        allFeatures{image} = features;
    end
    fprintf(1, '\n');
    
    %% select subset of activations
    fprintf(1, 'Computing bag-of-words: ');
    bow = NaN(nrLayers*nrPerLayer, 100);
    w = 1;
    for l = layers,
        fprintf(1, '.');
        sz = numel(allFeatures{1}{l});
        sel = randperm(ceil(sz * nrPercentPerLayer));

        activations = NaN(length(sel), 100);
        for image = 1:100,
            activations(:,image) = allFeatures{image}{l}(sel);
        end

        for image = 1:100,
            activations(:,image) = activations(:,image) / norm(activations(:,image), 1);
        end

        % encode
        p = randperm(100);
        centers = activations(:,p(1:nrPerLayer));
        options = struct();
        options.encoderThreshold = 2;
        options.logSpaceScaling = false;
        for image = 1:100,
            b = encode(activations(:,image), centers, 'nnTh', options);
%             if sum(b) == 0,
%                 b(:) = 1 / length(b);
%             end
            nrm = 1;
            if norm(b,2) > 0,
               nrm = norm(b,2);
            end
            bow(w:(w+nrPerLayer-1), image) = b / nrm;
        end

        w = w + nrPerLayer;
    end
    fprintf(1, '\n');
    
    % save
    if ~exist(fileparts(fnameBOW)),
        mkdir(fileparts(fnameBOW));
    end
    save(fnameBOW, 'bow');
end
return

%% COLOR HISTOGRAM %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for dataset = datasets,
    fprintf(1, '>> Computing color histgrams for dataset %d...\n', dataset);
    
    %% check if already computed
    res_desc = 'colorHistogram';
    fname = fullfile(getPreDir(), 'bow', res_desc, sprintf('%d.mat', ds));
    if exist(fname, 'file'),
        warning('Color Histogram features file "%s" already exists. Skipping.', fname);
        continue;
    end
    
    %% For reproducability
    rand('seed', 123456);
    
    %% load images and summaries
    [imagescolor, imagesgray] = get_images(dataset, false);
    [summaries, nrSummaries] = load_summaries(dataset);
    
    %% Compute and save histograms
    options = struct();
    options.featureType = 'colorHistogram';
    options.clustering = 'kmeans';
    options.nrVisualWords = 128;
    options.normalizeFeatures = 'l1';
    options.dataset = dataset;
    options.normalizeBOW = 'l1';
    options.verbose = true;
    
    options.encoderOptions = struct();
    options.encoderOptions.encoder = 'nn';
    options.encoderOptions.logSpaceScaling = false;

    [bow,fdesc,dictionary] = get_BOW4(imagescolor, imagesgray, options);
    
    save(fname, 'bow', 'fdesc', 'dictionary');
end
% 
% %% CLUSTER color histogram %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% clusterSizes = 5:5:20;
% for dataset = datasets,
%     fprintf(1, '>> Computing clustering of color histograms for dataset %d...\n', dataset);
%     
%     fnameGIST = sprintf('%s/precomputed/colorHistogram_ds_%d.mat', getPreDir(), dataset);
%     if ~exist(fnameGIST, 'file'),
%         error('GIST features have to be precomputed, but "%s" does not exists.', fnameGIST);
%         continue;
%     end
%     
%     load(fnameGIST);
%     for clusterSize = clusterSizes,
%         fprintf('Clustersize: %d\n', clusterSize);
%         
%         fname = sprintf('%s/precomputed/colorHistogram_cluster_ds_%d_size_%d.mat', getPreDir(), dataset, clusterSize);
%         if exist(fname, 'file'),
%             warning('Color histogram cluster file "%s" already exists. Skipping.', fname);
%             continue;
%         end
%         
%         %% For reproducability
%         rand('seed', 123456);
%         
%         %% cluster
%         [clusterCenters, clusterAssignment] = vl_kmeans(bow, clusterSize, 'NumRepetitions', 10);
%         distanceToClusterCenters = vl_alldist(clusterCenters, bow);
%         
%         %% save
%         save(fname, 'clusterCenters', 'clusterAssignment', 'distanceToClusterCenters');
%     end
% end

% for dataset = datasets,
%     fprintf(1, '>> Computing SIFT for dataset %d...\n', dataset);
%     
%     %% set options
%     options = struct();
%     options.clustering = 'random';
%     options.normalizeBOW = 'l1';
%     options.normalizeFeatures = 'l1';
%     % options.summaries = summaries;
%     options.dataset = dataset;
%     options.featureType = 'colorSQ';
%     options.verbose = true;
%     options.nrVisualWords = 5000;
% 
%     optionsEncoder = struct();
%     optionsEncoder.encoder = 'nnTh';
%     optionsEncoder.logSpaceScaling = false;
%     options.encoderOptions = optionsEncoder;
%     
%     %% check if already computed
%     fname = sprintf('%s/precomputed/%s', getPreDir(), get_BOWFilename(dataset, options));
%     if exist(fname, 'file'),
%         warning('Bag-of-words file "%s" already exists. Skipping.', fname);
%         continue;
%     end
%     
%     %% For reproducability
%     rand('seed', 123456);
%     
%     %% load images and summaries
%     [imagescolor, imagesgray] = get_images(dataset, false);
%     [summaries, nrSummaries] = load_summaries(dataset);
%     
%     %% SIFT, 5000 words %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%     % compute BOW and store
%     fprintf(1, 'Precomputing bow to "%s".\n', fname);
%     [bow,fdesc,dictionary] = get_BOW4(imagescolor, imagesgray, options);
%     save(fname, 'bow', 'fdesc', 'dictionary');
% end

%% BOW with different encoder thresholds %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% for dataset = datasets,
%     fprintf(1, '>> Computing SIFT for dataset %d...\n', dataset);
%     
%     for th = 0.25:0.25:1.75,
%         %% set options
%         options = struct();
%         options.clustering = 'random';
%         options.normalizeBOW = 'l1';
%         options.normalizeFeatures = 'l1';
%         % options.summaries = summaries;
%         options.dataset = dataset;
%         options.featureType = 'phow';
%         options.verbose = true;
%         options.nrVisualWords = 1000;
% 
%         optionsEncoder = struct();
%         optionsEncoder.encoder = 'nnTh';
%         optionsEncoder.logSpaceScaling = false;
%         optionsEncoder.encoderThreshold = th;
%         options.encoderOptions = optionsEncoder;
% 
%         %% check if already computed
%         fname = sprintf('%s/precomputed/bow_ds_%d_nr_%d_nnTh_%f.mat', getPreDir(), dataset, options.nrVisualWords, th);
%         if exist(fname, 'file'),
%             warning('Bag-of-words file "%s" already exists. Skipping.', fname);
%             %continue;
%         end
% 
%         %% For reproducability
%         rand('seed', 123456);
% 
%         %% load images and summaries
%         [imagescolor, imagesgray] = get_images(dataset, false);
%         [summaries, nrSummaries] = load_summaries(dataset);
% 
%         %% SIFT, 5000 words %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%         % compute BOW and store
%         fprintf(1, 'Precomputing bow to "%s".\n', fname);
%         [bow,fdesc,dictionary] = get_BOW4(imagescolor, imagesgray, options);
%         save(fname, 'bow', 'fdesc', 'dictionary');
%     end
% end


%%%%%%%%
for dataset = datasets,
    fprintf(1, '>> Computing SIFT for dataset %d...\n', dataset);
    
    %% set options
    options = struct();
    options.clustering = 'random';
    options.normalizeBOW = 'l1';
    options.normalizeFeatures = 'l1';
    % options.summaries = summaries;
    options.dataset = dataset;
    options.featureType = 'phow';
    options.verbose = true;
    options.nrVisualWords = 1000;
    options.appendColorHistogram = true;

    optionsEncoder = struct();
    optionsEncoder.encoder = 'nnSoft';
    optionsEncoder.encoderThreshold = 0.9;
    optionsEncoder.logSpaceScaling = false;
    options.encoderOptions = optionsEncoder;
    
    %% check if already computed
    fname = sprintf('%s/features/%s', getPreDir(), get_BOWFilename(dataset, options));
    if exist(fname, 'file'),
        warning('Bag-of-words file "%s" already exists. Skipping.', fname);
        continue;
    end
    
    %% For reproducability
    rand('seed', 123456);
    
    %% load images and summaries
    [imagescolor, imagesgray] = get_images(dataset, false);
    [summaries, nrSummaries] = load_summaries(dataset);
    
    %% SIFT, 5000 words %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % compute BOW and store
    fprintf(1, 'Precomputing bow to "%s".\n', fname);
    [bow,fdesc,dictionary] = get_BOW4(imagescolor, imagesgray, options);
    save(fname, 'bow', 'fdesc', 'dictionary');
end

%% CLUSTER SIFT
clusterSizes = 5:5:20;
for dataset = datasets,
    fprintf(1, '>> Computing clustering of bag-of-words for dataset %d...\n', dataset);
    
    fnameBOW = sprintf('%s/features/%s', getPreDir(), get_BOWFilename(dataset, options));
    if ~exist(fnameBOW, 'file'),
        error('Bag-of-words files have to be precomputed, but "%s" does not exists.', fnameBOW);
        continue;
    end
    
    load(fnameBOW);
    for clusterSize = clusterSizes,
        fprintf('Clustersize: %d\n', clusterSize);
        
        fname = sprintf('%s/features/bow_cluster_ds_%d_size_%d.mat', getPreDir(), dataset, clusterSize);
        if exist(fname, 'file'),
            warning('Bag-of-words cluster file "%s" already exists. Skipping.', fname);
            continue;
        end
        
        %% For reproducability
        rand('seed', 123456);
        
        %% cluster
        [clusterCenters, clusterAssignment] = vl_kmeans(bow, clusterSize, 'NumRepetitions', 10);
        distanceToClusterCenters = vl_alldist(clusterCenters, bow);
        
        %% save
        save(fname, 'clusterCenters', 'clusterAssignment', 'distanceToClusterCenters');
    end
end

end
