function [] = precompute_BOW_gist(datasets)

%% GIST %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for dataset = datasets,
    fprintf(1, '>> Computing GIST for dataset %d...\n', dataset);
    
    %% check if already computed
    fname = sprintf('%s/features/gist_ds_%d.mat', getPreDir(), dataset);
    if exist(fname, 'file'),
        warning('GIST features file "%s" already exists. Skipping.', fname);
        continue;
    end
    
    %% For reproducability
    rand('seed', 123456);
    
    %% load images and summaries
    [imagescolor, imagesgray] = get_images(dataset, false);
    [summaries, nrSummaries] = load_summaries(dataset);
    
    %% Compute and save GIST
    clear param
    param.imageSize = [300 300]; % set a normalized image size
    param.orientationsPerScale = [8 8 8 8]; % number of orientations per scale (from HF to LF)
    param.numberBlocks = 4;
    param.fc_prefilt = 4;
    nrImages = size(imagescolor,4);

    % Pre-allocate gist:
    Nfeatures = sum(param.orientationsPerScale)*param.numberBlocks^2;
    gist = zeros([size(imagescolor,4) Nfeatures]);

    % Load first image and compute gist:
    [gist(1, :), param] = LMgist(imagescolor(:,:,:,1), '', param); % first call
    % Loop:
    for img = 2:nrImages,
       gist(img, :) = LMgist(imagescolor(:,:,:,img), '', param); % the next calls will be faster
    end

    gist = gist';
    save(fname, 'gist');
end

%% GIST similarity %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
for lambda = 1:1:3,
    res_desc = sprintf('gist/lambda%d', lambda);
    for ds = datasets,
        fnameSIM = load_resource('similarity', res_desc, sprintf('%d.mat', ds), 1);
        if exist(fnameSIM, 'file'),
            fprintf(1, 'File ''%s'' already exists. Skipping.', fnameSIM);
            continue;
        end
        
        % TODO: Load using load_resource
        fnameGIST = sprintf('%s/features/gist_ds_%d.mat', getPreDir(), ds);
        if ~exist(fnameGIST, 'file'),
            error('GIST features have to be precomputed, but "%s" does not exists.', fnameGIST);
        end
        load(fnameGIST, 'gist');
        
        sim = zeros(100);
        for i = 1:100,
            for j = i:100,
                val = exp(-lambda*norm(gist(:,i) - gist(:,j)));
                sim(i,j) = val;
                sim(j,i) = val;
            end
        end
        
        if ~exist(fileparts(fnameSIM), 'dir'),
            mkdir(fileparts(fnameSIM));
        end
        save(fnameSIM, 'sim');
    end
end


%% CLUSTER GIST %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clusterSizes = 5:5:20;
for dataset = datasets,
    fprintf(1, '>> Computing clustering of GIST for dataset %d...\n', dataset);
    
    fnameGIST = sprintf('%s/features/gist_ds_%d.mat', getPreDir(), dataset);
    if ~exist(fnameGIST, 'file'),
        error('GIST features have to be precomputed, but "%s" does not exists.', fnameGIST);
        continue;
    end
    
    load(fnameGIST);
    for clusterSize = clusterSizes,
        fprintf('Clustersize: %d\n', clusterSize);
        
        fname = sprintf('%s/features/gist_cluster_ds_%d_size_%d.mat', getPreDir(), dataset, clusterSize);
        if exist(fname, 'file'),
            warning('GIST cluster file "%s" already exists. Skipping.', fname);
            continue;
        end
        
        %% For reproducability
        rand('seed', 123456);
        
        %% cluster
        [clusterCenters, clusterAssignment] = vl_kmeans(gist, clusterSize, 'NumRepetitions', 10);
        distanceToClusterCenters = vl_alldist(clusterCenters, gist);
        
        %% save
        save(fname, 'clusterCenters', 'clusterAssignment', 'distanceToClusterCenters');
    end
end

end