function [] = precompute_BOW_superpixels(datasets)

%% settings
nrPatchesInDictionary = 200;
sizeColorHistogram = 3*16;
sizeSIFTHistogram = 200;
encoderThreshold = 1;
selectPatchCenters = 'kmeans'; % kmeans

for dataset = datasets,
    fprintf(1, '>> DATASET %d\n', dataset);
    
    %% load images and segmentation (precomputeSegmentation.m)
    [images,imagesgray] = get_images(dataset, false, 'hsv');
    images = single(images);
    imagesagray = single(imagesgray);
    fname = sprintf('%s/Data/set%d/imagesSegmentation.mat', getPreDir(), dataset);
    load(fname);
    imagesLabels = single(imagesLabels);
    imagesSeg = single(imagesSeg);
    
    %% extract transcriptions for all patches
    nrPatches = 0;
    for img = 1:100,
        nrPatches = nrPatches + max(max(imagesLabels(:,:,img)));
    end
    nrPatches
    
    patchHistogramsColor = zeros(sizeColorHistogram, nrPatches, 'single');
    patchHistogramsSIFT = zeros(sizeSIFTHistogram, nrPatches, 'single');
    
    % options for histogram
    optionsColorHistogram = struct();
    optionsColorHistogram.histType = 'SQ';
    optionsColorHistogram.bins = [sizeColorHistogram/3,sizeColorHistogram/3,sizeColorHistogram/3];
    
    % get dictionary for SIFT encoding
    optionsSIFTHistogram = struct();
    optionsSIFTHistogram.featureType = 'phow';
    optionsSIFTHistogram.clustering = 'random';
    optionsSIFTHistogram.nrVisualWords = sizeSIFTHistogram;
    optionsSIFTHistogram.logSpaceScaling = false;
    optionsSIFTHistogram.encoderOptions = struct();
    optionsSIFTHistogram.encoderOptions.encoder = 'none';
    optionsSIFTHistogram.normalizeBOW = 'none';
    optionsSIFTHistogram.normalizeFeatures = 'l1';
    optionsSIFTHistogram.dataset = dataset;
    optionsSIFTHistogram.verbose = true;
    optionsSIFTHistogram.appendColorHistogram = true;
    [~,SIFTDescriptions,SIFTDictionary,SIFTFeatures] = get_BOW4(images, imagesgray, optionsSIFTHistogram);
    
    idx = 1;
    fprintf(1, 'Encoding patches: ');
    for img = 1:100,
        fprintf(1, '.');
        nrPatchesImage = max(max(imagesLabels(:,:,img)));
        ch1 = images(:,:,1,img);
        ch2 = images(:,:,2,img);
        ch3 = images(:,:,3,img);
        
        description = SIFTDescriptions{img};
        SIFTindices = sub2ind([size(images,1), size(images,2)], description(1,:), description(2,:));
        for p = 1:nrPatchesImage,
            indices = find(imagesLabels(:,:,img) == p);
            [c,iA,iB] = intersect(SIFTindices, indices);
            patchHistogramsColor(:,idx) = get_colorHistogramPatch([ch1(indices), ch2(indices), ch3(indices)], optionsColorHistogram);
            
            optionsEncoder = struct();
            optionsEncoder.logSpaceScaling = false;
            optionsEncoder.encoderThreshold = 4;
            patchHistogramsSIFT(:,idx) = encode(SIFTFeatures{img}(:,iA), SIFTDictionary, 'nn', optionsEncoder);
            idx = idx + 1;
        end
    end
    fprintf(1, '\n');
    patchHistogramsColor = bsxfun(@times, patchHistogramsColor, 1./sum(patchHistogramsColor,1));
    nrm = sum(patchHistogramsSIFT,1);
    nrm(nrm < 1e-6) = 1;
    patchHistogramsSIFT = bsxfun(@times, patchHistogramsSIFT, 1./nrm);
    
    %% extract dictionary of patches
    fprintf(1, 'Extracting patch-dictionary using k-means...\n');
    patchHistograms = vertcat(2*patchHistogramsColor, patchHistogramsSIFT);
    patchHistograms = bsxfun(@times, patchHistograms, 1./sum(patchHistograms,1));
    %patchHistograms = patchHistogramsSIFT;
    %clear('patchHisogramsColor', 'patchHistogramsSIFT');
    % TODO: Use Matlab KMEANS
    %pDictionary = vl_kmeans(patchHistograms, nrPatchesInDictionary / 10, 'distance', 'l1', 'algorithm', 'elkan', 'verbose');
    
    if strcmp(selectPatchCenters, 'random'),
        perm = randperm(size(patchHistograms,2));
        pDictionary = patchHistograms(:,perm(1:nrPatchesInDictionary));
    elseif strcmp(selectPatchCenters, 'kmeans'),
        [~,pDictionary] = kmeans(patchHistograms', nrPatchesInDictionary, 'Distance', 'city', 'EmptyAction', 'drop');
        pDictionary = pDictionary';
    else
        error('Invalid dictionary creation method.');
    end
    
    %% encode patches to create BOW
    fprintf(1, 'Encoding patches...\n');
    % encodeError = true;
    % while encodeError,
    idx = 1;
    bow = zeros(size(pDictionary, 2) + 100, 100);
    %     toAdd = [];
    %     encodeError = false;
    %     err = [];
    %     fprintf(1, '-- ROUND --\n');
    for img = 1:100,
        nrPatchesImage = max(max(imagesLabels(:,:,img)));
        tLabels = imagesLabels(:,:,img);
        areas = accumarray(tLabels(:), 1);
        garbage = 0;
        
        % find best Steuerbetrugmatching entry in dictionary
        dists = vl_alldist(pDictionary, patchHistograms(:,idx:(idx+nrPatchesImage-1)), 'l1');
        
        % TODO: If dist is too large, create new center
        [minDist, minDistIdx] = min(dists, [], 1);
        minDist(minDist < encoderThreshold) = 0;
        enc = accumarray(minDistIdx', max(zeros(size(minDistIdx)), encoderThreshold - minDist) * 1/encoderThreshold .* areas');
        if length(enc) < size(pDictionary,2),
            enc = vertcat(enc, zeros(size(pDictionary,2) - length(enc), 1));
        end
        fprintf(1, '[%3d] Encoding error: %f; Threshold not exceeded: #%d\n', img, sum(minDist .* areas') / sum(areas), sum(minDist > encoderThreshold));
        bow(1:(end-100),img) = enc;
        
        tidx = find(minDist > encoderThreshold);
        ctidx = setdiff(1:length(minDist), tidx);
        garbage = sum(minDist(ctidx) * 1/encoderThreshold .* areas(ctidx)');
        garbage = garbage + sum(ones(1,length(tidx)) .* areas(tidx)');
        % bow(size(bow,1)-100+img, img) = garbage;
        %         if ~isempty(tidx),
        %             encodeError = true;
        %             [~,t] = max(minDist);
        %             %toAdd = [toAdd, idx + t - 1];
        %             toAdd = horzcat(toAdd, idx + tidx - 1);
        %             % err = [err,max(minDist)];
        %             err = [err, minDist(tidx)];
        %             %fprintf(1, '[%3d] Adding patch. Max current error: %f\n', img, max(minDist));
        %         end
        %bow(end,img) = sum(minDist(tidx).*areas(tidx)')*encoderThreshold;
        
        idx = idx + nrPatchesImage;
    end
    %     [a,b] = sort(err, 'descend');
    % %     if isempty(toAdd) || length(toAdd) < 5,
    % %         break;
    % %     end
    % %     toAdd = find(err > mean(err) + 0.5*std(err));
    %      max(err)
    % %     [~,tmpDict,~,dists] = kmeans(patchHistograms(:,toAdd)', min(length(toAdd), 100), 'Distance', 'city', 'EmptyAction', 'drop');
    % %     tmpDict = tmpDict';
    % %     min(dists)
    % %     if isempty(tmpDict) || all(min(dists) > encoderThreshold),
    % %         disp('here')
    %         tmpDict = patchHistograms(:,toAdd(b(1:min(5,length(b)))));
    % %     else
    % %         tmpDict = tmpDict(:,logical(min(dists) <= encoderThreshold));
    % %     end
    %     fprintf(1, 'Added %d centers.\n', size(tmpDict,2));
    %     % pDictionary = horzcat(pDictionary, patchHistograms(:,toAdd(b(1:min(5,length(b))))));
    %     pDictionary = horzcat(pDictionary, tmpDict);
    % end
    nrm = sum(bow, 1);
    nrm(nrm < 1e-6) = 1;
    bow = bsxfun(@times, bow, 1./nrm);
    %bow(end,:) = 0;
    
    %% show similarity
    bowTFIDF = bow;
    bowTFIDF(1:(end-100),:) = applyTFIDF(bow(1:(end-100),:), get_TFIDF(bow(1:(end-100),:)));
    nrm = sum(bowTFIDF, 1);
    nrm(nrm < 1e-6) = 1;
    bowTFIDF = bsxfun(@times, bowTFIDF, 1./nrm);
    
    %subplot 211;
    %imagesc(BOWsimilarity(bow));
    %subplot 212;
    %imagesc(BOWsimilarity(bowTFIDF));
    
    %% compute merit measures, i.e. coverage and diversity
    
    %% score
    %bowTFIDF = bow;
    
    
    % summaries = pruneSummaries(dataset,100);
    %
    % perTrain = 0.5;
    % [summariesTrain, summariesTest] = partitionSummaries(summaries, perTrain);
    % randomSummaries = randomSummary(100, 10, 1000);
    %
    % % Compute scores
    % options = struct();
    % options.bow = bowTFIDF; %applyTFIDF(bow, get_TFIDF(bow));
    % options.summariesAsBow = computeBOWSummary(summariesTest, bowTFIDF);
    % [randScore, randScores] = summaryEvaluation(randomSummaries, summariesTest, 'rouge', options);
    % [sumScore, scores] = summaryEvaluation(summariesTrain, summariesTest, 'rouge', options);
    %
    % clear functions;
    % cov = coverage(summaries, bow, ones(size(bow,1), 1));
    % sim = similarity(summaries, bow, ones(size(bow,1),1));
    % fprintf(1, 'BOW: Coverage: %f / Similarity: %f (cov-sim: %f)\n', cov, sim, cov-sim);
    % clear functions;
    % cov = coverage(randomSummaries, bow, ones(size(bow,1), 1));
    % sim = similarity(randomSummaries, bow, ones(size(bow,1),1));
    % fprintf(1, 'RANDOM BOW: Coverage: %f / Similarity: %f (cov-sim: %f)\n', cov, sim, cov-sim);
    % clear functions;
    % cov = coverage(summaries, bowTFIDF, ones(size(bow,1), 1));
    % sim = similarity(summaries, bowTFIDF, ones(size(bow,1),1));
    % fprintf(1, 'BOW-TFIDF: Coverage: %f / Similarity: %f (cov-sim: %f)\n', cov, sim, cov-sim);
    % clear functions;
    % cov = coverage(randomSummaries, bowTFIDF, ones(size(bow,1), 1));
    % sim = similarity(randomSummaries, bowTFIDF, ones(size(bow,1),1));
    % fprintf(1, 'RANDOM BOW-TFIDF: Coverage: %f / Similarity: %f (cov-sim: %f)\n', cov, sim, cov-sim);
    %
    % scoresStr = sprintf('Human summaries/Random summaries: %f (+-%f)/%f (+-%f) (overlap: %f; ratio: %f)\n', sumScore, std(scores), randScore, std(randScores), computeGaussianOverlap(mean(scores), mean(randScores), std(scores), std(randScores)), mean(scores) / mean(randScores));
    % fprintf(1, '**** %s\n', scoresStr);
    %
    % [randScore, randScores] = summaryEvaluation(randomSummaries, summariesTest, 'largestSubset', options);
    % [sumScore, scores] = summaryEvaluation(summariesTrain, summariesTest, 'largestSubset', options);
    %
    % scoresStr = sprintf('Human summaries/Random summaries: %f (+-%f)/%f (+-%f) (overlap: %f; ratio: %f)\n', sumScore, std(scores), randScore, std(randScores), computeGaussianOverlap(mean(scores), mean(randScores), std(scores), std(randScores)), mean(scores) / mean(randScores));
    % fprintf(1, '**** LB: %s\n', scoresStr);
    
    fname = load_resource('bow', 'superpixels', dataset, true);
    mkdir(fileparts(fname));
    save(fname, 'bow');
    
end
end