function [summaries, prunedSummaries] = pruneSummaries(dataset, nrSummariesToPrune)
% PRUNESUMMARIES  Prunes bad scoring summaries
%   SUMMARIES = PRUNESUMMARIES(DATASET, NR) returns the human summaries for
%   dataset DATASET, where the worst NR summaries are pruned. 
%
%   [SUMMARIES, PRUNEDSUMMARIES] = PRUNESUMMARIES(DATASET, NR) also returns
%   the pruned summaries as PRUNEDSUMARRIES.

% (1) compose filename of precomputed data
filename = sprintf('%s/summary_pruning_set%d.mat', getPreDir(), dataset);

% (2) sanity check -- has the precomputation been performed
if ~exist(filename, 'file'),
    error('File ''%s'' does not exist. Did you run precomputePruning?', filename);
end

% (3) load pruning info and return summaries
load(filename);
nrSummaries = length(pruningInformation);
if nrSummaries < nrSummariesToPrune,
    error('There are less summaries available than the number of summaries that should be pruned.');
end

% TODO: REMOVE NEXT LINE
summaries = pruningInformation{nrSummariesToPrune}.summaries;
prunedSummaries = pruningInformation{nrSummariesToPrune}.removedSummaries;

end