function [] = startJobs(configTemplateFile, jobName, variant)

%% are we at ZID or SPSC?
[~,b] = system('hostname');
if strcmpi(strtrim(b), 'dcluster'),
   isZID = true;
else
   isZID = false;
end

%% >> Settings >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
if isZID,
    basePathGeneral = '/home/stschia/ECCV2014/BatchSystem/experiments';
    codePath = '/home/stschia/ECCV2014/Codes';
    precompPath = '/home/stschia/ECCV2014/precomputed';
else
    basePathGeneral = '/clusterFS/home/user/stschia/ECCV2014/BatchSystem/experiments';
    codePath = '/clusterFS/home/user/stschia/ECCV2014/Codes';
    precompPath = '/clusterFS/home/user/stschia/ECCV2014/precomputed';
end
datasets = 1:14;
%% << Settings <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

if nargin <= 2,
    error('At least two arguments must be provided.');
end

if nargin == 2,
    variant = 'single';
end

%% addpath to substitute config
addpath('../general/');

%% settings
basePath = fullfile(basePathGeneral, jobName);
saveResultsPath = fullfile(basePath, 'results');
configPath = fullfile(basePath, 'configs');
logPath = fullfile(basePath, 'logs');
if ~exist(basePath, 'dir')
    fprintf(1, 'Creating folder ''%s''.\n', basePath);
    mkdir(basePath);
end
if ~exist(saveResultsPath, 'dir'),
    fprintf(1, 'Creating folder ''%s''.\n', saveResultsPath);
    mkdir(saveResultsPath);
end
if ~exist(configPath, 'dir'),
    fprintf(1, 'Creating folder ''%s''.\n', configPath);
    [stat,msg] = mkdir(configPath);
    if stat == 0
        msg
        error('failed to create dir "%s"\n', configPath);
    end
end
if ~exist(logPath, 'dir'),
    fprintf(1, 'Creating folder ''%s''.\n', logPath);
    mkdir(logPath);
end

trainingsSetting = {}; % every row corrsponds to a training setting
trSettIdx = 1;
trainingsSetting{trSettIdx,1} = 'OneMinusRouge';
trainingsSetting{trSettIdx,2} = 'modmod';
%trSettIdx = trSettIdx + 1;
%trainingsSetting{trSettIdx,1} = 'OneMinusRouge';
%trainingsSetting{trSettIdx,2} = 'greedy';
%trSettIdx = trSettIdx + 1;
%trainingsSetting{trSettIdx,1} = 'SurrogateRougeLoss';
%trainingsSetting{trSettIdx,2} = 'greedy';

if ~exist(configTemplateFile, 'file'),
    error('Template of job file ''%s'' does not exist.', configTemplateFile);
end
configTemplate = fileread(configTemplateFile);

for settingsIdx = 1:size(trainingsSetting,1),
    for overlap = [0.10],% [0.10, 0.05],
	for dataset = datasets, 
	    lossFunction = trainingsSetting{settingsIdx,1};
	    algorithm    = trainingsSetting{settingsIdx,2};
	    filenameResult = fullfile(saveResultsPath, sprintf('results_dataset_%d_overlap_%1.2f_loss_%s_algorithm_%s.mat', dataset, overlap, lossFunction, algorithm));
	    
	    % check if results already exist -> skip
	    if exist(filenameResult, 'file'),
	        fprintf(1, 'Results file ''%s'' already exists. Skipping.\n', filenameResult);
	        continue;
	    end
	    
	    filenameJob = tempname();
	    filenameConfig = fullfile(configPath, sprintf('dataset%d_%s_%s.m', dataset, algorithm, lossFunction));
	    
	    % create and save config file
	    subst = struct();
	    subst.testDataset = sprintf('%d', dataset);
	    subst.filenameResult = filenameResult;
	    subst.jobname = sprintf('ImageSummarization_single_%d_overlap_%1.2f_loss_%s_algorithm_%s', dataset, overlap, lossFunction, algorithm);
	    subst.overlap = sprintf('%1.2f', overlap);
	    subst.lossFunction = lossFunction;
	    subst.inference = algorithm;
	    subst.basePath = basePath;
	    subst.configFilePath = filenameConfig;
	    
	    if strcmp(variant, 'single'),
	        subst.trainDatasets = sprintf('%d;', dataset);
	    else
	        subst.trainDatasets = sprintf('setdiff(1:14, %d);', dataset);
	    end
	    
	    configString = substituteConfig(configTemplate(:)', subst);
	    fh = fopen(filenameConfig, 'w');
	    if fh == -1,
                error('Could not create "%s".', filenameConfig);
            end
            fprintf(fh, strrep(configString, '%', '%%'));
	    fclose(fh);
	    
	    system(sprintf('screen -S runit -d -m matlabR2013a.afs -nodisplay -r "performMMtrainingAcrossDatasets(''%s'', ''config.components'')"', filenameConfig));
	end
    end
end

end
