function X = sfo_modmodimprovedv2(F, G, V, k, Xinit)
% The modular-modular procedure for maximizing a difference between
% submodular functions. This is a variant (v2) of the procedure, where you mimic the 
% supermodular-submodular algorithm under constraints. Essentially at every round, we perform constrained submodular maximization.

% mode = 1 : Random subgradient
% mode = 2: Efficient subgradient (default)
if nargin == 5,
    X = Xinit;
else
    X = [];
end
TOL = 1e-6;
bestVal = -inf;
% pi = V(randperm(length(V)));
[vecVminusj vecj] = find_vectors_fast(G, V);
H = sfo_fn_lincomb({F, G}, [1, -1]);

while(1)  
    Ga = sfo_fn_wrapper(@(A) mod_uppera(G, A, X, V, vecj) - mod_uppera(G, [], X, V, vecj));
    Gb = sfo_fn_wrapper(@(A) mod_upperb(G, A, X, V, vecVminusj) - mod_upperb(G, [], X, V, vecVminusj));
    FMa = sfo_fn_lincomb({F,Ga},[1,-1]);
    FMb = sfo_fn_lincomb({F,Gb},[1,-1]);
    Aa = greedySummary(FMa, k);
    Ab = greedySummary(FMb, k);
    if (F(Aa) - G(Aa) >= F(Ab) - G(Ab))
        X = Aa;
    else 
        X = Ab;
    end
    currVal = F(X) - G(X);
    disp(sprintf('The value at the current iteration is: %f', currVal));
    if currVal > bestVal + TOL
        bestVal = currVal;
    else
        break;
    end
end

end

function [val] = mod_uppera(F, A, B, V, vecj)
% Produces the modular upper bound-1 of a submodular function F, with
% respect to a set B. 

n = length(V);
val = F(B);
AminusB = sfo_setdiff_fast(A, B);
BminusA = sfo_setdiff_fast(B, A);
val = sum(vecj(AminusB));
currset = F(B);
for i = 1 : length(BminusA)
    j = BminusA(i);
    Bminusj = sfo_setdiff_fast(B, j);
    val = val + F(Bminusj) - currset;
end
end

function [val] = mod_upperb(F, A, B, V, vecVminusj)
% Produces the modular upper bound-1I of a submodular function F, with
% respect to a set B. 

n = length(V);
val = 0;
AminusB = sfo_setdiff_fast(A, B);
BminusA = sfo_setdiff_fast(B, A);
currset = F(B);
val = -sum(vecVminusj(BminusA));
for i = 1 : length(AminusB)
    j = AminusB(i);
    val = val + F([B, j]) - currset;
end
end

function [vecVminusj vecj] = find_vectors_fast(F, V)
n = length(V);
    for j = 1 : n
        Vminusj = sfo_setdiff_fast(V, j);
        vecVminusj(j) = F(V) - F(Vminusj);
        vecj(j) = F(j) - F([]);
    end
end
