function A = sfo_subsupimproveduc(f, g, V, mode, Xinit)
% The Submodular-Supermodular procedure (Narsimhan-Bilmes'05) for
% max_X f(X) - g(X)
% Below is a reimplementation with an efficient heuristic of subgradient

% mode = 1: Random subgradient
% mode = 2: Efficient choice of subgradient (default)
    if nargin == 3
        mode = 2;
        Xinit = [];
    end
    if(nargin == 4)
        Xinit = [];
    end
    X = Xinit;
    if ~exist('opt','var')
        opt = sfo_opt;
    end
    TOL = sfo_opt_get(opt,'ssp_tolerance',1e-6);
    % Reverse the definitions since the assumption is max_X f(X) - g(X),
    % The code below is for min_X F(X) - G(X)
    F = g;
    G = f;
    N = length(V);
    bestVal = inf;
    A = [];
    H = sfo_fn_lincomb({F, G}, [1, -1]);

    while 1
        if mode == 1
            pi = get_rand_perm(V, A);
        else
            pi = get_right_perm(H, V, A);
        end 
        Hw = sfo_ssp_modular_approx(G,pi);
        H = sfo_fn_wrapper(@(A) sum(Hw(sfo_unique_fast(A))));

        FM = sfo_fn_lincomb({F,H},[1,-1]);
        A = sfo_min_norm_point(FM,V,sfo_opt({'minnorm_init',A}));
        curVal = FM(A);
        if curVal<bestVal-TOL
            bestVal = curVal;
        else
            break;
        end    
    end
end

function pi = get_right_perm(H, V, X)
    % The function to get the permutations based on a hueristic 
    % of arranging the elements based on the current gains.
    D = sfo_setdiff_fast(V,X);
    diff = [];
    diffcord = [];
    for j = 1 : length(X)
        diff(j) = H(setdiff(X, X(j))) - H(X);
        diffcord(j) = X(j);
    end
    %[value k] = max(diff);
    %Xk = setdiff(X, X(k));
    addel = [];
    addelcord = [];
    for j = 1 : length(D)
        addel(j) = H([X, D(j)]) - H(X);
        addelcord(j) = D(j);
    end
    [~, diffsortorder] = sort(diff, 'descend');
    [~, addelsortorder] = sort(addel, 'ascend');
    pi = [diffcord(diffsortorder), addelcord(addelsortorder)];
end

function pi = get_rand_perm(V, X)
    D = sfo_setdiff_fast(V,X);
    D = D(randperm(length(D)));
    pi = [X D];
end

function H = sfo_ssp_modular_approx(G,pi)
    H = zeros(1,max(pi(:)));
    W = [];
    oldVal = G(W);
    for i = 1:length(pi)
        newVal = G([W pi(i)]);
        H(pi(i)) = newVal-oldVal;
        oldVal = newVal;
        W = [W pi(i)];
    end
end