function X = sfo_supsubimproveduc(F, G, V, mode, Xinit)
% The supermodular-submodular procedure for maximizing a difference between
% submodular functions, i.e \max_X F(X) - G(X). 
% It performs submodular maximization at every step 
% for maximizing F - mG, where mG is the modular upper bound of G.

% mode = 1: randomized greedy algorithm for submod max (default)
% mode = 2: local search algorithm for submod max
if(nargin == 3)
    mode = 1;
    Xinit = [];
end
if(nargin == 4)
    Xinit = [];
end
X = Xinit(:)';
TOL = 1e-6;
bestVal = F(X) - G(X); % change by Sebastian
disp(sprintf('Initial value: %f', bestVal));
[vecVminusj vecj] = find_vectors_fast(G, V);
while(1)  
    Ga = sfo_fn_wrapper(@(A) mod_uppera(G, A, X, V, vecj));
    Gb = sfo_fn_wrapper(@(A) mod_upperb(G, A, X, V, vecVminusj));
    
    FMa = sfo_fn_lincomb({F,Ga},[1,-1]);
    FMb = sfo_fn_lincomb({F,Gb},[1,-1]);
    if mode == 1
        Aa = sfo_randomgreedymax(FMa, V, 5);
        Ab = sfo_randomgreedymax(FMb, V, 5);
    else
        Aa = sfo_localsearch_lazy(FMa, V);
        Ab = sfo_localsearch_lazy(FMb, V);
    end
    if (F(Aa) - G(Aa) >= F(Ab) - G(Ab))
	X1 = Aa;
    else 
	X1 = Ab;
    end
    currVal = F(X1) - G(X1);
    if currVal > bestVal + TOL
        X = X1;
        bestVal = currVal;
    else
	break;
    end
    disp(sprintf('The value at the current iteration is: %f', currVal));
end

end


function [val] = mod_uppera(F, A, B, V, vecj)
% Produces the modular upper bound-1 of a submodular function F, with
% respect to a set B. 

n = length(V);
val = F(B);
AminusB = sfo_setdiff_fast(A, B);
BminusA = sfo_setdiff_fast(B, A);
val = sum(vecj(AminusB));
currset = F(B);
for i = 1 : length(BminusA)
    j = BminusA(i);
    Bminusj = setdiff(B, j);
    val = val + F(Bminusj) - currset;
end
end

function [val] = mod_upperb(F, A, B, V, vecVminusj)
% Produces the modular upper bound-1I of a submodular function F, with
% respect to a set B. 

n = length(V);
val = 0;
AminusB = sfo_setdiff_fast(A, B);
BminusA = sfo_setdiff_fast(B, A);
currset = F(B);
val = -sum(vecVminusj(BminusA));
for i = 1 : length(AminusB)
    j = AminusB(i);
    val = val + F([B, j]) - currset;
end
end

function [vecVminusj vecj] = find_vectors_fast(F, V)
n = length(V);
    for j = 1 : n
        Vminusj = sfo_setdiff_fast(V, j);
        vecVminusj(j) = F(V) - F(Vminusj);
        vecj(j) = F(j) - F([]);
    end
end