function X = sfo_supsubimproved(F, G, V, mode, f, g, Xinit)
% The supermodular-submodular procedure for maximizing a difference between
% submodular functions, i.e \max_X F(X) - G(X). 
% It performs submodular maximization at every step 
% for maximizing F - mG, where mG is the modular upper bound of G.

% mode = 1: randomized greedy algorithm for submod max (default)
% mode = 2: local search algorithm for submod max
if(nargin == 3)
    mode = 1;
end
X = [];
if nargin == 7,
    X = Xinit;
end

TOL = 1e-6;
bestVal = -inf;
[vecVminusj vecj] = find_vectors_fast(G, V);
ssimpuc = SupSubImproved(f, g, max(V));
while(1)
    %Gaorig = sfo_fn_wrapper(@(A) mod_uppera(G, A, X, V, vecj));
    %Gborig = sfo_fn_wrapper(@(A) mod_upperb(G, A, X, V, vecVminusj));
    modUpperA = ModUpperA(g, X, vecj);
    %Ga = sfo_fn_wrapper(@(A) modUpperA.eval(A));
    modUpperB = ModUpperB(g, X, vecVminusj);
    %Gb = sfo_fn_wrapper(@(B) modUpperB.eval(B));
    
    %FMaorig = sfo_fn_lincomb({F,Ga},[1,-1]);
    %FMborig = sfo_fn_lincomb({F,Gb},[1,-1]);
    FMat = LinCombFunction();
    FMat.addFunction(f);
    FMat.addFunction(modUpperA);
    FMat.setWeights([1,-1]);
    FMbt = LinCombFunction();
    FMbt.addFunction(f);
    FMbt.addFunction(modUpperB);
    FMbt.setWeights([1,-1]);
    FMa = sfo_fn_wrapper(@(A) FMat.eval(A));
    FMb = sfo_fn_wrapper(@(A) FMbt.eval(A));
    
    if mode == 1
        rand('seed', 1234);
        Aa = sfo_randomgreedymax(FMa, V, 5);
        Ab = sfo_randomgreedymax(FMb, V, 5);
    else
        Aa = sfo_localsearch_lazy(FMa, V);
        Ab = sfo_localsearch_lazy(FMb, V);
    end
    if (F(Aa) - G(Aa) >= F(Ab) - G(Ab))
        X1 = Aa;
    else
        X1 = Ab;
    end
    currVal = F(X1) - G(X1);
    if currVal > bestVal + TOL
        X = X1;
        bestVal = currVal;
    else
        break;
    end
    disp(sprintf('The value at the current iteration is: %f', currVal));
end

end

function [vecVminusj vecj] = find_vectors_fast(F, V)
n = length(V);
FV = F(V);
Femp = F([]);
    for j = 1 : n
        Vminusj = sfo_setdiff_fast(V, j);
        vecVminusj(j) = FV - F(Vminusj);
        vecj(j) = F(j) - Femp;
    end
end