function [obj, A, err] = sparseCoding(X, D, lambda)
% performs sparse coding and returns the coeffitients of the encoding 
% as well as the objective of the approximation (not including of course
% the correlation term)

%% settings
minChange = 1e-3;
maxIter = 100;

% TODO: Initial values are chosen as 1/k in the paper (page 952, right
% column, top)
A = ones(size(D,2), size(X, 2));
chg = inf;
iter = 0;
DX = D'*X;
DD = D'*D;
while (chg > minChange) && (iter < maxIter),
    Anew = A .* (DX) ./ (DD*A + lambda);
    
    chg = norm(Anew - A, 'fro') / norm(A, 'fro');
    A = Anew;
    iter = iter + 1;
end

if iter == maxIter,
    % warning('Maximum number of iterations reached in sparse coding.');
end

err = sum(sum((X-D*A).^2));
obj = err + lambda*sum(sum(abs(A)));

end