clear all;
clc;

%% settings
trainDatasets = 14;
testDataset = 14;
nrPatches = 1000;
patchSize = 15;
enorm = 10 / (255^2);
ezca = 0.01;

%% extract random patches of size 15x15 from images
nrPerDataset = ceil(nrPatches / length(trainDatasets));
nrPerImage = ceil(nrPerDataset / 100);
nrPatches = nrPerImage * 100 * length(trainDatasets);
nrPatches

fprintf(1, 'Extracting patches: ');
patches = zeros(3*patchSize^2, nrPatches);
idx = 1;
for ds = trainDatasets,
    images = get_images(ds, false);
    dim = size(images,1);
    fprintf(1, '.');
    for img = 1:100,
        for n = 1:nrPerImage,
            x = ceil(rand(1) * (dim - patchSize));
            y = ceil(rand(1) * (dim - patchSize));
            tpatch = images((x:(x+patchSize-1)), (y:(y+patchSize - 1)), :, img);
            patches(:,idx) = tpatch(:);
            idx = idx + 1;
        end
    end
end
fprintf(1, '\n');

%% display random subset of patches
nrToShow = 100;
nrPerCol = 10;
perm = randperm(nrPatches);

idx = 1;
clf;
col = 1;
row = 1;
for k = 1:nrToShow,
    if row > nrPerCol,
        row = 1;
        col = col + 1;
    end
    subplot(nrPerCol,ceil(nrToShow / nrPerCol),idx);
    imshow(reshape(patches(:,perm(idx)), patchSize, patchSize, 3));
    
    row = row + 1;
    idx = idx + 1;
end

%% whiten data and redisplay
fprintf(1, 'Normalizing input...\n');
for p = 1:size(patches, 2),
    tpatch = patches(:,p);
    tpatch = (tpatch - mean(tpatch)) / (sqrt(var(tpatch) + enorm));
    patches(:,p) = tpatch;
end

%% show again
nrToShow = 100;
nrPerCol = 10;
perm = randperm(nrPatches);

idx = 1;
clf;
col = 1;
row = 1;
for k = 1:nrToShow,
    if row > nrPerCol,
        row = 1;
        col = col + 1;
    end
    subplot(nrPerCol,ceil(nrToShow / nrPerCol),idx);
    imshow(reshape(patches(:,perm(idx)), patchSize, patchSize, 3));
    
    row = row + 1;
    idx = idx + 1;
end

%% whitening input
fprintf(1, 'Whitening input...\n');
[V,D] = eig(cov(patches'));
TZCA = V*((D + ezca*eye*(size(D,1)))^(-1/2))*V';
for p = 1:size(patches, 2),
    patches(:,p) = TZCA*patches(:,p);
end

%% display again
nrToShow = 100;
nrPerCol = 10;
perm = randperm(nrPatches);

idx = 1;
clf;
col = 1;
row = 1;
for k = 1:nrToShow,
    if row > nrPerCol,
        row = 1;
        col = col + 1;
    end
    subplot(nrPerCol,ceil(nrToShow / nrPerCol),idx);
    imshow(reshape(patches(:,perm(idx)), patchSize, patchSize, 3));
    
    row = row + 1;
    idx = idx + 1;
end

%% run kmeans
nrIter = 10;
nrCenters = 1000;
perm = randperm(size(patches,2));

% TODO: Implement better initalization
D = patches(:,perm(1:nrCenters));

S = zeros(nrPatches, nrCenters);
for iter = 1:nrIter,
    T = D'*patches;
    [a,b] = max(T, [], 1);
    for p = 1:size(patches,2),
        S(p,b(p)) = a(p);
    end
    D = patches*S + D;
    
    % renormalize dict
    for k = 1:nrCenters,
        D(:,k) = D(:,k) / norm(D(:,k),2);
    end
end

%% display dict
% nrToShow = size(D,2);
% nrPerCol = 10;
% 
% idx = 1;
% clf;
% col = 1;
% row = 1;
% for k = 1:nrToShow,
%     if row > nrPerCol,
%         row = 1;
%         col = col + 1;
%     end
%     subplot(nrPerCol,ceil(nrToShow / nrPerCol),idx);
%     t = reshape(D(:,idx), patchSize, patchSize, 3);
%     imagesc(t(:,:,1));
%     
%     row = row + 1;
%     idx = idx + 1;
% end

%% encode images
images = get_images(testDataset, false);
offset = 5;
threshold = 0.25;

% compute number of patches in there
nrPatchesImage = (((dim - patchSize) / offset))^2;
nrPatchesDim = sqrt(nrPatchesImage);
nrPatchesImage

imagePatches = zeros(nrPatchesDim, nrPatchesDim, nrCenters, 100);
fprintf(1, 'Encoding patches: ');
for img = 1:100,
    fprintf(1, '.');
    xstart = 1;
    for xp = 1:nrPatchesDim,
        xstart = xstart + offset;
        ystart = 1;
        for yp = 1:nrPatchesDim,
            xend = xstart + patchSize - 1;
            ystart = ystart + offset;
            yend = ystart + patchSize - 1;
            tpatch = images(xstart:xend, ystart:yend, :, img);
            tpatch = tpatch(:);
            tpatch = (tpatch - mean(tpatch)) / (sqrt(var(tpatch) + enorm));
            tpatch = TZCA*tpatch;
            
            % encode
            imagePatches(xp,yp,:,img) = max(zeros(nrCenters,1), D'*tpatch/patchSize - threshold);
        end
    end
end
fprintf(1, '\n');

%% pool regions
bow = zeros(4*nrCenters, 100);
for img = 1:100,
    t = imagePatches(1:10,1:10,:,img);
    t = squeeze(max(t, [], 1));% reshape(t, 100, 300);
    t = squeeze(max(t, [], 1));
    bow(1:1000,img) = max(t, [], 1);
    
    t = imagePatches(11:20,1:10,:,img);
    t = squeeze(max(t, [], 1));% reshape(t, 100, 300);
    t = squeeze(max(t, [], 1));
    bow(1001:2000,img) = max(t, [], 1);
    
    t = imagePatches(1:10,11:20,:,img);
    t = squeeze(max(t, [], 1));% reshape(t, 100, 300);
    t = squeeze(max(t, [], 1));
    bow(2001:3000,img) = max(t, [], 1);
    
    t = imagePatches(11:20,11:20,:,img);
    t = squeeze(max(t, [], 1));% reshape(t, 100, 300);
    t = squeeze(max(t, [], 1));
    bow(3001:400,img) = max(t, [], 1);
end

