clear all;
clc;

rand('seed', 123456);
ds = 6;

load(sprintf('data%d.mat', ds));

% drop to good random summaries
mH = mean(outputHuman);
maxH = max(outputHuman);
mR = mean(outputRandom);
minR = min(outputRandom);
th = 0.5*(mH+mR);
%th = maxH;
% th = 0;
idx = outputRandom > th;
outputRandom = outputRandom(idx);
inputRandom = inputRandom(idx,:);

% reduce random data to amount of human data
t = randperm(length(outputRandom));
t = t(1:length(outputHuman));
outputRandom = outputRandom(t);
inputRandom = inputRandom(t,:);

% create training data
output = [ones(size(outputHuman)); -ones(size(outputRandom))];
input = [inputHuman; inputRandom];

% create training and test set
t = randperm(length(output));
t = t(1:ceil(length(t)*0.7));
tC = setdiff(1:length(output), t);
outputTrain = output(t);
inputTrain = input(t,:);
outputTest = output(tC);
inputTest = input(tC,:);

options = svmsmoset('MaxIter', 10000000, 'Display', 'iter');
svm = svmtrain(inputTrain, outputTrain, 'KERNEL_FUNCTION', 'linear', 'AUTOSCALE', false, 'METHOD', 'SMO', 'SMO_OPTS', options);

%% classify data
groupsHuman = svmclassify(svm, inputHuman);
accHuman = mean(ones(size(outputHuman)) == groupsHuman);
groupsTrain = svmclassify(svm, inputTrain);
accTrain = mean(outputTrain == groupsTrain);
groupsTest = svmclassify(svm, inputTest);
accTest = mean(outputTest == groupsTest);
fprintf(1, 'Accuracy Train: %f, Accuracy Test: %f\n', accTrain, accTest);

% compute baseline performance
[a,b] = hist(output, [-1,1]);
trivAcc = max(a) / sum(a);
fprintf(1, 'Simple maj voting: Acc=%f\n', trivAcc);

%% consider scores of incorrectly classified summaris
mH
maxH
mR
minR
incorrectIdx = find(groupsHuman ~= ones(size(outputHuman)));
sc = outputHuman(incorrectIdx);
mI = mean(sc);
mI

%% perform least squares fitting
nrFun = size(inputHuman,2);
% w = lsqnonneg([inputHuman;inputRandom], [outputHuman;outputRandom]);
options = optimset('algorithm', 'interior-point');
options = optimset(options, 'display', 'iter');
options = optimset(options, 'MaxFunEvals', 5000);
w = fmincon(@(x) norm([inputHuman;inputRandom]*x - [outputHuman;outputRandom], 1) + 1e-2*norm(x), 0.1*rand(nrFun,1), [], [], [], [], zeros(nrFun,1), 10*ones(nrFun,1), [], options);
x = outputHuman;
y = inputHuman*w;
x2 = outputRandom;
y2 = inputRandom*w;
scatter(x,y, 'g');
hold on;
scatter(x2,y2, 'r');
a = min(min([x;x2]), min([y;y2]));
b = max(max([x;x2]), max([y;y2]));
xlim([a,b]);
ylim([a,b]);
hold on;
line([a,b], [a,b]);

xlabel('Human');
ylabel('Mapped');

